#!/usr/bin/perl -w

use strict;
use FileHandle;
use MyUtils;
use As;

my $prefix=$ARGV[0] || die "need a prefix for the outputfiles\n";
my $dir=$ARGV[1] || die "need directory with genomic sequence\n";
my $exonOnly=$ARGV[2];
my %loc;
my %genes;
my %ghelp;
#my $ILEN=300;
while(my $line=<STDIN>) {
  chomp($line);
  my $id="";
  my $classd="";
  #print "class: $classd\n";
  if( $line =~ /^(CG[0-9]+)/ ) {
    $id=$1;
  }
  my @exons;
  #print "line: $line\n";
  my $tmp=(split(/_/,$line))[1];
  my $chr=(split(/:/,$tmp))[0];
  if( !$chr || !$id ) {
    print "id: $id, chr: $chr\n";
    print "problem: $line\n";
    exit(1);
  }
  $loc{$id} = $chr;
  $genes{$id} .= "#" unless !$genes{$id};
  $genes{$id} .= $line;
  $ghelp{$id} .= "#" unless !$ghelp{$id};
}

writeFasta2($dir,\%loc,\%genes,$prefix,\%ghelp,$exonOnly);

sub writeFasta2 {
  my ($dir,$locRef,$gref,$prefix,$ghelpRef,$exonOnly) = @_;
  my %genome = (
           "2L" => "${dir}dmel-2L-chromosome-r4.2.1.fasta",
           "2R" => "${dir}dmel-2R-chromosome-r4.2.1.fasta",
           "3L" => "${dir}dmel-3L-chromosome-r4.2.1.fasta",
           "3R" => "${dir}dmel-3R-chromosome-r4.2.1.fasta",
           "4" => "${dir}dmel-4-chromosome-r4.2.1.fasta",
           "X" => "${dir}dmel-X-chromosome-r4.2.1.fasta"
	       );

  my %loc = %$locRef;
  my %genes = %$gref;
  my $fh = FileHandle::new();
  my $file = "$prefix";
  $fh->open(">$file") || die "failed to write file\n";
  my $last_chr="";
  my $seq="";
  foreach my $gid (sort { $loc{$a} cmp $loc{$b} } keys %loc) {
    if( !$genes{$gid}) {
      next;
    }
    #print "hello? $gid [$genes{$gid}]\n";
    my $chr=$loc{$gid};
    if( !($chr eq $last_chr) ) {
      #print "get_seq: $chr $last_chr\n";
      $seq = get_seq($genome{$chr});
    }
    $last_chr = $chr;
    my @exons=split(/\#/,$genes{$gid});
    for(my $iter=0; $iter <= $#exons; $iter++) {
      my $exon=$exons[$iter];
      mywrite($fh,\$seq,$exon,$chr,$exonOnly);
    }
  }
  $fh->close();
}

sub mywrite {
  my ($fh2,$seq,$pline,$chr,$exonOnly) = @_;
  #print "pl: $pline\n";
  my $strnd="";
  if( $pline =~ /_.{0,2}:{0,1}([+-])_/ ) {
    $strnd=$1;
  }
  my ($leftOff,$rghtOff)=(400,400);
  if( $exonOnly ) {
    ($leftOff,$rghtOff)=(1,1);
  }
  my ($left,$rght) = As::getExonBoundary($pline,$leftOff,$rghtOff);
  writeRange($seq,$left,$rght,$strnd,$pline,$fh2);
}

sub writeRange {
  my($seq,$beg,$end,$strnd,$header,$fh) = @_;
  my $len=abs($beg-$end)+1;
  my $str = substr($$seq,$beg-1,$len);
  if( $len  < 13 ) {
    print "warning really short exon: [$len] - may have some blast issues\n ";
    #print "param: $beg-1,$len ";
    #print "$header\n";
    #return;
    #exit(1);
  }
  if( $strnd eq "-" ) {
    $str =~ tr/ACGTUuacgtyrkm/TGCAAatgcarymk/;
    $str = reverse($str);
  }
  writeFasta(\$str,$header,$fh);
}

sub get_seq {
  my ($file) = @_;
  open(FILE,$file) || die "unable to open [$file]\n";
  <FILE>;
  my $seq="";
  while(my $line = <FILE>) {
    chomp($line);
    $seq .= $line;
  }
  close(FILE);
  return $seq;
}
