#!/usr/bin/perl -w

use strict;

# reads version 4.2.1
my %locus;
my %exon;
my %cds;
my %chrHash;

my %useExonLst;
my $exonLst = $ARGV[0] || die "need list of CG identifiers\n";
my $outFile = $ARGV[1] || die "need multi fasta output file where extracted sequences go\n";
my $dir = $ARGV[2] || die "need directory where D. melanogaster chromosome files are located\n";
if( $exonLst ) {
  %useExonLst = loadExonLst($exonLst);
}

while(my $line = <STDIN>) {
  chomp($line);
  if( $line =~ /^#/ ) {
    next;
  }
  my $chr=(split(/\t/,$line))[0];
  my $type=(split(/\t/,$line))[2];
  my $idval=(split(/\t/,$line))[8];
  if( $type eq "gene" && $idval =~ /ID=(C[GR]\w+);/ ) {
    my $gid=$1;
    $locus{$gid}.="$line\n";
    $chrHash{$gid}=$chr;
  } elsif( $type eq "gene" ) {
    # rna
    # print "RNA: [$line]\n";
  }
  if( $type =~ /CDS/ && $idval =~ /ID=(C[GR]\w+)-/ ) {
    my $gid=$1;
    $cds{$gid}.="$line\n";
  }
  elsif( $type =~ /CDS/ ) {
    print "error: [$line]\n";
  }
  if( $type eq "exon" && $idval =~ /ID=(C[GR]\w+):/ ) {
    my $gid=$1;
    $exon{$gid}.="$line\n";
  } elsif( $type eq "exon" ) {
    # rna
    # print "RNA2: [$line]\n";
  }
}

my $total=0;
my $valid=0;
my $result="";
foreach my $lid (keys %locus) {
  if( $cds{$lid} && $exon{$lid} ) {
    my $chr = $chrHash{$lid};
    my @cds_coords = countOrfs($cds{$lid});
    my %seKey;
    foreach my $coords (@cds_coords) {
      my @tv=split(/:/,$coords);
      my @ids=split(/ /,$tv[1]);
      my ($c5,$c3)=split(/ /,$tv[0]);
      my $val = clusterExons($exon{$lid},$c5,$c3,$tv[1],\%seKey,$chr,$exonLst,\%useExonLst);
      if( $val ) {
	$result .= "$val";
      }
    }
    $total++;
  } else {
    #print "non-coding gene? [$lid]\n";
  }
}

chomp($result);
my $cmd = "echo \"$result\" | exalt_getseq_dmel.pl $outFile $dir";
print "here: $cmd\n";
system($cmd);

sub countOrfs {
  my ($vals) = @_;
  my @vals=split(/\n/,$vals);
  my %coords;
  foreach my $line (@vals) {
    my @tv=split(/\t/,$line);
    my $id="$tv[3] $tv[4]";
    #print "$id\n";
    my $gid="";
    if( $tv[8] =~ /ID=(CG\d+-\w+)/ ) {
      $gid=$1;
      $gid =~ s/-\w/-R/;
    }  else {
      print "parse error\n";
      exit(0);
    }
    if( !$coords{$id} ) {
      $coords{$id}=$gid;
    } else {
      $coords{$id}.=" $gid";
    }
  }
  my $norf=0;
  my @crds;
  foreach my $tv (keys %coords) {
    $norf++;
    push(@crds,"${tv}:$coords{$tv}");
  }
  return @crds;
}

sub isIn {
  my ($pos,$left,$rght) = @_;
  my $status=0;
  if( $pos >= $left && $pos <= $rght ) {
    $status=1;
  }
  return $status;
}

sub isOverlap {
  my ($e1,$e2) = @_;
  my ($e51,$e31)=split(/ /,$e1);
  my ($e52,$e32)=split(/ /,$e2);
  my $status = 0;
  if( isIn($e51,$e52,$e32) || isIn($e31,$e52,$e32) ||
      isIn($e52,$e51,$e31) || isIn($e32,$e51,$e31) ) {
    $status=1;
  }
  return $status;
}



sub isSplice {
  my ($splice,$ids,$genesRef) = @_;
  my @id=split(/ /,$ids);
  my $pass=1;
  foreach my $lid (@id) {
    my @vals = split(/ /,$genesRef->{$lid});
    if( $splice == $vals[1] || $splice == $vals[$#vals] ) {
      $pass=0;
      last;
    }
  }
  return $pass;
}

sub clusterExons {
  my ($exons,$cEnd5,$cEnd3,$idLst,$sKey,$chr,$useLst,$exonLst) = @_;
  my $ret="";
  my @lines = split(/\n/,$exons);
  my $gid="";
  my %tid;
  my %ex2id;
  my %isoform;
  my %strndHash;
  #print "$idLst\n";
  foreach my $line (@lines) {
    my @vals=split(/\t/,$line);
    my ($strnd,$end5,$end3) = ($vals[6],$vals[3],$vals[4]);
    my $tid=$vals[8];
    my @tv=split(/;/,$tid);
    foreach my $id (@tv) {
      if( $id =~ /Parent=/ ) {
	while($id =~ /(CG\d+-\w+)/g ) {
	  my $lid=$1;
	  if( $lid=~ /(\w+)-/ ) {
	    $gid=$1;
	  }
	  $strndHash{$gid}=$strnd;
	  #print "look: [$lid]\n";
	  if( $idLst !~ /$lid/ ) {
	    next;
	  }
	  my $ec="$end5 $end3";
	  if( !$tid{$lid} ) {
	    $tid{$lid}=1;
	  }
	  if( !$isoform{$lid} ) {
	    $isoform{$lid} = "$strnd $end5 $end3";
	  } else {
	    $isoform{$lid} .= " $end5 $end3";
	  }
	  if( !$ex2id{$ec}) {
	    $ex2id{$ec} = "$lid";
	  } else {
	    $ex2id{$ec} .= " $lid";
	  }
	}
      }
    }
  }
  my %frameHash;
  foreach my $exon (keys %ex2id) {
    my @ids = split(/ /,$ex2id{$exon});
    foreach my $id (@ids) {
      my $gene = $isoform{$id};
      my $predFrame = getFrame($gene,$exon,$cEnd5,$cEnd3);
      $frameHash{"$exon $id"} = $predFrame;
    }
  }
  my @sexons = sort { (split(/ /,$a))[0] <=> (split(/ /,$b))[0] } keys %ex2id;
  #print "huh [@sexons]\n";
  my $exonStr="";
  my %sInv;
  my $currInv="";
  while( $#sexons >= 0 ) {
    my $currExon = shift @sexons;
    my ($e5,$e3) = split(/ /,$currExon);
    #print "check: [$currInv] [$currExon], [$exonStr]\n";
    if( $currInv && isOverlap($currExon,$currInv) ) {
      $currInv = updateInv($currExon,$currInv);
      $exonStr .= ":$currExon";
      #print "update cluster [$currInv][$exonStr]\n";
    } else {
      $sInv{$currInv} = $exonStr if($currInv);
      #print "new cluster\n";
      $currInv = $currExon;
      $exonStr = $currExon;
    }
    if( $#sexons < 0 ) {
      $sInv{$currInv} = $exonStr;
    }
  }
  my @sinvs = sort { (split(/ /,$a))[0] <=> (split(/ /,$b))[0] } keys %sInv;
  #print "crap $gid [@sinvs]\n";
  my $strnd = $strndHash{$gid};
  for(my $iter = 0; $iter <=$#sinvs; $iter++) {
    my $key = $sinvs[$iter];
    my $cluster = $sInv{$key};
    if( dirtyCluster($cluster,$sKey) ) {
      next;
    }
    my $lcluster = "";
    if( $iter > 0 ) {
      $lcluster = $sInv{$sinvs[$iter-1]};
    }
    my $rcluster = "";
    if( $iter < $#sinvs ) {
      $rcluster = $sInv{$sinvs[$iter+1]};
    }
    my $inExonLst=0;
    if( $useLst ) {
      my @exons = split(/:/,$cluster);
      foreach my $exon (@exons) {
	my $id = "$gid";
	#print "match? $id\n";
	if( $exonLst->{$id} ) {
	  #print "hit: [$id]\n";
	  $inExonLst=1;
	  last;
	}
      }
    } else {
      $inExonLst=1;
    }
    my $isPrc = spliceIsInPrc($cluster,$cEnd5,$cEnd3,\%ex2id,\%isoform);
    my $cnt=0;
    my $sid="";
    #print "status: $cluster [$isPrc] [$inExonLst]\n";
    if( $isPrc && $inExonLst ) {
      my $res = printClusters($lcluster,$cluster,$rcluster,\%ex2id,\%frameHash,$strnd);
      $res =~ s/ /_/g;
      #print "${gid}_${chr}:${strnd}_CLASS:_CS_$res\n";
      $ret .= "${gid}_${chr}:${strnd}_CLASS:_CS_$res\n";
    }
  }
  return $ret
}

sub markCluster {
  my ($cluster,$sKey) = @_;
  my @exons=split(/:/,$cluster);
  foreach my $exon (@exons) {
    $sKey->{$exon}=1;
  }
}

sub dirtyCluster {
  my ($cluster,$sKey) = @_;
  my $dirty=0;
  my @exons=split(/:/,$cluster);
  foreach my $exon (@exons) {
    if($sKey->{$exon}) {
      $dirty=1;
      last;
    }
  }
  return $dirty;
}

sub spliceIsInPrc {
  my ($cluster,$cEnd5,$cEnd3,$ex2id,$isoform) = @_;
  my $isTrue=1;
  my @exons=split(/:/,$cluster);
  my %cnt;
  foreach my $exon (@exons) {
    my ($end5,$end3)=split(/ /,$exon);
    my $splice5 = isSplice($end5,$ex2id->{$exon},$isoform);
    my $splice3 = isSplice($end3,$ex2id->{$exon},$isoform);
    if( ($splice5 && ($end5 < $cEnd5 || $end5 > $cEnd3 ) ) ||
	($splice3 && ($end3 < $cEnd5 || $end3 > $cEnd3 ) ) ) {
      $isTrue=0;
      last;
    }
  }
  return $isTrue;
}

sub updateInv {
  my ($cExon,$cInv) = @_;
  my ($e5,$e3) = split(/ /,$cExon);
  my ($i5,$i3) = split(/ /,$cInv);
  my $n5=$i5;
  if( $e5 < $i5 ) {
    $n5=$e5;
  }
  my $n3=$i3;
  if( $e3 > $i3 ) {
    $n3=$e3;
  }
  return "$n5 $n3";
}

sub compressInv {
  my ($cExon,$cInv) = @_;
  my ($e5,$e3) = split(/ /,$cExon);
  my ($i5,$i3) = split(/ /,$cInv);
  my $n5=$i5;
  if( $e5 > $i5 ) {
    $n5=$e5;
  }
  my $n3=$i3;
  if( $e3 < $i3 ) {
    $n3=$e3;
  }
  return "$n5 $n3";
}

sub multipleExonsPerTrans {
  my ($cls,$exRef) = @_;
  my $fail=0;
  my @exons=split(/:/,$cls);
  my %cnt;
  foreach my $exon (@exons) {
    my @ids=split(/ /,$exRef->{$exon});
    foreach my $id (@ids) {
      if( $cnt{$id} ) {
	$fail=1;
	last;
      } else {
	$cnt{$id}=1;
      }
    }
    if( $fail ) {
      last;
    }
  }
  return $fail;
}

sub transClusterIn {
  my ($elem,$set,$ex2id) = @_;
  my $isIn=1;
  my $lid="";
  if ( $set ) {
    my @lexons=split(/:/,$set);
    foreach my $lexon (@lexons) {
      if ( !$lid ) {
	$lid = "$ex2id->{$lexon}";
      } else {
	$lid .= ":$ex2id->{$lexon}";
      }
    }
    my $tid="";
    my @exons=split(/:/,$elem);
    foreach my $exon (@exons) {
      my @ids=split(/ /,$ex2id->{$exon});
      foreach my $id (@ids) {
	if( $lid && $lid !~ /$id/ ) {
	  $isIn=0;
	  last;
	}
      }
      if( !$isIn ) {
	last;
      }
    }
  } else {
    $isIn=0;
  }
  return $isIn;
}

sub printClusters {
  my ($lcluster,$cluster,$rcluster,$ex2id,$fHash,$strnd) = @_;
  my $lid="";
  my %sFrame;
  if ( $lcluster ) {
    my @lexons=split(/:/,$lcluster);
    foreach my $lexon (@lexons) {
      if ( !$lid ) {
	$lid = "$ex2id->{$lexon},$lexon";
      } else {
	$lid .= ":$ex2id->{$lexon},$lexon";
      }
      if ( $strnd eq "-" ) {
	my @ids = split(/ /,$ex2id->{$lexon});
	foreach my $id (@ids) {
	  my $frame = $fHash->{"$lexon $id"};
	  $sFrame{$frame}=1;
	}
      }
    }
  }
  my $rid="";
  if ( $rcluster ) {
    my @rexons=split(/:/,$rcluster);
    foreach my $rexon (@rexons) {
      if ( !$rid ) {
	$rid = "$ex2id->{$rexon},$rexon";
      } else {
	$rid .= ":$ex2id->{$rexon},$rexon";
      }
      if ( $strnd eq "+" ) {
	my @ids = split(/ /,$ex2id->{$rexon});
	foreach my $id (@ids) {
	  my $frame = $fHash->{"$rexon $id"};
	  $sFrame{$frame}=1;
	}
      }
    }
  }
  my $tid="";
  my @exons=split(/:/,$cluster);
  my %sFrame2;
  foreach my $exon (@exons) {
    if ( !$tid ) {
      $tid = "$ex2id->{$exon},$exon";
    } else {
      $tid .= ":$ex2id->{$exon},$exon";
    }
    my @ids = split(/ /,$ex2id->{$exon});
    foreach my $id (@ids) {
      my $frame = $fHash->{"$exon $id"};
      $sFrame2{$frame}=1;
    }

  }
  my @frames2 = keys %sFrame2;
  if ( $#frames2 < 0 ) {
    @frames2=(0);
  }
  my @frames = keys %sFrame;
  if ( $#frames < 0 ) {
    @frames=(0);
  }
  my $res  = "[$lid], [$tid], [$rid] (@frames2) (@frames)";
  $res =~ s/CG\d+-//g;
  return $res;
}

sub getFrame {
  my ($gstr,$exon,$cEnd5,$cEnd3) = @_;
  my @vals=split(/ /,$gstr);
  my $strnd=shift @vals;
  my @scoords = sort { $a <=> $b } @vals;
  my $pframe = 0;
  my $prevOverHang=$pframe;
  my $verb=0;
  if( $strnd eq "+" ) {
    print "[@scoords]\n" if($verb);
    for(my $iter = 0; $iter < $#scoords; $iter+=2) {
      my ($e5,$e3) = ($scoords[$iter],$scoords[$iter+1]);
      my $cexon = "$e5 $e3";
      ## initial coding exon
      my $clen=-1;
      $prevOverHang=$pframe;
      if( $cEnd5 >= $e5 && $cEnd5 <= $e3 ) {
	$clen = ($e3-$cEnd5)+1;
	$pframe = $clen % 3;
      } elsif( $e5 >= $cEnd5 && $e5 <= $cEnd3 ) {
	$clen = ($e3-$e5)+1;
	my $sub=0;
	if( $pframe == 1 ) {
	  $sub=2;
	} elsif( $pframe == 2 ) {
	  $sub=1;
	}
	$clen -= $sub;
	$pframe = $clen % 3;
      } else {
	#print "noncoding ";
      }
      print "$cexon $clen overhang: $pframe\n" if($verb);
      #print "proc: [$cexon] $pframe\n";
      if( $cexon eq $exon ) {
	print "set: [$exon] to $prevOverHang\n" if($verb);
	last;
      }
    }
  } else {
    for(my $iter = $#scoords; $iter > 0; $iter-=2) {
      my ($e5,$e3) = ($scoords[$iter-1],$scoords[$iter]);
      my $cexon = "$e5 $e3";
      ## initial coding exon
      $prevOverHang=$pframe;
      if( $cEnd3 >= $e5 && $cEnd3 <= $e3 ) {
	my $clen = ($cEnd3-$e5)+1;
	$pframe = $clen % 3;
      } elsif( $e3 >= $cEnd5 && $e3 <= $cEnd3 ) {
	my $clen = ($e3-$e5)+1;
	my $sub=0;
	if( $pframe == 1 ) {
	  $sub=2;
	} elsif( $pframe == 2 ) {
	  $sub=1;
	}
	$clen -= $sub;
	$pframe = $clen % 3;
      }
      if( $cexon eq $exon ) {
	last;
      }
    }
  }
  return $prevOverHang;
}

sub loadExonLst {
  my ($fileName) = @_;
  open(FILE,$fileName) || die "unable to open [$fileName]\n";
  my %save;
  while(my $line = <FILE>) {
    chomp($line);
    $save{$line} = 1;
  }
  close(FILE);
  return %save;
}
