#ifndef _IHMC_H
#define _IHMC_H

#include "Model.h"
#include "MarkovChain.h"
#include <vector>
#include <string>
#include <iostream>


/**
 * @class pwm - Position Weight Matrix
 * This object can be used in several forms
 * and is also known as a inhomogeneous markov model
 * If we specify dinucleotide frequences this is
 * referred to as a Weight Array Model (IHMC).
 */
class IhMc : public Model {
 public:

 public:
  IhMc(const Model*,int,double);

  IhMc(const IhMc&);
  ~IhMc();

  /**
   * during the build process, call this function 
   * to increment the couns of a particular position in the IhMc
   * @param const string& = nucleotide sequence
   * @param int = the start position in the nucleotide sequence
   *              the count will iterate across the width of the IhMc.
   */
  void 
  update(const std::string&,int,int=0);

  const Model&
    getModel(unsigned pos) const { return *_vec[pos]; }

  void gnuPlot(std::ostream&) const;
  Model* clone() const;
  Model& operator=(const Model&);
  Model* 
    operator-(const Model&) const;
  void operator/=(int);
  void operator+=(const Model&);
  void operator*=(double);
  void operator*=(const Model&);
  void operator-=(const Model&);
  void add(double);

  void write(std::ostream&) const;
  void read(std::istream&);

  void 
  normalize();

  virtual double 
  getLogProb(const std::string&,int,int) const;

  unsigned 
  getWidth() const { return _vec.size();}
  
  int 
    getOrder() const { return _vec[0]->getOrder(); }

  const Model* getElem(unsigned pos) const { return _vec[pos]; }
 private:

  Model* getElem(unsigned pos) { return _vec[pos]; }

  double getVal(const std::string&, int, unsigned) const;

  std::ostream&
    print(std::ostream&) const;

  friend std::ostream&
  operator<< (std::ostream&, const IhMc&); 

 private:
  std::vector<Model*> _vec;
};

#endif // _IHMC_H
