#ifndef _MARKOV_CHAIN_H
#define _MARKOV_CHAIN_H

#include "Model.h"
#include <vector>
#include <ostream>

class MarkovChain : public Model {
 public:
  friend class IhMc;
  MarkovChain(int, double = 0.0, int = 4);
  MarkovChain(const MarkovChain&);
  
  ~MarkovChain() { }

  void gnuPlot(std::ostream&) const;

  void add(double);
  Model* clone() const;
  Model& operator=(const Model&);
  void operator-=(const Model&);
  void operator*=(double);
  void operator*=(const Model&);
  void operator/=(int);
  void operator+=(const Model&);
  Model* 
    operator-(const Model&) const;

  void write(std::ostream&) const;
  void read(std::istream&);

  MarkovChain*
    makeOrder0(const std::string&, int) const;

  int
    getOrder() const;

  unsigned
    getWidth() const { return 0; }
  
  const Model&
    getModel(unsigned) const { return *this; }

  double
    getVal(const std::string&, int, unsigned=0) const;
  
  void
    update(const std::string&, int, int);
  virtual void
    normalize();

  double 
    getLogProb(const std::string&,int,int) const;

  std::ostream&
    print(std::ostream&) const;

 private:

  //Only called by makeOrder0, to build a 0 order model base on input counter and chain
  MarkovChain(const std::vector<std::vector<double> >&, int chain); 

  int 
    getTail(const std::string&, int) const;
  
  double
  getCount(int,int) const;

  void
    count(int,int); 


 protected:
  std::vector<std::vector<double> > _counter;
  int _cOrder;
 
};

#endif // _MARKOV_CHAIN_H
