#include "AnnotationInfo.h"
#include <fstream>
#include <iostream>

using std::string;
using std::list;
using std::endl;
using std::cout;

typedef std::string::size_type size_type;
void
AnnotationInfo::load(const string& filename) { //, const hmap_str_t& idList) {
  std::ifstream ifs(filename.c_str()); 
  if( !ifs ) {
    std::cerr<<"Unabel to open ["<<filename<<"]"<<endl;
    assert(0);
  }
  const int buff_size = 4096;
  char buff[buff_size];
  while(ifs.getline(buff,buff_size)) {
    if( *buff == '#' ) {
      continue;
    } 
    GffFeature* gff = new GffFeature;
    gff->fillFromBuffer(buff);
    const string& c2 = gff->getCol3();
    const string& c8 = gff->getCol8();
    if( c2 == string("CDS") ) {
      const size_type p1 = c8.find("-");
      const string lid = c8.substr(3,p1-3);
      _codeh[lid].push_back(gff);
    } else if( c2 == string("exon") ) {
      //cout<<"buff: "<<buff<<endl;
      //cout<<"c8:["<<c8<<"]"<<endl;
      size_type iter = c8.find("Parent=");
      iter += 6;
      while( iter != string::npos ) {
	size_type next = c8.find_first_of(",;",iter+1);
	size_type lpos = next==string::npos ? c8.length()+1 : next;
	string pid = c8.substr(iter+1,lpos-1-iter);
	if( pid.find("NULL:") == string::npos &&
	    pid.find("dbxref") == string::npos &&
	    pid.find("synonym") == string::npos ) {
	  //cout<<"pid:["<<pid<<"]"<<endl;
	  list<GeneList*>& tmp = _geneh[pid];
	  list<GeneList*>::iterator mt;
	  bool makeNew = tmp.empty() ? true : false;
	  if(!tmp.empty()) {
	    mt = findGeneList(tmp,pid);
	    if( mt == tmp.end() ) {
	      makeNew = true;
	    }
	  }
	  if(makeNew) {
	    GeneList* add = new GeneList(pid);
	    add->push_back(gff);
	    tmp.push_back(add);
	  } else {
	    GeneList* fnd = *mt;
	    fnd->push_back(gff);
	  }
	}
	iter = next;
      }
    } else {
      delete gff;
      continue;
    }
  }
}

std::list<GeneList*>::const_iterator
AnnotationInfo::findGeneList(const std::list<GeneList*>& lst, const std::string& id) {
  list<GeneList*>::const_iterator iter = lst.begin();
  const list<GeneList*>::const_iterator stop = lst.end();
  list<GeneList*>::const_iterator match = lst.end();
  for(; iter != stop; ++iter) {
    const GeneList* gl = *iter;
    //const GffFeature* gff = gl->front();
    //const string& str = gff->getCol8();
    //size_t pos = str.find(id);
    if( gl->getId() == id ) {
      match = iter;
      break;
    }
  }
  return match;
}

std::list<GeneList*>::iterator
AnnotationInfo::findGeneList(std::list<GeneList*>& lst, const std::string& id) {
  list<GeneList*>::iterator iter = lst.begin();
  const list<GeneList*>::iterator stop = lst.end();
  list<GeneList*>::iterator match = lst.end();
  for(; iter != stop; ++iter) {
    const GeneList* gl = *iter;
    //const GffFeature* gff = gl->front();
    //const string& str = gff->getCol8();
    //size_t pos = str.find(id);
    if( gl->getId() == id ) {
      match = iter;
      break;
    }
  }
  return match;
}
