#include "ConsHmm.h"
#include <fstream>
#include <iostream>

using std::vector;
using std::string;
using std::ostream;
using std::ofstream;
using std::ifstream;
using std::endl;
using std::cout;

ConsHmm::ConsHmm(int size): _init(size,0.0), _trans(size) {
  for(unsigned i = 0; i < _trans.size(); ++i) {
    _trans[i].resize(size,0.0);
  }
}

ConsHmm::ConsHmm(int size, const vector<double>& init, const vector<vector<double> >& trans): _init(size,0.0), _trans(size) {
  for(unsigned i = 0; i < _init.size(); ++i) {
    _init[i] = init[i];
  }
  for(unsigned i = 0; i < trans.size(); ++i) {
    _trans[i].resize(trans[i].size());
    for(unsigned j = 0; j < trans[i].size(); ++j) {
      _trans[i][j] = trans[i][j];
    }
  }
}

ConsHmm::ConsHmm(const ConsHmm& ch): _init(ch._init.size(),0.0), _trans(ch._trans.size()) {
  for(unsigned i = 0; i < _init.size(); ++i) {
    _init[i] = ch._init[i];
  }
  for(unsigned i = 0; i < _trans.size(); ++i) {
    _trans[i].resize(ch._trans[i].size());
    for(unsigned j = 0; j < _trans[i].size(); ++j) {
      _trans[i][j] = ch._trans[i][j];
    }
  }
}

void
ConsHmm::operator+=(const ConsHmm& ch) {
  for(unsigned i = 0; i < _init.size(); ++i) {
    _init[i] += ch._init[i];
  }
  for(unsigned i = 0; i < _trans.size(); ++i) {
    for(unsigned j = 0; j < _trans[i].size(); ++j) {
      _trans[i][j] += ch._trans[i][j];
    }
  }
}

void
ConsHmm::operator/=(int div) {
  for(unsigned i = 0; i < _init.size(); ++i) {
    _init[i] /= div;
  }
  for(unsigned i = 0; i < _trans.size(); ++i) {
    for(unsigned j = 0; j < _trans[i].size(); ++j) {
      _trans[i][j] /= div;
    }
  }
}

void ConsHmm::update(int st) {
  assert(st < (signed)_init.size());
  _init[st]++;
}

void ConsHmm::update(int from, int to) {
  assert(from < (signed)_trans.size());
  assert(to < (signed)_trans[from].size());
  _trans[from][to]++;
}

void ConsHmm::normalize() {
  double sum =0;
  for(unsigned i = 0; i < _init.size(); ++i) {
    sum += _init[i];
  }
  if( sum > 0 ) {
    for(unsigned i = 0; i < _init.size(); ++i) {
      _init[i] /= sum;
    }
  } else {
    cout<<"no initial states? something weird going on!!!"<<endl;
  }
  for(unsigned i = 0; i < _trans.size(); ++i) {
    double sum =0;
    for(unsigned j = 0; j < _trans[i].size(); ++j) {
      sum += _trans[i][j];
    }
    if( sum > 0 ) {
      for(unsigned j = 0; j < _trans[i].size(); ++j) {
	_trans[i][j] /= sum;
      }
    }
  }
}

double
ConsHmm::getProb(int st) const {
  assert(st < (signed)_init.size());
  return _init[st];
}

double
ConsHmm::getProb(int fr, int to) const {
  assert(fr < (signed)_trans.size());
  assert(to < (signed)_trans[fr].size());
  return _trans[fr][to];
}

void
ConsHmm::write(std::ostream& ofs) const {
  ofs<<"InitTrans"<<endl;
  for(unsigned i = 0; i < _init.size(); ++i) {
    ofs<<_init[i];
    if( i < _init.size()-1 ) {
      ofs<<" ";
    }
  }
  ofs<<endl;
  ofs<<"TransProb"<<endl;
  for(unsigned i = 0; i < _trans.size(); ++i) {
    //ofs<<i<<" ";
    for(unsigned j = 0; j < _trans.size(); ++j) {
      //ofs<<setprecision(3)<<setw(8)<<_trans[i][j];
      ofs<<_trans[i][j];
      if(j < _trans.size()-1)
	ofs<<" ";
    }
    ofs<<endl;
  }
}

void
ConsHmm::print(ostream& os) const {
  os<<"InitTrans"<<endl;
  bool hit=false;
  for(unsigned i = 0; i < _init.size(); ++i) {
    if( _init[i] >0) {
      hit=true;
      os<<i<<","<<_init[i];
      if( i < _init.size()-1 ) {
	os<<" ";
      }
    }
  }
  if(hit)os<<endl;
  os<<"TransProb"<<endl;
  for(unsigned i = 0; i < _trans.size(); ++i) {
    hit=false;
    for(unsigned j = 0; j < _trans.size(); ++j) {
      if( _trans[i][j] > 0) {
	hit=true;
	os<<i<<","<<j<<","<<" "<<_trans[i][j];
	if(j < _trans.size()-1)
	  os<<" ";
      }
    }
    if(hit) os<<endl;
  }
}

void
ConsHmm::read(std::istream& ifs) {
  string dum;
  ifs>>dum;
  for(unsigned i = 0; i < _init.size(); ++i) {
    ifs>>_init[i];
  }
  ifs>>dum;
  for(unsigned i = 0; i < _trans.size(); ++i) {
    for(unsigned j = 0; j < _trans.size(); ++j) {
      ifs>>_trans[i][j];
    }
  }
}
