#ifndef _CONSHMMINSTANCE_H
#define _CONSHMMINSTANCE_H

#include "ConsHmm.h"
#include "ModelType.h"
#include "stype.h"
#include "RateHmm.h"
#include <iostream>
#include <utility>

// should be called ConsHmmTrellis - rename

class ConsHmmInstance {
 public:
  typedef ModelType::cname_t cname_t;
  typedef std::pair<htype_t,int> hpair_t;
  struct cons_state_t {
    cons_state_t() { init(); }
    cons_state_t(htype_t ht, int time, int clevel) : _ht(ht), _time(time), _clevel(clevel) {}
    cons_state_t(const cons_state_t& c) : _ht(c._ht), _time(c._time), _clevel(c._clevel) {}
    cons_state_t& operator=(const cons_state_t& c) {
      _ht=c._ht; _time=c._time; _clevel=c._clevel;
      return *this;
    }
    friend std::ostream& operator<<(std::ostream& os, const cons_state_t& cs) {
      os<<cs._ht<<" "<<cs._time<<" "<<cs._clevel;
      return os;
    }
    void init () { _ht=eEmptyState; _time=-1; _clevel=0; }
    htype_t _ht;
    int _time;
    int _clevel;
  };
  typedef std::pair<cons_state_t,spair_t> svpair_t; 

  static RateHmm* buildHmm(htype_t);

 public:
  ConsHmmInstance() { }
  ConsHmmInstance(htype_t);
  ConsHmmInstance(const ConsHmmInstance&);
  ~ConsHmmInstance() { }

  static hpair_t
    extract_htype(const std::vector<std::string*>& svec,int, cname_t cn, int phase);
  static std::vector<std::vector<double> > getCodedTrans(cname_t,bool);
  static std::vector<double> getCodedInit(cname_t,bool);
  ConsHmmInstance& operator=(const ConsHmmInstance&);
  spair_t getBestScore(htype_t,int) const;
  unsigned size() const { return _states.size(); }
  void print(std::ostream&) const;
  cons_state_t getState(int idx) const;
  spair_t getScore(const cons_state_t&) const;
  void set(const cons_state_t& cs, spair_t v);
  static void update(ConsHmm&, const cons_state_t&, const cons_state_t&);

  static double getTransProb(const ConsHmm&, const cons_state_t&, const cons_state_t&); 
  static double getInitProb(const ConsHmm&, const cons_state_t&); 
  static double getFalseTransProb();
  static double getFalseInitProb();

  static bool isTrans(const cons_state_t&, const cons_state_t&);
  static int getStateIdx(htype_t, int, int);

  static double state2rate(const cons_state_t&);
  bool
    idx2cons(htype_t ht, int id);

  static int getPrevTime(htype_t,int);
  static int 
    getTimes(htype_t);
  static int 
    getSize(htype_t);
  static int 
    getLevels(htype_t);

  static cons_state_t
    nextConsState(htype_t, int, int);

  static int
    cons_state2int(const cons_state_t& cs);

  
  static htype_t
    cname2htype(ModelType::cname_t,bool);

  static htype_t mname2htype(ModelType::mname_t,bool);

 private:
  std::vector<svpair_t> _states;
};
 
#endif // _CONSHMMINSTANCE_H
