#include "ConsManager.h"
#include "Trellis.h"
#include <iostream>
#include <fstream>

using std::ifstream;
using std::ofstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::vector;

ConsManager::ConsManager(const ConsManager& cm) {
  for(unsigned i = 0; i < (unsigned)ModelType::eCnameEnd; ++i) {
    cname_t id = (cname_t)i;
    //htype_t ht = ConsHmmInstance::cname2htype(id,false);
    //int size = ConsHmmInstance::getSize(ht);
    RateHmmHashT::const_iterator match = cm._rateHash.find(id); 
    assert(match != cm._rateHash.end());
    const RateHmm* rh = (*match).second;
    _rateHash[id] = new RateHmm(*rh);
  }
}

ConsManager::ConsManager() {
  for(unsigned i = 0; i < (unsigned)ModelType::eCnameEnd; ++i) {
    cname_t id = (cname_t)i;
    htype_t ht = ConsHmmInstance::cname2htype(id,false);
    int size = ConsHmmInstance::getSize(ht);
    _consHash[id] = new ConsHmm(size);
  }
}

ConsManager::~ConsManager() {
  RateHmmHashT::iterator citer = _rateHash.begin();
  const RateHmmHashT::iterator cstop = _rateHash.end();
  for(; citer != cstop; ++citer) {
    RateHmm* md = (*citer).second;
    delete md;
  }
  ConsHmmHashT::iterator citer1 = _consHash.begin();
  const ConsHmmHashT::iterator cstop1 = _consHash.end();
  for(; citer1 != cstop1; ++citer1) {
    ConsHmm* md = (*citer1).second;
    delete md;
  }
}

HmmComponent*
ConsManager::find(cname_t cn, const HmmComponent& comp) {
  RateHmm* rh = getRateHmm(cn);
  return rh->find(comp);
}

void 
ConsManager::normalize1() {
  RateHmmHashT::iterator citer = _rateHash.begin();
  const RateHmmHashT::iterator cstop = _rateHash.end();
  for(; citer != cstop; ++citer) {
    cname_t id = (*citer).first;
    RateHmm* md = (*citer).second;
    //cout<<"Before: "<<ModelType::cname2str(id)<<endl;
    //md->print(cout); cout<<endl;
    const htype_t ht1 = Trellis::cname2htype(id,false);
    md->normalize1(ht1);
    //cout<<"After: "<<ModelType::cname2str(id)<<endl;
    //md->print(cout); cout<<endl;
  }
}

void 
ConsManager::normalize2() {
  RateHmmHashT::iterator citer = _rateHash.begin();
  const RateHmmHashT::iterator cstop = _rateHash.end();
  for(; citer != cstop; ++citer) {
    cname_t id = (*citer).first;
    RateHmm* md = (*citer).second;
    const htype_t ht1 = Trellis::cname2htype(id,false);
    md->normalize2(ht1);
  }
}

void 
ConsManager::normalize() {
  RateHmmHashT::iterator citer = _rateHash.begin();
  const RateHmmHashT::iterator cstop = _rateHash.end();
  for(; citer != cstop; ++citer) {
    cname_t id = (*citer).first;
    RateHmm* md = (*citer).second;
    //cout<<"Before: "<<ModelType::cname2str(id)<<endl;
    //md->print(cout); cout<<endl;
    const htype_t ht1 = Trellis::cname2htype(id,false);
    md->normalize(ht1);
    cout<<"After: "<<ModelType::cname2str(id)<<endl;
    md->print(cout); cout<<endl;
  }
#if 0
  ConsHmmHashT::iterator citer1 = _consHash.begin();
  const ConsHmmHashT::iterator cstop1 = _consHash.end();
  for(; citer1 != cstop1; ++citer1) {
    cname_t id = (*citer1).first;
    ConsHmm* md = (*citer1).second;
    cout<<"Before: "<<ModelType::cname2str(id)<<endl;
    md->print(cout); cout<<endl;
    md->normalize();
    cout<<"After: "<<ModelType::cname2str(id)<<endl;
    md->print(cout); cout<<endl;
  }
#endif
}

void 
ConsManager::print(std::ostream& os) const {
  RateHmmHashT::const_iterator citer = _rateHash.begin();
  const RateHmmHashT::const_iterator cstop = _rateHash.end();
  for(; citer != cstop; ++citer) {
    cname_t id = (*citer).first;
    const RateHmm* md = (*citer).second;
    os<<"ConsManager: "<<ModelType::cname2str(id)<<endl;
    md->print(os); os<<endl;
  }
}

bool
ConsManager::read(const string& file) {
  ifstream ifs(file.c_str()); 
  if( !ifs ) {
    cerr<<"Could not open ["<<file<<"]"<<endl;
    return false;
  }
  char buff[512];
  unsigned cnt =0;
  while(ifs.getline(buff,512) ) {
    string id = string(buff);
    cname_t cn = ModelType::str2cname(id);
    assert(cn != ModelType::eCnameEnd);
    //htype_t ht1 = Trellis::cname2htype(cn,false);
    //_rateHash[cn] = Trellis::buildHmm(ht1);
    _rateHash[cn] = new RateHmm(cn); //Trellis::buildHmm(ht1);
    RateHmm* look = _rateHash[cn];
    look->read(ifs);
    ++cnt;
  }
  return cnt==0 ? false : true;
}

#if 1
RateHmm*
ConsManager::getRateHmm(cname_t cn) {
  assert(cn != ModelType::eCnameEnd); 
  cname_t rn = cn;
  if( cn== ModelType::eCintronUp ) {
    rn = ModelType::eCintronDn;
  }
  else if( cn== ModelType::eAintronUp ) {
    rn = ModelType::eAintronDn;
  } 
  return _rateHash[rn];
}
#else
RateHmm*
ConsManager::getRateHmm(cname_t cn) {
  assert(cn != ModelType::eCnameEnd); 
  cname_t rn = cn;
  if( cn== ModelType::eCintronUp ) {
    rn = ModelType::eCintronDn;
  } else if( cn== ModelType::eAintronUp ) {
    rn = ModelType::eCintronDn;
  } else if( cn==ModelType::eAintronDn ) {
    rn = ModelType::eCintronDn;
  } else if( cn==ModelType::eAexonNpc ) {
    rn = ModelType::eCexonNpc;
  } else if( cn==ModelType::eAexonPrc ) {
    rn = ModelType::eCexonPrc;
  }
  return _rateHash[rn];
}
#endif

void
ConsManager::create() {
  for(unsigned i = 0; i < (unsigned)ModelType::eCnameEnd; ++i) {
    const cname_t cn = (cname_t)i;
    const htype_t ht = ConsHmmInstance::cname2htype(cn,false);
    const int size = ConsHmmInstance::getSize(ht);
    const vector<double> init = ConsHmmInstance::getCodedInit(cn,false);
    const vector<vector<double> > trans = ConsHmmInstance::getCodedTrans(cn,false);
    _consHash[cn] = new ConsHmm(size,init,trans);
    const htype_t ht1 = Trellis::cname2htype(cn,false);
    _rateHash[cn] = Trellis::buildHmm(ht1,cn);
  }
}

double
ConsManager::getTransProb(cname_t cn, const cons_state_t& ps, const cons_state_t& cs) const {
  assert(cn != ModelType::eCnameEnd);
  ConsHmmHashT::const_iterator iter  = _consHash.find(cn);
  const ConsHmm* ch  = (*iter).second;
  double val = ConsHmmInstance::getTransProb(*ch,ps,cs);
  return val;
}

double
ConsManager::getInitProb(cname_t cn, const cons_state_t& cs) const {
  assert(cn != ModelType::eCnameEnd);
  ConsHmmHashT::const_iterator iter  = _consHash.find(cn);
  const ConsHmm* ch  = (*iter).second;
  double val = ConsHmmInstance::getInitProb(*ch,cs);
  return val;
}

void
ConsManager::write(const string& file) const {
  ofstream ofs(file.c_str()); 
  if( !ofs ) {
    cerr<<"Could not open ["<<file<<":"<<endl;
    assert(0);
  }
  RateHmmHashT::const_iterator citer = _rateHash.begin();
  const RateHmmHashT::const_iterator cstop = _rateHash.end();
  for(; citer != cstop; ++citer) {
    cname_t id = (*citer).first;
    if( id == ModelType::eCnameEnd ) {
      // need to find out where this is coming from should not be ehere\n";
      continue;
    }
    const RateHmm* md = (*citer).second;
    assert(md);
    ofs<<ModelType::cname2str(id)<<endl;
    md->write(ofs);
    ofs<<endl;
  }
}
