#ifndef _CONSMANAGER_H
#define _CONSMANAGER_H

#include <ext/hash_map>
#include "Model.h"
#include "RateHmm.h"
#include "SeqLabel.h"
#include "ConsHmm.h"
#include "ConsHmmInstance.h"
#include <iostream>
#include <utility>

class ConsManager {
 public:
  typedef ConsHmmInstance::cons_state_t cons_state_t;
  typedef ModelType::cname_t cname_t;
  struct myhash { size_t operator()(const cname_t& __s) const { 
    return __s; }};

  typedef __gnu_cxx::hash_map<cname_t,ConsHmm*, myhash > ConsHmmHashT;
  typedef __gnu_cxx::hash_map<cname_t,RateHmm*, myhash > RateHmmHashT;
  
 public:
  ConsManager();
  ~ConsManager();
  ConsManager(const ConsManager&);
  void operator+=(const ConsManager&);
  ConsManager& operator=(const ConsManager&);
  void operator/=(int);

  void print(std::ostream&) const;
  void update(cname_t, const cons_state_t&, const cons_state_t&);
  double getInitProb(cname_t, const cons_state_t&) const;
  double getTransProb(cname_t, const cons_state_t&, const cons_state_t&) const;

  void create();

  bool read(const std::string& file); 
  void write(const std::string& file) const; 
  void normalize();
  void normalize1();
  void normalize2();
  RateHmm* getRateHmm(cname_t cn);

  HmmComponent* find(cname_t,const HmmComponent& );
 private:
  ConsHmmHashT _consHash;
  RateHmmHashT _rateHash;
  RateHmmHashT _rateValsHash;
};
 
#endif // _CONSMANAGER_H
