#include "ConsThresh.h"


using std::cerr;
using std::endl;
using std::cout;
using std::string;
using std::vector;
using std::list;
using std::ofstream;
using std::ifstream;

ConsThresh::ConsThresh() : 
  _sdistr(ModelType::eCnameEnd), _thresh(ModelType::eCnameEnd,dpair_t(0,0)), 
  _sdistrNuc(ModelType::eCnameEnd), _threshNuc(ModelType::eCnameEnd,dpair_t(0,0)),
  _range(eConsvEnd,dpair_t(0,0)) 
    
{
}

void
ConsThresh::write(const string& file) const {
  ofstream ofs(file.c_str());
  if( !ofs ) {
    throw "unable to open file";
  }
  for(unsigned i = 0; i < _thresh.size(); ++i) {
    ofs<<_thresh[i].first<<" "<<_thresh[i].second<<endl;
  }
  for(unsigned i = 0; i < _threshNuc.size(); ++i) {
    ofs<<_threshNuc[i].first<<" "<<_threshNuc[i].second<<endl;
  }
  //exit(0);
  for(unsigned i = 0; i < _range.size(); ++i) {
    ofs<<_range[i].first<<" "<<_range[i].second<<endl;
  }
}


void
ConsThresh::read(const string& file) {
  ifstream ifs(file.c_str());
  if( !ifs ) {
    throw "unable to open file";
  }
  for(unsigned i = 0; i < 2*_thresh.size(); ++i) {
    double v1=0,v2=0;
    if( !(ifs>>v1>>v2) ) {
      cerr<<"problems parsing: "<<file<<endl;
      throw;
    }
    if( i >= _thresh.size() ) {
      const unsigned base = i-_thresh.size();
      _threshNuc[base].first = v1;
      _threshNuc[base].second = v2;
    } else {
      _thresh[i].first = v1;
      _thresh[i].second = v2;
    }
  }
  for(unsigned i = 0; i < _range.size(); ++i) {
    double v1=0,v2=0;
    if( !(ifs>>v1>>v2) ) {
      cerr<<"problems parsing: "<<file<<endl;
      throw;
    }
    _range[i].first = v1;
    _range[i].second = v2;
  }
}

void
ConsThresh::makeThresholdsHelp(vector<dpair_t>& thresh, vector<list<double> >& sdistr, vector<dpair_t>& range) {
  const bool findRange = (range.size() > 0);
  vector<bool> first(eConsvEnd,true);
  for(unsigned i = 0; i < thresh.size(); ++i) {
    sdistr[i].sort();
    unsigned size = sdistr[i].size();
    double pcnt = 1.0; //0.12; //0.200
    ModelType::cname_t cn = ((ModelType::cname_t)i);
    if( ModelType::isIntron(cn) ) {
      pcnt = 0.2;
    }
#if 0
    if( ModelType::isSplice(cn) ) {
      pcnt = 0.95; // just split it in 1/2 due to lack of data
    } else if( ModelType::isPrcExon(cn) ) {
      pcnt = 0.36; //0.46
    } else if( cn==ModelType::eCexonNpc ) {
	   pcnt = 1;
    }
#endif
    double num = pcnt*(double)size;
    cout<<"Sample Size total for "<<i<<" is "<<size<<" (using top "<<pcnt*100<<") = "<<num<<endl;
    if( size > 0 ) {
      list<double>::const_iterator iter = sdistr[i].begin();
      list<double>::const_iterator stop = sdistr[i].end();
      for(unsigned cnt=0; iter != stop && cnt < num; ++iter, ++cnt) { }
      list<double>::const_reverse_iterator riter = sdistr[i].rbegin();
      list<double>::const_reverse_iterator rstop = sdistr[i].rend();
      for(unsigned cnt=0; riter != rstop && cnt < num; ++riter, ++cnt) { }
      cout<<"The thresholds for "<<i<<" are "<<*riter<<" and "<<*iter<<endl;
      thresh[i].first = *riter;
      thresh[i].second = *iter;
      if( findRange ) {
	const double tmin = sdistr[i].front();
	const double tmax = sdistr[i].back();
	consv_t cv = cname2consv((ModelType::cname_t)i);
	if( first[cv] ) {
	  range[cv].first=tmax;
	  range[cv].second=tmin;
	  first[cv] = false;
	} else {
	  if( tmax > range[cv].first ) {
	    range[cv].first = tmax;
	  }
	  if( tmin < range[cv].second) {
	    range[cv].second = tmin;
	  }
	}
      }
      sdistr[i].clear();
    }
  }
}

void
ConsThresh::makeThresholds() {
  vector<dpair_t> dum;
  makeThresholdsHelp(_thresh,_sdistr,dum);
  makeThresholdsHelp(_threshNuc,_sdistrNuc,_range);
  for(unsigned i = 0; i < _range.size(); ++i) {
    cout<<"Range: "<<_range[i].first<<" "<<_range[i].second<<endl;
  }
}

dpair_t
ConsThresh::getRange(cname_t cn) const {
  consv_t cv = cname2consv((ModelType::cname_t)cn);
  return _range[cv];
}

ConsThresh::consv_t
ConsThresh::cname2consv(cname_t cn) {
  switch(cn) {
  case ModelType::eCintronDn:
  case ModelType::eAintronDn:
    return eIntronDn;
  case ModelType::eCintronUp:
  case ModelType::eAintronUp:
    return eIntronUp;
  case ModelType::eCexonPrc:
  case ModelType::eAexonPrc:
    return eExonPrc;
  case ModelType::eCexonNpc:
  case ModelType::eAexonNpc:
    return eExonNpc;
  case ModelType::eCdon:
  case ModelType::eAdon:
    return eDon;
  case ModelType::eCacc:
  case ModelType::eAacc:
    return eAcc;
  default:
    cerr<<"SOMETHING BAD HAPPENED!!!! STOP!"<<endl;
    return eConsvEnd;
  }
}
