#include "ExonDescrip.h"
#include "AnnotationInfo.h"
#include "utils.h"
#include <iostream>
#include <algorithm>

using std::pair;
using std::sort;
using std::vector;
using std::swap;
using std::string;
using std::cout;
using std::endl;
using std::list;

typedef string::size_type size_type;

ExonDescrip::ExonDescrip(const ExonDescrip& ed) {
  std::cerr<<"Try to avoid using: "<<endl;
  assert(0);
}

void
ExonDescrip::setStartFrame(const list<int>& lst) {
  unsigned size = lst.size();
  _startFrame.resize(size);
  list<int>::const_iterator iter = lst.begin();
  const list<int>::const_iterator stop = lst.end();
  for(unsigned cnt = 0; iter != stop; iter++, ++cnt) {
    assert(cnt<_startFrame.size());
    _startFrame[cnt] = *iter;
  }
}

void
ExonDescrip::setNextFrame(const list<int>& lst) {
  unsigned size = lst.size();
  _nextFrame.resize(size,0);
  list<int>::const_iterator iter = lst.begin();
  const list<int>::const_iterator stop = lst.end();
  for(unsigned cnt = 0; iter != stop; iter++, ++cnt) {
    _nextFrame[cnt] = *iter;
  }
}

void
ExonDescrip::print(std::ostream& os) const {
  os<<"codonr: "<<_beg<<" "<<_end<<endl;
  SeqLabel::printLst(os,_labelLst);
  GffFeature::printLstSpecial(os,_startStopLst);
}

int 
ExonDescrip::compRel(int crd, char strnd, int beg, int end) {
  int nc=0;
  if ( strnd == '-') {
    nc = (end-crd)+1;
  } else if (strnd == '+') {
    nc = (crd-beg)+1;
  }
  return nc;
}

int 
ExonDescrip::compAbs(int crd, char strnd, int beg, int end) {
  int nc=0;
  if ( strnd == '-') {
    nc = (end-crd)+1;
  } else if (strnd == '+') {
    nc = (crd+beg)-1;
  }
  return nc;
}

int 
ExonDescrip::compAbs(int crd) const {
  const char strnd = _strnd;
  const int beg = _beg;
  const int end = _end;
  int nc=0;
  if ( strnd == '-') {
    nc = (end-crd)+1;
  } else if (strnd == '+') {
    nc = (crd+beg)-1;
  }
  return nc;
}

int 
ExonDescrip::countCds() const {
  list<const GffFeature*>::const_iterator iter = _startStopLst.begin();
  const list<const GffFeature*>::const_iterator beg = _startStopLst.begin();
  const list<const GffFeature*>::const_iterator stop = _startStopLst.end();
  int cnt = 0;
  for(; iter != stop; ++iter) {
    const GffFeature* feat = *iter;
    list<const GffFeature*>::const_iterator iter2 = iter;
    bool isNew=true;
    if( iter2 != beg ) {
      --iter2;
      for(; iter2 != stop; --iter2) {
	const GffFeature* feat2 = *iter2;
	if( feat->getEnd5()==feat2->getEnd5() && feat->getEnd3()==feat2->getEnd3() ) {
	  isNew = false;
	  break;
	}
      }
    }
    if( isNew ) {
      ++cnt;
    }
  }
  return cnt;
}

bool
ExonDescrip::isPrcAlt() const {
  bool isAlter=false;
  list<const GffFeature*>::const_iterator iter = _startStopLst.begin();
  const list<const GffFeature*>::const_iterator stop = _startStopLst.end();
  const GffFeature* first = *iter;
  ++iter;
  for(; iter != stop; ++iter) {
    const GffFeature* curr = *iter;
    if( first->getEnd5() != curr->getEnd5() || first->getEnd3() != curr->getEnd3() ) {
      isAlter=true;
      break;
    }
  }
  return isAlter;
}

bool
ExonDescrip::isProteinCoding(int pos, const SeqLabel& llabel, const SeqLabel& rlabel) const {
  const int lbnd = llabel.getExonBnd();
  const int rbnd = rlabel.getExonBnd();
  const int aleft = compAbs(lbnd,_strnd,_beg,_end);
  const int arght = compAbs(rbnd,_strnd,_beg,_end);
  int end5 = -1, end3 = -1;
  // match to 5' end
  if( SeqLabel::isAcc(llabel.getLabel()) || llabel.getLabel()==SeqLabel::eBeg ) {
    end5 = aleft;
    
  } 
  // match to 3' end
  else if( SeqLabel::isDon(rlabel.getLabel()) ) {
    end3 = arght;
  }
  bool res = false;
  std::list<GeneList*>::const_iterator iter = _geneList.begin();
  const std::list<GeneList*>::const_iterator stop = _geneList.end();
  for(; iter != stop; ++iter) {
    const GeneList* gl = *iter;
    const GffFeature* pass = NULL;
    if( end5 != -1 ) {
      pass = gl->getMatchEnd5(end5,_strnd);
    }
    if( end3 != -1 ) {
      pass = gl->getMatchEnd3(end3,_strnd);
    }
    if(pass) {
    std::list<const GffFeature*>::const_iterator match = findCds(gl->getId());
    assert(match != _startStopLst.end());
    const GffFeature* cdsFeat = *match;
    int mn = compRel(cdsFeat->getEnd5(),_strnd,_beg,_end);
    int mx = compRel(cdsFeat->getEnd3(),_strnd,_beg,_end);
    //const int startc = mn, stopc = mx;
    if( mn > mx ) {
      swap(mn,mx);
    }
    //if( pos >= mn && pos <= mx ) {
    const bool isPrc = isPrcCheck(pos,mn,mx);
    if( isPrc ) {
      const int npos = compAbs(pos,_strnd,_beg,_end);
      const GffFeature* gff = gl->getOverlap(npos);
      if( gff ) {
	res = true;
	break;
      }
    }
    }
  }
  return res;
}

int
ExonDescrip::getTargPos(int pos) const {
  return pos;
}

int
ExonDescrip::findFrameHelp(const GeneList* gl, int apos) const {
  int frame = -1;
  std::list<const GffFeature*>::const_iterator match = findCds(gl->getId());
  assert(match != _startStopLst.end());
  const GffFeature* cdsFeat = *match;
  int mn = cdsFeat->getEnd5();
  int mx = cdsFeat->getEnd3();
  int start = mn, stop = mx;
  if( _strnd == '-' ) {
    swap(start,stop);
  }
  if( apos >= mn && apos <= mx ) {
    const GffFeature* gff = gl->getOverlap(apos);
    if( gff ) {
      assert(gff->getFrame() != -1);
      int end5 = gff->getEnd5();
      int end3 = gff->getEnd3();
      if( _strnd == '-' ) {
	swap(end5,end3);
      }
      const int shift = gff->getFrame() > 0 ? 3-gff->getFrame() : 0;
      if( _strnd == '+') {
	const int left = start > end5 ? start : end5;
	if( (left+shift)<=apos ) {
	  frame = (apos-(left+shift)) % 3;
	  assert(frame != 3);
	} else {
	  frame = gff->getFrame()+(apos-left);
	  assert(frame != 3);
	}
      } else {
	assert(end5>=end3);
	const int rght = start < end5 ? start : end5;
	if( (rght-shift) >= apos ) {
	  frame = ((rght-shift)-apos) % 3;
	  assert(frame != 3);
	} else {
	  frame = gff->getFrame()+(rght-apos);
	  assert(frame != 3);
	}
      }
    }
  }
  return frame;
}

void
ExonDescrip::sortLabelLst(const string& cstr) {
  vector<SeqLabel> vec(_labelLst.size());
  std::list<SeqLabel>::const_iterator iter = _labelLst.begin();
  const std::list<SeqLabel>::const_iterator stop = _labelLst.end();
  unsigned accTot=0, donTot=0;
  for(unsigned cnt = 0; iter != stop; ++iter, ++cnt) {
    const SeqLabel& sl = *iter;
    if( SeqLabel::isDon(sl.getLabel()) ) {
      ++donTot;
    }
    if( SeqLabel::isAcc(sl.getLabel()) ) {
      ++accTot;
    }
    vec[cnt] = sl;
  }
  sort(vec.begin(),vec.end());
  _labelLst.clear();
  assert(vec.size()>= 1);
  unsigned accCnt = 0, donCnt = 0;
  if( cstr == "IR"  || cstr == "PT:IR" ) {
    unsigned num_acc_clust=0;
    unsigned num_don_clust=0;
    unsigned facc = 0;
    for(facc = 1; facc < vec.size()-1; ) {
      if(vec[facc].getLabel()==SeqLabel::eAcc && vec[facc-1].getLabel()==SeqLabel::eBeg) {
	num_acc_clust=1;
	if( vec[facc+1].getLabel()==SeqLabel::eAcc ) {
	  vec[facc].setLabel(SeqLabel::eFma1);
	  vec[facc].setEvent(SeqLabel::eAlt);
	  vec[facc+1].setLabel(SeqLabel::eFma2);
	  vec[facc+1].setEvent(SeqLabel::eAlt);
	  ++facc;
	} else {
	  vec[facc].setLabel(SeqLabel::eNea);
	  vec[facc].setEvent(SeqLabel::eCon);
	}
      } else if( SeqLabel::isAcc(vec[facc].getLabel()) && SeqLabel::isAcc(vec[facc-1].getLabel()) ) {
	const SeqLabel::blabel_t bl = num_acc_clust==1 ? SeqLabel::eAfma : SeqLabel::eIrAfma;
	vec[facc].setLabel(bl);
	vec[facc].setEvent(SeqLabel::eAlt);
      } else if(vec[facc].getLabel()==SeqLabel::eAcc && SeqLabel::isDon(vec[facc-1].getLabel())) {
	num_acc_clust=2;
	if( vec[facc+1].getLabel()==SeqLabel::eAcc ) {
	  vec[facc].setLabel(SeqLabel::eIrFma1);
	  vec[facc].setEvent(SeqLabel::eAlt);
	  vec[facc+1].setLabel(SeqLabel::eIrFma2);
	  vec[facc+1].setEvent(SeqLabel::eAlt);
	  ++facc;
	} else {
	  vec[facc].setLabel(SeqLabel::eIrSa);
	  vec[facc].setEvent(SeqLabel::eAlt);
	}
      } else if( SeqLabel::isDon(vec[facc].getLabel()) && SeqLabel::isDon(vec[facc-1].getLabel()) ) {
	const SeqLabel::blabel_t bl = num_don_clust==2 ? SeqLabel::eAfmd : SeqLabel::eIrAfmd;
	vec[facc].setLabel(bl);
	vec[facc].setEvent(SeqLabel::eAlt);
      } else if( SeqLabel::isDon(vec[facc].getLabel()) && SeqLabel::isAcc(vec[facc-1].getLabel()) ) {
	if( num_acc_clust == 2 ) {
	  num_don_clust = 2;
	  if( vec[facc+1].getLabel() == SeqLabel::eDon ) {
	    vec[facc].setLabel(SeqLabel::eFmd1);
	    vec[facc].setEvent(SeqLabel::eAlt);
	    vec[facc+1].setLabel(SeqLabel::eFmd2);
	    vec[facc+1].setEvent(SeqLabel::eAlt);
	    ++facc;
	  } else {
	    vec[facc].setLabel(SeqLabel::eNed);
	    vec[facc].setEvent(SeqLabel::eCon);
	  }
	}
	else {
	  num_don_clust = 1;
	  if( vec[facc+1].getLabel() == SeqLabel::eDon ) {
	    vec[facc].setLabel(SeqLabel::eIrFmd1);
	    vec[facc].setEvent(SeqLabel::eAlt);
	    vec[facc+1].setLabel(SeqLabel::eIrFmd2);
	    vec[facc+1].setEvent(SeqLabel::eAlt);
	    ++facc;
	  } else {
	    vec[facc].setLabel(SeqLabel::eIrSd);
	    vec[facc].setEvent(SeqLabel::eAlt);
	  }
	}
      } else if( SeqLabel::isDon(vec[facc].getLabel()) && vec[facc-1].getLabel()==SeqLabel::eBeg ) {
	num_don_clust = 1;
	if( vec[facc+1].getLabel() == SeqLabel::eDon ) {
	  vec[facc].setLabel(SeqLabel::eIrFmd1);
	  vec[facc].setEvent(SeqLabel::eAlt);
	  vec[facc+1].setLabel(SeqLabel::eIrFmd2);
	  vec[facc+1].setEvent(SeqLabel::eAlt);
	  ++facc;
	} else {
	  vec[facc].setLabel(SeqLabel::eIrSd);
	  vec[facc].setEvent(SeqLabel::eAlt);
	}
      }
      ++facc;
    }
    for(unsigned i = 0; i < vec.size(); ++i) {
      SeqLabel& sl = vec[i];
      if( i == vec.size()-1 ) {
	assert(i>0);
	if( vec[i-1].getEvent()==SeqLabel::eAlt ) {
	  sl.setEvent(SeqLabel::eAlt);
	}
      }
      _labelLst.push_back(sl);
    }
  } else {
    for(unsigned i = 0; i < vec.size(); ++i) {
      SeqLabel& sl = vec[i];
      //cout<<"sl: "<<sl<<endl;
      if( cstr == "MS" || cstr == "PT:MS" ) {
	if( sl.getLabel()==SeqLabel::eDon ) {
	  if(sl.getEvent() == SeqLabel::eAlt && donCnt <= 1 && donTot > 1 ) {
	    if( donCnt == 0 )
	      sl.setLabel(SeqLabel::eFmd1);
	    else
	      sl.setLabel(SeqLabel::eFmd2);
	    ++donCnt;
	  } else if( sl.getEvent() == SeqLabel::eAlt && donCnt > 1 && donTot > 1 ) {
	    sl.setLabel(SeqLabel::eAfmd);
	  } else if( sl.getEvent()== SeqLabel::eCon ) {
	    sl.setLabel(SeqLabel::eNed);
	    sl.setEvent(SeqLabel::eCon);
	  }
	} else if( sl.getLabel()==SeqLabel::eAcc ) {
	  if(sl.getEvent() == SeqLabel::eAlt && accCnt <= 1 && accTot > 1 ) {
	    if( accCnt == 0 ) 
	      sl.setLabel(SeqLabel::eFma1);
	    else
	      sl.setLabel(SeqLabel::eFma2);
	    ++accCnt;
	  } else if( sl.getEvent() == SeqLabel::eAlt && accCnt > 1 && accTot > 1 ) {
	    sl.setLabel(SeqLabel::eAfma);
	  } else if( sl.getEvent()==SeqLabel::eCon ) {
	    sl.setLabel(SeqLabel::eNea);
	    sl.setEvent(SeqLabel::eCon);
	  }
	} else {
	  // no additional modifications needed
	}

      } else if( cstr == "SE" ) {
	assert(donTot==1 &&  accTot==1);
	if( sl.getLabel()==SeqLabel::eDon ) {
	  sl.setLabel(SeqLabel::eCed);
	} else if( sl.getLabel()==SeqLabel::eAcc) {
	  sl.setLabel(SeqLabel::eCea);
	}
      } else if(cstr == "SE:MS" ) {
	if( sl.getLabel()==SeqLabel::eDon ) {
	  if(sl.getEvent() == SeqLabel::eAlt && donCnt <= 1 && donTot > 1 ) {
	    if( donCnt == 0 )
	      sl.setLabel(SeqLabel::eFmd1);
	    else
	      sl.setLabel(SeqLabel::eFmd2);
	    donCnt++;
	  } else if(sl.getEvent() == SeqLabel::eAlt && donCnt > 1 && donTot > 1 ) {
	    sl.setLabel(SeqLabel::eAfmd);
	  } else {
	    sl.setLabel(SeqLabel::eCed);
	  }
	}
	else if( sl.getLabel()==SeqLabel::eAcc ) {
	  if(sl.getEvent() == SeqLabel::eAlt && accCnt <= 1 && accTot > 1 ) {
	    if( accCnt == 0 ) 
	      sl.setLabel(SeqLabel::eFma1);
	    else
	      sl.setLabel(SeqLabel::eFma2);
	    accCnt++;
	  } else if(sl.getEvent() == SeqLabel::eAlt && donCnt > 1 && donTot > 1 ) {
	    sl.setLabel(SeqLabel::eAfma);
	  } else {
	    sl.setLabel(SeqLabel::eCea);
	  }
	}
      } else if( cstr == "CS" ) {
	//assert(donTot<=1 &&  accTot<=1 && (donTot>0||accTot>0));
	const bool normal = (donTot<=1 &&  accTot<=1 && (donTot>0||accTot>0));
	if( !normal ) {
	  cout<<"warning this is odd an unspliced gene well, it can be used but didnt know I used them\n";
	}
	if( sl.getLabel()==SeqLabel::eDon ) {
	  sl.setLabel(SeqLabel::eNed);
	} else if( sl.getLabel()==SeqLabel::eAcc) {
	  sl.setLabel(SeqLabel::eNea);
	}
      } else {
	assert(0);
      }
      if( i == vec.size()-1 ) {
	assert(i>0);
	if( vec[i-1].getEvent()==SeqLabel::eAlt ) {
	  sl.setEvent(SeqLabel::eAlt);
	}
      }
      _labelLst.push_back(sl);
    }
  }
}

int
ExonDescrip::getFrame(unsigned pos, const SeqLabel& llabel, const SeqLabel& rlabel) const {
  assert(pos >= (unsigned)llabel.getRght()+1 && pos <= (unsigned)rlabel.getLeft()-1);
  const int rpos  = compAbs(pos,_strnd,_beg,_end);
  const int lbnd = llabel.getExonBnd();
  const int rbnd = rlabel.getExonBnd();
  const int aleft = compAbs(lbnd,_strnd,_beg,_end);
  const int arght = compAbs(rbnd,_strnd,_beg,_end);
  int end5 = -1, end3 = -1, apos = -1;
  // match to 5' end
  if( SeqLabel::isAcc(llabel.getLabel()) || llabel.getLabel()==SeqLabel::eBeg ) {
    end5 = aleft;
  } 
  // match to 3' end
  else if( SeqLabel::isDon(rlabel.getLabel()) ) { //&& !SeqLabel::isDon(llabel.getLabel()) ) {
    end3 = arght;
  } else {
    assert(SeqLabel::isDonIr(llabel.getLabel()) && SeqLabel::isAccIr(rlabel.getLabel()));
    apos  = compAbs(pos,_strnd,_beg,_end);
  }
  assert(end5 != -1 || end3 != -1 || apos != -1);
  int lastFrame = -1;
  std::list<GeneList*>::const_iterator iter = _geneList.begin();
  const std::list<GeneList*>::const_iterator stop = _geneList.end();
  for(; iter != stop; ++iter) {
    const GeneList* gl = *iter;
    const GffFeature* pass = NULL;
    if( end5 != -1 ) {
      //cout<<"Match5: "<<end5<<" "<<_strnd<<endl;
      pass = gl->getMatchEnd5(end5,_strnd);
    } else if( end3 != -1 ) {
      //cout<<"Match3: "<<end3<<" "<<_strnd<<endl;
      pass = gl->getMatchEnd3(end3,_strnd);
    } else if( apos != -1) {
      pass = gl->getOverlap(apos);
    }
    if( pass ) {
      const int frame = findFrameHelp(gl,rpos);
      //cout<<"HMM: noncoding region?\n";
      if( frame != -1 ) {
	if( lastFrame != -1 && lastFrame != frame ) {
	  //std::cout<<"Possibly a complicated case of multiple frames"<<endl;
	}
	lastFrame = frame;
      }
    }
  }
  return lastFrame;
}

bool
ExonDescrip::makeFrames() {
  std::list<GeneList*>::iterator iter = _geneList.begin();
  const std::list<GeneList*>::iterator stop = _geneList.end();
  bool status = true;
  for(; iter != stop; ++iter) {
    GeneList* gl = *iter;
    std::list<const GffFeature*>::const_iterator match = findCds(gl->getId());
    assert(match != _startStopLst.end());
    const GffFeature* cdsFeat = *match;
    bool res = gl->makeFrame(cdsFeat->getEnd5(),cdsFeat->getEnd3(),_strnd);
    if( !res ) {
      status = false;
      break;
    }
  }
  return status;
}

pair<string,string>
ExonDescrip::classType(int end5, int end3, char strnd) const {
  pair<string,string> res;
  std::list<GeneList*>::const_iterator iter = _geneList.begin();
  const std::list<GeneList*>::const_iterator stop = _geneList.end();
  for(; iter != stop; ++iter) {
    const GeneList* gl = *iter;
    pair<string,string> pr = gl->classType(end5,end3,strnd);
    if(pr.first.length() > 0 ) {
      res = pr;
      break;
    }
  }
  if( res.first.length()<=0) {
	//classType(end5,end3,strnd);
     //assert(res.first.length()>0);
	cout<<"THIS IS A ERROR, tell CALLER TO NOT PROCESS THIS EXON, MARK FOR REMOVEAL"<<endl;
  }
  return res;
}

ExonDescrip::ipair_t
ExonDescrip::getStartStop() const {
  list<const GffFeature* >::const_iterator iter1 = _startStopLst.begin();
  list<const GffFeature* >::const_iterator stop1 = _startStopLst.end();
  int min = -1, max = -1;
  for(; iter1 != stop1; ++iter1) {
    const GffFeature* gff = *iter1;
    if(min==-1 || gff->getEnd5() <min)
      min = gff->getEnd5();
    if(max==-1 || gff->getEnd3() >max)
      max = gff->getEnd3();
  }
  int rmin = compRel(min,_strnd,_beg,_end);
  int rmax = compRel(max,_strnd,_beg,_end);
  if( rmin > rmax ) {
    swap(rmin,rmax);
  }
  return ipair_t(rmin,rmax);
}

list<const GffFeature*>::const_iterator
ExonDescrip::findCds(const string& id) const {
  const size_type lp1 = id.find("-");
  const string id1 = id.substr(lp1+1);
  list<const GffFeature*>::const_iterator stop = _startStopLst.end();
  for(list<const GffFeature*>::const_iterator iter = _startStopLst.begin(); iter != stop; ++iter) {
    const GffFeature* gff = *iter;
    const string& c8 = gff->getCol8();
    size_type p1 = c8.find("-");
    size_type p2 = c8.find(";",p1+1);
    string id2 = c8.substr(p1+1,p2-p1-1);
    if( id2[1] == id1[1] ) {
      return iter;
    }
  }
  return stop;
}

bool
ExonDescrip::hasLabel(SeqLabel::blabel_t label) const {
  bool res = false;
  std::list<SeqLabel>::const_iterator iter = _labelLst.begin();
  const std::list<SeqLabel>::const_iterator stop = _labelLst.end();
  for(; iter != stop; ++iter) {
    if( (*iter).getLabel() == label ) {
      res = true;
      break;
    }
  }
  return res;
}

void
ExonDescrip::printAlignment(std::ostream& os, const std::vector<string*>& seqs) const {
  std::list<SeqLabel>::const_iterator iter = _labelLst.begin();
  const std::list<SeqLabel>::const_iterator stop = _labelLst.end();
  ::printAlignment(os,iter,stop,seqs);
}
