#ifndef _EXONDESCRIP_H
#define _EXONDESCRIP_H

#include "GffFeature.h"
#include "AnnotationInfo.h"
#include "SeqLabel.h"
#include "seqtype.h"
#include <utility>

class ExonDescrip {
 public:
  typedef SeqLabel::ipair_t ipair_t;

 public:
  ExonDescrip() { }
  ExonDescrip(const ExonDescrip&);

  int countCds() const;
  void printAlignment(std::ostream&,const std::vector<std::string*>&) const;
  std::pair<std::string,std::string> classType(int,int,char) const;
  ipair_t getStartStop() const;
  void setBeg(int val) { _beg = val; }
  void setEnd(int val) { assert(val >= 1);_end = val; }
  void setId(const std::string& id) { _tid = id; }
  void setStrnd(char strnd) { _strnd = strnd; }
  void setChr(const std::string& chr) { _chr = chr; }

  void addGene(GeneList* gl) { _geneList.push_back(gl); }
  void addFeature(const GffFeature* gffFeat) { _startStopLst.push_back(gffFeat); }
  void addSeqLabel(const SeqLabel& sl) { _labelLst.push_back(sl); }

  bool isPrcAlt() const;
  void print(std::ostream&) const;
  const std::string& getId() const { return _tid; }
  const std::list<SeqLabel>& getSeqLabelLst() const { return _labelLst;}
  bool hasLabel(SeqLabel::blabel_t) const;
  bool isProteinCoding(int,const SeqLabel&,const SeqLabel&) const;
  int getTargPos(int) const;
  int getBegRel() const { return compRel(_beg,_strnd,_beg,_end); }
  int getEndRel() const { return compRel(_end,_strnd,_beg,_end); }
  int getFrame(unsigned,const SeqLabel&,const SeqLabel&) const;
  std::string getChr() const { return _chr; }
  char getStrnd() const { return _strnd; }
  
  std::list<SeqLabel>& getLabelLst() { return _labelLst; }
  void sortLabelLst(const std::string&);
  ExonDescrip*
    adjustCoords(const fasta_seq&, const fasta_seq&);

  static int
    compRel(int, char, int, int);
  static int
    compAbs(int, char, int, int);

  int
    compAbs(int) const;

  bool makeFrames();
  void setStartFrame(const std::list<int>&);
  void setNextFrame(const std::list<int>&);
  const std::vector<int>& getStartFrame() const { return _startFrame; }
  const std::vector<int>& getNextFrame() const { return _nextFrame; }
 private:
  int
    findFrameHelp(const GeneList*, int) const;

  std::list<const GffFeature*>::const_iterator
    findCds(const std::string& id) const;

 private:
  std::vector<int> _startFrame;
  std::vector<int> _nextFrame;
  std::list<SeqLabel> _labelLst;
  std::list<const GffFeature* > _startStopLst;
  std::list<GeneList* > _geneList;
  std::string _tid;
  std::string _chr;
  char _strnd;
  // if "-" strnd, left=stop codon, rght=start codon
  // if "+" strnd, reverse...
  //int _leftCodon, _rghtCodon;
  int _beg, _end;
};

#endif // _EXONDESCRIP_H
