#include "FelChurchModel.h"
#include "base.h"
#include <cmath>
#include <iostream>

using std::string;
using std::cout;
using std::endl;

static int
kroneckerDelta(int i, int j);

FelChurchModel::FelChurchModel(const MarkovChain& mc, double tnTvRatio, double mRate) : 
  _mc(mc), _tnTvRatio(tnTvRatio), _mRate(mRate), _alpha(0), _beta(0) {
  init();
}

void 
FelChurchModel::init() {
  double pyFreq = 0, prFreq = 0;
  for(unsigned i = 0; i < 4; ++i) {
    const char c = int2base(i);
    string str(1,c);
    if( isPy(c) ) {
      pyFreq += _mc.getVal(str,1);
    } else if ( isPr(c) ) {
      prFreq += _mc.getVal(str,1);
    } else {
      assert(0);
    }
  }
  const double mod = (getMutRate()/(1+getTnTvRatio()));
  //cout<<"mod: "<<mod<<endl;

  //const double numer1 = 2*prFreq*getBaseFreq('t')*getTnTvRatio();
  const double numer1 = 2*prFreq*pyFreq*getTnTvRatio();
  //cout<<"numer1: "<<numer1<<endl;

  const double prProd = getBaseFreq('a')*getBaseFreq('g');
  const double pyProd = getBaseFreq('c')*getBaseFreq('t');
  const double numer2 = (2*prProd)+(2*pyProd);
  //cout<<"numer2: "<<numer2<<endl;

  const double denom1 = prFreq > 0 ? (2*prProd)/prFreq : 0;
  const double denom2 = pyFreq > 0 ? (2*pyProd)/pyFreq : 0;
  const double denom = denom1+denom2;
  //cout<<"denom1: "<<denom1<<" denom2: "<<denom2<<" denom: "<<denom<<endl;
  _alpha = denom > 0 ? ((numer1-numer2)/denom)*mod : 0;
  // this seems to assume both non 0 R and Y prob, rather than R+Y>0 , figure out why
  double cnt = 0;
  double val = 1.0;
  if(prFreq > 0) {
    ++cnt;
    val *= prFreq;
  }
  if(pyFreq > 0) {
    ++cnt;
    val *= pyFreq;
  }
  _beta = (1/(cnt*val))*mod;
}


double
FelChurchModel::getBaseFreq(char ch) const {
  string str(1,ch);
  return _mc.getVal(str,1);
}

static int
kroneckerDelta(int i, int j) {
  return (i==j) ? 1 : 0;
}

double
FelChurchModel::getProb(const string& to, const string& fr, double blen) const {
  const char tc = to[0];
  const char fc = fr[0];
  const unsigned ti = base2int(tc);
  const unsigned fi = base2int(fc);
  if( ti == 4 || fi == 4 ) {
    assert(0);
  }
  const int kd = kroneckerDelta(ti,fi);
  const double alpha = getAlpha();
  const double beta = getBeta();
  const double noMut = exp((-(alpha+beta))*blen)*kd;

  double tv = exp(-(beta*blen))*(1-exp(-(alpha*blen)));
  //if(tv<0) {
    //cout<<"Warning tv: not consistent  may need to change tntv ratio: now setting to prob 0: "<<tv<<endl;
    //tv=0;
  //}

  const int sb=sameBaseType(tc,fc);
  const double toFreq = getBaseFreq(tc);
  double sum = 0;
  for(unsigned i = 0; i < 4; ++i) {
    const char base = int2base(i);
    int lsb=sameBaseType(fc,base);
    sum += getBaseFreq(base)*lsb;
  }
  const double pcnt = sum > 0 ? (toFreq/sum)*sb : 0;
  
  const double nothing = (1-exp(-beta*blen))*toFreq;

  double res = noMut+(tv*pcnt)+nothing;
  if(res > 1) {
    //cout<<"Warning res: not consistent  may need to change tntv ratio: now setting to prob 1: "<<res<<endl;
    res = 1;
  }
  if( res < 0 ) {
    res = 0;
  }
  assert(res>=0 && res<=1);
  return res;
}

double
FelChurchModel::getProb(char tc, char fc, double blen) const {
  const unsigned ti = base2int(tc);
  const unsigned fi = base2int(fc);
  if( ti == 4 || fi == 4 ) {
    assert(0);
  }
  const int kd = kroneckerDelta(ti,fi);
  const double alpha = getAlpha();
  const double beta = getBeta();
  const double noMut = exp((-(alpha+beta))*blen)*kd;

  double tv = exp(-(beta*blen))*(1-exp(-(alpha*blen)));
  //if(tv<0) {
    //cout<<"modify tv: "<<tv<<endl;
    //tv=0;
  //}

  const int sb=sameBaseType(tc,fc);
  const double toFreq = getBaseFreq(tc);
  double sum = 0;
  for(unsigned i = 0; i < 4; ++i) {
    const char base = int2base(i);
    int lsb=sameBaseType(fc,base);
    sum += getBaseFreq(base)*lsb;
  }
  const double pcnt = sum > 0 ? (toFreq/sum)*sb : 0;
  
  const double nothing = (1-exp(-beta*blen))*toFreq;

  double res = noMut+(tv*pcnt)+nothing;
  if(res > 1) {
    //cout<<"modify res: "<<res<<endl;
    res = 1;
  }
  if(res < 0) {
    //cout<<"modify res: "<<res<<endl;
    res = 0;
  }
  assert(res>=0 && res<=1);
  return res;
}
