#include "GeneList.h"
#include <iostream>
#include <algorithm>
#include <utility>

using std::pair;
using std::string;
using std::swap;
using std::list;
using std::cout;
using std::endl;

static bool DEBUG = false;

const GffFeature* 
GeneList::getOverlap(int pos) const {
  const GffFeature* res = NULL;
  list<GffFeature*>::const_iterator iter = begin();
  const list<GffFeature*>::const_iterator stop_iter = end();
  for(iter = begin(); iter != stop_iter; ++iter) {
    const GffFeature* fexon = *iter;
    const int end5 = fexon->getEnd5();
    const int end3 = fexon->getEnd3();
    if( pos >= end5 && pos <= end3 ) {
      res = fexon;
      break;
    }
  }
  return res;
    //cout<<end5<<" "<<end3<<" "<<strnd<<" "<<start<<" "<<stop<<" "<<frame<<endl;
}

pair<string,string>
GeneList::classType(int end5, int end3, char strnd) const {
  string leftStr,rghtStr;
  list<GffFeature*>::const_iterator iter = begin();
  const list<GffFeature*>::const_iterator stop_iter = end();
  unsigned cnt = 0;
  for(iter = begin(); iter != stop_iter; ++iter, ++cnt) {
    const GffFeature* fexon = *iter;
    const int left = fexon->getEnd5();
    const int rght = fexon->getEnd3();
    if( left== end5 && rght==end3 ) {
      if( strnd == '-' ) {
	leftStr = cnt > 0 ? string("don") : string("other");
	rghtStr = cnt < size()-1 ? "acc" : "other";
      } else {
	leftStr = cnt > 0 ? "acc" : "other";
	rghtStr = cnt < size()-1 ? "don" : "other";
      }
      break;
    }
  }
  return pair<string,string>(leftStr,rghtStr);
}

const GffFeature* 
GeneList::getMatchEnd5(int end5, char strnd) const {
  const GffFeature* res = NULL;
  list<GffFeature*>::const_iterator iter = begin();
  const list<GffFeature*>::const_iterator stop_iter = end();
  for(iter = begin(); iter != stop_iter; ++iter) {
    const GffFeature* fexon = *iter;
    const int left = fexon->getEnd5();
    const int rght = fexon->getEnd3();
    //if( left == end5 && rght == end3 ) {
    if( (strnd == '+' && end5 == left) ||
	(strnd == '-' && end5 == rght) ) {
      res = fexon;
      break;
    }
  }
  return res;
    //cout<<end5<<" "<<end3<<" "<<strnd<<" "<<start<<" "<<stop<<" "<<frame<<endl;
}

const GffFeature* 
GeneList::getMatchEnd3(int end3, char strnd) const {
  const GffFeature* res = NULL;
  list<GffFeature*>::const_iterator iter = begin();
  const list<GffFeature*>::const_iterator stop_iter = end();
  for(iter = begin(); iter != stop_iter; ++iter) {
    const GffFeature* fexon = *iter;
    const int left = fexon->getEnd5();
    const int rght = fexon->getEnd3();
    if( (strnd == '+' && end3 == rght) ||
	(strnd == '-' && end3 == left) ) {
      res = fexon;
      break;
    }
  }
  return res;
    //cout<<end5<<" "<<end3<<" "<<strnd<<" "<<start<<" "<<stop<<" "<<frame<<endl;
}

template<class MyIterator>
static bool
helpMakeFrame(int start, int stop, char strnd, MyIterator beg, MyIterator stop_iter) {
  int frame = -1;
  if( strnd == '-' ) {
    std::swap(start,stop);
  }
  for(MyIterator iter = beg; iter != stop_iter; ++iter) {
    GffFeature* fexon = *iter;
    const int mn = fexon->getEnd5();
    const int mx = fexon->getEnd3();
    int end5 = mn, end3 = mx;
    if( strnd == '-' ) {
      std::swap(end5,end3);
    }
    if(DEBUG)
      cout<<end5<<" "<<end3<<" "<<strnd<<" "<<start<<" "<<stop<<" "<<frame<<endl;
    // initial exon
    if( start >= mn && start <= mx ) { 
      fexon->setFrame(0);
      if(strnd == '+') {
	const int rght = stop < end3 ? stop : end3;
	const int len = (rght-start)+1;
	assert(len>=1);
	frame = len % 3;
      } else {
	const int rght = stop > end3 ? stop : end3;
	const int len = (start-rght)+1;
	assert(len>=1);
	frame = len % 3;
      }
      // non-initial protein coding exon
    } else if( (strnd=='+' && end5 >= start && end5 <= stop) || (strnd=='-' && end5 <= start && end5 >= stop) ) {
      fexon->setFrame(frame);
      int len = -1;
      if(strnd == '+') {
	const int shift = frame > 0 ? 3-frame : frame;
	const int left = stop < end3 ? stop : end3;
	len = (left-(end5+shift))+1;
	assert(len>=0);
	frame = len % 3;
      } else {
	const int shift = frame > 0 ? 3-frame : frame;
	const int rght = stop > end3 ? stop : end3; 
	len = ((end5-shift)-rght)+1;
	assert(len>=0);
	frame = len % 3;
      }
      // should be last prc exon, set frame=0 for debug 
      assert(len>=0);
      if( len == 0 ) {
	frame=0;
      }
    } else {
      //cout<<"Non-coding exon"<<endl;
    }
    //cout<<"set frame: "<<*fexon<<endl;
  }
  bool status=false;
  if( frame == 0 ) {
    status=true;
    ///assert(frame==0);
  }
  return status;
}

bool
GeneList::makeFrame(int start, int stop, char strnd) {
#if 0
  if(start == 3865402) {
    DEBUG=true;
  } else {
    DEBUG=false;
  }
#endif
  bool status = false;
  if(strnd == '+' ) {
    status = helpMakeFrame(start,stop,strnd,begin(),end());
  } else {
    status = helpMakeFrame(start,stop,strnd,rbegin(),rend());
  }
  if( DEBUG ) {
    cout<<"MADE FRAMES, "<<start<<" "<<stop<<" "<<strnd<<endl;
    cout<<*this<<endl;
  }
  return status;
}

namespace std {
  ostream& operator<<(ostream& os, const GeneList& gi) {
    GeneList::const_iterator iter = gi.begin();
    const GeneList::const_iterator stop = gi.end();
    for(; iter != stop; ++iter) {
      os<<**iter<<endl;
    }
    return os;
  }
}
