#include "Histogram.h"
#include <algorithm>
#include <math.h>
#include <iostream>

using std::list;
using std::min;
using std::max;
using std::istream;
using std::ostream;
using std::cout;
using std::endl;

Histogram::Histogram() : _nbins(0) { }

Histogram::Histogram(double mn, double mx, int nbin, double base) : _mn(mn), _mx(mx), _nbins(nbin) { 
  _mn = mn;
  _mx = mx;
  _nbins = nbin;
  _hist.clear();
  const int nbins = _nbins;
  _hist.resize(nbins,base);
  double diff = fabs(_mn-_mx);
  _binSize = (int)(diff / (double)(nbins));
}

Histogram::Histogram(const Histogram& hs) {
  _mn = hs._mn;
  _mx = hs._mx;
  _nbins = hs._nbins;
  _hist.resize(_nbins);
  _binSize = hs._binSize;
  for(unsigned i = 0; i < _hist.size(); ++i) {
    _hist[i] = hs._hist[i];
  }
}

Histogram&
Histogram::operator=(const Histogram& hs) {
  _mn = hs._mn;
  _mx = hs._mx;
  _nbins = hs._nbins;
  _hist.resize(_nbins);
  _binSize = hs._binSize;
  for(unsigned i = 0; i < _hist.size(); ++i) {
    _hist[i] = hs._hist[i];
  }
  return *this;
}

void
Histogram::init(double mn, double mx, int nbin, double base) {
  _mn = mn;
  _mx = mx;
  _nbins = nbin;
  _hist.clear();
  const int nbins = _nbins;
  _hist.resize(nbins,base);
  double diff = fabs(_mn-_mx);
  _binSize = (int)(diff / (double)(nbins));
}


void
Histogram::store(double val) {
  //  _vals.push_back(val);

  double numer = val-_mn;
  int bin = (int)(numer / _binSize);
  if( bin < 0 )
    bin = 0;
  if( bin >= (signed)_hist.size() ) {
    bin = _hist.size()-1;
  }
  //cout<<numer<<" "<<bin<<endl;
  _hist[bin]++;
}

Histogram&
Histogram::operator+=(const Histogram& hst) {
  assert(get_nbins()==hst.get_nbins());
  assert(get_min()==hst.get_min());
  assert(get_max()==hst.get_max());
  assert(_hist.size() > 0 && _hist.size()==hst._hist.size());
  for(unsigned i = 0; i < _hist.size(); ++i) {
    _hist[i] += hst._hist[i];
  }
  return *this;
}

Histogram&
Histogram::operator/=(double val) {
  for(unsigned i = 0; i < _hist.size(); ++i) {
    _hist[i] /= val;
  }
  return *this;
}

void
Histogram::finish() {
  double sum = 0;
  for(unsigned i = 0; i < _hist.size(); ++i) {
    sum += _hist[i];
  }
  if( sum > 0 ) {
    for(unsigned i = 0; i < _hist.size(); ++i) {
      _hist[i] /= sum;
      //assert(_hist[i] <= 1);
    }
  }
}

double
Histogram::get_prob(double val) const {
  assert(val-_mn >= 0 );
  unsigned bin = (unsigned)((val-_mn) / _binSize);
  //assert(bin < _hist.size());
  if( bin>=_hist.size())
    bin = _hist.size()-1;
  
  return _hist[bin];
}

ostream&
operator<<(ostream& os, const Histogram& hst) {
  double xval = hst._mn;
  for(unsigned i = 0; i < hst._hist.size(); ++i, xval += hst._binSize) {
    os<<xval<<" "<<hst._hist[i];
    if( i+1 < hst._hist.size() ) os<<endl;
  }
  return os;
}

istream&
operator>>(istream& is, Histogram& hst) {
  double xval = 0;
  for(unsigned i = 0; i < hst._hist.size(); ++i, xval += hst._binSize) {
    is>>xval>>hst._hist[i];
  }
  return is;
}
