#ifndef _HISTOGRAM_H
#define _HISTOGRAM_H

#include <vector>
#include <list>
#include <iostream>
#include <assert.h>
#include <utility>

class Histogram {
 public:
  typedef std::pair<Histogram,Histogram> pair_t;

 public:
  Histogram();
  Histogram(const Histogram&);
  Histogram(double,double,int,double base=0);

  Histogram& operator=(const Histogram&);

  void finish();
  void store(double);
  void init(double,double,int,double base=0);
  double get_prob(double) const;
 
  Histogram&
    operator+=(const Histogram&);

  Histogram&
    operator/=(double);

  friend std::ostream& 
  operator<<(std::ostream&,const Histogram&);
  friend std::istream& 
  operator>>(std::istream&,Histogram&);

  unsigned size() const { return _hist.size(); }
  double get_min() const { return _mn; }
  int get_binSize() const { return _binSize; }
  double get_val(unsigned idx) const { assert(idx<_hist.size()); return _hist[idx]; }
 private:
  void set_nbins(int val) { _nbins = val; }

  double get_max() const { return _mx; }
  int get_nbins() const { return _nbins; }

 private:
  std::vector<double> _hist;
  double _mn,_mx;
  int _binSize, _nbins;
};

#endif // _HISTOGRAM_H
