#include "Histogram3d.h"
#include <algorithm>
#include <math.h>
#include <iostream>

using std::list;
using std::min;
using std::max;
using std::ostream;
using std::cout;
using std::endl;

Histogram3d::Histogram3d(unsigned num_elem, double mn, double mx, int nbin, double base) 
: _hist(num_elem) {

  for(unsigned i = 0; i < num_elem; ++i) {
    _hist[i].init(mn,mx,nbin,base);
  }
}

Histogram3d::Histogram3d(const Histogram3d& hs) : _hist(hs.size()) {
  for(unsigned i = 0; i < size(); ++i) {
    _hist[i] = hs._hist[i];
  }
}

Histogram3d&
Histogram3d::operator=(const Histogram3d& hs) {
  _hist.clear();
  _hist.resize(hs.size());
  for(unsigned i = 0; i < size(); ++i) {
    _hist[i] = hs._hist[i];
  }
  return *this;
}

void
Histogram3d::init(unsigned num_elem, double mn, double mx, int nbin, double base) {
  _hist.clear();
  _hist.resize(num_elem);
  for(unsigned i = 0; i < size(); ++i) {
    _hist[i].init(mn,mx,nbin,base);
  }
}


void
Histogram3d::store(unsigned idx, double val) {
  //  _vals.push_back(val);
  _hist[idx].store(val);
}

Histogram3d&
Histogram3d::operator+=(const Histogram3d& hst) {
  assert(size()==hst.size());
  for(unsigned i = 0; i < size(); ++i) {
    _hist[i] += hst._hist[i];
  }
  return *this;
}

Histogram3d&
Histogram3d::operator/=(double val) {
  for(unsigned i = 0; i < size(); ++i) {
    Histogram& hist = _hist[i];
    hist/=val;
  }
  return *this;
}

void
Histogram3d::finish() {
  for(unsigned k = 0; k < size(); ++k) {
    _hist[k].finish();
  }
}

double
Histogram3d::get_prob(unsigned idx, double val) const {
  return _hist[idx].get_prob(val);
}

ostream&
operator<<(ostream& os, const Histogram3d& hst) {
  for(unsigned k = 0; k < hst.size(); ++k) {
    double xval = hst._hist[k].get_min();
    //const vector<double>& vals = hst._hist[k].get
    for(unsigned i = 0; i < hst._hist[k].size(); ++i, xval += hst._hist[k].get_binSize()) {
      os<<k<<" "<<xval<<" "<<hst._hist[k].get_val(i)<<endl;
    }
    os<<endl;
  }
  os<<endl;
  return os;
}
