#ifndef _INITNUCDISTR_H
#define _INITNUCDISTR_H

#include <ext/hash_map>
#include "Model.h"
#include "ModelType.h"
#include "ExonDescrip.h"
#include "ParamFile.h"
#include "PhyloModel.h"
#include "utils.h"
#include "MultiModel.h"
#include "stype.h"
#include <utility>


class InitNucDistr {
 public:
  typedef ModelType::cname_t cname_t;
  struct myhash { size_t operator()(const cname_t& __s) const { 
    return __s; }};
  typedef __gnu_cxx::hash_map<cname_t,MultiModel*, myhash > MultiModelHashT;
  typedef __gnu_cxx::hash_map<cname_t,PhyloModel*, myhash > PhyloModelHashT;

 public:
  InitNucDistr();
  ~InitNucDistr();

  void del();
  void
   train(const fasta&); 

  spair_t
    compScore(const std::vector<std::string*>&, int, int, cname_t);

  ipair_t
    getDonCnt() const { return _dnCnt; }

 private:
  mcpair_t getMc(ModelType::cname_t,const std::vector<std::string*>&,int,int);
  
  //void normalize();

  void
    updateModel(const std::vector<std::string*>&,
		const SeqLabel&, const SeqLabel&, const ExonDescrip&);

  cname_t
    getModel(const SeqLabel&, const SeqLabel&, const ExonDescrip&, unsigned);

  void
    createModel(cname_t);

  void
    createPhyloModel(cname_t, double);


 private:
  MultiModelHashT _mm;
  PhyloModelHashT _pmSlow;
  PhyloModelHashT _pmFast;
  ipair_t _dnCnt;
};

#endif // _INITNUCDISTR_H
