#include "LabelSeq.h"
#include "Options.h"
#include "InvScoreCache.h"
#include "ParseHeader.h"
#include "StateTrans.h"
#include <math.h>
#include <iostream>
#include <stdexcept>
#include <iomanip>

using std::setprecision;
using std::list;
using std::vector;
using std::string;
using std::cout;
using std::cerr;
using std::endl;

extern bool XDEBUGON;
bool DEBUGON=false;

list<SeqLabelVal>*
LabelSeq::traceBack(smatrix_t* lastPtr, int& bframe) {
  const smatrix_t& last = *lastPtr;
  const SeqLabelVal* bestVal = NULL;
  double maxVal = 1;
  smatrix_t::const_iterator iter = last.begin();
  const smatrix_t::const_iterator stop = last.end();
  for(; iter != stop; ++iter) {
    const vector<SeqLabelVal>& pslvVec = *iter;
    for(unsigned frame = 0; frame < 3; ++frame) {
      const SeqLabelVal& pslv = pslvVec[frame];
      const double cval = pslv.getVal();
      if( (cval > maxVal ||  !bestVal) && pslv.isValid() ) {
   maxVal = cval;
   bestVal = &pslv;
   bframe = frame;
      }
    }
  }
  list<SeqLabelVal>* res = new list<SeqLabelVal>;
  while(bestVal) {
    res->push_front(*bestVal);
    bestVal = bestVal->getPrev();
  }
  return res;
}

static int
compCodingLen(const SeqLabelVal& slv, const SeqLabelVal* iPrev, int startc, int stopc) {
  const SeqLabelVal* curr = &slv;
  const SeqLabelVal* prev = iPrev;
  const SeqLabelVal* lBnd = NULL;
  int rbnd = slv.getExonBnd();
  if( stopc < rbnd ) {
    rbnd = stopc;
  }
  while(prev) {
    if( SeqLabel::isExon(prev->getLabel(),curr->getLabel()) ) {
      lBnd = prev;
      if( startc >= prev->getExonBnd() ) {
   if( startc > curr->getExonBnd()) {
     return -1;
   }
   break;
      }
    } else {
      lBnd = curr;
      break;
    }
    curr=prev;
    prev=prev->getPrev();
  }
  assert(lBnd);
  const int lbnd = lBnd->getExonBnd() > startc ? lBnd->getExonBnd() : startc;
  const int len = (rbnd-lbnd)+1;
  return len;
}


void
LabelSeq::cacheScore(const InvScoreCache& lbnd, InvScoreCache& rbnd,  const vector<string*>& svec, int start_codon, int stop_codon, pred_t predType) const {
  //iterate through aexon, cexon, aintron, cintron, 
  const SeqLabel& ll = lbnd.getBnd();
  const SeqLabel& rl = rbnd.getBnd();
  const int lpos = ll.getRght()+1;
  const int rpos = !rl.isEnd() ? rl.getRght() : rl.getRght()-1;
  if( lpos <= rpos ) {
    for(unsigned i = 0; i < ModelType::eInvCnameEnd; ++i) {
      ModelType::inv_t inv = (ModelType::inv_t)i;
      const bool isPrc = true; //changed in scoreInterval
      ModelType::cname_t cn = ModelType::inv2cname(inv,isPrc);
      const bool verb  = false;
      if( predType==eBasicFixedRate || predType==eLearnedFixedRate ) {
   rbnd.getInvScore(i) = _mm.scoreInterval5a(lpos,rpos,svec,start_codon,stop_codon,cn,predType,verb);
      } else {
   rbnd.getInvScore(i) = _mm.scoreInterval5b(lpos,rpos,svec,start_codon,stop_codon,cn,verb);
      }
      if(DEBUGON) {
   const vector<spair_t>& vec = rbnd.getInvScore(i);
   cout<<"cachecheck: "<<ll<<" to "<<rl<<" modelidx: "<<ModelType::cname2str(cn)<<" isPrc: "<<(isPrc?1:0)<<" val: "<<vec[0]<<" "<<vec[1]<<" "<<vec[2]<<endl;
      }
    }
  }
}

bool
LabelSeq::hasGcDon(const vector<string*>& svec, int start, int stop, const list<SeqLabel>& lst) const {
  const list<SeqLabel>::const_iterator end = lst.end();
  list<SeqLabel>::const_iterator iter = lst.begin();
  bool isGc=false;
  const string& tseq = *svec[0];
  for(; iter != end; iter++) {
    const SeqLabel* cslv = &(*iter);
    if( SeqLabel::isDon(cslv->getLabel()) ) {
      int pos = cslv->getExonBnd()+1;
      if( tolower(tseq[pos-1])=='g' && tolower(tseq[pos]) == 'c' ) {
   isGc=true;
   break;
      }
    }
  }
  return isGc;
}

InvScoreCache* 
getPrevNonOverlap(const vector<InvScoreCache*>& invVec, const SeqLabel& sl, int& idx) {
  int niter = idx;
  InvScoreCache* match = NULL;
  // ignore overlaps
  for(; niter >=0; --niter ) {
    const SeqLabel& pl = invVec[niter]->getBnd();
    if( pl.getRght() < sl.getLeft() ) {
      match = invVec[niter];
      idx = niter;
      break;
    }
  }
  assert(idx >=0);
  assert(match);
  return match;
}

InvScoreCache* 
getPrevOverlap(const vector<InvScoreCache*>& invVec, const SeqLabel& sl, int& idx) {
  int niter = idx;
  InvScoreCache* match = NULL;
  // ignore overlaps
  for(; niter >=0; --niter ) {
    const SeqLabel& pl = invVec[niter]->getBnd();
    const bool isOverlap = (sl.isIn(pl.getLeft()) || sl.isIn(pl.getRght()));
    const bool isSameType = ((SeqLabel::isAcc(pl.getLabel())&&SeqLabel::isAcc(sl.getLabel()))||(SeqLabel::isDon(pl.getLabel())&&SeqLabel::isDon(sl.getLabel())));
    const bool isValidOverlap = (isOverlap && isSameType);
    if( pl.getRght() < sl.getLeft() || isValidOverlap ) {
      assert(pl.getExonBnd()<sl.getExonBnd());
      match = invVec[niter];
      idx = niter;
      break;
    } 
  }
  assert(idx >=0);
  assert(match);
  return match;
}


list<SeqLabelVal>*
LabelSeq::labelF(const vector<string*>& svec, int start_codon, int stop_codon, int start_pos, int stop_pos, int& retFrame, bool useCons, pred_t predType, 
       const vector<int>& limitFrame, const vector<int>& limitNextFrame, const vector<int>& incl, bool isInitExon, bool isLastExon) {
  assert(svec[0]);
  list<InvScoreCache*> invLst;
  const string& str = *svec[0];
  invLst.push_back(new InvScoreCache(SeqLabel(SeqLabel::eBeg,SeqLabel::eUnk,start_pos,start_pos)));
  const int slen = (signed)str.size();
  const int beg = start_pos < 6 ? 6 : start_pos+1;
  for(unsigned i = beg; i < str.size(); ++i) {
    ipair_t ap = SeqLabel::getSigDim1(SeqLabel::eAcc,i);
    ipair_t dp = SeqLabel::getSigDim1(SeqLabel::eDon,i);
    bool justOnce=false;
    if( tolower(str[i-4]) == 'a' && tolower(str[i-3])=='g' &&
	ap.first > start_pos && ap.second <= slen ) {
      SeqLabel sl(SeqLabel::eAcc,SeqLabel::eUnk,ap.first,ap.second);
      invLst.push_back(new InvScoreCache(sl));
      justOnce=true;
    } 
    if( incl.size() > 0 && !justOnce ) {
      const int inPut = incl[0];
      if( (i-1) == (unsigned)inPut && inPut > start_pos ) {
	SeqLabel sl(SeqLabel::eAcc,SeqLabel::eUnk,ap.first,ap.second);
	cout<<"look: "<<sl<<endl;
	invLst.push_back(new InvScoreCache(sl));
      }
    }
    justOnce=false;
    //if( tolower(str[i-6]) == 'g' && (tolower(str[i-5])=='t' ||tolower(str[i-5])=='c')
    if( tolower(str[i-6]) == 'g' && tolower(str[i-5])=='t'
	&&
	dp.first > start_pos && dp.second <= slen ) {
      SeqLabel sl(SeqLabel::eDon,SeqLabel::eUnk,dp.first,dp.second);
      justOnce=true;
      invLst.push_back(new InvScoreCache(sl));
    }
    if( incl.size() > 1 && !justOnce ) {
      const int inPut = incl[1];
      if( ((i-6) == (unsigned)inPut) && inPut < stop_pos ) {
	SeqLabel sl(SeqLabel::eDon,SeqLabel::eUnk,dp.first,dp.second);
	invLst.push_back(new InvScoreCache(sl));
      }
    }
  }
  invLst.push_back(new InvScoreCache(SeqLabel(SeqLabel::eEnd,SeqLabel::eUnk,stop_pos,stop_pos)));
  
  vector<InvScoreCache*> invVec(invLst.size());
  list<InvScoreCache*>::iterator iter = invLst.begin();
  const list<InvScoreCache*>::iterator stop = invLst.end();
  for(unsigned cnt = 0; iter != stop; ++cnt, ++iter) {
    assert( cnt < invVec.size());
    invVec[cnt] = *iter;
  }
  for(unsigned cnt = 1; cnt < invVec.size(); ++cnt) {
    const SeqLabel& sl = invVec[cnt]->getBnd();
    signed niter = (signed)cnt-1;
    InvScoreCache* match = getPrevNonOverlap(invVec,sl,niter);
    cacheScore(*match,*invVec[cnt],svec,start_codon,stop_codon,predType);
  }
  //iterator through asplice, csplice
  vector<smatrix_t*> matr(invVec.size(),NULL);
  dpUpdate(invVec,matr,-1,0,svec,start_codon,stop_codon,useCons,predType,limitFrame,limitNextFrame,incl,isInitExon,isLastExon);
  for(unsigned cnt = 1; cnt < invVec.size(); ++cnt) {
    const SeqLabel& sl = invVec[cnt]->getBnd();
    for(signed prv = (signed)cnt; prv > 0; ) {
      signed niter = prv-1;
      getPrevOverlap(invVec,sl,niter);

      dpUpdate(invVec,matr,niter,cnt,svec,start_codon,stop_codon,useCons,predType,limitFrame,limitNextFrame,incl,isInitExon,isLastExon);
      prv = niter;
      assert(matr[niter] && matr[cnt]);
    }
  }
  unsigned eIdx = matr.size()-1;
  list<SeqLabelVal>* res = traceBack(matr[eIdx],retFrame); 
  for(unsigned i = 0; i <invVec.size(); ++i) {
    delete invVec[i];
    delete matr[i];
  }
  if( res->empty() && limitFrame.size()>0) {
    cout<<"can't predict exon within constraints, relaxing\n";
    vector<int> tmp;
    return labelF(svec,start_codon,stop_codon,start_pos,stop_pos,retFrame,useCons,predType,tmp,limitNextFrame,incl,isInitExon,isLastExon);
      
  }
  return res;
}


static vector<spair_t>
compScore(ModelManager& mm, const vector<InvScoreCache*>& isc, int cidx, 
	  const vector<string*>& svec, int start_codon, int stop_codon, const SeqLabel& psl, const SeqLabel& csl, pred_t predType) {
  vector<spair_t> tsum(3,spair_t(true,0));
  ModelType::inv_t inv = ModelType::sl2inv(psl,csl);
  if( inv == ModelType::eInvCnameEnd ) {
    cout<<"Danger will robinson, danger! unhandled interval case abandon ship: "<<psl<<" "<<csl<<endl;
    tsum[0]=tsum[1]=tsum[2]=spair_t(false,0);
    return tsum;
  }
  const bool verb = true;
  // allow psl and csl, to be acc/acc or don/don pairs, with overlapping windows
  const bool isOverlap = (csl.isIn(psl.getLeft()) || csl.isIn(psl.getRght()));
  const bool isSameType = ((SeqLabel::isAcc(psl.getLabel())&&SeqLabel::isAcc(csl.getLabel()))||(SeqLabel::isDon(psl.getLabel())&&SeqLabel::isDon(csl.getLabel())));
  if( isOverlap && isSameType ) {
    assert(psl.getExonBnd() < csl.getExonBnd());
    if( predType==eBasicFixedRate||predType==eLearnedFixedRate ) {
      tsum = mm.scoreInterval1a(psl,csl,svec,start_codon,stop_codon,&csl,NULL,predType,verb);
    } else {
      tsum = mm.scoreInterval1b(psl,csl,svec,start_codon,stop_codon,&csl,NULL,verb);
    }
    return tsum;
  }


  int pidx = cidx-1;
  getPrevNonOverlap(isc,csl,pidx);
  const SeqLabel& tp = isc[pidx]->getBnd();
  const int lpos = tp.getRght()+1;
  vector<spair_t> ts;
  list<spair_t> lst; //debug
  if( lpos < csl.getRght() && !(csl.isEnd() && csl.getLeft()==lpos) ) {
    assert(lpos <= csl.getLeft());
    if( predType==eBasicFixedRate||predType==eLearnedFixedRate ) {
      ts = mm.scoreInterval1a(psl,csl,svec,start_codon,stop_codon,&tp,NULL,predType,verb);
    } else {
      ts = mm.scoreInterval1b(psl,csl,svec,start_codon,stop_codon,&tp,NULL,verb);
    }
    for(unsigned x = 0; x < 3; ++x) {
      if( ts[x].first && tsum[x].first ) {
	tsum[x].second += ts[x].second;
      } else {
	tsum[x].first = false;
      }
    }
  }

  if( DEBUGON ) {
    const int pframe = 0;
    //cout<<"onfly: "<<ts[0]<<" "<<ts[1]<<" "<<ts[2]<<" "<<lpos<<" to "<<csl.getRght()<<" sum1: "<<tsum[1]<<endl;
    cout<<"onfly: "<<ts[pframe]<<" "<<lpos<<" to "<<csl.getRght()<<" "<<tsum[pframe]<<endl;
    lst.push_front(ts[pframe]);
  }
  if( tp.getLeft()==psl.getLeft() && tp.getRght()==psl.getRght() ) {
    pidx = -1;
  }
  int ridx = pidx;
  bool finish=false;
  while(ridx > 0) {
    vector<spair_t> ts = isc[ridx]->getInvScore((unsigned)inv);
    const SeqLabel& rghtBnd = isc[ridx]->getBnd();

    int lidx = ridx-1;
    getPrevNonOverlap(isc,rghtBnd,lidx);
    const SeqLabel& leftBnd = isc[lidx]->getBnd();
    //if( psl.getRght() >= leftBnd.getLeft() && (rghtBnd.getLeft() != psl.getLeft() || rghtBnd.getRght() != psl.getRght()  )) {
    if( leftBnd.getRght() < psl.getLeft() || (SeqLabel::isOverlap(psl,leftBnd) && (rghtBnd.getLeft() != psl.getLeft() || rghtBnd.getRght() != psl.getRght()  ))) {

      // overlap
      const bool verb = true;
      if( predType==eBasicFixedRate||predType==eLearnedFixedRate ) {
	ts = mm.scoreInterval1a(psl,csl,svec,start_codon,stop_codon,NULL,&rghtBnd,predType,verb);
      } else {
	ts = mm.scoreInterval1b(psl,csl,svec,start_codon,stop_codon,NULL,&rghtBnd,verb);
      }
      finish=true;
    }
#if 0
    if( !ts[2].first ) {
      int stop =0;
    }
#endif
    for(unsigned x = 0; x < 3; ++x) {
      if( ts[x].first && tsum[x].first ) {
	tsum[x].second += ts[x].second;
      } else {
	tsum[x].first = false;
      }
    }
    if( DEBUGON ) {
      int pos = leftBnd.getRght()+1;
      if( finish ) {
	pos = psl.getRght()+1;
      }
      const string str = finish ? "onfl2: " : "cache: ";
      const int pframe = 0;
      cout<<str<<ts[pframe]<<" "<<pos<<" to "<<rghtBnd.getRght()<<" "<<tsum[pframe]<<endl;
      lst.push_front(ts[pframe]);
    }
    if(finish || (leftBnd.getLeft() == psl.getLeft() && leftBnd.getRght()==psl.getRght())) break;
    ridx = lidx;

  }
  if(DEBUGON) {
    list<spair_t>::const_iterator iter = lst.begin();
    const list<spair_t>::const_iterator stop = lst.end();
    spair_t sum(true,0);
    for(; iter != stop; ++iter) {
      sum.second += (*iter).second;
      cout<<*iter<<" "<<sum<<endl;
    }
  }
  return tsum;
}

void
LabelSeq::dpUpdate(const vector<InvScoreCache*>& isc, vector<smatrix_t*>& sm, int pidx, int cidx, 
		   const vector<string*>& svec, int start_codon, int stop_codon, bool isCons, pred_t predType, 
		   const vector<int>& limitFrame, const vector<int>& limitNextFrame, const vector<int>& incl, bool isInitExon, bool isFinalExon) {
  const SeqLabel& tsl = isc[cidx]->getBnd();
  bool initVal = pidx < 0 ? true : false;
  SeqLabelVal orig_sl(tsl,spair_t(initVal,0));
  smatrix_t* possiblePtr = sm[cidx];
  if( !sm[cidx] ) {
    if(isCons) {
      sm[cidx] = possiblePtr = orig_sl.splitLabelCons2();
    } else {
      sm[cidx] = possiblePtr = orig_sl.splitLabel1();
    }
  }
  smatrix_t& possible = *possiblePtr;
  bool debugOn= DEBUGON;
  if( incl.size() > 0 ) {
    if( incl[0]==orig_sl.getExonBnd() && SeqLabel::isDon(orig_sl.getLabel() ) ) return;
  }
  if( SeqLabel::isDon(orig_sl.getLabel()) && ( start_codon == orig_sl.getExonBnd() || start_codon == (orig_sl.getExonBnd()-1) ) ) {
    return;
  }
  if(  orig_sl.isSplice() && (orig_sl.getExonBnd() < start_codon || orig_sl.getExonBnd() > stop_codon) ) {
    return;
  }
  //if( sl.isIn(start_codon) ) {
  //return;
  //}
  if( pidx >= 0 ) {
    assert(sm[pidx]);
    const smatrix_t& prev = *sm[pidx];
    smatrix_t::const_iterator iter = prev.begin();
    const smatrix_t::const_iterator stop = prev.end();
    for(; iter != stop; ++iter) {
      const vector<SeqLabelVal>& pslvVec = *iter; //prev[i];
      if( !pslvVec[0].isValid() && !pslvVec[1].isValid() && !pslvVec[2].isValid() ) {
	continue;
      }
      const SeqLabel& psl = pslvVec[0];
#if 0
      if( (orig_sl.getLeft()==867 && psl.getLeft()==0) ||
	  (orig_sl.getLeft()==905 && psl.getLeft()==867) ||
	  (orig_sl.getLeft()==1725 && psl.getLeft()==905) || 
	  (orig_sl.getLeft()==1784 && psl.getLeft()==1725)  
	  ) {
	int stop = 0;
      }
#endif
      vector<vector<spair_t> > scores(2);
      for(unsigned k = 0; k < scores.size(); ++k) {
	scores[k].resize(3,spair_t(true,0));
      }
      smatrix_t::iterator citer = possible.begin();
      const smatrix_t::iterator cstop = possible.end();
      //for(unsigned j = 0; j < possible.size(); ++j) {
      for(; citer != cstop; ++citer) {
	vector<SeqLabelVal>& cslvVec = *citer; //possible[j];
	const SeqLabel& csl = cslvVec[0];
	if( incl.size() > 0 ) {
	  int pillar=-1;
	  assert(incl[0]<incl[1]);
	  for(signed i =incl.size()-1 ; i>=0; --i) {
	    if( csl.getExonBnd() > incl[i] ) {
	      pillar = incl[i];
	      break;
	    }
	  }
	  if( psl.getExonBnd() < pillar ) {
	    return; // valid for all csl
	  }
	  ModelType::cname_t mt = ModelType::sl2cnameInv(psl,csl,true);
	  if( (psl.getLeft() == incl[0] && incl[0] == 0 && !ModelType::isExon(mt)) || 
	      (orig_sl.getLeft() == incl[1] && incl[1] == (signed)svec[0]->size() && !ModelType::isExon(mt)) ) {
	    continue;
	  }
	}
#if 0
	if(
	   (psl.getLeft()==43 && psl.getLabel()== SeqLabel::eNea && csl.getLeft()==292 && csl.getLabel()== SeqLabel::eEnd) ||
	   (psl.getLeft()==491 && psl.getLabel()== SeqLabel::eIrSa && csl.getLeft()==651 && csl.getLabel()== SeqLabel::eNed) ||
	   (psl.getLeft()==819 && psl.getLabel()== SeqLabel::eEnd && csl.getLeft()==651 && csl.getLabel()== SeqLabel::eNed) ) {
	  int stop=0;
	}
	if( psl.getLeft()==10 && psl.getLabel()==SeqLabel::eFma1 && csl.getLeft()==79 && csl.getLabel()==SeqLabel::eFma2) {
	  int stop = 0;
	}
#endif
	bool isFixedAcc=false, isFixedDon=false;
	if( incl.size() > 0 ) {
	  if( incl[0]==orig_sl.getExonBnd() ) {
	    isFixedAcc=true;
	  }
	  if( incl[1]==orig_sl.getExonBnd() || (csl.isEnd() && incl[1]==(csl.getExonBnd()+1)) ) {
	    isFixedDon=true;
	  }
	}
	if( !isFixedAcc && SeqLabel::isAcc(csl.getLabel()) && !SeqLabel::isAccIr(csl.getLabel()) && start_codon < csl.getExonBnd() && start_codon > 0 ) {
	  continue;
	}
	if( limitFrame.size() > 0 && isInitExon &&
	    ((psl.getLabel()==SeqLabel::eBeg && /*!isFixedAcc &&*/ SeqLabel::isAcc(csl.getLabel()) && !SeqLabel::isAccIr(csl.getLabel())) ||
	     (!isFinalExon && psl.getLabel()==SeqLabel::eBeg && csl.getLabel()==SeqLabel::eEnd1)  || 
	     (psl.getLabel()==SeqLabel::eBeg && csl.getLabel()==SeqLabel::eEnd))
	    ) {
	  continue;
	}
	if( limitFrame.size() > 0 && isFinalExon &&
	    ((csl.getLabel()==SeqLabel::eEnd && SeqLabel::isDon(psl.getLabel()) && !SeqLabel::isDonIr(psl.getLabel())) ||
	     (!isInitExon && psl.getLabel()==SeqLabel::eBeg && csl.getLabel()==SeqLabel::eEnd1) ||
	     (psl.getLabel()==SeqLabel::eBeg && csl.getLabel()==SeqLabel::eEnd))
	    ) {
	  continue;
	}
	if( limitFrame.size() > 0 && !isFinalExon && !isInitExon &&
	    ((psl.getLabel()==SeqLabel::eBeg && SeqLabel::isDon(csl.getLabel())) ||
	     (SeqLabel::isAcc(psl.getLabel()) && csl.getLabel()==SeqLabel::eEnd) ||
	     (psl.getLabel()==SeqLabel::eBeg && csl.getLabel()==SeqLabel::eEnd1)) ) {
	  continue;
	}
	// don't allow introns that disrupt frame
	if( limitFrame.size() > 0 && !isFinalExon && !isInitExon && psl.getLabel()==SeqLabel::eBeg && csl.getLabel()==SeqLabel::eEnd ) {
	  bool pass=false;
	  for(unsigned en=0; en < limitFrame.size(); ++en) {
	    for(unsigned en1=0; en1 < limitNextFrame.size(); ++en1) {
	      if(limitFrame[en]==limitNextFrame[en1] ) {
		pass=true;
		break;
	      }
	    }
	    if(pass) break;
	  }
	  if( !pass ) continue;
	}
	StateTrans::state_t st = StateTrans::label2state(psl,csl,false);
	const SeqLabel& cleft = psl;
	const SeqLabel& crght = csl;
	if( psl.getEvent() != csl.getEvent() && (psl.getLabel()==SeqLabel::eBeg||csl.isEnd())) {
	  continue;
	}
	if( st != StateTrans::eStateSize ) {
	  const unsigned qidx = (SeqLabel::event_t)csl.getEvent();
	  bool done=false;
	  for(unsigned x=0; x < scores[qidx].size();++x) {
	    if(!scores[qidx][x].first) {
	      done=true;
	    }
	  }
	  if( !done ) {
	    scores[qidx] = compScore(_mm,isc,cidx,svec,start_codon,stop_codon,psl,csl,predType);
	  }
	  for(unsigned frame = 0; frame < 3; ++frame) {
	    bool force=false;
	    spair_t sc = scores[qidx][frame];
	    if( !sc.first ) {
	      if(incl.size()==0) {
		continue;
	      } else {
		bool cMatch =  (csl.getExonBnd()==incl[1] || 
				(csl.isEnd() && incl[1]==(csl.getExonBnd()+1)) );
		bool pMatch =  (psl.getExonBnd()==incl[0] || 
				(psl.getLabel()==SeqLabel::eBeg && incl[0]==(psl.getExonBnd()-1)) );
		force = cMatch && pMatch;
		if( !force) {
		  continue;
		}
	      }
	    }
	    const int rframe = frame;
	    SeqLabelVal& cslv = cslvVec[rframe];
	    const SeqLabelVal& pslv = pslvVec[frame];
	    bool passFrameTest=true;
	    if( limitFrame.size() > 0 && SeqLabel::isAcc(cslv.getLabel()) && !SeqLabel::isAccIr(cslv.getLabel()) ) {
	      passFrameTest=false;
	      for(unsigned ti=0; ti< limitFrame.size(); ++ti) {
		const int iframe = limitFrame[ti];
		if((int)frame == frame2exalt(iframe,cslv.getExonBnd(),start_codon) ) {
		  passFrameTest=true;
		  break;
		}
	      }
	    }
#if 1
	    if( limitFrame.size() > 0 && SeqLabel::isAccIr(cslv.getLabel()) ) {
	      passFrameTest=false;
	      for(unsigned ti=0; ti< limitFrame.size(); ++ti) {
		const int iframe = limitFrame[ti];
		//the irsa must connect with the irsd
		if( pslv.getPrev() ) {
		  int sub=0;
		  if( iframe == 2 ) sub=1;
		  else if( iframe == 1 ) sub=2;
		  const int clen = compCodingLen(pslv,pslv.getPrev(),start_codon,stop_codon) - sub;
		  const int nframe = clen % 3;
		  const int eframe = frame2exalt(nframe,cslv.getExonBnd(),start_codon);
		  if( eframe == (signed)frame ) {
		    passFrameTest=true;
		    break;
		  }
		}
	      }
	    }
#endif
	    if( !passFrameTest ) {
	      continue;
	    }
	    ModelType::cname_t tc=ModelType::sl2cnameInv(pslv,cslv,true);
	    if( limitNextFrame.size() > 0 && 
		((SeqLabel::isDon(cslv.getLabel()) && !SeqLabel::isDonIr(cslv.getLabel())) ||
		 (csl.getLabel()==SeqLabel::eEnd && ModelType::isExon(tc)))
		&& (pslv.getPrev() || pslv.getLabel()==SeqLabel::eBeg)) {
	      passFrameTest=false;
	      for(unsigned en=0; en < limitFrame.size(); ++en) {
		int sub=0;
		if( limitFrame[en] == 2 ) sub=1;
		else if( limitFrame[en] == 1 ) sub=2;
		const int clen = compCodingLen(cslv,&pslv,start_codon,stop_codon) - sub;
		if( clen > 3 ) {
		  for(unsigned ti=0; ti< limitNextFrame.size(); ++ti) {
		    const int nframe = clen % 3;
		    if(limitNextFrame[ti] == nframe ) {
		      passFrameTest=true;
		      break;
		    }
		  }
		}
	      }   
	    }
	    if( !passFrameTest ) {
	      continue;
	    }
	    double tprob = 0;
	    if( st != StateTrans::eStateSize && pslv.isValid() ) {
	      if( pslv.getPrev() ) {
		const SeqLabel& pleft = *(pslv.getPrev());
		const SeqLabel& prght = pslv;
		tprob = _mm.scoreTrans(pleft,prght,cleft,crght);
	      } else {
		tprob = _mm.scoreTrans(cleft,crght);
	      }
	    }
	    if( (sc.first && tprob > 0) || force ) {
	      sc.second += log2(tprob);
	      if( force ) {
		sc.second=0;
	      }
	      if( debugOn ) {
		cout<<"sc: "<<pslv<<" "<<cslv<<" "<<sc<<" "<<pslv.getVal()<<" tprob: "<<tprob<<endl;
	      }
	      if( (sc.second+pslv.getVal() > cslv.getVal() || cslv.isNew()) && pslv.isValid() ) {
		double val = sc.second+pslv.getVal();
		cslv.setVal(spair_t(true,val));
		cslv.setPred(&pslv);
		if(debugOn) {
		  cout<<"connecting: frame "<<frame<<" "<<pslv<<" "<<cslv<<endl;
		}
	      }
	    }
	  }
	}
      }
    }
  }
}

static int 
findIdx(const vector<InvScoreCache*>& inv, const SeqLabel& sl) {
  int idx = -1;
  for(unsigned i = 0; i < inv.size(); ++i) {
    const SeqLabel& tl = inv[i]->getBnd();
    if( tl.getRght() == sl.getRght() && tl.getLeft() == sl.getLeft() ) {
      idx = i;
      break;
    }
  }
  assert(idx != -1);
  return idx;
}

spair_t
LabelSeq::scoreF(const vector<string*>& svec, int start_codon, int stop_codon, const std::list<SeqLabel>& slst,int labelFrame, pred_t predType) const {
  std::ostream* osPtr = Options::getInstance()->_logStream;
  const string& str = *svec[0];
  list<SeqLabel>::const_iterator ti = slst.begin();
  const list<SeqLabel>::const_iterator st = slst.end();
  bool isValid=true;
  const SeqLabel* lastPtr = NULL;
  for(; ti != st; ++ti) {
    const SeqLabel& sl = *ti;
    const int pos = sl.getExonBnd();
    if( SeqLabel::isDon(sl.getLabel()) ) {
      // remember pos is 1 based
      //isValid = (tolower(str[pos]) == 'g' && (tolower(str[pos+1]) == 't' || tolower(str[pos+1]) == 'c') );
      isValid = (tolower(str[pos]) == 'g' && tolower(str[pos+1]) == 't');
      if( !isValid ) {
	break;
      }
    } else if( SeqLabel::isAcc(sl.getLabel()) ) {
      isValid = (tolower(str[pos-3]) == 'a' && tolower(str[pos-2]) == 'g');
      if(!isValid ) {
	break;
      }
    }
    lastPtr = &sl;
  }
  if( !isValid ) {
    //cout<<"Only deal with GT/GC AG splice sites in this round (that's tough enough!) and no overlaps - skipping from analysis"<<endl;
    if( osPtr ) (*osPtr)<<"limited to scoring AG/GT exons currently, sorry.\n";
    return spair_t(false,0);
  }
  assert(svec[0]);
  list<InvScoreCache*> invLst;
  int start_pos = slst.front().getLeft();
  invLst.push_back(new InvScoreCache(SeqLabel(SeqLabel::eBeg,SeqLabel::eUnk,start_pos,start_pos)));
  const int slen = (signed)str.size();
  const int beg = start_pos < 6 ? 6 : start_pos+1;
  for(unsigned i = beg; i < str.size(); ++i) {
    ipair_t ap = SeqLabel::getSigDim1(SeqLabel::eAcc,i);
    ipair_t dp = SeqLabel::getSigDim1(SeqLabel::eDon,i);
    if( tolower(str[i-4]) == 'a' && tolower(str[i-3])=='g' &&
	ap.first > start_pos && ap.second <= slen ) {
      SeqLabel sl(SeqLabel::eAcc,SeqLabel::eUnk,ap.first,ap.second);
      invLst.push_back(new InvScoreCache(sl));
    }
    //if( tolower(str[i-6]) == 'g' && (tolower(str[i-5])=='t' ||tolower(str[i-5])=='c')
    if( tolower(str[i-6]) == 'g' && tolower(str[i-5])=='t' 
	&&
	dp.first > start_pos && dp.second <= slen ) {
      SeqLabel sl(SeqLabel::eDon,SeqLabel::eUnk,dp.first,dp.second);
      invLst.push_back(new InvScoreCache(sl));
    }
  }
  int stop_pos = slst.back().getLeft();
  invLst.push_back(new InvScoreCache(SeqLabel(SeqLabel::eEnd,SeqLabel::eUnk,stop_pos,stop_pos)));
  
  vector<InvScoreCache*> invVec(invLst.size());
  list<InvScoreCache*>::iterator iter = invLst.begin();
  const list<InvScoreCache*>::iterator stop = invLst.end();
  for(unsigned cnt = 0; iter != stop; ++cnt, ++iter) {
    assert( cnt < invVec.size());
    invVec[cnt] = *iter;
    //cout<<"what: "<<invVec[cnt]->getBnd()<<endl;
  }
  for(unsigned cnt = 1; cnt < invVec.size(); ++cnt) {
    const SeqLabel& sl = invVec[cnt]->getBnd();
    signed niter = (signed)cnt-1;
    InvScoreCache* match = getPrevNonOverlap(invVec,sl,niter);
    cacheScore(*match,*invVec[cnt],svec,start_codon,stop_codon,predType);
  }
  const SeqLabel* pleft = NULL, * prght = NULL;
  vector<spair_t> sum(3,spair_t(true,0));
  list<SeqLabel>::const_iterator siter = slst.begin();
  const list<SeqLabel>::const_iterator send = slst.end();
  const SeqLabel* psl = &(*siter);
  ++siter;
  bool fail=false;
  bool status = false;
  for(unsigned intervalCnt=0; siter != send; ++siter,++intervalCnt) {
    const SeqLabel* csl = &(*siter);
    const int cidx = findIdx(invVec,*csl);
    //   DEBUGON=true;
    vector<spair_t> scores = compScore(_mm,invVec,cidx,svec,start_codon,stop_codon,*psl,*csl,predType);
    double tprob = 0;
    if( labelFrame == -1 ) {
      if( intervalCnt == 0 ) {
	tprob = _mm.scoreTrans(*psl,*csl);
      } else {
	tprob = _mm.scoreTrans(*pleft,*prght,*psl,*csl);
      }
      if( tprob <= 0 ) {
	//fail = true;
	tprob=1;
	if( osPtr ) {
	  (*osPtr)<<"Warning unseen transition, may want to use pseudo counts"<<endl;
	  if( intervalCnt == 0 ) (*osPtr)<<"boundaries: "<<*psl<<" "<<*csl<<endl;
	  else (*osPtr)<<"boundaries: "<<*pleft<<" "<<*prght<<" "<<*psl<<" "<<*csl<<endl;
	}
	break;
      }
    } else {
      tprob=1;
    }
    for(unsigned i = 0; i < sum.size(); ++i) {
      if( scores[i].first && sum[i].first ) {
	const double wght = log2(tprob);
	sum[i].second += (scores[i].second + wght);
      } else {
	sum[i].second = 0;
	sum[i].first = false;
      }
    }
    if( labelFrame == -1 && osPtr ) {
      (*osPtr)<<"here we scr: "<<*psl<<" "<<*csl<<"|"<<scores[0]<<"|"<<scores[1]<<"|"<<scores[2]<<endl;
      (*osPtr)<<"here we sum: "<<*psl<<" "<<*csl<<"|"<<sum[0]<<"|"<<sum[1]<<"|"<<sum[2]<<endl;
    }
    if(!sum[0].first && !sum[1].first && !sum[2].first) {
      fail=true;
      break;
    }
    pleft = psl;
    prght = csl;
    psl = csl;
  }
  double best = -1;
  if( !fail ) {
    if( labelFrame == -1 ) {
      for(unsigned frame = 0; frame < 3; ++frame) {
	if( (!status || sum[frame].second > best) && sum[frame].first ) {
	  best =sum[frame].second;
	  status = true;
	  labelFrame=frame;
	}
      }
      if( osPtr ) (*osPtr)<<"Score we found: "<<labelFrame<<" "<<status<<" "<<best<<endl;
    } else {
      best = sum[labelFrame].second;
      status = true;
    }
  } else {
    if( osPtr ) (*osPtr)<<"failed to compute score"<<endl;
  }
  for(unsigned i = 0; i <invVec.size(); ++i) {
    delete invVec[i];
  }
  return spair_t(status,best);
}
