#include "ModelHash.h"
#include <fstream>
#include <iostream>

using std::ostream;
using std::string;
using std::cerr;
using std::endl;
using std::ofstream;

Model*
ModelHash::cDiff(ModelType::mname_t left, ModelType::mname_t rght) const {
  ModelHash::const_iterator li = find(left);
  const Model* lmd = (*li).second;
  ModelHash::const_iterator ri = find(rght);
  const Model* rmd = (*ri).second;
  return *lmd-*rmd;
}

Model* 
ModelHash::compDiff(ModelType::mname_t  m1, ModelType::mname_t m2) const {
  Model* diff = cDiff(m1,m2);
  return diff;
}

void ModelHash::operator+=(const ModelHash& mm) {
  ModelHash::const_iterator iter = mm.begin();
  const ModelHash::const_iterator end = mm.end();
  for(; iter != end; ++iter) {
    ModelType::mname_t id = (*iter).first;
    assert(id != ModelType::eError);
    if( id == ModelType::eTerm ) {
      continue;
    }
    const Model* md = (*iter).second;
    Model& thisModel = *((*this)[id]);
    thisModel += *md;
  }
}

void 
ModelHash::print(ostream& os) const {
  ModelHash::const_iterator iter = begin();
  const ModelHash::const_iterator end = this->end();
  for(; iter != end; ++iter) {
    ModelType::mname_t id = (*iter).first;
    assert(id != ModelType::eError);
    if( id == ModelType::eTerm ) {
      continue;
    }
    const Model* md = (*iter).second;
    md->print(os);
  }
}

void ModelHash::operator/=(int div) {
  ModelHashT::iterator iter = begin();
  const ModelHashT::iterator end = this->end();
  for(; iter != end; ++iter) {
    ModelType::mname_t id = (*iter).first;
    assert(id != ModelType::eError);
    if( id == ModelType::eTerm ) {
      continue;
    }
    Model* md = (*iter).second;
    Model& thisModel = *md;
    thisModel /= div;
  }
}


ModelHash& 
ModelHash::operator=(const ModelHash& mm) {
  ModelHashT::const_iterator iter = mm.begin();
  const ModelHashT::const_iterator end = mm.end();
  for(; iter != end; ++iter) {
    ModelType::mname_t id = (*iter).first;
    assert(id != ModelType::eError);
    if( id == ModelType::eTerm ) {
      continue;
    }
    const Model* md = (*iter).second;
    (*this)[id] = md->clone();
  }
  return *this;
}
