#ifndef _MODELMANAGER_H
#define _MODELMANAGER_H

#include <ext/hash_map>
#include "ParamSearch.h"
#include "RateManager.h"
#include "ConsHmmInstance.h"
#include "Trellis.h"
#include "ModelType.h"
#include "ModelHash.h"
#include "Model.h"
#include "ConsHmm.h"
#include "Kmer.h"
#include "ParamFile.h"
#include "ExonDescrip.h"
#include "PhyloModel.h"
#include "StateTrans.h"
#include "ConsManager.h"
#include "ConsThresh.h"
#include "stype.h"
#include "MultiModel.h"
#include <utility>


class ModelManager {
 public:
  typedef ModelType::conserve_t conserve_t;
  typedef ModelType::cname_t cname_t;
  typedef ModelType::mname_t mname_t;
  typedef ConsManager::cons_state_t cons_state_t;
  typedef ConsHmmInstance::hpair_t hpair_t;
  typedef __gnu_cxx::hash_map<ModelType::mname_t,PhyloModel*, myhash > PhyloModelHashT;


 public:
  ModelManager(bool=false);
  ~ModelManager();

  void setST(StateTrans* st) { _stateTrans = st; }
  void setRM(RateManager* rm) { _rm = rm; }
  void setCM(ConsManager* cm) { _cm = cm; }
  static tp_t getCname(const std::list<SeqLabel>& lst, int pos, bool isPrc);
  ParamSearchPair* selectParam(ParamSearchHashT&);
  void updateParams(ParamSearchHashT&);
  void write(const std::string&) const;
  bool readLFR(const std::string&) ;
  bool read(const std::string&) ;
  RateManager& getRateManager() { return *_rm; }
  const RateManager& getRateManager() const { return *_rm; }
  const ConsManager& getConsManager() const { return *_cm; }
  ConsManager* getConsManager() { return _cm; }
  void
   train(const fasta&); 
  void
    trainRates(const fasta&); 
  int discTrain(const std::vector<std::string*>&,int,int, const std::vector<std::list<SeqLabel>*>&, const std::vector<int>&, ParamSearchHashT& );
  void compare3(const std::vector<std::string*>&,int,int, const std::vector<std::list<SeqLabel>*>& );
  void compare(const std::vector<std::string*>&,int,int, const std::vector<std::list<SeqLabel>*>&, pred_t );
  dpair_t scoreSpliceConservation(const SeqLabel&, const std::vector<std::string*>&);
  dpair_t scoreIntronConservation(const SeqLabel&, const SeqLabel&, const std::vector<std::string*>&,int,int);
  double scoreBnd(const SeqLabel&, const std::vector<std::string*>&, bool = false);
  std::vector<spair_t>
    scoreInterval1a(const SeqLabel&, const SeqLabel&, const std::vector<std::string*>&, int, int, const SeqLabel*, const SeqLabel*, pred_t, bool = false);
  std::vector<spair_t>
    scoreInterval1b(const SeqLabel&, const SeqLabel&, const std::vector<std::string*>&, int, int, 
		    const SeqLabel*, const SeqLabel*, bool = false);
  std::vector<spair_t>
    scoreInterval5a(int,int, const std::vector<std::string*>&, int, int, cname_t, pred_t, bool = false);
  std::vector<spair_t>
    scoreInterval5b(int,int, const std::vector<std::string*>&, int, int, cname_t, bool = false);
  void
    scoreInterval2(int,int,const std::vector<std::string*>&,int,int, int);
  double scoreTrans(const SeqLabel& left, const SeqLabel& rght) const;
  double scoreTrans(const SeqLabel& pleft, const SeqLabel& prght, const SeqLabel& cleft, const SeqLabel& crght) const; 
  const StateTrans& getStateTrans() const { return *_stateTrans; }
  const ModelHash& getModelKmerHash() const { return _kmer; }
  int
    scorePosHelp3(bool, unsigned, const std::vector<std::string*>& svec, ModelType::cname_t cn, std::vector<htype_t>&,
		  bool&, std::vector<int>&, std::vector<spair_t>& psum, std::vector<Trellis*>&);
  int
    scorePosHelp2(bool, unsigned, const std::vector<std::string*>& svec, ModelType::cname_t cn, std::vector<htype_t>&,
		  bool&, std::vector<int>&, std::vector<spair_t>& psum, std::vector<Trellis*>&);
  int
    compProb3(unsigned pos, int phase, 
	      const std::vector<std::string*>&, const std::string&, cname_t cn, hpair_t hp, htype_t,
	      Trellis& chi, bool isInit, bool verbose);
 private:

  int getIntronConsLevel(unsigned pos, const std::vector<std::string*>&, cname_t);
  void normalize();
  void
    trainStage(const fasta&,int); 
  void
    trainFalse(const fasta&); 
  bool
    updateModel(const std::string&,
		const SeqLabel&, const SeqLabel&, const ExonDescrip&,
		const std::vector<std::string*>&, bool,int);
  void
    updateFalseModel(const std::string&,
		const SeqLabel&, const SeqLabel&, const ExonDescrip&,
		const std::vector<std::string*>&, bool);
  std::pair<ModelType::mname_t,int>
    getModel(const SeqLabel&, const SeqLabel&, const ExonDescrip&, unsigned,
	     const std::vector<std::string*>&, bool,int,bool&);
  void
    createPhyloModel(ModelType::mname_t); 
  void
    createModelKmer(ModelType::mname_t);
  void
    createModel(ModelType::mname_t);

 private:
  ipair_t
    getConsType(unsigned,const std::vector<std::string*>&, cname_t, const ExonDescrip&,
		const SeqLabel&, const SeqLabel&,int,bool&);
  int
    scorePosHelp(bool,unsigned, const std::vector<std::string*>&, ModelType::cname_t, std::vector<htype_t>&, bool, 
		 std::vector<int>&, std::vector<spair_t>&, std::vector<ConsHmmInstance*>&);
  ipair_t
    getSpliceConsType(unsigned, const std::vector<std::string*>&, cname_t, int,int,bool&);

  int
    compProb2(unsigned, int, 
	      const std::vector<std::string*>&,
	      const std::string&, 
	      cname_t, 
	      hpair_t,
	      ConsHmmInstance&,
	      bool,
	      bool=false); 

  bool
    runBM(const SeqLabel& leftLabel, const SeqLabel& rghtLabel, const ExonDescrip& targetCoords,
	  const std::vector<string*>& svec, std::vector<double>&, std::vector<double>&);

  spair_t
    compProb1(unsigned, int, const std::vector<std::string*>&,const std::string&,cname_t,const std::string&, pred_t, bool, int, bool=false);

 private:
  RateManager* _rm;
  ConsManager* _cm;
  MultiModelHashT _mm;
  ModelHash _kmer;
  PhyloModelHashT _pm;
  const ParamFile* _pf;
  StateTrans* _stateTrans;
  //InitNucDistr _initDistr;
  ConsThresh _cthresh;
  bool _sepNonCons;
};

#endif // _MODELMANAGER_H
