#include "ModelManager.h"
#include <cmath>
#include <stdexcept>

using std::pair;
using std::cerr;
using std::vector;
using std::cout;
using std::endl;
using std::string;
using std::list;

extern bool DEBUGON;
extern bool XDEBUGON;

vector<spair_t>
ModelManager::scoreInterval5a(int lpos, int rpos, const vector<string*>& svec, 
			      int start, int stop, cname_t icn, pred_t predType, bool verbose) {
  assert(lpos > 0);
  vector<int> ftrack(3,-1);
  getPhases(ftrack,lpos,start);
  bool lastPrc = false;
  bool lDebug = false;
  if( lpos == 76 && rpos == 116 && icn == ModelType::eCexonPrc ) {
    lDebug=true;
  }
  vector<spair_t> psum(3,spair_t(true,0));
  for(signed i = lpos; i <= rpos; ++i) {
    if( i < 4 ) continue;
    const string& str = *svec[0];
    if( str[i] == '-' ) {
      continue;
    }
    const bool isPrc = isPrcCheck(i,start,stop); 
    //bool isPrc = (i >= start && i<= stop);
    cname_t cn = icn;
    if( ModelType::isExon(icn) && isPrc ) {
      cn = ModelType::isAlt(icn) ? ModelType::eAexonPrc : ModelType::eCexonPrc;
    } else if( ModelType::isExon(icn) && !isPrc) {
      cn = ModelType::isAlt(icn) ? ModelType::eAexonNpc : ModelType::eCexonNpc;
    }
    if(!isPrc || !ModelType::isExon(cn)) {
      ftrack[0]=ftrack[1]=ftrack[2] = -1;
    }
    // this means we've switch into prc
    // and thie 1st time, and we know only frame=0 is valid
    else if(  ftrack[0] == -1 && isPrc && ModelType::isExon(cn) ) {
      ftrack[0]=0;
      ftrack[1]=ftrack[2]=-1;
    }
    const unsigned num_org = svec.size();
    string column(num_org,'-');
    fillColumn(column,svec,i);
    //cout<<"column "<<column<<" "<<i<<endl;
    spair_t pscore(false,0);
    for(unsigned x = 0; x < 3; ++x) {
      int& frame = ftrack[x];
      if( frame == -1 && ModelType::isExon(cn) && isPrc ) {
	psum[x] = spair_t(false,0);
	continue;
      }
      // if non-coding you don't need to compute for different frame
      if( frame == -1 && x > 0 && psum[x].first ) {
	assert(pscore.first);
	psum[x].second += pscore.second;
      } else if( psum[x].first || (frame == -1 && x==0)) {
	//cout<<"lframe: "<<x<<endl;
	if( lDebug && i == 91 ) {
	  XDEBUGON=true;
	} else {
	  XDEBUGON=false;
	}
	const spair_t score =compProb1(i,frame,svec,column,cn,str,predType,false,start,verbose);
	pscore = score;
	if( score.first && psum[x].first ) {
	  psum[x].second += score.second;
	} else {
	  psum[x] = spair_t(false,0);
	}
      }
      if( isPrc && ModelType::isExon(cn) ) {
	++frame;
	frame = frame > 2 ? 0 : frame;
      } else if( ModelType::isSplice(cn) ) {
	++frame;
      }
    }
#if 0
    if(XDEBUGON) {
      cout<<"here: "<<psum[0]<<" "<<psum[1]<<" "<<psum[2]<<" "<<ftrack[0]<<" "<<ftrack[1]<<" "<<ftrack[2]<<endl;
    }
#endif
    lastPrc = isPrc;
    if( !psum[0].first && !psum[1].first && !psum[2].first ) {
      break;
    }
  }
#if 0
  if(XDEBUGON) {
    exit(0);
  }
#endif
  return psum;
}

vector<spair_t>
ModelManager::scoreInterval5b(int lpos, int rpos, const vector<string*>& svec, int start, int stop, cname_t icn, bool verbose) {
  if(svec.size()==1) {
    assert(0);
  }
  vector<int> ftrack(3);
  getPhases(ftrack,lpos,start);
  vector<spair_t> psum(3,spair_t(true,0));
  vector<Trellis*> cTrellis;
  vector<htype_t> save_ht;
  vector<htype_t> last_ht;
  bool isInit=true;
  const int strsize = (signed)svec[0]->size();
  for(signed i = lpos; i <= rpos; ++i) {
    if( i < 4 ) continue; 
    if( i >= strsize ) break;
    if( !psum[0].first && !psum[1].first && !psum[2].first ) break;
    const bool isPrc = isPrcCheck(i,start,stop);
    //bool isPrc = (i >= start && i<= stop);
    cname_t cn = icn;
    if( ModelType::isExon(icn) && isPrc ) {
      cn = ModelType::isAlt(icn) ? ModelType::eAexonPrc : ModelType::eCexonPrc;
    } else if( ModelType::isExon(icn) && !isPrc) {
      cn = ModelType::isAlt(icn) ? ModelType::eAexonNpc : ModelType::eCexonNpc;
    }
    scorePosHelp2(false,i,svec,cn,last_ht,isInit,ftrack,psum,cTrellis);
    save_ht = last_ht;
    isInit=false;
  }
  //kinda like to know if null for now
  assert(!cTrellis.empty());
  if(!cTrellis.empty()) {
    for(unsigned x = 0; x < 3; ++x) {
      const int cphase = ftrack[x]==-1?0:ftrack[x];
      unsigned idx = cTrellis.size()==1 ? 0 : x;
      htype_t sht = save_ht[idx];
      if( sht != eEmptyState && psum[x].first ) {
	const int pphase = ConsHmmInstance::getPrevTime(sht,cphase);
	const spair_t sp = cTrellis[idx]->getBestScore(sht,pphase);
	if( sp.first ) {
	  psum[x].first = true;
	  psum[x].second += sp.second;
	}
      }
    }
    for(unsigned x = 0; x < cTrellis.size(); ++x) {
      if(cTrellis[x])
	delete cTrellis[x];
    }
  }
  return psum;
}

int
ModelManager::scorePosHelp2(bool verbose, unsigned i, const vector<string*>& svec, ModelType::cname_t cn, vector<htype_t>& last_ht,
			   bool& saveInit, vector<int>& ftrack, vector<spair_t>& psum, vector<Trellis*>& cTrellis) {
  if( i < 4 ) return -1;
  saveInit = false;
  const unsigned tsize = ModelType::isPrcExon(cn) ? 3 : 1;
  if( cTrellis.empty() ) {
    cTrellis.resize(tsize,NULL);
  }
  if( last_ht.empty() ) {
    last_ht.resize(tsize,eEmptyState);
  }
  vector<htype_t> save_ht(tsize,eEmptyState);
  const unsigned num_org = svec.size();
  string column(num_org,'-');
  fillColumn(column,svec,i);
  if( verbose ) {
    cout<<"Pos: "<<i<<" "<<column<<endl;
  }
  const unsigned num_frames = tsize;
  vector<hpair_t> hpv(3);
  vector<bool> isInit(num_frames,false);
  for(unsigned x = 0; x < 3; ++x) {
    const unsigned tidx = cTrellis.size()==1 ? 0 : x;
    const unsigned nidx = tsize==1 ? 0 : x;
    const int cphase = ftrack[x]==-1?0:ftrack[x];
    if( x < 1 || ModelType::isPrcExon(cn) ) {
      hpv[nidx] = ConsHmmInstance::extract_htype(svec,i,cn,cphase);
    }
    htype_t ht = hpv[nidx].first;
    if( ht==eEmptyState && cTrellis[tidx] ) {
      //assert(last_ht[tidx]==ePrc1 || last_ht[tidx]==ePrc2);
      assert(ModelType::isPrcExon(cn) );
      psum[x].first = false;
    }
    if( ht != eEmptyState && ht != last_ht[tidx] && last_ht[tidx] != eEmptyState ) {
      if( psum[x].first ) {
	const int pphase = Trellis::getPrevTime(last_ht[tidx],cphase);
	const spair_t sp =  cTrellis[tidx]->getBestScore(last_ht[tidx],pphase);
	if( sp.first ) {
	  psum[x].second += sp.second;
	}
      }
      isInit[nidx]=true;
    } else if(last_ht[tidx] == eEmptyState){
      isInit[nidx]=true;
    }
    save_ht[nidx] = ht;
  }
  if( tsize != cTrellis.size() ) {
    for(unsigned x = 0; x < cTrellis.size(); ++x) {
      delete cTrellis[x];
      cTrellis[x] = NULL;
    }
    cTrellis.resize(tsize,NULL);
  }
  for(unsigned x = 0; x < tsize; ++x) {
    if( isInit[x] && hpv[x].first == eEmptyState ) {
      if(cTrellis[x])
	delete cTrellis[x];
      cTrellis[x] = NULL;
    } else if(isInit[x] ) {
      if(cTrellis[x])
	delete cTrellis[x];
      RateHmm* rhmm = _cm->getRateHmm(cn);
      cTrellis[x] = new Trellis(rhmm);
    }
    if( hpv[x].first == eEmptyState ) {
      // should arise at the stage only from checking for stop codons
      assert(ModelType::isPrcExon(cn));
      psum[x].first = false;
    }
  }
  for(unsigned x = 0; x < 3; ++x) {
    const hpair_t hp = hpv[x];
    const htype_t ht = hpv[x].first;
    //const unsigned lidx = last_ht.size()==1 ? 0 : x;
    const unsigned idx = cTrellis.size()==1 ? 0 : x;
    //if( !psum[x].first ) continue;
    if(x==0 && !ModelType::isPrcExon(cn) && !ModelType::isSplice(cn) && isInit[x] ) {
      ftrack[0]=0;
      ftrack[1]=ftrack[2] = -1;
    } 
    // this means we've switch into prc
    // and thie 1st time, and we know only frame=0 is valid
    else if(x==0 &&  !ModelType::isPrcExon(cn) && ModelType::isSplice(cn) && isInit[x] ) {
      ftrack[0]=0;
      ftrack[1]=ftrack[2]=-1;
    } 
    int& frame = ftrack[x];
    if( frame == -1 && ModelType::isPrcExon(cn) ) {
      // simple time saver, if in inial exon, only valid frame is 0
      psum[x] = spair_t(false,0);
      continue;
    }
    if(psum[x].first || ModelType::isSplice(cn) || ModelType::isIntron(cn) || ModelType::isNpcExon(cn) ) {
      // only need to compute multiple phases for PrcExon
      int pass = 0; //valid
      if( (x < 1 || ModelType::isPrcExon(cn)) && ht != eEmptyState  ) {
	assert(x < cTrellis.size());
	Trellis* trel = cTrellis[idx];
	const bool isi = isInit[x];
	assert(trel);
	pass = compProb3(i,frame,svec,column,cn,hp,last_ht[idx],*trel,isi,verbose);
	if( pass == -1 ) {
	  psum[x].first = false;
	}
      }
      if( num_frames==1 && (hp.second==-1 || pass==-1) ) {
	for(unsigned all=0; all<3; ++all) {
	  psum[all] = spair_t(false,0);
	}
      }
      if(verbose) {
	if(frame==-1)frame=0;
	if(hp.second != -1 && cTrellis[idx] ) {
	  const int idx = x >= cTrellis.size() ? 0 : x;
	  spair_t sp = cTrellis[idx]->getBestScore(ht,frame);
	  cout<<"["<<ModelType::cname2str(cn)<<" "<<" f:"<<x<<" p:"<<frame<<" s:"<<sp<<" ps: "<<psum[x]<<" "<<psum[x].second+sp.second<<"]";
	}
      }
    }
    if( ModelType::isPrcExon(cn) ) {
      frame++;
      frame = frame > 2 ? 0 : frame;
    } else if(ModelType::isSplice(cn) ) {
      frame++; 
      // allow back to back
      const int thresh = ModelType::isDon(cn) ? SeqLabel::getSigWidth(SeqLabel::eDon) : SeqLabel::getSigWidth(SeqLabel::eAcc);
      frame = frame >= thresh ? 0 : frame;
    }
  }
  if(verbose)
    cout<<endl;
  if( last_ht.size() != tsize) {
    last_ht.resize(tsize,eEmptyState);
  }
  last_ht=save_ht;
  for(unsigned x = 0; x < 3; ++x) {
    if( isInit[x] ) {
      saveInit = true;
      break;
    }
  }
  return 0;
}

int
ModelManager::compProb3(unsigned pos, int phase, 
			const vector<string*>& svec, const string& column, cname_t cn, hpair_t hp, htype_t last_ht,
			Trellis& chi, bool isInit, bool verbose) {
  bool isConsCol = true;
  if( hp.second != 0 ) 
    isConsCol=false;
  htype_t ht = hp.first;
  const bool val = true; 
  int clevel = 0;
  int  tval = 0; 
  bool foundOneValid=false;
  const int times = Trellis::getTimes(ht);
  const int numMutRates = Trellis::getLevels(ht);
  RateTrans* bTrans = NULL;
  RateHmm* rhmm = _cm->getRateHmm(cn);
  //cTrellis[x]->setHmm(rhmm);
  chi.add(isInit,last_ht,phase,rhmm);
  for(signed currRateIdx = 0; currRateIdx < numMutRates; ++currRateIdx) {
    int phaseIdx = phase >= times ? (times-1) : phase;
    unsigned idx = (currRateIdx*times)+phaseIdx;
    RateState& cstate = chi.getState(idx);
    const double rate = cstate.getRate();
    spair_t bscore(false,0), hscore(false,0);
    ModelType::mname_t con_mn = ModelType::cname2mname(cn,tval);
    MultiModel* tmcm = _mm[con_mn];
    ModelType::mname_t mn = ModelType::cname2mname(cn,clevel);
    PhyloModel* pm = _pm[mn];
    double cprob = pm->getProb(svec,pos,phase,column,tmcm,rate);
    ModelType::mname_t con_fn = ModelType::getFalseName(mn,val);
    MultiModel* fmcm = _mm[con_fn];
    ModelType::mname_t fn = ModelType::getFalseName(mn,val);
    PhyloModel* fm = _pm[fn];
    double fprob = fm->getProb(svec,pos,phase,column,fmcm,1.0);
    if( isInit && fprob != 0 && cprob != 0 ) {
      spair_t upto = chi.getScoreCurr(cstate);
      const double initProbPred = cstate.getInitProb();
      const double initProbFals = 1.0 / (double)numMutRates;
      assert(initProbPred>0 && initProbPred<=1);
      assert(initProbFals>0 && initProbFals<=1);
      double cscore = log2(cprob*initProbPred)-log2(fprob*initProbFals);
      if( upto.first ) {
	cscore += upto.second;
      }
      hscore.first = bscore.first = true;
      hscore.second = bscore.second = cscore;

      if(verbose)
	cout<<"cs1: "<<cscore<<endl;
    } else if( !isInit && fprob != 0 && cprob != 0 ) {
      list<RateTrans*>& inlst = cstate.getInLst();
      const double ftrans = 1.0/(double)inlst.size();
      list<RateTrans*>::iterator iter = inlst.begin();
      const list<RateTrans*>::iterator stop = inlst.end();
      double hs = 0;
      for(; iter != stop; ++iter) {
	RateTrans& trans = **iter;
	RateState& pstate = trans.getFr();
	const spair_t pscore = chi.getScorePrev(pstate);
	if( pscore.first ) {
	  const double transProbPred = trans.getProb();
	  const double transProbFals = ftrans;
	  assert(transProbPred>0 && transProbPred<=1);
	  assert(transProbFals>0 && transProbFals<=1);
	  const double tscore = log2(cprob*transProbPred)-log2(fprob*transProbFals);
	  const double cscore = pscore.second + tscore;
	  if( cscore > bscore.second || !bscore.first ) {
	    hscore.first = bscore.first = true;
	    bscore.second = cscore;
	    hscore.second = hs = tscore;
	    bTrans = &trans;
	  }
	} else {
	  //if fprob==0 || cprob==0 , assume adjusting rates wont change this
	  // so we can just qut
	  //apparently some very small #'s get pushed to 0
	  //assert(prevRateIdx==0);//confirm assumption(failed)
	  //break;
	}
      }
      if(verbose)
	cout<<"cs: "<<hs<<endl;
    }
    if( bscore.first ) 
      foundOneValid=true;
    chi.set(cstate,bscore,bTrans,hscore);
    if( DEBUGON && pos >= 145 && pos <= 217 ) {
      cout<<"what up: "<<pos<<" "<<ModelType::cname2str(cn)<<" "<<bscore<<" ";
      cstate.write(cout);
    }
  }
  return !foundOneValid ? -1 : clevel;
}

void
ModelManager::compare3(const vector<string*>& svec, int start, int stop, const vector<list<SeqLabel>*>& cvec) {
  vector<vector<int> > ftrack(cvec.size());
  for(unsigned d=0;d<ftrack.size(); ++d) {
    ftrack[d].resize(3,d);
  }
  vector<vector<spair_t> > psum(cvec.size());
  for(unsigned d=0;d<psum.size(); ++d) {
    psum[d].resize(3,spair_t(true,0));
  }
  int left = cvec[0]->front().getLeft()+1;
  int rght = cvec[0]->back().getRght()-1;
  vector<vector<Trellis*> > cTrellis(cvec.size());
  vector<vector<htype_t> > save_ht(cvec.size()), last_ht(cvec.size());
  vector<bool> isInit(cvec.size(),true), saveInit(cvec.size(),true);
  for(signed i = left; i <= rght; ++i) {
    if( i < 4 ) continue; 
    string column(svec.size(),'-');
    fillColumn(column,svec,i);
    //cout<<"pos: "<<i<<" ";
    //cout<<column<<endl;
    const bool isPrc = isPrcCheck(i,start,stop);
    //bool isPrc = (i >= start && i<= stop);
    for(unsigned k = 0; k < cvec.size(); ++k) {
      if( !psum[k][0].first && !psum[k][1].first && !psum[k][2].first ) continue;
      const list<SeqLabel>& lst = *cvec[k];
      const tp_t tp = ModelManager::getCname(lst,i,isPrc);
      const cname_t cn = tp.first;
      cout<<"PARSE: "<<k<<endl;
      const bool verbose=true;
      if( !last_ht[k].empty() ) {
	if( (eDon==last_ht[k][0]||eAcc==last_ht[k][0]) && ModelType::isPrcExon(cn) ) {
	  const int start = last_ht[k][0]==eDon ? i-SeqLabel::getSigWidth(SeqLabel::eDon) : i-SeqLabel::getSigWidth(SeqLabel::eAcc);
	  SeqLabel::blabel_t bl = last_ht[k][0]==eDon ? SeqLabel::eDon : SeqLabel::eAcc;
	  const SeqLabel tmp(bl,SeqLabel::eUnk,start,i-1);
	  getPhases(ftrack[k],tmp,start);
	}
      }
      bool here = false;
      scorePosHelp2(verbose,i,svec,cn,last_ht[k],here,ftrack[k],psum[k],cTrellis[k]);
      saveInit[k] = here;
      isInit[k]=false;
      save_ht[k]=last_ht[k];
    }
  }
  for(unsigned k = 0; k < cvec.size(); ++k) {
    vector<htype_t> last_ht = save_ht[k];
    for(unsigned x = 0; x < psum[k].size(); ++x) {
      unsigned idx = cTrellis[k].size()==1 ? 0 : x;
      const int cphase = ftrack[k][x]==-1?0:ftrack[k][x];
      if(last_ht[idx] != eEmptyState && cTrellis[k][idx]) {
	const int pphase = ConsHmmInstance::getPrevTime(last_ht[idx],cphase);
	spair_t sp = cTrellis[k][idx]->getBestScore(last_ht[idx],pphase);
	if(sp.first && psum[k][x].first) {
	  psum[k][x].second += sp.second;
	}
      } else {
	assert(!psum[k][x].first);
      }
      cout<<"frame: "<<x<<" k: "<<k<<" sum: "<<psum[k][x]<<endl;
    }
    for(unsigned x = 0; x < cTrellis[k].size(); ++x) {
      delete cTrellis[k][x];
    }
  }
  //cout<<"what's it look like: "<<endl;
  //_cm->normalize1();
  
}
#if 0
void
ModelManager::discTrain(const vector<string*>& svec, int start, int stop, const vector<list<SeqLabel>*>& cvec, const vector<int>& useFrame) {
  vector<vector<int> > ftrack(cvec.size());
  for(unsigned d=0;d<ftrack.size(); ++d) {
    ftrack[d].resize(3);
    for(unsigned i = 0; i < 3; ++i) {
      ftrack[d][i] = i;
    }
  }
  vector<vector<spair_t> > psum(cvec.size());
  for(unsigned d=0;d<psum.size(); ++d) {
    psum[d].resize(3,spair_t(true,0));
  }
  int left = cvec[0]->front().getLeft()+1;
  int rght = cvec[0]->back().getRght()-1;
  vector<vector<Trellis*> > cTrellis(cvec.size());
  vector<vector<htype_t> > save_ht(cvec.size()), last_ht(cvec.size());
  vector<bool> saveInit(cvec.size(),true);
  vector<cname_t> ptype(cvec.size(),ModelType::eCnameEnd);
  const bool verbose=false;
  const int tsize =3;
  for(unsigned i = 0; i < cTrellis.size(); ++i) {
    cTrellis[i].resize(tsize,NULL);
    last_ht[i].resize(tsize,eEmptyState);
  }
  cout<<"The input frames are: "<<useFrame[0]<<" "<<useFrame[1]<<endl;
  for(signed i = left; i <= rght; ++i) {
    if( i < 4 ) continue; 
    string column(svec.size(),'-');
    fillColumn(column,svec,i);
    const bool isPrc = isPrcCheck(i,start,stop);
    //bool isPrc = (i >= start && i<= stop);
    for(unsigned k = 0; k < cvec.size(); ++k) {
      if( !psum[k][0].first && !psum[k][1].first && !psum[k][2].first ) continue;
      const list<SeqLabel>& lst = *cvec[k];
      const tp_t tp = ModelManager::getCname(lst,i,isPrc);
      const cname_t cn = tp.first;
      //cout<<"PARSE: "<<k<<endl;
      if( !last_ht[k].empty() ) {
	if( (eDon==last_ht[k][0]||eAcc==last_ht[k][0]) && ModelType::isPrcExon(cn) ) {
	  getPhases(ftrack[k],i,start);
	}
      }
      bool save=false;
      ptype[k] = cn;
      scorePosHelp2(verbose,i,svec,cn,last_ht[k],save,ftrack[k],psum[k],cTrellis[k]);
      saveInit[k] = save;
      save_ht[k]=last_ht[k];
    }
    // compare and update ....
    assert(cvec.size()==2);
    HmmComponent* ansComp = NULL;
    HmmComponent* prdComp = NULL;
    spair_t ansScore(false,0);
    spair_t prdScore(false,0);
    for(unsigned k = 0; k < 2; ++k) {
      vector<htype_t> last_ht = save_ht[k];
      const int x = useFrame[k];
      unsigned idx = cTrellis[k].size()==1 ? 0 : x;
      const int cphase = ftrack[k][x]==-1?0:ftrack[k][x];
      HmmComponent* comp = NULL;
      spair_t sp(false,0);
      if(last_ht[idx] != eEmptyState && cTrellis[k][idx]) {
	const int pphase = ConsHmmInstance::getPrevTime(last_ht[idx],cphase);
	comp = cTrellis[k][idx]->getBestScore1(last_ht[idx],pphase,sp,saveInit[k]);
      }
      if( k == 0 && sp.first ) {
	ansComp = comp;
	ansScore = sp;
      } else if( k == 1 && sp.first ) {
	prdComp = comp;
	prdScore = sp;
      }
    }
    //if( prdScore.second > ansScore.second && prdComp && ansComp && prdScore.first && ansScore.first ) {
    if( ptype[0] != ptype[1] && prdComp && ansComp && prdScore.first && ansScore.first ) {
      prdComp->update(-1.0);
      ansComp->update(1.0);
      //cout<<"what up: "<<ansScore<<" "<<prdScore<<endl; //" diff: "<<val<<endl;
      prdComp->write(cout);
      ansComp->write(cout);
    }
  }
  for(unsigned k = 0; k < cvec.size(); ++k) {
    vector<htype_t> last_ht = save_ht[k];
    const int x = useFrame[k];
    unsigned idx = cTrellis[k].size()==1 ? 0 : x;
    const int cphase = ftrack[k][x]==-1?0:ftrack[k][x];
    if(last_ht[idx] != eEmptyState && cTrellis[k][idx]) {
      const int pphase = ConsHmmInstance::getPrevTime(last_ht[idx],cphase);
      spair_t sp = cTrellis[k][idx]->getBestScore(last_ht[idx],pphase);
      if(sp.first && psum[k][x].first) {
	psum[k][x].second += sp.second;
      }
    } else {
      assert(!psum[k][x].first);
    }
    cout<<"frame: "<<x<<" k: "<<k<<" sum: "<<psum[k][x]<<endl;
    for(unsigned x = 0; x < cTrellis[k].size(); ++x) {
      delete cTrellis[k][x];
    }
  }
}
#endif
int
ModelManager::scorePosHelp3(bool verbose, unsigned i, const vector<string*>& svec, ModelType::cname_t cn, vector<htype_t>& last_ht,
			    bool& saveInit, vector<int>& ftrack, vector<spair_t>& psum, vector<Trellis*>& cTrellis) {
  if( i < 4 ) return -1;
  saveInit = false;
  const unsigned tsize = cTrellis.size(); //ModelType::isPrcExon(cn) ? 3 : 1;
  vector<htype_t> save_ht(tsize,eEmptyState);
  const unsigned num_org = svec.size();
  string column(num_org,'-');
  fillColumn(column,svec,i);
  if( verbose ) {
    cout<<"Pos: "<<i<<" "<<column<<endl;
  }
  const unsigned num_frames = tsize;
  vector<hpair_t> hpv(3);
  vector<bool> isInit(num_frames,false);
  for(unsigned x = 0; x < 3; ++x) {
    const int cphase = ftrack[x]==-1?0:ftrack[x];
    hpv[x] = ConsHmmInstance::extract_htype(svec,i,cn,cphase);
    const htype_t ht = hpv[x].first;
    if( ht==eEmptyState && cTrellis[x] ) {
      //psum[x].first = false;
      delete cTrellis[x];
      cTrellis[x]=NULL;
    }
    if( ht != eEmptyState && ht != last_ht[x] && last_ht[x] != eEmptyState ) {
      //if( psum[x].first ) {
	//const int pphase = Trellis::getPrevTime(last_ht[x],cphase);
	//const spair_t sp =  cTrellis[x]->getBestScore(last_ht[x],pphase);
	//if( sp.first ) {
	  //psum[x].second += sp.second;
	//}
      //}
      isInit[x]=true;
    } else if(last_ht[x] == eEmptyState){
      isInit[x]=true;
    }
    save_ht[x] = ht;
  }
  for(unsigned x = 0; x < tsize; ++x) {
    if( isInit[x] && hpv[x].first == eEmptyState ) {
      if(cTrellis[x])
	delete cTrellis[x];
      cTrellis[x] = NULL;
    } else if(isInit[x] ) {
      //RateHmm* rhmm = _cm->getRateHmm(cn);
      //assert(cTrellis[x]);
      //cTrellis[x]->setHmm(rhmm);
    }
  }
  for(unsigned x = 0; x < 3; ++x) {
    if(cTrellis[x]) {
      if(isInit[x] && ModelType::isSplice(cn) ) {
	ftrack[x]=0;
      } 
      const hpair_t hp = hpv[x];
      const htype_t ht = hpv[x].first;
      int& frame = ftrack[x];
      // only need to compute multiple phases for PrcExon
      int pass = 0; //valid
      assert(x < cTrellis.size());
      Trellis* trel = cTrellis[x];
      const bool isi = isInit[x];
      assert(trel);
      if( DEBUGON && i == 218 && x == 0) {
	DEBUGON=true;
      } else {
	DEBUGON=false;
      }
      pass = compProb3(i,frame,svec,column,cn,hp,last_ht[x],*trel,isi,verbose);
      if( pass==-1 ) {
	delete cTrellis[x];
	cTrellis[x]=NULL;
      }
      if(verbose) {
	if(frame==-1)frame=0;
	if(hp.second != -1 && cTrellis[x] ) {
	  spair_t sp = cTrellis[x]->getBestScore(ht,frame);
	  cout<<"["<<ModelType::cname2str(cn)<<" "<<" f:"<<x<<" p:"<<frame<<" s:"<<sp<<" ps: "<<psum[x]<<" "<<psum[x].second+sp.second<<"]";
	}
      }
      if( ModelType::isPrcExon(cn) ) {
	frame++;
	frame = frame > 2 ? 0 : frame;
      } else if(ModelType::isSplice(cn) ) {
	frame++; 
	// allow back to back
	const int thresh = ModelType::isDon(cn) ? SeqLabel::getSigWidth(SeqLabel::eDon) : SeqLabel::getSigWidth(SeqLabel::eAcc);
	frame = frame >= thresh ? 0 : frame;
      }
    }
  }
  if(verbose)
    cout<<endl;
  last_ht=save_ht;
  for(unsigned x = 0; x < 3; ++x) {
    if( isInit[x] ) {
      saveInit = true;
      break;
    }
  }
  return 0;
}

int
ModelManager::discTrain(const vector<string*>& svec, int start, int stop, const vector<list<SeqLabel>*>& cvec, const vector<int>& useFrame,
			ParamSearchHashT& phash) {
  list<SeqLabel>::iterator iter0 = cvec[0]->begin();
  list<SeqLabel>::iterator iter1 = cvec[1]->begin();
  list<SeqLabel>::iterator end = cvec[1]->end();
  bool isSame = false;
  if( cvec[0]->size() == cvec[1]->size() ) {
    bool pass=true;
    for(; iter1 != end; ++iter0, ++iter1) {
      if( !(*iter1 == *iter0) ) {
	pass = false;
	break;
      }
    }
    isSame=pass;
  }
  if( isSame ) {
    return 0;
  }

  vector<vector<int> > ftrack(cvec.size());
  for(unsigned d=0;d<ftrack.size(); ++d) {
    ftrack[d].resize(3);
    for(unsigned i = 0; i < 3; ++i) {
      ftrack[d][i] = i;
    }
  }
  vector<vector<spair_t> > psum(cvec.size());
  for(unsigned d=0;d<psum.size(); ++d) {
    psum[d].resize(3,spair_t(true,0));
  }
  int left = cvec[0]->front().getLeft()+1;
  int rght = cvec[0]->back().getRght()-1;
  vector<vector<Trellis*> > cTrellis(cvec.size());
  vector<vector<htype_t> > save_ht(cvec.size()), last_ht(cvec.size());
  vector<bool> saveInit(cvec.size(),true);
  vector<cname_t> ptype(cvec.size(),ModelType::eCnameEnd);
  const bool verbose=false;
  const int tsize =3;
  for(unsigned i = 0; i < cTrellis.size(); ++i) {
    cTrellis[i].resize(tsize,NULL);
    last_ht[i].resize(tsize,eEmptyState);
    for(unsigned j = 0; j < cTrellis[i].size(); ++j) {
      cTrellis[i][j] = new Trellis;
    }
  }
  cout<<"The input frames are: "<<useFrame[0]<<" "<<useFrame[1]<<endl;
  for(signed i = left; i <= rght; ++i) {
    if( i < 4 ) continue; 
    string column(svec.size(),'-');
    fillColumn(column,svec,i);
    const bool isPrc = isPrcCheck(i,start,stop);
    //bool isPrc = (i >= start && i<= stop);
    for(unsigned k = 0; k < cvec.size(); ++k) {
      if( !cTrellis[k][0] && !cTrellis[k][1] && !cTrellis[k][2] ) {
	continue;
      }
      if( i == 218 && k == 0 ) {
	DEBUGON=true;
      } else {
	DEBUGON=false;
      }
      const list<SeqLabel>& lst = *cvec[k];
      const tp_t tp = ModelManager::getCname(lst,i,isPrc);
      const cname_t cn = tp.first;
      //cout<<"PARSE: "<<k<<endl;
      if( !last_ht[k].empty() ) {
	if( (eDon==last_ht[k][0]||eAcc==last_ht[k][0]) && ModelType::isPrcExon(cn) ) {
	  getPhases(ftrack[k],i,start);
	}
      }
      bool save=false;
      ptype[k] = cn;
      scorePosHelp3(verbose,i,svec,cn,last_ht[k],save,ftrack[k],psum[k],cTrellis[k]);
      saveInit[k] = save;
      save_ht[k]=last_ht[k];
    }
  }
  vector< vector<ParamSearchPos>*> paths(cvec.size());
  for(unsigned k = 0; k < cvec.size(); ++k) {
    vector<htype_t> last_ht = save_ht[k];
    const int x = useFrame[k];
    const int cphase = ftrack[k][x]==-1?0:ftrack[k][x];
    if(last_ht[x] != eEmptyState && cTrellis[k][x]) {
      const int pphase = ConsHmmInstance::getPrevTime(last_ht[x],cphase);
      const htype_t ht = last_ht[x];
      paths[k] = cTrellis[k][x]->getPath(ht,pphase);
    }
  }
  for(unsigned k = 0; k < cvec.size(); ++k) {
    for(unsigned x = 0; x < cTrellis[k].size(); ++x) {
      if(cTrellis[k][x])
	delete cTrellis[k][x];
    }
  }
  assert(cvec.size()==2);
  assert( paths[0]->size()==paths[1]->size() );
  int num_diffs = 0;
  for(unsigned pos = 0; pos < paths[0]->size(); ++pos) {
    ParamSearchPos ansComp = (*paths[0])[pos];
    ParamSearchPos prdComp = (*paths[1])[pos];
    if( ansComp.getComp() != prdComp.getComp() ) {
      phash[prdComp.getComp()].push_back(ParamSearchPair(ansComp,prdComp,svec));
      ++num_diffs;
    }
#if 0
    //cout<<"compare paths: ";
    //ansComp->write(cout);
    //prdComp->write(cout);
    if( ansComp.first != prdComp.first && prdComp.first && ansComp.first ) {
      ++num_diffs;
      double negWght = ansComp.second.second-prdComp.second.second;
      prdComp.first->update(negWght);
      double posWght = prdComp.second.second-ansComp.second.second;
      ansComp.first->update(posWght);
      cout<<"There's a diff"<<" "<<posWght<<endl;
      //cout<<"what up: "<<ansScore<<" "<<prdScore<<endl; //" diff: "<<val<<endl;
      ansComp.first->write(cout);
      prdComp.first->write(cout);
    }
#endif
  }
  delete paths[0];
  delete paths[1];
  return num_diffs;
}

#if 1
void ModelManager::updateParams(ParamSearchHashT& psh) {
  const int  tval = 0; 
  ParamSearchHashT::iterator iter = psh.begin();
  ParamSearchHashT::iterator stop = psh.end();
  const double cMargin = 0.0001;
  //const double cPosReinf = 1.2;
  ParamSearchHashT deltaHash;
  for(; iter != stop; ++iter) {
    //double tSum = 0, pSum = 0;
    list<ParamSearchPair>& pspLst = (*iter).second;
    HmmComponent* neg_param = (*iter).first;
    list<ParamSearchPair>::iterator piter =  pspLst.begin();
    const list<ParamSearchPair>::iterator pstop =  pspLst.end();
    for(; piter != pstop; ++piter) {
      ParamSearchPair& psp = *piter;
      ParamSearchPos& ans = psp.getAns();
      ParamSearchPos& prd = psp.getPrd();
      int pos = ans.getPos();
      assert(ans.getPos()==prd.getPos());

      const vector<string*>& svec = psp.getSeq();
      const unsigned num_org = svec.size();
      string column(num_org,'-');
      fillColumn(column,svec,pos);

      const ModelType::cname_t pcn = prd.getHmm()->getType();
      const double prate = prd.getComp()->getRate();
      const int pphase = prd.getPhase();
      const ModelType::mname_t pmn = ModelType::cname2mname(pcn,tval);
      MultiModel* pmcm = _mm[pmn];
      PhyloModel* ppm = _pm[pmn];
      double pprob = ppm->getProb(svec,pos,pphase,column,pmcm,prate);

      const ModelType::cname_t tcn = ans.getHmm()->getType();
      const double trate = ans.getComp()->getRate();
      const int tphase = ans.getPhase();
      const ModelType::mname_t tmn = ModelType::cname2mname(tcn,tval);
      MultiModel* tmcm = _mm[tmn];
      PhyloModel* tpm = _pm[tmn];
      double tprob = tpm->getProb(svec,pos,tphase,column,tmcm,trate);
      HmmComponent* pos_param = ans.getComp();
      double ttransProb = pos_param->getProb();
      double ptransProb = neg_param->getProb();
      
      const double ftprob = tprob*ttransProb;
      const double fpprob = pprob*ptransProb;
      if( ftprob < fpprob ) {
	const double denom = ftprob+fpprob;
	const double numer = cMargin - (ftprob-fpprob);
	const double delta = numer/denom<1 ? numer/denom : 0.99;
	//assert(delta >= -1.0 && delta <= 1);
	
	psp.setDelta(delta);
	if( delta == 1 || delta == 0 ) {
	  cout<<"Warhing this don't look good: delta= "<<delta<<endl;
	}
	list<ParamSearchPair>::const_iterator match = find(deltaHash[pos_param].begin(),deltaHash[pos_param].end(),psp);
	if( match == deltaHash[pos_param].end() ) {
	  deltaHash[pos_param].push_back(psp);
	  deltaHash[neg_param].push_back(psp);
	}
	double posVal = ttransProb+(delta*ttransProb);
	double negVal = ptransProb-(delta*ptransProb);
	//cout<<"delta: "<<delta<<" pv: "<<posVal<<" nv: "<<negVal<<" tp: "<<ftprob<<" fp: "<<fpprob<<endl;
	pos_param->update(posVal);
	neg_param->update(negVal);
      }
    }
  }
#if 0
  iter = deltaHash.begin();
  stop = deltaHash.end();
  for(; iter != stop; ++iter) {
    HmmComponent* comp=(*iter).first;
    list<ParamSearchPair>& pspLst = (*iter).second;
    typedef pair<HmmComponent*,list<ParamSearchPair>*> lpair_t;
    list<lpair_t> open;
    ParamSearchHashT once;
    open.push_front(lpair_t(comp,&pspLst));
    while(!open.empty()) {
      lpair_t lp = open.front();
      open.pop_front();
      HmmComponent* param = lp.first;
      assert(param);
      list<ParamSearchPair>* pspLstPtr = lp.second;
      assert(pspLstPtr);
      assert(!pspLstPtr->empty());

      ParamSearchPair& psp = pspLstPtr->front();
      const RateHmm* rh = (param==psp.getPrd().getComp() ? psp.getPrd().getHmm() : psp.getAns().getHmm());
      assert(rh);
      const ModelType::cname_t cn = rh->getType();
      cout<<"check: "<<ModelType::cname2str(cn)<<endl;
      param->write(cout);
      once[param].push_back(pspLstPtr->front());
      list<ParamSearchPair>::iterator piter =  pspLstPtr->begin();
      const list<ParamSearchPair>::iterator pstop =  pspLstPtr->end();
      for(; piter != pstop; ++piter) {
	ParamSearchPair& psp = *piter;
	const string param_str = param==psp.getAns().getComp() ? "answ": "pred";
	const string other_str = param==psp.getAns().getComp() ? "pred": "answ";
	HmmComponent* other = param==psp.getAns().getComp() ? psp.getPrd().getComp() : psp.getAns().getComp();
	assert(other);
	const RateHmm* ohmm = param==psp.getAns().getComp() ? psp.getPrd().getHmm() : psp.getAns().getHmm();
	assert(ohmm);
	const ModelType::cname_t ocn = ohmm->getType();
	const double pdelta = psp.getDelta(param);
	const double odelta = psp.getDelta(other);
	cout<<param_str<<"="<<pdelta<<" "<<other_str<<"="<<odelta<<" "<<ModelType::cname2str(ocn)<<" ";
	other->write(cout);
	if( once[other].empty() ) {
	  list<ParamSearchPair>& lp = deltaHash[other];
	  open.push_front( lpair_t(other,&lp) );
	  once[other].push_back(psp);
	}
      }
    }

  }
#endif
}
#endif


ParamSearchPair* 
ModelManager::selectParam(ParamSearchHashT& psh) {
  ParamSearchHashT::iterator iter = psh.begin();
  const ParamSearchHashT::iterator stop = psh.end();
  //const int  tval = 0; 
  //double cMargin = 0.0001;
  list<ParamSearchPair>* best = NULL;
  for(; iter != stop; ++iter) {
    list<ParamSearchPair>& pspLst = (*iter).second;
    unsigned bsize = best ? best->size() : 0;
    unsigned csize = pspLst.size();
    if( (csize < bsize || !best) && !pspLst.empty()  ) {
      bsize = pspLst.size();
      best = &pspLst;
    }
  }
#if 0
  list<ParamSearchPair>::iterator piter =  best->begin();
  const list<ParamSearchPair>::iterator pstop =  best->end();
  double negReinf = 0, posReinf = 0;
  for(; piter != pstop; ++piter) {
    ParamSearchPair& psp = *piter;
    ParamSearchPos& ans = psp.getAns();
    ParamSearchPos& prd = psp.getPrd();
    int pos = ans.getPos();
    assert(ans.getPos()==prd.getPos());

    const vector<string*>& svec = psp.getSeq();
    const unsigned num_org = svec.size();
    string column(num_org,'-');
    fillColumn(column,svec,pos);

    const ModelType::cname_t pcn = prd.getHmm()->getType();
    const double prate = prd.getComp()->getRate();
    const int pphase = prd.getPhase();
    const ModelType::mname_t pmn = ModelType::cname2mname(pcn,tval);
    MultiModel* pmcm = _mm[pmn];
    PhyloModel* ppm = _pm[pmn];
    double pprob = ppm->getProb(svec,pos,pphase,column,pmcm,prate);

    const ModelType::cname_t tcn = ans.getHmm()->getType();
    const double trate = ans.getComp()->getRate();
    const int tphase = ans.getPhase();
    const ModelType::mname_t tmn = ModelType::cname2mname(tcn,tval);
    MultiModel* tmcm = _mm[tmn];
    PhyloModel* tpm = _pm[tmn];
    double tprob = tpm->getProb(svec,pos,tphase,column,tmcm,trate);
    double transProb = ans.getComp()->getProb();
    tprob *= (transProb*0.8);

    const double targVal = tprob-cMargin > 0 ? tprob-cMargin : cMargin;
    const double newParam = targVal / pprob;
    posReinf += (transProb*0.8);
    negReinf += newParam;
  }
  double nsize = best->size();
  const double negAvg = -1.0 * (negReinf / (double)nsize);
  const double posAvg = posReinf / (double)nsize;
  cout<<"Update pos= "<<posAvg<<" neg= "<<negAvg<<endl;
#endif
  if( best ) {
    //HmmComponent* posParam = best->front().getAns().getComp();
    //HmmComponent* negParam = best->front().getPrd().getComp();
    return new ParamSearchPair(best->front());
  } else {
    return NULL;
  }
#if 0
    list<ParamSearchPair>::iterator piter =  pspLst.begin();
    const list<ParamSearchPair>::iterator pstop =  pspLst.end();
    double tSum = 0, pSum = 0;
    for(; piter != pstop; ++piter) {
      ParamSearchPair& psp = *piter;
      ParamSearchPos& ans = psp.getAns();
      ParamSearchPos& prd = psp.getPrd();
      int pos = ans.getPos();
      assert(ans.getPos()==prd.getPos());

      const vector<string*>& svec = psp.getSeq();
      const unsigned num_org = svec.size();
      string column(num_org,'-');
      fillColumn(column,svec,pos);

      const ModelType::cname_t pcn = prd.getHmm()->getType();
      const double prate = prd.getComp()->getRate();
      const int pphase = prd.getPhase();
      const ModelType::mname_t pmn = ModelType::cname2mname(pcn,tval);
      MultiModel* pmcm = _mm[pmn];
      PhyloModel* ppm = _pm[pmn];
      double pprob = ppm->getProb(svec,pos,pphase,column,pmcm,prate);

      const ModelType::cname_t tcn = ans.getHmm()->getType();
      const double trate = ans.getComp()->getRate();
      const int tphase = ans.getPhase();
      const ModelType::mname_t tmn = ModelType::cname2mname(tcn,tval);
      MultiModel* tmcm = _mm[tmn];
      PhyloModel* tpm = _pm[tmn];
      double tprob = tpm->getProb(svec,pos,tphase,column,tmcm,trate);
      tprob *= ans.getComp()->getProb();

      //const double targVal = tprob-cMargin > 0 ? tprob-cMargin : cMargin;
      //const double newParam = targVal / pprob;
      //prd.getComp()->update(newParam);
      tSum += log2(tprob);
      pSum += log2(pprob);
    }
    double nsize = pspLst.size();
    double diff = ((tSum-cMargin)-pSum)/(double)nsize;
    double nParam = pow(2,diff); 
    HmmComponent* param = (*iter).first;
    param->update(nParam);
  }

  iter = psh.begin();
  for(; iter != stop; ++iter) {
    HmmComponent* comp=(*iter).first;
    list<ParamSearchPair>& pspLst = (*iter).second;
    ParamSearchPair& psp = pspLst.front();
    const RateHmm* rh = psp.getPrd().getHmm();
    const ModelType::cname_t cn = rh->getType();
    cout<<"check: "<<ModelType::cname2str(cn);
    comp->write(cout);
  }
#endif
}
