#include "ModelType.h"

using std::string;


static ModelType::conserve_t
clevel2conserve(int clevel) {
  switch(clevel) {
  case 0: return ModelType::eConserved;
  case 3: return ModelType::eNotConserved;
  default: return ModelType::eAmbiguous;
  }
}

ModelType::mname_t
ModelType::toSplice(SeqLabel::event_t rev, SeqLabel::blabel_t st, int clevel) {
  conserve_t consType = clevel2conserve(clevel);
  mname_t mn = eError;
  if( st == SeqLabel::eBeg || st == SeqLabel::eEnd ) {
    mn = eTerm;
  } else if( rev == SeqLabel::eAlt ) {
    if( consType==eConserved ) {
      mn = SeqLabel::isDon(st) ? eAdonCon : eAaccCon;
    } else {
      mn = SeqLabel::isDon(st) ? eDonNonc : eAccNonc;
    }
  } else if( rev == SeqLabel::eCon ) {
    if( consType==eConserved ) {
      mn = SeqLabel::isDon(st) ? eCdonCon : eCaccCon;
    } else {
      mn = SeqLabel::isDon(st) ? eDonNonc : eAccNonc;
    }
  } else {
    assert(0);
  }
  return mn;
}

ModelType::mname_t
ModelType::toUpstreamIntron(SeqLabel::event_t rev, int clevel, bool notStrict) {
  conserve_t consType = clevel2conserve(clevel);
  mname_t mn = eError;
  if( rev == SeqLabel::eAlt ) {
    if( consType == eConserved ) {
      mn = eAintronConUp;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = eAintronNonConUp;
    }
  } else if( rev == SeqLabel::eCon ) {
    if( consType == eConserved ) {
      mn = eCintronConUp;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = eCintronNonConUp;
    }
  } else {
    assert(0);
  }
  return mn;
}

ModelType::mname_t
ModelType::toDnstreamIntron(SeqLabel::event_t rev, int clevel, bool notStrict) {
  conserve_t consType = clevel2conserve(clevel);
  mname_t mn = eError;
  if( rev == SeqLabel::eAlt ) {
    if( consType == eConserved ) {
      mn = eAintronConDn;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = eAintronNonConDn;
    }
  } else if( rev == SeqLabel::eCon ) {
    if( consType == eConserved ) {
      mn = eCintronConDn;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = eCintronNonConDn;
    }
  } else {
    assert(0);
  }
  return mn;
}

ModelType::mname_t
ModelType::toExon(SeqLabel::event_t rev, int clevel, bool isPrc, bool notStrict) {
  conserve_t consType = clevel2conserve(clevel);
  mname_t mn = eError;
  if( rev == SeqLabel::eAlt ) {
    if( consType == eConserved ) {
      mn = isPrc ? eAexonConPrc : eAexonConNpc;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = isPrc ? eAexonNonConPrc : eAexonNonConNpc;
    }
  } else if( rev == SeqLabel::eCon ) {
    if( consType == eConserved ) {
      mn = isPrc ? eCexonConPrc : eCexonConNpc;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = isPrc ? eCexonNonConPrc : eCexonNonConNpc;
    }
  } else {
    assert(0);
  }
  return mn;
}

string
ModelType::mname2str(mname_t mn) {
  switch(mn) {
  case eAccNonc:
    return "eAccNonc";
  case eAaccCon:
    return "eAaccCon";
  case eCaccCon:
    return "eCaccCon";
  case eCdonCon:
    return "eCdonCon";
  case eAdonCon:
    return "eAdonCon";
  case eDonNonc:
    return "eDonNonc";
  case eCintronNonConDn:
    return "eCintronNonConDn";
  case eCintronConDn: 
    return "eCintronConDn"; 
  case eAintronNonConDn:
    return "eAintronNonConDn";
  case eAintronConDn:
    return "eAintronConDn";
  case eCintronNonConUp:
    return "eCintronNonConUp";
  case eCintronConUp:
    return "eCintronConUp";
  case eAintronNonConUp:
    return "eAintronNonConUp";
  case eAintronConUp:
    return "eAintronConUp";
  case eCexonNonConPrc:
    return "eCexonNonConPrc";
  case eCexonConPrc:
    return "eCexonConPrc";
  case eAexonConPrc:
    return "eAexonConPrc";
  case eAexonNonConPrc:
    return "eAexonNonConPrc";
  case eAexonConPrcBindSite:
    return "eAexonConPrcBindSite";
  case eCexonNonConNpc:
    return "eCexonNonConNpc";
  case eCexonConNpc:
    return "eCexonConNpc";
  case eAexonConNpc:
    return "eAexonConNpc";
  case eAexonNonConNpc:
    return "eAexonNonConNpc";
  case eAretainedIntronPrc:
    return "eAretainedIntronPrc";
  case eAretainedIntronNpc:
    return "eAretainedIntronNpc";
  case eTerm:
    return "eTerm";
  case eError:
    return "eError";
  default:
    return "error";
  }
}

#if 0
//static
ModelType::mname_t
ModelType::label2Model(SeqLabel::blabel_t bl) {
  switch(bl) {
  case SeqLabel::eDon:
  case SeqLabel::eNed:
    return eCdon;
  case SeqLabel::eCed:
  case SeqLabel::eIrSd:
  case SeqLabel::eIrAfmd:
  case SeqLabel::eIrFmd:
  case SeqLabel::eAfmd:
  case SeqLabel::eFmd:
    return eAdon;
  case SeqLabel::eAcc:
  case SeqLabel::eNea:
    return eCacc;
  case SeqLabel::eCea:
  case SeqLabel::eIrSa:
  case SeqLabel::eIrAfma:
  case SeqLabel::eIrFma:
  case SeqLabel::eAfma:
  case SeqLabel::eFma:
    return eAacc;
  case SeqLabel::eBeg:
  case SeqLabel::eEnd:
    return eTerm;
  default:
    return eError;
  }
}
#endif


ModelType::mname_t
ModelType::str2mname(const string& str) {
  if( str == "eAaccCon")
    return eAaccCon;
  if( str == "eCaccCon")
    return eCaccCon;
  if( str == "eCdonCon")
    return eCdonCon;
  if( str == "eAdonCon")
    return eAdonCon;
  if( str == "eDonNonc")
    return eDonNonc;
  if( str == "eAccNonc")
    return eAccNonc;
  if( str == "eCintronNonConDn")
    return eCintronNonConDn;
  if( str == "eCintronConDn") 
    return eCintronConDn; 
  if( str == "eAintronNonConDn")
    return eAintronNonConDn;
  if( str == "eAintronConDn")
    return eAintronConDn;
  if( str == "eCintronNonConUp")
    return eCintronNonConUp;
  if( str == "eCintronConUp")
    return eCintronConUp;
  if( str == "eAintronNonConUp")
    return eAintronNonConUp;
  if( str == "eAintronConUp")
    return eAintronConUp;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonConPrc")
    return eCexonConPrc;
  if( str == "eAexonConPrc")
    return eAexonConPrc;
  if( str == "eAexonNonConPrc")
    return eAexonNonConPrc;
  if( str == "eAexonConPrcBindSite")
    return eAexonConPrcBindSite;
  if( str == "eCexonNonConNpc")
    return eCexonNonConNpc;
  if( str == "eCexonConNpc")
    return eCexonConNpc;
  if( str == "eAexonConNpc")
    return eAexonConNpc;
  if( str == "eAexonNonConNpc")
    return eAexonNonConNpc;
  if( str == "eAretainedIntronPrc")
    return eAretainedIntronPrc;
  if( str == "eAretainedIntronNpc")
    return eAretainedIntronNpc;
  if( str == "eTerm")
    return eTerm;
  if( str == "eError")
    return eError;
  else 
    return eError;
}

ConsManager::cname_t
ModelType::mname2cname(mname_t mn, bool isAlt) {
  switch(mn) {
  case eDonNonc:
    return isAlt ? ConsManager::eAdon : ConsManager::eCdon;
  case eAdonCon:
    return ConsManager::eAdon;
  case eCdonCon:
    return ConsManager::eCdon;
  case eAaccCon:
    return ConsManager::eAacc;
  case eCaccCon:
    return ConsManager::eCacc;
  case eAccNonc:
    return isAlt ? ConsManager::eAacc : ConsManager::eCacc;
  case eCintronNonConDn:
  case eCintronConDn:
    return ConsManager::eCintronDn;
  case eAintronNonConDn:
  case eAintronConDn:
    return ConsManager::eAintronDn;
  case eCintronNonConUp:
  case eCintronConUp:
    return ConsManager::eCintronUp;
  case eAintronNonConUp:
  case eAintronConUp:
    return ConsManager::eAintronUp;
  case eCexonNonConPrc:
  case eCexonConPrc:
    return ConsManager::eCexonPrc;
  case eAexonNonConPrc:
  case eAexonConPrc:
    return ConsManager::eAexonPrc;
  case eCexonNonConNpc:
  case eCexonConNpc:
    return ConsManager::eCexonNpc;
  case eAexonNonConNpc:
  case eAexonConNpc:
    return ConsManager::eAexonNpc;
  default:
    assert(0);
    return ConsManager::eCnameEnd;
  }
}

ModelType::mname_t
ModelType::cname2mname(cname_t cn, int clevel) {
  switch(cn) {
  case ConsManager::eAdon:  
    return clevel == 0 ? eAdonCon : eDonNonc;
  case ConsManager::eCdon:  
    return clevel == 0 ? eCdonCon : eDonNonc;
  case ConsManager::eAacc:  
    return clevel == 0 ? eAaccCon : eAccNonc;
  case ConsManager::eCacc:  
    return clevel == 0 ? eCaccCon : eAccNonc;
  case ConsManager::eCintronDn:
    return clevel == 0 ? eCintronConDn : eCintronNonConDn;
  case ConsManager::eAintronDn:
    return clevel == 0 ? eAintronConDn : eAintronNonConDn;
  case ConsManager::eCintronUp:
    return clevel == 0 ? eCintronConUp : eCintronNonConUp;
  case ConsManager::eAintronUp:
    return clevel == 0 ? eAintronConUp : eAintronNonConUp;
  case ConsManager::eCexonPrc:
    return clevel == 0 ? eCexonConPrc : eCexonNonConPrc;
  case ConsManager::eAexonPrc:
    return clevel == 0 ? eAexonConPrc : eAexonNonConPrc;
  case ConsManager::eCexonNpc:
    return clevel == 0 ? eCexonConNpc : eCexonNonConNpc;
  case ConsManager::eAexonNpc:
    return clevel == 0 ? eAexonConNpc : eAexonNonConNpc;
  default:
    assert(0);
    return eError;
  }
}

ConsHmmInstance::htype_t
ModelType::mname2htype(mname_t mn) {
  switch(mn) {
  case eCintronNonConDn:
  case eCintronConDn:
  case eAintronNonConDn:
  case eAintronConDn:
  case eCintronNonConUp:
  case eCintronConUp:
  case eAintronNonConUp:
  case eAintronConUp:
  case eCexonNonConNpc:
  case eCexonConNpc:
  case eAexonNonConNpc:
  case eAexonConNpc:
    return ConsHmmInstance::eNpc;
  case eCexonNonConPrc:
  case eCexonConPrc:
  case eAexonNonConPrc:
  case eAexonConPrc:
    return ConsHmmInstance::ePrc;
  case eCdonCon:
  case eAdonCon:
  case eDonNonc:
    return ConsHmmInstance::eDon;
  case eCaccCon:
  case eAaccCon:
  case eAccNonc:
    return ConsHmmInstance::eAcc;
  default:
    return ConsHmmInstance::eEmptyState;
  }
}

bool 
ModelType::isPrcExon(mname_t mn) {
  switch(mn) {
  case eAretainedIntronPrc:
  case eAexonConPrc:
  case eAexonNonConPrc:
  case eCexonConPrc:
  case eCexonNonConPrc:
  case eAexonConPrcBindSite:
    return true;
  default:
    return false;
  }
}
