#ifndef _MODELTYPE_H
#define _MODELTYPE_H

#include <string>
#include "SeqLabel.h"

class ModelType {
 public:
   enum consv_t {
    eIntronDn = 0, eIntronUp, eExonPrc, eExonNpc, eAcc, eDon, eConsvEnd
  };
  enum cname_t { 
    eCintronDn=0,eAintronDn,eCintronUp,eAintronUp,
    eCexonPrc,eAexonPrc,eCexonNpc,eAexonNpc,
    eAdon,eCdon,eAacc,eCacc,
    eCnameEnd
  };

  enum inv_t { 
    eInvCintron=0,eInvAintron,
    eInvCexon,eInvAexon,eInvCnameEnd
  };
  enum mname_t { eTerm=0, eAaccCon, eCaccCon, eAdonCon, eCdonCon, eAccNonc, eDonNonc,
		 eCintronNonConDn,eCintronConDn,eAintronNonConDn,eAintronConDn,

		 eCintronNonConUp,eCintronConUp,eAintronNonConUp,eAintronConUp,
		 eCexonNonConPrc,eCexonConPrc,eAexonConPrc,eAexonNonConPrc,

		 eDonFalseNonc,eDonFalseCons,eAccFalseNonc,eAccFalseCons,
		 eBackgroundCons,eBackgroundNonc,
		 eCexonNonConNpc,eCexonConNpc,eAexonConNpc,eAexonNonConNpc,
		 eError
  };

  enum conserve_t { eNotConserved, eConserved, eAmbiguous };

  inline static bool isAlt(cname_t);
  inline static bool isNonCons(mname_t);
  inline static mname_t getOppCons(mname_t);
  inline static mname_t getOppSide(mname_t);
  inline static cname_t getOppSplice(cname_t);
  inline static bool isIntron(cname_t);
  inline static mname_t getFalseName(mname_t,bool);

  inline static cname_t inv2cname(inv_t,bool);

  inline static bool isAcc(cname_t);
  inline static bool isDon(cname_t);

  inline static bool isSplice(cname_t);
  inline static bool isExon(cname_t cn) { return isNpcExon(cn) || isPrcExon(cn); }
  inline static bool isNpcExon(cname_t); 
  inline static bool isPrcExon(cname_t); 
  inline static inv_t
    sl2inv(const SeqLabel&, const SeqLabel&);
  inline static cname_t
    sl2cnameInv(const SeqLabel&, const SeqLabel&,bool);
  inline static cname_t
    sl2cnamePos(const SeqLabel&);
  inline static std::string cname2str(cname_t);
  inline static cname_t str2cname(const std::string&); 
  inline static cname_t mname2cname(mname_t,bool);
  inline static mname_t cname2mname(cname_t,int);
  inline static std::string mname2str(mname_t); 
  inline static mname_t str2mname(const std::string&);
  static mname_t label2Model(SeqLabel::blabel_t);
  inline static bool isPrcExon(mname_t);
  inline static mname_t toSplice(SeqLabel::event_t, SeqLabel::blabel_t,int);
  inline static mname_t
    toUpstreamIntron(SeqLabel::event_t, int,bool);
  inline static mname_t
    toDnstreamIntron(SeqLabel::event_t, int,bool);
  inline static mname_t
    toExon(SeqLabel::event_t, int, bool,bool);

};


typedef std::pair<ModelType::cname_t,int> tp_t;
typedef std::pair<tp_t,int> tp2_t;

#include "ModelType.hpp"

#endif // _MODELTYPE_H
