#include "ModelType.h"
#include <iostream>

using std::string;
using std::cout;
using std::endl;

bool
ModelType::isAlt(cname_t cn) {
  switch(cn) {
  case eAintronDn:
  case eAintronUp:
  case eAexonPrc:
  case eAexonNpc:
  case eAdon:
  case eAacc:
    return true;
  default:
    return false;
  }
}

static ModelType::conserve_t
clevel2conserve(int clevel) {
  switch(clevel) {
  case 0: return ModelType::eConserved;
  case 3: return ModelType::eNotConserved;
  default: return ModelType::eAmbiguous;
  }
}

bool
ModelType::isNonCons(mname_t mn) {
  switch(mn) {
  case eCintronNonConDn: 
  case eCintronNonConUp:
  case eCexonNonConPrc:
  case eCexonNonConNpc:
  case eAintronNonConDn:
  case eAintronNonConUp:
  case eAexonNonConPrc:
  case eAexonNonConNpc:
    return true;
  default:
    return false;
  }
}

ModelType::cname_t 
ModelType::getOppSplice(cname_t cn) {
  switch(cn) {
  case eAdon: return eCdon;
  case eAacc: return eCacc;
  case eCdon: return eAdon;
  case eCacc: return eAacc;
  default:
    assert(0); return eCnameEnd;
  }
}

ModelType::mname_t
ModelType::getOppCons(mname_t mn) {
  switch(mn) {
  case eCintronNonConDn: return eAintronNonConDn;
  case eCintronNonConUp: return eAintronNonConUp;
  case eCexonNonConPrc: return eAexonNonConPrc;
  case eCexonNonConNpc: return eAexonNonConNpc;
  case eAintronNonConDn: return eCintronNonConDn;
  case eAintronNonConUp: return eCintronNonConUp;
  case eAexonNonConPrc: return eCexonNonConPrc;
  case eAexonNonConNpc: return eCexonNonConNpc;
  default: assert(0); return eError;
  }
}

ModelType::mname_t
ModelType::getOppSide(mname_t mn) {
  switch(mn) {
  case eCintronNonConDn: return eCintronNonConUp;
  case eCintronConDn: return eCintronConUp;
  case eCintronNonConUp: return eCintronNonConDn;
  case eCintronConUp: return eCintronConDn;

  case eAintronNonConDn: return eAintronNonConUp;
  case eAintronConDn: return eAintronConUp;
  case eAintronNonConUp: return eAintronNonConDn;
  case eAintronConUp: return eAintronConDn;
  default: assert(0); return eError;
  }
}

ModelType::mname_t
ModelType::toSplice(SeqLabel::event_t rev, SeqLabel::blabel_t st, int clevel) {
  conserve_t consType = clevel2conserve(clevel);
  mname_t mn = eError;
  if( st == SeqLabel::eBeg || SeqLabel::isEnd(st) ) {
    mn = eTerm;
  } else if( rev == SeqLabel::eAlt ) {
    if( consType==eConserved ) {
      mn = SeqLabel::isDon(st) ? eAdonCon : eAaccCon;
    } else {
      mn = SeqLabel::isDon(st) ? eDonNonc : eAccNonc;
    }
  } else if( rev == SeqLabel::eCon ) {
    if( consType==eConserved ) {
      mn = SeqLabel::isDon(st) ? eCdonCon : eCaccCon;
    } else {
      mn = SeqLabel::isDon(st) ? eDonNonc : eAccNonc;
    }
  } else {
    assert(0);
  }
  return mn;
}

ModelType::mname_t
ModelType::toUpstreamIntron(SeqLabel::event_t rev, int clevel, bool notStrict) {
  conserve_t consType = clevel2conserve(clevel);
  mname_t mn = eError;
  if( rev == SeqLabel::eAlt ) {
    if( consType == eConserved ) {
      mn = eAintronConUp;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = eAintronNonConUp;
    }
  } else if( rev == SeqLabel::eCon ) {
    if( consType == eConserved ) {
      mn = eCintronConUp;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = eCintronNonConUp;
    }
  } else {
    assert(0);
  }
  return mn;
}

ModelType::mname_t
ModelType::toDnstreamIntron(SeqLabel::event_t rev, int clevel, bool notStrict) {
  conserve_t consType = clevel2conserve(clevel);
  mname_t mn = eError;
  if( rev == SeqLabel::eAlt ) {
    if( consType == eConserved ) {
      mn = eAintronConDn;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = eAintronNonConDn;
    }
  } else if( rev == SeqLabel::eCon ) {
    if( consType == eConserved ) {
      mn = eCintronConDn;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = eCintronNonConDn;
    }
  } else {
    assert(0);
  }
  return mn;
}

ModelType::mname_t
ModelType::toExon(SeqLabel::event_t rev, int clevel, bool isPrc, bool notStrict) {
  conserve_t consType = clevel2conserve(clevel);
  mname_t mn = eError;
  if( rev == SeqLabel::eAlt ) {
    if( consType == eConserved ) {
      mn = isPrc ? eAexonConPrc : eAexonConNpc;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = isPrc ? eAexonNonConPrc : eAexonNonConNpc;
    }
  } else if( rev == SeqLabel::eCon ) {
    if( consType == eConserved ) {
      mn = isPrc ? eCexonConPrc : eCexonConNpc;
    } else if( (consType == eNotConserved) || (notStrict && consType == eAmbiguous) ) {
      mn = isPrc ? eCexonNonConPrc : eCexonNonConNpc;
    }
  } else {
    assert(0);
  }
  return mn;
}

ModelType::mname_t 
ModelType::getFalseName(mname_t mn, bool isCons) {
  switch(mn) {
  case eAccNonc:
  case eAaccCon:
  case eCaccCon:
    return isCons ? eAccFalseCons : eAccFalseNonc;
  case eDonNonc:
  case eCdonCon:
  case eAdonCon:
    return isCons ? eDonFalseCons : eDonFalseNonc;
  default:
    return isCons ? eBackgroundCons : eBackgroundNonc;
  }
}

string
ModelType::mname2str(mname_t mn) {
  switch(mn) {
  case eAccFalseNonc:
    return "eAccFalseNonc";
  case eAccFalseCons:
    return "eAccFalseCons";
  case eAccNonc:
    return "eAccNonc";
  case eAaccCon:
    return "eAaccCon";
  case eCaccCon:
    return "eCaccCon";
  case eCdonCon:
    return "eCdonCon";
  case eAdonCon:
    return "eAdonCon";
  case eDonNonc:
    return "eDonNonc";
  case eDonFalseNonc:
    return "eDonFalseNonc";
  case eDonFalseCons:
    return "eDonFalseCons";
  case eCintronNonConDn:
    return "eCintronNonConDn";
  case eCintronConDn: 
    return "eCintronConDn"; 
  case eAintronNonConDn:
    return "eAintronNonConDn";
  case eAintronConDn:
    return "eAintronConDn";
  case eCintronNonConUp:
    return "eCintronNonConUp";
  case eCintronConUp:
    return "eCintronConUp";
  case eAintronNonConUp:
    return "eAintronNonConUp";
  case eAintronConUp:
    return "eAintronConUp";
  case eCexonNonConPrc:
    return "eCexonNonConPrc";
  case eCexonConPrc:
    return "eCexonConPrc";
  case eAexonConPrc:
    return "eAexonConPrc";
  case eAexonNonConPrc:
    return "eAexonNonConPrc";
  case eCexonNonConNpc:
    return "eCexonNonConNpc";
  case eCexonConNpc:
    return "eCexonConNpc";
  case eAexonConNpc:
    return "eAexonConNpc";
  case eAexonNonConNpc:
    return "eAexonNonConNpc";
  case eBackgroundNonc:
    return "eBackgroundNonc";
  case eBackgroundCons:
    return "eBackgroundCons";
  case eTerm:
    return "eTerm";
  case eError:
    return "eError";
  default:
    return "error";
  }
}

ModelType::mname_t
ModelType::str2mname(const string& str) {
  if( str == "eAaccCon")
    return eAaccCon;
  if( str == "eCaccCon")
    return eCaccCon;
  if( str == "eCdonCon")
    return eCdonCon;
  if( str == "eAdonCon")
    return eAdonCon;
  if( str == "eDonNonc")
    return eDonNonc;
  if( str == "eAccNonc")
    return eAccNonc;
  if( str == "eDonFalseCons")
    return eDonFalseCons;
  if( str == "eAccFalseCons")
    return eAccFalseCons;
  if( str == "eDonFalseNonc")
    return eDonFalseNonc;
  if( str == "eAccFalseNonc")
    return eAccFalseNonc;
  if( str == "eCintronNonConDn")
    return eCintronNonConDn;
  if( str == "eCintronConDn") 
    return eCintronConDn; 
  if( str == "eAintronNonConDn")
    return eAintronNonConDn;
  if( str == "eAintronConDn")
    return eAintronConDn;
  if( str == "eCintronNonConUp")
    return eCintronNonConUp;
  if( str == "eCintronConUp")
    return eCintronConUp;
  if( str == "eAintronNonConUp")
    return eAintronNonConUp;
  if( str == "eAintronConUp")
    return eAintronConUp;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonNonConPrc")
    return eCexonNonConPrc;
  if( str == "eCexonConPrc")
    return eCexonConPrc;
  if( str == "eAexonConPrc")
    return eAexonConPrc;
  if( str == "eAexonNonConPrc")
    return eAexonNonConPrc;
  if( str == "eCexonNonConNpc")
    return eCexonNonConNpc;
  if( str == "eCexonConNpc")
    return eCexonConNpc;
  if( str == "eAexonConNpc")
    return eAexonConNpc;
  if( str == "eAexonNonConNpc")
    return eAexonNonConNpc;
  if( str == "eBackgroundNonc")
    return eBackgroundNonc;
  if( str == "eBackgroundCons")
    return eBackgroundCons;
  if( str == "eTerm")
    return eTerm;
  if( str == "eError")
    return eError;
  else 
    return eError;
}

ModelType::cname_t
ModelType::mname2cname(mname_t mn, bool isAlt) {
  switch(mn) {
  case eDonNonc:
    return isAlt ? eAdon : eCdon;
  case eAdonCon:
    return eAdon;
  case eCdonCon:
    return eCdon;
  case eAaccCon:
    return eAacc;
  case eCaccCon:
    return eCacc;
  case eAccNonc:
    return isAlt ? eAacc : eCacc;
  case eCintronNonConDn:
  case eCintronConDn:
    return eCintronDn;
  case eAintronNonConDn:
  case eAintronConDn:
    return eAintronDn;
  case eCintronNonConUp:
  case eCintronConUp:
    return eCintronUp;
  case eAintronNonConUp:
  case eAintronConUp:
    return eAintronUp;
  case eCexonNonConPrc:
  case eCexonConPrc:
    return eCexonPrc;
  case eAexonNonConPrc:
  case eAexonConPrc:
    return eAexonPrc;
  case eCexonNonConNpc:
  case eCexonConNpc:
    return eCexonNpc;
  case eAexonNonConNpc:
  case eAexonConNpc:
    return eAexonNpc;
  default:
    assert(0);
    return eCnameEnd;
  }
}

ModelType::mname_t
ModelType::cname2mname(cname_t cn, int clevel) {
  switch(cn) {
  case eAdon:  
    return clevel == 0 ? eAdonCon : eDonNonc;
  case eCdon:  
    return clevel == 0 ? eCdonCon : eDonNonc;
  case eAacc:  
    return clevel == 0 ? eAaccCon : eAccNonc;
  case eCacc:  
    return clevel == 0 ? eCaccCon : eAccNonc;
  case eCintronDn:
    return clevel == 0 ? eCintronConDn : eCintronNonConDn;
  case eAintronDn:
    return clevel == 0 ? eAintronConDn : eAintronNonConDn;
  case eCintronUp:
    return clevel == 0 ? eCintronConUp : eCintronNonConUp;
  case eAintronUp:
    return clevel == 0 ? eAintronConUp : eAintronNonConUp;
  case eCexonPrc:
    return clevel == 0 ? eCexonConPrc : eCexonNonConPrc;
  case eAexonPrc:
    return clevel == 0 ? eAexonConPrc : eAexonNonConPrc;
  case eCexonNpc:
    return clevel == 0 ? eCexonConNpc : eCexonNonConNpc;
  case eAexonNpc:
    return clevel == 0 ? eAexonConNpc : eAexonNonConNpc;
  default:
    assert(0);
    return eError;
  }
}

bool 
ModelType::isPrcExon(mname_t mn) {
  switch(mn) {
    //case eAretainedIntronPrc:
  case eAexonConPrc:
  case eAexonNonConPrc:
  case eCexonConPrc:
  case eCexonNonConPrc:
      //case eAexonConPrcBindSite:
    return true;
  default:
    return false;
  }
}


string
ModelType::cname2str(cname_t cn) {
  if( cn == eCintronDn ) {
    return "eCintronDn";
  } else if(cn == eAintronDn) {
    return "eAintronDn";
  } else if(cn == eCintronUp) {
    return "eCintronUp";
  } else if(cn == eAintronUp) {
    return "eAintronUp";
  } else if(cn == eCexonPrc) {
    return "eCexonPrc";
  } else if(cn == eAexonPrc) {
    return "eAexonPrc";
  } else if(cn == eCexonNpc) {
    return "eCexonNpc";
  } else if(cn == eAexonNpc) {
    return "eAexonNpc";
  } else if(cn == eAdon) {
    return "eAdon";
  } else if(cn == eCdon) {
    return "eCdon";
  } else if(cn == eAacc) {
    return "eAacc";
  } else if(cn == eCacc) {
    return "eCacc";
  } else if(cn == eCnameEnd) {
    return "eCnameEnd";
  } else {
    assert(0);
    return "eCnameEnd";
  }
}

ModelType::cname_t
ModelType::str2cname(const string& str) {
  if( str == "eCintronDn" ) {
    return eCintronDn;
  } else if(str == "eAintronDn") {
    return eAintronDn;
  } else if(str == "eCintronUp") {
    return eCintronUp;
  } else if(str == "eAintronUp") {
    return eAintronUp;
  } else if(str == "eCexonPrc") {
    return eCexonPrc;
  } else if(str == "eAexonPrc") {
    return eAexonPrc;
  } else if(str == "eCexonNpc") {
    return eCexonNpc;
  } else if(str == "eAexonNpc") {
    return eAexonNpc;
  } else if(str == "eAdon") {
    return eAdon;
  } else if(str == "eCdon") {
    return eCdon;
  } else if(str == "eAacc") {
    return eAacc;
  } else if(str == "eCacc") {
    return eCacc;
  } else if(str == "eCnameEnd") {
    return eCnameEnd;
  } else {
    assert(0);
    return eCnameEnd;
  }
}


ModelType::cname_t
ModelType::sl2cnameInv(const SeqLabel& sll, const SeqLabel& srl, bool isPrc) {
  cname_t res = eCnameEnd;
  SeqLabel::blabel_t ll=sll.getLabel();
  SeqLabel::blabel_t rl=srl.getLabel();
  if( ll==SeqLabel::eBeg && rl==SeqLabel::eEnd ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eAintronDn : eCintronDn; // moving to using a single intron type, (not up versus down)
  }
  else if( ll==SeqLabel::eBeg && rl==SeqLabel::eEnd1 ) {
    if( isPrc ) {
      res = srl.getEvent()==SeqLabel::eCon ? eCexonPrc : eAexonPrc;
    } else {
      res = srl.getEvent()==SeqLabel::eCon ? eCexonNpc : eAexonNpc;
    }
  }
  else if( ((SeqLabel::isAcc(ll)||ll==SeqLabel::eBeg) && (SeqLabel::isDon(rl)||rl==SeqLabel::eEnd)) ||
      (SeqLabel::isAcc(ll)&&SeqLabel::isAcc(rl))||
      (SeqLabel::isDon(ll)&&SeqLabel::isDon(rl)) ||
      (SeqLabel::isDonIr(ll)&&SeqLabel::isAccIr(rl)) ) {
    if( isPrc ) {
      res = (sll.getEvent()==SeqLabel::eCon||sll.getLabel()==SeqLabel::eBeg) && srl.getEvent()==SeqLabel::eCon ? eCexonPrc : eAexonPrc;
    } else {
      res = (sll.getEvent()==SeqLabel::eCon||sll.getLabel()==SeqLabel::eBeg) && srl.getEvent()==SeqLabel::eCon ? eCexonNpc : eAexonNpc;
    }
  }
  else if( SeqLabel::isDon(ll) && rl==SeqLabel::eEnd ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eAintronDn : eCintronDn;
  }
  else if( ll==SeqLabel::eBeg && SeqLabel::isAcc(rl) ) {
    //res = srl.getEvent()==SeqLabel::eAlt ? eAintronUp : eCintronUp;
    res = srl.getEvent()==SeqLabel::eAlt ? eAintronDn : eCintronDn; // moving to using a single intron type, (not up versus down)
  } 
  return res;
}

#if 0
ModelType::inv_t
ModelType::sl2inv(const SeqLabel& sll, const SeqLabel& srl) {
  inv_t res = eInvCnameEnd;
  SeqLabel::blabel_t ll=sll.getLabel();
  SeqLabel::blabel_t rl=srl.getLabel();
  if( ll==SeqLabel::eBeg && rl==SeqLabel::eEnd ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eInvAintron : eInvCintron; // moving to using a single intron type, (not up versus down)
  }
  else if( ll==SeqLabel::eBeg && rl==SeqLabel::eEnd1 ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eInvAexon : eInvCexon; // moving to using a single intron type, (not up versus down)
  }
  else if( ((SeqLabel::isAcc(ll)||ll==SeqLabel::eBeg) && (SeqLabel::isDon(rl)||rl==SeqLabel::eEnd)) ||
      (SeqLabel::isAcc(ll)&&SeqLabel::isAcc(rl))||
      (SeqLabel::isDon(ll)&&SeqLabel::isDon(rl)) ||
      (SeqLabel::isDonIr(ll)&&SeqLabel::isAccIr(rl)) ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eInvAexon : eInvCexon;
  }
  else if( SeqLabel::isDon(ll) && rl==SeqLabel::eEnd ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eInvAintron : eInvCintron;
  }
  else if( ll==SeqLabel::eBeg && SeqLabel::isAcc(rl) ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eInvAintron : eInvCintron;
  }
  return res;
}
#endif

ModelType::inv_t
ModelType::sl2inv(const SeqLabel& sll, const SeqLabel& srl) {
  inv_t res = eInvCnameEnd;
  SeqLabel::blabel_t ll=sll.getLabel();
  SeqLabel::blabel_t rl=srl.getLabel();
  if( ll==SeqLabel::eBeg && rl==SeqLabel::eEnd ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eInvAintron: eInvCintron; // moving to using a single intron type, (not up versus down)
  }
  else if( ll==SeqLabel::eBeg && rl==SeqLabel::eEnd1 ) {
    res = srl.getEvent()==SeqLabel::eCon ? eInvCexon : eInvAexon;
  }
  else if( ((SeqLabel::isAcc(ll)||ll==SeqLabel::eBeg) && (SeqLabel::isDon(rl)||rl==SeqLabel::eEnd)) ||
      (SeqLabel::isAcc(ll)&&SeqLabel::isAcc(rl))||
      (SeqLabel::isDon(ll)&&SeqLabel::isDon(rl)) ||
      (SeqLabel::isDonIr(ll)&&SeqLabel::isAccIr(rl)) ) {
    res = (sll.getEvent()==SeqLabel::eCon||sll.getLabel()==SeqLabel::eBeg) && srl.getEvent()==SeqLabel::eCon ? eInvCexon : eInvAexon;
  }
  else if( SeqLabel::isDon(ll) && rl==SeqLabel::eEnd ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eInvAintron: eInvCintron;
  }
  else if( ll==SeqLabel::eBeg && SeqLabel::isAcc(rl) ) {
    res = srl.getEvent()==SeqLabel::eAlt ? eInvAintron : eInvCintron; // moving to using a single intron type, (not up versus down)
  } 
  return res;
}


inline ModelType::cname_t
ModelType::sl2cnamePos(const SeqLabel& srl) {
  cname_t res = eCnameEnd;
  SeqLabel::blabel_t ll=srl.getLabel();
  SeqLabel::event_t ev=srl.getEvent();
  if( SeqLabel::isAcc(ll) ) {
    res = (ev == SeqLabel::eAlt ? eAacc : eCacc);
  } else if (SeqLabel::isDon(ll) ) {
    res = (ev == SeqLabel::eAlt ? eAdon : eCdon);
  }
  return res;
}

bool 
ModelType::isPrcExon(cname_t mn) {
  switch(mn) {
  case eCexonPrc:
  case eAexonPrc:
    return true;
  default:
    return false;
  }
}

bool 
ModelType::isNpcExon(cname_t mn) {
  switch(mn) {
  case eCexonNpc:
  case eAexonNpc:
    return true;
  default:
    return false;
  }
}

bool 
ModelType::isDon(cname_t mn) {
  switch(mn) {
  case eAdon:
  case eCdon:
    return true;
  default:
    return false;
  }
}

bool 
ModelType::isAcc(cname_t mn) {
  switch(mn) {
  case eAacc:
  case eCacc:
    return true;
  default:
    return false;
  }
}

bool 
ModelType::isSplice(cname_t mn) {
  switch(mn) {
  case eAdon:
  case eCdon:
  case eAacc:
  case eCacc:
    return true;
  default:
    return false;
  }
}
    
ModelType::cname_t
ModelType::inv2cname(inv_t inv, bool isPrc) {
  switch(inv) {
  case eInvCintron: return eCintronDn;
  case eInvAintron: return eAintronDn;
  case eInvCexon: return isPrc ? eCexonPrc : eCexonNpc;
  case eInvAexon: return isPrc ? eAexonPrc : eAexonNpc;
  default:
    return eCnameEnd;
  }
}

bool
ModelType::isIntron(cname_t cn) {
  switch(cn) {
  case eCintronDn:
  case eAintronDn:
  case eCintronUp:
  case eAintronUp:
    return true;
  default:
    return false;
  }
}
