#include "MultiModel.h"
#include "TamuraNei.h"
#include <cctype>
#include <iostream>

using std::list;
using std::endl;
using std::cout;
using std::string;
using std::vector;

extern bool DEBUGON;
extern bool GDEBUG;

MultiModel::MultiModel(Model* mod, unsigned size) : _vec(size,NULL) {
  init(mod);
}

void
MultiModel::init(Model* mod) {
  const double pseudoCnt = 0.00001;
  //const double pseudoCnt = 0.01;
  _vec[0]=mod;
  _vec[0]->add( pseudoCnt );
  for(unsigned i = 1; i < _vec.size(); ++i) {
    _vec[i] = mod->clone();
    _vec[i]->add( pseudoCnt );
  }
  const Model* md = _vec[0];
  const MarkovChain& tmp =  dynamic_cast<const MarkovChain&>(md->getModel((unsigned)0));
  const int forder = tmp.getOrder();
  _order = forder;
}

MultiModel::~MultiModel() {
  for(unsigned i = 0; i < _vec.size(); ++i) {
    delete _vec[i];
  }
  hashmc_t::iterator iter = _mcHash.begin();
  const hashmc_t::iterator end = _mcHash.end();
  for(; iter != end; ++iter) {
    Model* md = (*iter).second;
    delete md;
  }
}

void 
MultiModel::update(const std::vector<std::string*>& svec, int pos, int time) {
  for(unsigned i = 0; i < _vec.size(); ++i) {
    if( svec[i] ) {
      _vec[i]->update(*svec[i],pos,time);
    }
  }
}

void 
MultiModel::update1(const std::vector<std::string*>& svec, int pos, int time) {
  for(unsigned i = 1; i < _vec.size(); ++i) {
    if( svec[i] ) {
      _vec[i]->update(*svec[i],pos,time);
    }
  }
}


static void
makeKey1(const vector<string*>& svec, int pos, int time, int order, string& keyStr) {
  pos-=1; // capture string preceding pos
  if( (pos-1) >= 0 ) {
    assert(order >=0);
    signed width = order;
    for(unsigned x = 0, istart=0; x < svec.size(); ++x, istart+=width) {
      if( svec[x] ) {
	const string& gstr = *svec[x];
	signed gidx = (pos-1), ccnt = 0;
	for(signed idx = istart; gidx >= 0 && ccnt <= width && idx < (signed)istart+width; ) {
	  assert(gidx >=0 && gidx < (signed)gstr.size());
	  const char tchr = gstr[gidx];
	  if( !std::islower(tchr) && tchr != '-' ) {
	    assert(idx >=0 && idx <(signed)keyStr.size());
	    keyStr[idx]=tchr;
	    ++ccnt;
	    --gidx;
	    ++idx;
	  } else {
	    --gidx;
	  }
	}
      }
    }      
    unsigned sidx = keyStr.size();
    if( time < 0 )
      time = 0;
    keyStr[sidx-1] = getDigitPos0(time);
    if(time >= 10) {
      assert(time < 100); // one char per digit (assume max of 2 digits for now);
      keyStr[sidx-2] = getDigitPos1(time);
    }
  }
}

static void
makeKey2(const vector<string*>& svec, int pos, int time, int order, const list<int>& olst, string& keyStr) {
  pos-=1; // capture string preceding pos
  if( (pos-1) >= 0 ) {
    assert(order>=0);
    signed width = order;
    list<int>::const_iterator iter = olst.begin();
    const list<int>::const_iterator stop = olst.end();
    for(unsigned istart=0; iter != stop; ++iter, istart+=width) {
      int x = *iter;
      if( svec[x] ) {
	const string& gstr = *svec[x];
	signed gidx = (pos-1), ccnt = 0;
	for(signed idx = istart; gidx >= 0 && ccnt <= width && idx < (signed)istart+width; ) {
	  assert(gidx >=0 && gidx <(signed) gstr.size());
	  const char tchr = gstr[gidx];
	  if( !std::islower(tchr) && tchr != '-' ) {
	    assert(idx >=0 && idx <(signed)keyStr.size());
	    keyStr[idx]=tchr;
	    ++ccnt;
	    --gidx;
	    ++idx;
	  } else {
	    --gidx;
	  }
	}
      }
    }      
    unsigned sidx = keyStr.size();
    if( time < 0 )
      time = 0;
    keyStr[sidx-1] = getDigitPos0(time);
    if(time >= 10) {
      assert(time < 100); // one char per digit (assume max of 2 digits for now);
      keyStr[sidx-2] = getDigitPos1(time);
    }
  }
}

const MarkovChain*
MultiModel::getMc(const std::vector<std::string*>& svec, int pos, int time) {
  const int forder = getOrder();
  MarkovChain* result = NULL;
  if( pos > forder ) {
    unsigned nSize = time >= 0 && time <= 9 ? 1 : 2;
    unsigned strSize = nSize+(svec.size()*forder);
    string keyStr(strSize,'-');
    makeKey1(svec,pos,time,forder,keyStr);
    if(_mcHash[keyStr]) {
      result = _mcHash[keyStr];
    } else {
      result = new MarkovChain(0);
      for(unsigned i = 0; i < _vec.size(); ++i) {
	if( svec[i] ) {
	  const string& str = *svec[i];
	  Model* md = _vec[i];
	  const MarkovChain& tmp =  dynamic_cast<const MarkovChain&>(md->getModel((unsigned)time));
	  // if there gaps in the chain region, don't contribute 
	  // frequency pool
	  MarkovChain* tmc = tmp.makeOrder0(str,pos);
	  if( tmc ) {
	    *result += *tmc;
	    if( GDEBUG ) {
	      tmc->print(cout);
	      result->print(cout);
	    }
	    delete tmc;
	  }
	}
      }
      if(result) 
	result->normalize();
      _mcHash[keyStr] = result;
    }
  }
  if(GDEBUG) {
    cout<<"THE RESULT IS: "<<endl;
    if( result ) {
      result->print(cout);
    }
    cout<<"-------------- "<<endl;
  }
  return result;
}

#if 0
int
MultiModel::getOrder() const {
  const Model* md = _vec[0];
  const MarkovChain& tmp =  dynamic_cast<const MarkovChain&>(md->getModel((unsigned)0));
  const int forder = tmp.getOrder();
  return forder;
}
#endif

static bool
hasStr(const string* strPtr, int pos, int order, string& addKey) {
  bool status = false;
  if(strPtr) {
    const string& str = *strPtr;
    int start = (pos-1)-order;
    assert(start>=0);
    signed cnt = 0;
    for(; start < (pos-1); ++start) {
      const char tchr = str[start];
      if( std::isupper(tchr) && tchr != '-' ) {
	addKey[cnt]=tchr;
	++cnt;
      } else { 
	const unsigned s = addKey.size();
	addKey.resize(s,'-');
	break;
      }
    }
    if( cnt == order ) {
      status = true;
    }
  }
  return status;
}

MarkovChain*
MultiModel::getClosestTarget(const std::vector<std::string*>& svec, int pos, int time, int order, const list<int>& olst, string& addKey) const {
  int idx = -1;
  if(olst.size()==1 && olst.front()==2) {
    idx = 3;
  } 
  else if(olst.size()==1 && olst.front()==3) {
    idx = 2;
  } else {
    idx = 0;
  }
  assert(idx !=-1);
  if( !hasStr(svec[idx],pos,order,addKey) ) {
    assert(idx != 0);
    idx = 0;
    hasStr(svec[idx],pos,order,addKey);
  }
  assert(idx>=0&&idx<=9);
  unsigned sidx = addKey.size();
  addKey[sidx-1] = getDigitPos0(idx);
  const string& str = *svec[idx];
  Model* md = _vec[idx];
  const MarkovChain& tmp =  dynamic_cast<const MarkovChain&>(md->getModel((unsigned)time));
  // if there gaps in the chain region, don't contribute 
  MarkovChain* tmc = tmp.makeOrder0(str,pos);
  return tmc;
}

const SubModel*
MultiModel::getMc(const std::vector<std::string*>& svec, int pos, int time, Phylogeny::shash_t& mcHash, const list<int>& olst, double tnTvRatio, double mutRate) const {
  const int forder = getOrder();
  const SubModel* result = NULL;
  if( pos > forder ) {
    unsigned nSize = time >= 0 && time <= 9 ? 1 : 2;
    unsigned strSize = nSize+(olst.size()*forder);
    string keyStr(strSize,'-');
    makeKey2(svec,pos,time,forder,olst,keyStr);
    if(mcHash[keyStr]) {
      result = mcHash[keyStr];
    } else {
      MarkovChain* mcResult = new MarkovChain(0);
      unsigned add = 0;
      list<int>::const_iterator iter = olst.begin();
      const list<int>::const_iterator stop = olst.end();
      for(; iter != stop; ++iter) {
	int i = *iter;
	assert(i < (signed)svec.size());
	
	if( svec[i] ) {
	  const string& str = *svec[i];
	  Model* md = _vec[i];
	  const MarkovChain& tmp =  dynamic_cast<const MarkovChain&>(md->getModel((unsigned)time));
	  MarkovChain* tmc = tmp.makeOrder0(str,pos);
	  if(tmc) {
	    ++add;
	    *mcResult += *tmc;
	    if( GDEBUG ) {
	      tmc->print(cout);
	      mcResult->print(cout);
	    }
	    delete tmc;
	  } else if( str[pos-1] != '-' ) {
	    // need to add 0th order markov models for beginning of sequence
	    signed tc=0;
	    for(signed k=1;k < pos;++k) { if(str[k-1]!='-') ++tc; }
	    assert(tc < forder);
	    // for now 
	    const string& str = *svec[0];
	    Model* md = _vec[0];
	    const MarkovChain& tmp =  dynamic_cast<const MarkovChain&>(md->getModel((unsigned)time));
	    MarkovChain* tmc = tmp.makeOrder0(str,pos);
	    assert(tmc);
	    *mcResult += *tmc;
	    ++add;
       delete tmc;
	  }
	}
      }
#if 0
      // possible at beginning of sequence - just use target sequence
      if( add == 0 ) {
	const string& str = *svec[0];
	//const string str = "AAAAA";
	Model* md = _vec[0];
	const MarkovChain& tmp =  dynamic_cast<const MarkovChain&>(md->getModel((unsigned)time));
	MarkovChain* tmc = tmp.makeOrder0(str,pos);
	//MarkovChain* tmc = tmp.makeOrder0(str,5);
	*mcResult += *tmc;
      }
#endif
      if( add > 0 ) {
      if(mcResult) 
	mcResult->normalize();
      result = mcHash[keyStr] = mcResult ? new TamuraNei(*mcResult,tnTvRatio,mutRate) : NULL;
      }
      if(mcResult) delete mcResult;
    }
  }
  return result;
}

void 
MultiModel::write(std::ostream& os) const {
  for(unsigned i = 0; i < _vec.size(); ++i) {
    _vec[i]->write(os);
  }
}

void 
MultiModel::print(std::ostream& os) const {
  for(unsigned i = 0; i < _vec.size(); ++i) {
    _vec[i]->print(os);
  }
}

void 
MultiModel::read(std::istream& is) {
  for(unsigned i = 0; i < _vec.size(); ++i) {
    _vec[i]->read(is);
  }
}
