//////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2003, The Institute for Genomic Research (TIGR), Rockville,
// Maryland, U.S.A.  All rights reserved.
//////////////////////////////////////////////////////////////////////////////
#include "Options.h"
#include <iostream>
#include <fstream>
#include <unistd.h>
#include <sstream>

using std::cerr;
using std::cout;
using std::endl;
using std::runtime_error;
using std::string;

Options* Options::_instance = NULL;

Options::Options(int argc, char** argv, const string& id) : _logStream(NULL),
  _quit(false), _verbose(false), _oneExon(false), _useFrame(false), _useExon(false) {
    parse(argc,argv,id); 
  }

Options::~Options() {
  if(_logStream) delete _logStream;
}

void Options::parse(int argc, char** argv, const string& id) {
  int opt;
  unsigned cnt =0;
  while( (opt=getopt(argc,argv,"p:shfeVvo:m:i:l:")) != -1) {
    switch(opt) {
    case 'l':
      _logStream = new std::ofstream(optarg);
      if( ! (*_logStream) ) {
	cout<<"Unable to open ["<<optarg<<"]"<<endl;
	_quit = true;
      } else {
	cout<<"Output debug info to log file: "<<optarg<<endl;
      }
      break;
    case 'i':
      _header = optarg;
      cout<<"Run on header: "<<_header<<endl;
      break;
    case 'p':
      ++cnt;
      _paramFile = optarg;
      cout<<"Using parameter file: "<<_paramFile<<endl;
      break;
    case 'V':
      _verbose = true;
      break;
    case 'v':
      versionStatement(cout,id);
      exit(0);
    case 's': 
      _oneExon = true;
      break;
    case 'o':
      ++cnt;
      assert(optarg);
      _gffFile = optarg;
      cout<<"Output to file: "<<optarg<<endl;
      break;
    case 'f':
      _useFrame=true;
      break;
    case 'e':
      _useExon=true;
      break;
    case 'm':
      ++cnt;
      _modelFileBase = optarg;
      cout<<"Using models in: "<<_modelFileBase<<endl;
      break;
    case 'h':
      _quit = true;
      break;
    default:
      cerr<<"Unhandeled case: ["<<(char)opt<<"]\n";
      _quit = true;
      throw runtime_error("unable to parse the command line arguments sorry");
      break;
    }
  }
  for(; optind < argc; ++optind) {
    cout<<"Unhandeled argument: "<<argv[optind]<<endl;
  }
  if( cnt < 3 ) {
    _quit=true;
  }
}

void Options::versionStatement(std::ostream& os, const string& id) {
  os<<id<<endl;
}
