#ifndef _PARAMSEARCH_H
#define _PARAMSEARCH_H

#include "HmmComponent.h"
#include "RateHmm.h"
#include <vector>
#include <string>

class ParamSearch {
 public:
  ParamSearch() : _param(NULL), _source(NULL) { }
  ParamSearch(HmmComponent* hc, RateHmm* hmm) : _param(hc), _source(hmm) { }
  ParamSearch(const ParamSearch& ps) : _param(ps._param), _source(ps._source) { }
  const HmmComponent* getComp() const { return _param; }
  HmmComponent* getComp() { return _param; }
  RateHmm* getHmm() { return _source; }
 protected:
  HmmComponent* _param;
  RateHmm* _source;
};

class ParamSearchPos : public ParamSearch {
 public:
  ParamSearchPos() : _pos(-1), _phase(-1) { }
  ParamSearchPos(HmmComponent* hc, RateHmm* hmm, int pos, int phase) : 
			 ParamSearch(hc,hmm), _pos(pos), _phase(phase) {}
  ParamSearchPos(const ParamSearchPos& psp) : ParamSearch(psp), _pos(psp._pos), _phase(psp._phase) { }
  int getPos() const { return _pos; }
  int getPhase() const { return _phase; }
 private:
  int _pos, _phase;
};


class ParamSearchPair {
 public:
  ParamSearchPair(const ParamSearchPos& ans, const ParamSearchPos& pred, const std::vector<std::string*>& seq) :
  _ans(ans), _pred(pred), _seq(seq), _delta(0) { }
  ParamSearchPair(const ParamSearchPair& psp) : _ans(psp._ans), _pred(psp._pred), _seq(psp._seq), _delta(psp._delta) {}

  const ParamSearchPos& getAns() const{ return _ans; }
  ParamSearchPos& getAns() { return _ans; }
  const ParamSearchPos& getPrd() const { return _pred; }
  ParamSearchPos& getPrd() { return _pred; }
  const std::vector<std::string*>& getSeq() const { return _seq; }
  void setDelta(double d) { _delta = d; }
  double getDelta(const HmmComponent* hc) const {
    return (_ans.getComp()==hc) ? _delta*_ans.getComp()->getProb()+_ans.getComp()->getProb() : (_pred.getComp()->getProb()-_delta*_pred.getComp()->getProb());
  }
  
  bool operator==(const ParamSearchPair& psp) const {
    return ( _delta==psp._delta && _ans.getComp()==psp.getAns().getComp() &&
	_pred.getComp()==psp.getPrd().getComp() );
  }

 private:
  ParamSearchPos _ans;
  ParamSearchPos _pred;
  const std::vector<std::string*>& _seq;
  double _delta;
};

  struct hchash
  {
    std::size_t operator()(const HmmComponent* __s) const { return size_t(__s); }
  };

typedef __gnu_cxx::hash_map<HmmComponent*,std::list<ParamSearchPair>, hchash > ParamSearchHashT;

#endif // _PARAMSEARCH_H
