#include "ParseHeader.h"
#include "ParamFile.h"
#include "utils.h"
#include <iostream>
#include <string>
#include <sstream>
#include <stdlib.h>
#include <unistd.h>

typedef std::string::size_type size_type;
using std::istringstream;
using std::string;
using std::endl;
using std::cerr;
using std::cout;
using std::list;
using std::vector;
using std::pair;

static int
compCodingLen(int lbnd, int rbnd, int startc, int stopc); 

typedef std::pair<int,int> ipair_t;
static ipair_t getMinMax(const string& exns); 

struct TObj {
  bool operator()(const ipair_t& a, const ipair_t& b) const {
    return a.first < b.first;
  }
};

struct TObj1 {
  bool operator()(const ipair_t& a, const ipair_t& b) const {
    return a.second < b.second;
  }
};

static int 
compRel(int, char, int, int); 

static
list<GffFeature*>::const_iterator
lfind(const list<GffFeature*>::const_iterator& start, const list<GffFeature*>::const_iterator& stop, const string& id) {
  for(list<GffFeature*>::const_iterator iter = start; iter != stop; ++iter) {
    const GffFeature* gff = *iter;
    const string& c8 = gff->getCol8();
    size_type p1 = c8.find("-");
    size_type p2 = c8.find(";",p1+1);
    string id2 = c8.substr(p1+1,p2-p1-1);
    if( id2[1] == id[1] ) {
      return iter;
    }
  }
  return stop;
}

static bool DEBUG = false;
void ParseHeader::assignAnnotationInfo(const string& exns, const AnnotationInfo& ai) {
  const string& id = _klst.getId();
  if(DEBUG) 
    cout<<"lookup id: ["<<id<<"]"<<endl;
  const CodeHashT& chash = ai.getCodeHash();
  CodeHashT::const_iterator match = chash.find(id);
  assert(match != chash.end());
  const list<GffFeature*> feats = (*match).second;

  size_type rght_pos = exns.size(), last = 0;
  while(last != string::npos) {
    size_type cursor = exns.find_first_of(":",last);
    //size_type right_pos = (cursor == string::npos) ? exns.size() : cursor;
    string elst = exns.substr(last,rght_pos-last);
    size_type p2 = elst.find(',');
    if(DEBUG)
      cout<<"sexons: "<<elst<<endl;
    string idlst = elst.substr(0,p2);
    if(DEBUG)
      cout<<"idlst: "<<idlst<<endl;
    size_type slast = 0;
    while( slast != string::npos ) {
      size_type pos = idlst.find('_',slast);
      string gid;
      if( pos != string::npos ) {
	gid = idlst.substr(slast,pos-slast);
      } else { 
	gid = idlst.substr(slast,idlst.size()-slast);
      }
      list<GffFeature*>::const_iterator mi = lfind(feats.begin(),feats.end(),gid);
      if( mi == feats.end()) {
	cout<<"Warning could not find match for: "<<id<<" "<<gid<<endl;
      } else {
	if(DEBUG) 
	  cout<<"Add feat: "<<**mi<<endl;
	_klst.addFeature(*mi);
	//
	// IN HERE ADD FRAME DATA, make CodeHash - GeneStructureHash
      }
      //cout<<"gid: "<<gid<<endl;
      const GeneHashT& ghash = ai.getGeneHash();
      const string fgid = id + string("-") + gid;
      //cout<<"lookup : "<<fgid<<endl;
      GeneHashT::const_iterator gmatch = ghash.find(fgid);
      if(gmatch == ghash.end()) {
	//cout<<"fgid: "<<fgid<<endl;
	gmatch = ghash.find(fgid);
	assert(gmatch != ghash.end());
      }
      const list<GeneList*>& geneList = (*gmatch).second;
      list<GeneList*>::const_iterator gi = AnnotationInfo::findGeneList(geneList,fgid);
      if( gi == geneList.end()) {
	cout<<"Warning could not find match for: "<<id<<" "<<gid<<endl;
	assert(0);
      } else {
	if(DEBUG) 
	  cout<<"Add gene: "<<**mi<<endl;
	//cout<<"What's it look like: "<<endl<<**gi<<endl;
	_klst.addGene(*gi);
      }
      slast = pos;
      if( slast != string::npos ) 
	slast = pos + 1;
    }
    if(cursor == string::npos) {
      break;
    }
    last = cursor + 1;
  }
  //_klst.storeCodonBoundary();
}

string
ParseHeader::getSpliceEvent(const fasta_seq& fs) const {
  const string& head = fs.get_header();
  const size_type tp = head.find_first_of(':');
  const size_type ll = head.find('[');
  const size_type tp2 = head.find_first_of(':',tp+1);
  string cstr = head.substr(tp2+2,ll-tp2-3);
  return cstr;
}

bool 
ParseHeader::parse(const fasta_seq& fs) {
  ParamFile* pf = ParamFile::getInstance();
  const int offSet = pf->getExonOffset();
  //const int offSet = 400;
  const string& head = fs.get_header();
  const size_type tp = head.find_first_of(':');
  const char strnd = head[tp+1];
  const string locId = head.substr(0,tp);
  const size_type tpos = locId.find('_');
  const string tid = locId.substr(0,tpos);
  const string chr = locId.substr(tpos+1,locId.size());
  _klst.setStrnd(strnd);
  _klst.setId(tid);
  _klst.setChr(chr);

  const size_type ll = head.find('[');
  const size_type lr = head.find(']',ll);
  const size_type ml = head.find('[',lr);
  const size_type mr = head.find(']',ml);
  const size_type rl = head.find('[',mr);
  const size_type rr = head.find(']',rl);
  const size_type tp2 = head.find_first_of(':',tp+1);
  _isInitExon=false;
  _isLastExon=false;
  if(strnd == '+' && lr-ll==1) {
    _isInitExon=true;
  } else if( strnd == '-' && lr-ll==1 ) {
    _isLastExon=true;
  }
  if(strnd == '-' && rr-rl==1) {
    _isInitExon=true;
  } else if(strnd == '+' && rr-rl==1) {
    _isLastExon=true;
  }
  string cstr = head.substr(tp2+2,ll-tp2-3);

  const size_type fp1 = head.find('(');
  const size_type fp2 = head.find(')',fp1);
  string f1str = head.substr(fp1+1,(fp2-fp1)-1);

  const size_type fp3 = head.find('(',fp2);
  const size_type fp4 = head.find(')',fp3);
  string f2str = head.substr(fp3+1,(fp4-fp3)-1);

  size_type last_pos=0;
  while( last_pos != string::npos ) {
    const size_type pos = f1str.find('_',last_pos+1);
    f1str[pos]=' ';
    last_pos=pos;
  }
  list <int> flst;
  istringstream istrm(f1str.c_str());
  int frame;
  while( istrm>>frame ) {
    flst.push_back(frame);
  }
  _klst.setStartFrame(flst);

 last_pos=0;
  while( last_pos != string::npos ) {
    const size_type pos = f2str.find('_',last_pos+1);
    f2str[pos]=' ';
    last_pos=pos;
  }
  list <int> flst2;
  istringstream istrm2(f2str.c_str());
  while( istrm2>>frame ) {
    flst2.push_back(frame);
  }
  _klst.setNextFrame(flst2);
  
  string left,exns,rght;
  if( lr-ll-1 > 0)
    left = head.substr(ll+1,lr-ll-1);
  assert(mr-ml-1> 0);
  exns = head.substr(ml+1,mr-ml-1);
  if( rr-rl-1 > 0)
    rght = head.substr(rl+1,rr-rl-1);
  list<ipair_t> raw_elst;
  ipair_t cbnd = getMinMax(exns);
  ipair_t lbnd(cbnd.first,cbnd.first);
  ipair_t rbnd(cbnd.second,cbnd.second);;
  if( left.size() > 0 ) {
    lbnd = getMinMax(left);
  } 
  if( rght.size() > 0 ) { 
    rbnd = getMinMax(rght);
  }
  ipair_t theBnd(lbnd.second,rbnd.first);

  size_type rght_pos = exns.size();
  size_type last = exns.find(',') + 1;
  while(last != string::npos) {
    size_type cursor = exns.find_first_of(",:",last);
    if(cursor == string::npos) {
      rght_pos = exns.size();
    } else {
      rght_pos = cursor;
    }
    string coords = exns.substr(last,rght_pos-last);
    size_type pos = coords.find('_');
    if( pos != string::npos ) {
      coords[pos] = ' ';
      std::istringstream istrm(coords);
      int end5,end3;
      if(istrm>>end5>>end3) {
	raw_elst.push_back(ipair_t(end5,end3));
      } else {
	//cout<<"could not read coords: "<<coords<<endl;
      }
    }
    if(cursor == string::npos) {
      break;
    }
    last = cursor + 1;
  }
  if( cbnd.first-offSet > lbnd.second) {
    theBnd.first = cbnd.first-offSet;
  }
  if( cbnd.second+offSet < rbnd.first) {
    theBnd.second = cbnd.second+offSet;
  }
  list<ipair_t>::const_iterator iter  = raw_elst.begin();
  const list<ipair_t>::const_iterator stop  = raw_elst.end();
  const int beg = theBnd.first;
  const int end = theBnd.second;
  _klst.setBeg(beg);
  _klst.setEnd(end);
  int relb = compRel(beg,strnd,beg,end);
  int rele = compRel(end,strnd,beg,end);
  if(relb>rele) {
    std::swap(relb,rele);
  }
  if( _ai ) {
    assignAnnotationInfo(exns,*_ai);
  }
  hmap_str_t hmap;
  hmap_unsigned_t cnt;
  list<ipair_t> rel_elst;
  for(; iter != stop; ++iter) {
    ipair_t ip = *iter;
    int rel5 = compRel(ip.first,strnd,beg,end);
    int rel3 = compRel(ip.second,strnd,beg,end);
    if(rel5>rel3) {
      std::swap(rel5,rel3);
    }
    rel_elst.push_back(ipair_t(rel5,rel3));
    //cout<<"TheExon: "<<rel5<<" "<<rel3<<endl;
    pair<string,string> pr = _klst.classType(ip.first,ip.second,strnd);
    if(pr.first.length()<=0) {
       cout<<"MARK FOR REMOVEAL "<<head<<endl;
		 return false;
    }
    if( strnd == '-' ) {
      swap(pr.first,pr.second);
    }
    if(DEBUG) {
      cout<<"raw: "<<ip.first<<" "<<pr.first<<" "<<ip.second<<" "<<pr.second<<" rel: "<<rel5<<" "<<rel3<<endl;
    }
    //cout<<pr.first<<" "<<pr.second<<endl;
    hmap[rel5] = pr.first;
    hmap[rel3] = pr.second;
    cnt[rel5]++;
    cnt[rel3]++;
  }
  ipair_t nr = *std::min_element(rel_elst.begin(),rel_elst.end(),TObj());
  ipair_t xr = *std::max_element(rel_elst.begin(),rel_elst.end(),TObj1());
  const unsigned min = nr.first;
  const unsigned max = xr.second;
  const unsigned num_exons = rel_elst.size();
  hmap_str_t::const_iterator hiter = hmap.begin();
  hmap_str_t::const_iterator hstop = hmap.end();
  SeqLabel::event_t levent = SeqLabel::eUnk;
  for(; hiter != hstop; ++hiter) {
    pair<unsigned,string> hc = *hiter;
    unsigned sloc = hc.first;
    SeqLabel::blabel_t label = SeqLabel::eUndef;
    if( hc.second == "don" ) {
      label = SeqLabel::eDon;
    } else if( hc.second == "acc" ) {
      label = SeqLabel::eAcc;
    } else if( sloc == min ) {
      label = SeqLabel::eBeg;
      assert(sloc>=1);
      sloc-=1;
    } else if( sloc == max ) {
      label = SeqLabel::eEnd;
      sloc+=1;
    } else {
      //cerr<<"Not handle right yet: just ignored ["<<sloc<<"] ["<<hc.second<<"]"<<" min: "<<min<<" max: "<<max<<endl;
      continue;
      //label = SeqLabel::eOther;
      // ignore everything else for now
    }
    SeqLabel::event_t event = SeqLabel::eUnk;
    //cout<<"event: "<<cstr<<endl;
    if( label == SeqLabel::eBeg || label == SeqLabel::eEnd) {
      event=SeqLabel::eCon;
    } else if( cstr == "MS" || cstr == "PT:MS" ) {
      event=SeqLabel::eCon;
      if( cnt[sloc] < num_exons ) {
	event=SeqLabel::eAlt;
      }
    } else if( cstr == "IR"  || cstr == "PT:IR" ) {
      event = SeqLabel::eCon;
      unsigned loc = sloc;
      if( label == SeqLabel::eBeg ) {
	loc = sloc+1;
      } else if(label == SeqLabel::eEnd) {
	loc = sloc-1;
      }
      if( cnt[sloc] < num_exons-1 && loc > min && loc < max ) {
	event = SeqLabel::eAlt;
      }
    } else if( cstr == "SE" || cstr == "SE:MS" ) {
      event = SeqLabel::eAlt;
    } else if( cstr == "CS" ) {
      event = SeqLabel::eCon;
    } else {
      assert(0);
    }
    if( sloc == max ) {
      levent = event;
    }
    SeqLabel::ipair_t ip = SeqLabel::getSigDim(label,sloc);
    if(ip.first > 0 ) {
      SeqLabel sl(label,event,ip.first,ip.second);
      _klst.addSeqLabel(sl);
    }
  }
  SeqLabel::event_t etype = SeqLabel::eUnk;
  if( cstr == "IR" || cstr == "PT:IR" || cstr == "CS" ) {
    etype = SeqLabel::eCon;
  } else if( levent == SeqLabel::eAlt ) {
    etype = SeqLabel::eAlt;
  } else {
    etype = SeqLabel::eCon;
  }
  if( !_klst.hasLabel(SeqLabel::eBeg) ) {
    SeqLabel sl(SeqLabel::eBeg,etype,relb-1,relb-1);
    _klst.addSeqLabel(sl);
  }
  if( !_klst.hasLabel(SeqLabel::eEnd) ) {
    SeqLabel sl(SeqLabel::eEnd,etype,rele,rele);
    _klst.addSeqLabel(sl);
  }
  bool status = _klst.makeFrames();
  if(DEBUG) {
    _klst.print(cout);
    assert(0);
  }
  _klst.sortLabelLst(cstr);
  SeqLabel& lst = _klst.getLabelLst().back();
  assert(lst.getLabel()==SeqLabel::eEnd);
  const string& seq = fs.get_seq();
  if( lst.getRght() > (signed)seq.size() ) {
    //cout<<"end should always be an index of 1 position past last char in str: correcting"<<endl;
    lst.setEnds(seq.size(),seq.size());
  }
  return status;
}


int compRel(int crd, char strnd, int beg, int end) {
  int nc=0;
  if ( strnd == '-') {
    nc = (end-crd)+1;
  } else if (strnd == '+') {
    nc = (crd-beg)+1;
  }
  return nc;
}

ipair_t getMinMax(const string& exns) {
  size_type rght_pos = exns.size();
  size_type last = exns.find(',') + 1;
  int mnLeft = -1, mxRght = -1;
  while(last != string::npos) {
    size_type cursor = exns.find_first_of(",:",last);
    if(cursor == string::npos) {
      rght_pos = exns.size();
    } else {
      rght_pos = cursor;
    }
    string coords = exns.substr(last,rght_pos-last);
    size_type pos = coords.find('_');
    if( pos != string::npos ) {
      coords[pos] = ' ';
      std::istringstream istrm(coords);
      int end5,end3;
      //cout<<coords<<endl;
      istrm>>end5>>end3;
      if( end5 < mnLeft || mnLeft==-1 ) {
	mnLeft = end5;
      }
      if( end3 > mxRght || mxRght == -1 ) {
	mxRght = end3;
      }
    }
    if(cursor == string::npos) {
      break;
    }
    last = cursor + 1;
  }
  return ipair_t(mnLeft,mxRght);
}


list<int> 
ParseHeader::getExon(const fasta_seq& fs, const vector<int>& limitFrame, const vector<int>& limitNextFrame, int start_codon, int stop_codon) const {
  ParamFile* pf = ParamFile::getInstance();
  const int offSet = pf->getExonOffset();
  //const int offSet = 400;
  const string& head = fs.get_header();
  const size_type tp = head.find_first_of(':');
  const char strnd = head[tp+1];
  const string locId = head.substr(0,tp);
  const size_type tpos = locId.find('_');
  const string tid = locId.substr(0,tpos);
  const string chr = locId.substr(tpos+1,locId.size());

  const size_type ll = head.find('[');
  const size_type lr = head.find(']',ll);

  const size_type tp2 = head.find_first_of(':',tp+1);
  string cstr = head.substr(tp2+2,ll-tp2-3);
  const size_type ml = head.find('[',lr);
  const size_type mr = head.find(']',ml);
  const size_type rl = head.find('[',mr);
  const size_type rr = head.find(']',rl);

  const size_type fp1 = head.find('(');
  const size_type fp2 = head.find(')',fp1);
  string f1str = head.substr(fp1+1,(fp2-fp1)-1);

  const size_type fp3 = head.find('(',fp2);
  const size_type fp4 = head.find(')',fp3);
  string f2str = head.substr(fp3+1,(fp4-fp3)-1);

  string left,exns,rght;
  if( lr-ll-1 > 0)
    left = head.substr(ll+1,lr-ll-1);
  assert(mr-ml-1> 0);
  exns = head.substr(ml+1,mr-ml-1);
  if( rr-rl-1 > 0)
    rght = head.substr(rl+1,rr-rl-1);
  list<ipair_t> raw_elst;
  ipair_t cbnd = getMinMax(exns);
  ipair_t lbnd(cbnd.first,cbnd.first);
  ipair_t rbnd(cbnd.second,cbnd.second);;
  if( left.size() > 0 ) {
    lbnd = getMinMax(left);
  } 
  if( rght.size() > 0 ) { 
    rbnd = getMinMax(rght);
  }
  ipair_t theBnd(lbnd.second,rbnd.first);

  size_type rght_pos = exns.size();
  size_type last = exns.find(',') + 1;
  while(last != string::npos) {
    size_type cursor = exns.find_first_of(",:",last);
    if(cursor == string::npos) {
      rght_pos = exns.size();
    } else {
      rght_pos = cursor;
    }
    string coords = exns.substr(last,rght_pos-last);
    size_type pos = coords.find('_');
    if( pos != string::npos ) {
      coords[pos] = ' ';
      std::istringstream istrm(coords);
      int end5,end3;
      if(istrm>>end5>>end3) {
	raw_elst.push_back(ipair_t(end5,end3));
      }
    }
    if(cursor == string::npos) {
      break;
    }
    last = cursor + 1;
  }
  if( cbnd.first-offSet > lbnd.second) {
    theBnd.first = cbnd.first-offSet;
  }
  if( cbnd.second+offSet < rbnd.first) {
    theBnd.second = cbnd.second+offSet;
  }
  list<ipair_t>::const_iterator iter  = raw_elst.begin();
  const list<ipair_t>::const_iterator stop  = raw_elst.end();
  const int beg = theBnd.first;
  const int end = theBnd.second;
  int relb = compRel(beg,strnd,beg,end);
  int rele = compRel(end,strnd,beg,end);
  if(relb>rele) {
    std::swap(relb,rele);
  }
  hmap_str_t hmap;
  hmap_unsigned_t cnt;
  list<ipair_t> rel_elst;
  for(; iter != stop; ++iter) {
    ipair_t ip = *iter;
    int rel5 = compRel(ip.first,strnd,beg,end);
    int rel3 = compRel(ip.second,strnd,beg,end);
    if(rel5>rel3) {
      std::swap(rel5,rel3);
    }
    bool passFrameTest=false;
    for(unsigned en=0; en < limitFrame.size(); ++en) {
      int sub=0;
      if( limitFrame[en] == 2 ) sub=1;
      else if( limitFrame[en] == 1 ) sub=2;
      const int clen = compCodingLen(rel5,rel3,start_codon,stop_codon) - sub;
      if( clen >= 3 ) {
	for(unsigned ti=0; ti< limitNextFrame.size(); ++ti) {
	  const int nframe = clen % 3;
	  if(limitNextFrame[ti] == nframe ) {
	    passFrameTest=true;
	    break;
	  }
	}
      }
    }	
    if( !passFrameTest ) {
      continue;
    }
    pair<string,string> pr = _klst.classType(ip.first,ip.second,strnd);
    assert(pr.first.length()>0);
    if( strnd == '-' && pr.second == "other" ) {
      rel5=0;
    }
    if( strnd == '-' && pr.first == "other" ) {
      rel3=rele;
    }
    if( strnd == '+' && pr.second == "other" ) {
      rel3=rele;
    }
    if( strnd == '+' && pr.first == "other" ) {
      rel5=0;
    }
    //cout<<"valid test exon: "<<rel5<<" "<<rel3<<" "<<pr.first<<" "<<pr.second<<endl;
    rel_elst.push_back(ipair_t(rel5,rel3));
  }
  unsigned size = rel_elst.size();
  if( size == 0 ) {
		cout<<"coding length is probably too small here, exit gracefully"<<endl;
		exit(0);
  }
  // for now pick exon at random
  //unsigned idx = rand() % size;
  unsigned idx = 0; // assumes headers preprocessed to generate random order (which they were for dmel)
  list<ipair_t>::const_iterator ti = rel_elst.begin();
  const list<ipair_t>::const_iterator ts = rel_elst.end();
  ipair_t save(0,0);
  for(unsigned i = 0; i <= idx; ++i, ++ti) {
    assert(ti != ts);
    save = *ti;
  }
  list<int> res;
  res.push_back(save.first);
  res.push_back(save.second);
  return res;
}

static int
compCodingLen(int lbnd, int rbnd, int startc, int stopc) {
  //cout<<"in exon: "<<lbnd<<" "<<rbnd<<" "<<startc<<" "<<stopc<<endl;
  if( stopc < rbnd ) {
    rbnd = stopc;
  }
  if( startc > lbnd ) {
    lbnd = startc;
  }
  const int len = (rbnd-lbnd)+1;
  //cout<<"proc exon: "<<lbnd<<" "<<rbnd<<" "<<len<<endl;
  return len;
}
