#ifndef _PHYLOMODEL_H
#define _PHYLOMODEL_H

#include "Phylogeny.h"
#include "SubModel.h"
#include "Model.h"
#include "MarkovChain.h"
#include "utils.h"
#include "MultiModel.h"
#include <vector>
#include <string>
#include <ext/hash_map>

class PhyloModel {
 public:
  typedef std::pair<double,bool> ppair2_t;
  typedef __gnu_cxx::hash_map<std::string, dhelp, strhash, eqstr> hash_t;
  typedef Phylogeny::tnode tnode;

 public:
  PhyloModel(const Phylogeny&, double, int);

  ~PhyloModel();

  double getProb(const std::vector<std::string*>&,int,int,const std::string&,MultiModel*,double=-1);

 private:
  ppair2_t
    calcProbHelp(tnode*,const std::string&, char anc, double, bool debug);
  ppair2_t
    calcProbHelp2(tnode*,const std::string&, char anc, double, bool debug);
  
  double
    calcProb(const std::vector<std::string*>& svec, int pos, int time, const std::string& column, MultiModel*, bool debug);

  const SubModel*
    buildModel(tnode* node) const;


 private:
  Phylogeny* _phy;
  const double _tnTvRatio; //(2.0)
  double _mutRate; 
  hash_t _cache;
  const std::vector<std::string*>* _tgSvec;
  int _tgPos, _tgTime;
  const MultiModel* _tgMultiModel;
};

#endif // _PHYLOMODEL_H
