#ifndef _PHYLOGENY_H
#define _PHYLOGENY_H

#include <string>
#include <vector>
#include <ext/hash_map>
#include <list>
#include "OrgIdx.h"
#include "utils.h"

class SubModel;
class MarkovChain;

/**
 * Defines a phylogenetic tree
 * with implicit parameter  T (topology)
 * and explicit paremeters B (set of branch lengths)
 */
class Phylogeny {
 public:
  typedef __gnu_cxx::hash_map<std::string, const SubModel*, strhash, eqstr> shash_t;

  Phylogeny(unsigned s=1) : _root(s,NULL) { }
  ~Phylogeny();
  /**
   * load tree from file
   */
  void read(const OrgIdx& oidx, const std::string&);
  void print(std::ostream&) const;
  //const OrgIdx& getOrgIdx() const { return _oidx; }

  /** felsenstein dynamic programming
   * algorithm.
   * @param - substitution model
   * @param - column from multiple alignment
   */
  double calcProb(const MarkovChain&, const SubModel&, const std::string&, bool = false) const;
  double
  calcProb2(const MarkovChain&, const SubModel&, const std::string&, bool) const;

  struct tnode {
    tnode(struct tnode* n, double d, const std::string& id = "*") : 
      _blen(d), _parent(n), _left(NULL), _rght(NULL), _id(id), _idx(-1), _subHash(NULL) { }
	 ~tnode(); 
    double _blen;
    struct tnode* _parent, *_left, *_rght;
    std::string _id;
    int _idx;
    std::list<int> _dlst;
    shash_t* _subHash;
  };
  Phylogeny* copy() const;
  std::list<int> storeDescendants(tnode*);

  tnode* getRoot(int idx) { return _root[idx]; }

 private:
  tnode*
    buildTree(const OrgIdx&, tnode*, const char*) const;
  void del();
  unsigned str2idx(const std::string&) const;
 private:
  //OrgIdx _oidx;
  // defined to be binary tree
  // each element contains the branch length
  // to the ancestor with a 0 at the root.
  std::vector<tnode*> _root;
};

#endif // _PHYLOGENY_H
