#include "RateManager.h"
#include "stype.h"
#include "Trellis.h"
#include <iostream>
#include <fstream>

using std::ifstream;
using std::ofstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::vector;

static RateHmm*
initRates(ModelType::cname_t cn) {
  const int time = 0;
  RateHmm* retHmm = NULL;
  if( ModelType::isPrcExon(cn) ) {
    const unsigned num_states = 2;
    retHmm = new RateHmm(num_states,cn);
    double rate = 0.7;
    int idx = 0;
    RateState* retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);
    rate = 1.7;
    idx = 1;
    retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);
  } else if( ModelType::isIntron(cn) || ModelType::isNpcExon(cn) ) {
    const unsigned num_states = 2;
    retHmm = new RateHmm(num_states,cn);
    double rate = 1.0;
    int idx = 0;
    RateState* retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);

    rate = 1.7;
    idx = 1;
    retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);
  } else if( ModelType::isSplice(cn) ) {
    const unsigned num_states = 1;
    retHmm = new RateHmm(num_states,cn);
    double rate = 0.9;
    int idx = 0;
    RateState* retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);
  }
  assert(retHmm);
  return retHmm;
}

static RateHmm*
initChange(ModelType::cname_t cn) {
  const int time = 0;
  RateHmm* retHmm = NULL;
  if( ModelType::isPrcExon(cn) ) {
    const unsigned num_states = 2;
    retHmm = new RateHmm(num_states,cn);
    double rate = 0.1;
    int idx = 0;
    RateState* retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);
    rate = 0.1;
    idx = 1;
    retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);
  } else if( ModelType::isIntron(cn) || ModelType::isNpcExon(cn) ) {
    const unsigned num_states = 2;
    retHmm = new RateHmm(num_states,cn);
    double rate = 0.1;
    int idx = 0;
    RateState* retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);

    rate = 0.1;
    idx = 1;
    retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);
  } else if( ModelType::isSplice(cn) ) {
    const unsigned num_states = 1;
    retHmm = new RateHmm(num_states,cn);
    double rate = 0.1;
    int idx = 0;
    RateState* retState = new RateState(idx,time,rate);
    retHmm->setState(idx,retState);
  }
  assert(retHmm);
  return retHmm;
}

RateManager::RateManager() {
  for(unsigned i = 0; i < (unsigned)ModelType::eCnameEnd; ++i) {
    cname_t id = (cname_t)i;
    _hash[id] = new RateHmm(id);
  }
}

RateManager::RateManager(const RateManager& cm) {
  for(unsigned i = 0; i < (unsigned)ModelType::eCnameEnd; ++i) {
    cname_t id = (cname_t)i;
    const RateHmm* rh = cm.getRateHmm(id); 
    _hash[id] = new RateHmm(*rh);
  }
}

#if 0
RateManager&
RateManager::operator=(const RateManager& cm) {
  for(unsigned i = 0; i < (unsigned)ModelType::eCnameEnd; ++i) {
    cname_t id = (cname_t)i;
    const RateHmm* rh = cm.getRateHmm(id); 
    *(_hash[id]) = *rh;
  }
  return *this;
}
#endif

RateManager::~RateManager() {
  RateHmmHashT::iterator citer = _hash.begin();
  const RateHmmHashT::iterator cstop = _hash.end();
  for(; citer != cstop; ++citer) {
    RateHmm* md = (*citer).second;
    delete md;
  }
}

void 
RateManager::print(std::ostream& os) const {
  RateHmmHashT::const_iterator citer = _hash.begin();
  const RateHmmHashT::const_iterator cstop = _hash.end();
  for(; citer != cstop; ++citer) {
    cname_t id = (*citer).first;
    const RateHmm* md = (*citer).second;
    os<<"RateManager: "<<ModelType::cname2str(id)<<endl;
    md->print(os); os<<endl;
  }
}

bool
RateManager::read(const string& file) {
  ifstream ifs(file.c_str()); 
  if( !ifs ) {
    cerr<<"Could not open ["<<file<<"]"<<endl;
    return false;
  }
  char buff[512];
  unsigned cnt =0;
  while(ifs.getline(buff,512) ) {
    string id = string(buff);
    cname_t cn = ModelType::str2cname(id);
    assert(cn != ModelType::eCnameEnd);
    RateHmm* look = _hash[cn];
    look->read(ifs);
    ++cnt;
  }
  return cnt==0 ? false : true;
}

RateHmm*
RateManager::getRateHmm(cname_t cn) {
  assert(cn != ModelType::eCnameEnd); 
  cname_t rn = cn;
  if( cn== ModelType::eCintronUp ) {
    rn = ModelType::eCintronDn;
  }
  else if( cn== ModelType::eAintronUp ) {
    rn = ModelType::eAintronDn;
  } 
  return _hash[rn];
}

const RateHmm*
RateManager::getRateHmm(cname_t cn) const {
  assert(cn != ModelType::eCnameEnd); 
  cname_t rn = cn;
  if( cn== ModelType::eCintronUp ) {
    rn = ModelType::eCintronDn;
  }
  else if( cn== ModelType::eAintronUp ) {
    rn = ModelType::eAintronDn;
  } 
  RateHmmHashT::const_iterator match = _hash.find(rn);
  assert(match != _hash.end());
  return (*match).second;
}

void
RateManager::create() {
  for(unsigned i = 0; i < (unsigned)ModelType::eCnameEnd; ++i) {
    const cname_t cn = (cname_t)i;
    _hash[cn] = initRates(cn);
  }
}

void
RateManager::create1() {
  for(unsigned i = 0; i < (unsigned)ModelType::eCnameEnd; ++i) {
    const cname_t cn = (cname_t)i;
    _hash[cn] = initChange(cn);
  }
}


void
RateManager::write(const string& file) const {
  ofstream ofs(file.c_str()); 
  if( !ofs ) {
    cerr<<"Could not open ["<<file<<":"<<endl;
    assert(0);
  }
  RateHmmHashT::const_iterator citer = _hash.begin();
  const RateHmmHashT::const_iterator cstop = _hash.end();
  for(; citer != cstop; ++citer) {
    cname_t id = (*citer).first;
    if( id == ModelType::eCnameEnd ) {
      // need to find out where this is coming from should not be ehere\n";
      continue;
    }
    const RateHmm* md = (*citer).second;
    assert(md);
    ofs<<ModelType::cname2str(id)<<endl;
    md->write(ofs);
    ofs<<endl;
  }
}


static void 
updateRateManager(ModelType::cname_t, RateManager& best_rm, TestModel& best_tm, const RateManager& curr_rm, const TestModel* curr_tm) {
  const double best_score = best_tm.calcFscore();
  assert(curr_tm);
  const double curr_score = curr_tm->calcFscore();
  if( curr_score > best_score ) {
    for(unsigned typ = 0; typ < ModelType::eCnameEnd; ++typ) {
      ModelType::cname_t st = (ModelType::cname_t)typ;
      cout<<"updateRates: "<<ModelType::cname2str(st)<<" found better val: "<<curr_score<<" old="<<best_score<<endl;
      RateHmm* best_rhmm = best_rm.getRateHmm(st);
      const RateHmm* curr_rhmm = curr_rm.getRateHmm(st);
      const unsigned num_states = best_rhmm->getNumStates();
      assert(num_states==curr_rhmm->getNumStates());
      for(unsigned iter = 0; iter < num_states; ++iter) {
	RateState& best_state = best_rhmm->getState(iter);
	const RateState& curr_state = curr_rhmm->getState(iter);
	const double curr_rate = curr_state.getRate();
	cout<<"new rate for state "<<iter<<" is "<<curr_rate<<" old rate = "<<best_state.getRate()<<endl;
	best_state.setRate(curr_rate);
      }
    }
    best_tm = *curr_tm;
  }
}

void 
RateManager::updateRates(const RateManager& curr_rm ) {
  RateManager& best_rm = *this;
  for(unsigned typ = 0; typ < ModelType::eCnameEnd; ++typ) {
    ModelType::cname_t st = (ModelType::cname_t)typ;
    cout<<"updateRates: "<<ModelType::cname2str(st)<<" initializing "<<endl;
    RateHmm* best_rhmm = best_rm.getRateHmm(st);
    const RateHmm* curr_rhmm = curr_rm.getRateHmm(st);
    const unsigned num_states = best_rhmm->getNumStates();
    assert(num_states==curr_rhmm->getNumStates());
    for(unsigned iter = 0; iter < num_states; ++iter) {
      RateState& best_state = best_rhmm->getState(iter);
      const RateState& curr_state = curr_rhmm->getState(iter);
      const double curr_rate = curr_state.getRate();
      cout<<"new rate for state "<<iter<<" is "<<curr_rate<<" old rate = "<<best_state.getRate()<<endl;
      best_state.setRate(curr_rate);
    }
  }
}

static void 
setInitRatesHelp(ModelType::cname_t st, RateManager& retRm, const double dcRANGE) {
  RateHmm * val = retRm.getRateHmm(st);
  const unsigned num_states = val->getNumStates();
  assert(num_states==val->getNumStates());
  double sVal = 0;
  vector<dpair_t> max(num_states,dpair_t(0,0));
  if( ModelType::isDon(st) || ModelType::isAcc(st) ) {
    max[0].first = 0.5;
    max[0].second = 1.1;
    assert(num_states==1);
  } else if( ModelType::isPrcExon(st) ) {
    max[0].first = 0.5;
    max[0].second = 1.1;
    max[1].first = 1.2;
    max[1].second = 2.1;
  } else {
    max[0].first = 1.0;
    max[0].second = 1.5;
    max[1].first = 1.6;
    max[1].second = 2.1;
  }
  for(unsigned iter = 0; iter < num_states; ++iter) {
    if( iter == 1 ) {
      max[iter].first=sVal+dcRANGE+0.1;
      max[iter].second=max[iter].first+dcRANGE;
    }
    const double range = (max[iter].second-max[iter].first)*10.0;
    assert(range>=0);
    const double rval = (double)(rand() % (int)range)*0.1;
    const double nVal = max[iter].first+rval;
    RateState& s0 = val->getState(iter);
    cout<<"set random range: "<<max[iter].first<<" to "<<max[iter].second<<endl;
    cout<<"Set Random Rates: "<<ModelType::cname2str(st)<<" state: "<<iter<<" nval: "<<nVal<<" old val= "<<s0.getRate()<<endl;
    s0.setRate(nVal);
    if( iter == 0 ) {
      sVal = nVal;
    }
  }
  if( num_states ==2 ) {
    assert(val->getState(0).getRate() <= val->getState(1).getRate());
  }
}

void
RateManager::setInitRates() {
  double dcRANGE = 0.5; // based on cRANGE;
  unsigned cnt = 0;
  vector<int> rvec(10,0);
  for(unsigned i = 0; i < rvec.size(); ++i) {
    //rvec[i] = rand() % 100;
    rvec[i] = 100;
    //if( rvec[i] >= 50 ) {
      ++cnt;
    //}
  }
  // assure at least one parameter is adjusted
  if( cnt==0) {
    int rval =rand() % rvec.size();
    rvec[rval] = 100;
  }
  if(rvec[0] >= 50 ) 
    setInitRatesHelp(ModelType::eAdon,*this,dcRANGE);
  if(rvec[1] >= 50 ) 
    setInitRatesHelp(ModelType::eCdon,*this,dcRANGE);
  if(rvec[2] >= 50 ) 
    setInitRatesHelp(ModelType::eAacc,*this,dcRANGE);
  if(rvec[3] >= 50 ) 
    setInitRatesHelp(ModelType::eCacc,*this,dcRANGE);
  if(rvec[4] >= 50 ) 
    setInitRatesHelp(ModelType::eCexonPrc,*this,dcRANGE);
  if(rvec[5] >= 50 ) 
    setInitRatesHelp(ModelType::eAexonPrc,*this,dcRANGE);
  if(rvec[6] >= 50 ) 
    setInitRatesHelp(ModelType::eAexonNpc,*this,dcRANGE);
  if(rvec[7] >= 50 ) 
    setInitRatesHelp(ModelType::eCexonNpc,*this,dcRANGE);
  if(rvec[8] >= 50 ) 
    setInitRatesHelp(ModelType::eAintronDn,*this,dcRANGE);
  if(rvec[9] >= 50 ) 
    setInitRatesHelp(ModelType::eCintronDn,*this,dcRANGE);
  //return retRm;
}

void
RateManager::updateRates(TestModel& best_tm, const RateManager& curr, TestModel* curr_tm) {
  updateRateManager(ModelType::eAdon,*this,best_tm,curr,curr_tm);
}

static bool 
adjustRatesHelp(ModelType::cname_t st, RateManager& retRm, const RateManager& cr, unsigned iter, int mag) {
  const RateHmm * chm = cr.getRateHmm(st);
  const unsigned num_states = chm->getNumStates();
  RateHmm * val = retRm.getRateHmm(st);
  assert(num_states==val->getNumStates());
  const RateState& c0 = chm->getState(iter);
  RateState& s0 = val->getState(iter);
  const double nVal = s0.getRate()+((double)mag*c0.getRate());
  //cout<<"Tweak: "<<ModelType::cname2str(st)<<" change rate: "<<cval<<" state: "<<iter<<" nval: "<<nVal<<" old val= "<<s0.getRate()<<endl;
  cout<<"Tweak: "<<ModelType::cname2str(st)<<" state: "<<iter<<" nval: "<<nVal<<" old val= "<<s0.getRate()<<endl;
  assert(nVal>=0 && nVal<10);
  s0.setRate(nVal);
  bool fail = false;
  if( num_states ==2 ) {
    fail = !(val->getState(0).getRate() <= val->getState(1).getRate());
  }
  return fail;
}

void 
resetVal(ModelType::cname_t type, RateManager& set, const RateManager& best, unsigned idx) {
  const RateHmm * chm = best.getRateHmm(type);
  //const unsigned num_states = chm->getNumStates();
  RateHmm* val = set.getRateHmm(type);
  const RateState& c0 = chm->getState(idx);
  RateState& s0 = val->getState(idx);
  const double nVal = c0.getRate();
  s0.setRate(nVal);
}

void
RateManager::adjustRates(RateManager& rm, const RateManager& brm, tp2_t& pr, const int cRANGE) {
  //RateManager* retRm = new RateManager(rm);
  int& counter = pr.first.second;
  ModelType::cname_t type = pr.first.first;
  // assure at least one parameter is adjusted
  int idx = pr.second;
  cout<<"Adjust: "<<ModelType::cname2str(type)<<" idx: "<<idx<<" counter: "<<counter<<endl;
  const bool fail = adjustRatesHelp(type,rm,*this,idx,counter);
  --counter;
  if( fail ) {
    counter = 0;
  }
}
