#include "RateState.h"
#include "RateTrans.h"

using std::list;
using std::ostream;
using std::istream;
using std::endl;
using std::cout;

RateState::RateState(int id, int phase, double rate) : _id(id), _phase(phase), _rate(rate),
						       _initProb(0), _cnt1(0) {
}

RateState::RateState(const RateState& rs) : _id(rs._id), _phase(rs._phase), _rate(rs._rate), _initProb(rs._initProb), _cnt1(rs._cnt1) {
}


bool 
RateState::operator==(const HmmComponent& hc) const {
  bool rval=false;
  const RateState* rs = dynamic_cast<const RateState*>(&hc);
  if(rs) {
    rval = (rs->getId()==getId());
  }
  return rval;
}

void
RateState::transferUpdate(HmmComponent& hc) {
  RateState* rs = dynamic_cast<RateState*>(&hc);
  assert(rs);
  list<double>::const_iterator iter = rs->_initCnt.begin();
  const list<double>::const_iterator stop = rs->_initCnt.end();
  for(; iter != stop; ++iter) {
    _initCnt.push_back(*iter);
  }
  rs->_initCnt.clear();
}

void
RateState::update1(double val) {
  _cnt1+= val;
}

void
RateState::update(double val) {
  _initCnt.push_back(val);
}

double 
RateState::getCnt() const {
  double sum = 0;
  list<double>::const_iterator iter = _initCnt.begin();
  const list<double>::const_iterator stop = _initCnt.end();
  for(; iter != stop; ++iter) {
    sum += *iter;
  }
  const unsigned size = _initCnt.size();
  return size > 0 ? sum/(double)size : 0;
}

double 
RateState::getCntSum() const {
  double sum = 0;
  list<double>::const_iterator iter = _initCnt.begin();
  const list<double>::const_iterator stop = _initCnt.end();
  for(; iter != stop; ++iter) {
    sum += *iter;
  }
  return sum;
}

void
RateState::write(ostream& os) const {
  os<<_id<<" "<<_phase<<" "<<_rate<<" "<<_initProb<<" "<<endl;
  //cout<<"cnt: "<<_cnt1<<endl;
  //list<double>::const_iterator iter = _initCnt.begin();
  //const list<double>::const_iterator stop = _initCnt.end();
  //for(; iter != stop; ++iter) {
    //cout<<*iter<<endl;
  //}
#if 0
  list<RateTrans*>::const_iterator iter = _inLst.begin();
  const list<RateTrans*>::const_iterator stop = _inLst.end();
  for(; iter != stop; ++iter) {
    const RateTrans* rt = *iter;
    os<<rt->getLeftId()<<" "<<rt->getRghtId()<<endl;
  }
  os<<endl;
  list<RateTrans*>::const_iterator iter = _outLst.begin();
  const list<RateTrans*>::const_iterator stop = _outLst.end();
  for(; iter != stop; ++iter) {
    const RateTrans* rt = *iter;
    os<<rt->getLeftId()<<" "<<rt->getRghtId()<<endl;
  }
  os<<endl;
#endif
}

void
RateState::read(istream& is) {
  is>>_id>>_phase>>_rate>>_initProb;
}
