#include "RateTrans.h"
#include "RateState.h"

using std::list;
using std::cout;
using std::istream;
using std::ostream;
using std::endl;

RateTrans::RateTrans(RateState& from, RateState& to) : _from(from), _to(to), _prob(0), _cnt1(0) { 
}

RateTrans::RateTrans(RateState& from, RateState& to, double prob, double cnt) : _from(from), _to(to), _prob(prob), _cnt1(cnt) { 
}

bool 
RateTrans::operator==(const HmmComponent& hc) const {
  bool rval=false;
  const RateTrans* rt = dynamic_cast<const RateTrans*>(&hc);
  if(rt) {
    rval = ((rt->getFr().getId()==_from.getId()) &&
	    (rt->getTo().getId()==_to.getId()));
  }
  return rval;
}

void
RateTrans::transferUpdate(HmmComponent& hc) {
  RateTrans* rs = dynamic_cast<RateTrans*>(&hc);
  assert(rs);
  list<double>::const_iterator iter = rs->_cnt.begin();
  const list<double>::const_iterator stop = rs->_cnt.end();
  for(; iter != stop; ++iter) {
    _cnt.push_back(*iter);
  }
  rs->_cnt.clear();
}

void
RateTrans::update1(double val) {
  _cnt1 += val;
}

void
RateTrans::update(double val) {
  _cnt.push_back(val);
}

double
RateTrans::getRate() const {
  return _to.getRate(); 
}

int
RateTrans::getLeftId() const {
  return _from.getId();
}

int
RateTrans::getRghtId() const {
  return _to.getId();
}

double 
RateTrans::getCnt() const {
  double sum = 0;
  list<double>::const_iterator iter = _cnt.begin();
  const list<double>::const_iterator stop = _cnt.end();
  for(; iter != stop; ++iter) {
    sum += *iter;
  }
  const unsigned size = _cnt.size();
  return sum>0 ? sum/(double)size : 0;
}

double 
RateTrans::getCntSum() const {
  double sum = 0;
  list<double>::const_iterator iter = _cnt.begin();
  const list<double>::const_iterator stop = _cnt.end();
  for(; iter != stop; ++iter) {
    sum += *iter;
  }
  return sum;
}

void
RateTrans::write(ostream& os) const {
  //os<<"fr: "<<_from.getId()<<" to: "<<_to.getId()<<" p: "<<_prob<<" c: "<<_cnt<<endl;
  os<<_from.getId()<<" "<<_to.getId()<<" "<<_prob<<endl;
  //cout<<"cnt1: "<<_cnt1<<endl;
  //list<double>::const_iterator iter = _cnt.begin();
  //const list<double>::const_iterator stop = _cnt.end();
  //for(; iter != stop; ++iter) {
    //cout<<*iter<<endl;
  //}
}

void
RateTrans::read(istream& is, ipair_t& ip, double& prob, double& cnt) {
  is>>ip.first>>ip.second>>prob;
}
