#include "RunModel.h"
#include "Options.h"
#include "ParamFile.h"
#include "AnnotationInfo.h"
#include "SeqLabelVal.h"
#include "ParseHeader.h"
#include "LabelSeq.h"
#include "TestModel.h"
#include <string>
#include <algorithm>
#include <iostream>
#include <sstream>

using std::pair;
using std::ofstream;
using std::ostringstream;
using std::string;
using std::endl;
using std::cout;
using std::cerr;
using std::vector;
using std::list;

extern bool SPECIAL_DEBUG;
extern bool XDEBUGON;
extern bool GDEBUG;
extern bool DEBUGON;
const int cRANGE1=5; //5
const int cRANGE2=10; //10
const int cNUM_EX=50000; //50000

static void
countDiCons(const vector<string*>& svec, const std::list<SeqLabel>& slst, vector<int>& counter, vector<int>& total); 
static list<SeqLabel>
isAllDiCons(const vector<string*>& svec, const std::list<SeqLabel>& slst); 


static double
calcPcntId(const vector<string*>& svec, int start_codon, int stop_codon, const std::list<SeqLabel>& slst, ModelType::cname_t mmodel) {
  assert(svec.size()==2);
  const string& str = *svec[0];
  assert(svec[1]);
  const string& str2 = *svec[1];
  list<SeqLabel>::const_iterator ti = slst.begin();
  const list<SeqLabel>::const_iterator st = slst.end();
  const SeqLabel* lastPtr = NULL;
  const bool assumePrcOnly = true;
  double match = 0, total=0;
  for(; ti != st; ++ti) {
    const SeqLabel& sl = *ti;
    int spos=-1, rpos=-1;
    ModelType::cname_t cn = ModelType::eCnameEnd;
    if( lastPtr && !ModelType::isSplice(mmodel) ) {
      cn = ModelType::sl2cnameInv(*lastPtr,sl,assumePrcOnly);
      spos = lastPtr->getRght()+1;
      rpos = sl.getLeft();
    } else if( ModelType::isSplice(mmodel) ) {
      cn = ModelType::sl2cnamePos(sl);
      spos = sl.getLeft();
      rpos = sl.getRght()+1;
    }
    if( cn == mmodel ) {
      assert(svec.size()==2);
      assert(spos != -1 && rpos != -1);
      for(signed i = spos; i < rpos; ++i) {
	bool isValid=true;
	if( ModelType::isExon(cn) ) {
	  isValid = (i >= start_codon && i <= stop_codon);
	}
	if( isValid ) {
	  if( str[i-1]==str2[i-1] ) {
	    ++match;
	  }
	  ++total;
	}
      }
    }
    lastPtr = &sl;
  }
  return total>0 ? match/total : -1;
}

#if 0
void
RunModel::countPcntId(const fasta& aligned_targ, vector<vector<int> >& counter, vector<vector<int> >& total, const string& mheader) {
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  unsigned cnt = 0;
  int tot_exons = 0;
  for(;iter != stop; ++iter, ++cnt) {
    const fasta_seq& aligned_fs = **iter;
    if( !mheader.empty() ) {
      const string& header = aligned_fs.get_header();
      if( mheader != header ) {
	continue;
      }
    }
    const fasta_seq& targ_fs = aligned_fs;
    if(targ_fs.get_seq().length() > 10000) {
      //cout<<"too damn long skipping for debug phase: "<<targ_fs.get_header()<<" "<<targ_fs.get_seq().length()<<endl;
      continue;
    }
    ParseHeader ph(ai);
    bool status = ph.parse(targ_fs);
    if( !status ) {
      cout<<"skiping????"<<targ_fs.get_header()<<endl;
      continue;
    }
    ExonDescrip& ed = ph.getExonDescrip();
    bool isMultOrf=ed.isPrcAlt();
    //ed.print(); cout<<endl;
    ipair_t ip = ed.getStartStop();
    list<SeqLabel> klst = ed.getSeqLabelLst();
    const vector<string*> svec = getMatchSeq(targ_fs);
    const int start_codon = ip.first;
    const int stop_codon = ip.second; //3rd position of last codon
    if( svec[1] && svec.size()==2 ) {
      cout<<"start: "<<start_codon<<" stop: "<<stop_codon<<endl;
      ed.printAlignment(svec);
      bool isM3=SeqLabel::checkThree(klst,start_codon,stop_codon);
      bool isPrc=SeqLabel::isAllPrc(klst,start_codon,stop_codon);
      list<SeqLabel> missing;
      bool fail = !missing.empty();
      if( fail ) {
	double pcntId = calcPcntId(*(svec[0]),*(svec[1]),1,svec[0]->size());
	cout<<"failed: "<<targ_fs.get_header()<<" pcntId "<<pcntId<<" pres="<<(isM3?1:0)<<" prc="<<(isPrc?1:0)<<" oneOrf="<<(!isMultOrf?1:0)<<" ";
      } else {
	double pcntId = calcPcntId(*(svec[0]),*(svec[1]),1,svec[0]->size());
	cout<<"pass: "<<targ_fs.get_header()<<" pcntId "<<pcntId<<" pres="<<(isM3?1:0)<<" prc="<<(isPrc?1:0)<<" oneOrf="<<(!isMultOrf?1:0);
      }
      list<SeqLabel>::const_iterator it = missing.begin();
      const list<SeqLabel>::const_iterator st = missing.end();
      for(; it != st; ++it) {
	cout<<*it<<" ";
      }
      cout<<endl;
      ++tot_exons;
    }
    const bool isGcDon = hasGcDon(*(svec[0]),klst);
    cout<<"check 2: "<<targ_fs.get_header()<<" hasGc="<<(isGcDon?1:0)<<endl;
				  
  }
  cout<<"total matching exons this round: "<<tot_exons<<endl;
}
#endif

static void 
printHelp(const vector<string*>& svec, int start_codon, int stop_codon, const list<SeqLabel>& klst, ModelType::cname_t cn) {
  double pcntId = calcPcntId(svec,start_codon,stop_codon,klst,cn);
  cout<<" "<<ModelType::cname2str(cn)<<" "<<(pcntId*100);
}

void
RunModel::countPcntId1(const fasta& aligned_targ, const string& mheader, vector<int>& counter, vector<int>& total) {
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  unsigned cnt = 0;
  for(;iter != stop; ++iter, ++cnt) {
    const fasta_seq& aligned_fs = **iter;
    if( !mheader.empty() ) {
      const string& header = aligned_fs.get_header();
      if( mheader != header ) {
	continue;
      }
    }
    const fasta_seq& targ_fs = aligned_fs;
    if(targ_fs.get_seq().length() > 10000) {
      //cout<<"too damn long skipping for debug phase: "<<targ_fs.get_header()<<" "<<targ_fs.get_seq().length()<<endl;
      continue;
    }
    ParseHeader ph(ai);
    bool status = ph.parse(targ_fs);
    if( !status ) {
      cout<<"skiping????"<<targ_fs.get_header()<<endl;
      continue;
    }
    //cout<<targ_fs.get_header()<<endl;
    ExonDescrip& ed = ph.getExonDescrip();
    bool isMultOrf=ed.isPrcAlt();
    list<SeqLabel> klst = ed.getSeqLabelLst();
    const vector<string*> svec = getMatchSeq(targ_fs);
    ipair_t ip = ed.getStartStop();
    const int start_codon = ip.first;
    const int stop_codon = ip.second; //3rd position of last codon
    if( svec[1] && svec.size()==2 ) {
      cout<<"start: "<<start_codon<<" stop: "<<stop_codon<<endl;
      ed.printAlignment(cout,svec);
      bool isM3=SeqLabel::checkThree(klst,start_codon,stop_codon);
      bool isPrc=SeqLabel::isAllPrc(klst,start_codon,stop_codon);
      const double allPcntId = calcPcntId(*(svec[0]),*(svec[1]),1,svec[0]->size());
      const list<SeqLabel> missing = isAllDiCons(svec,klst);
      const bool allCons = missing.empty();
      cout<<"res: "<<targ_fs.get_header()<<" pcntid="<<allPcntId<<" allc="<<(allCons?1:0)<<" pres="<<(isM3?1:0)<<" prc="<<(isPrc?1:0)<<" oneOrf="<<(!isMultOrf?1:0);
      printHelp(svec,start_codon,stop_codon,klst,ModelType::eAexonPrc);
      printHelp(svec,start_codon,stop_codon,klst,ModelType::eCexonPrc);
      printHelp(svec,start_codon,stop_codon,klst,ModelType::eAintronDn);
      printHelp(svec,start_codon,stop_codon,klst,ModelType::eCintronDn);
      printHelp(svec,start_codon,stop_codon,klst,ModelType::eCdon);
      printHelp(svec,start_codon,stop_codon,klst,ModelType::eCacc);
      printHelp(svec,start_codon,stop_codon,klst,ModelType::eAdon);
      printHelp(svec,start_codon,stop_codon,klst,ModelType::eAacc);
      double numer = -1, denom = -1;
      countDiCons(svec,klst,counter,total);
      const double pcntId = calcPcntId(svec,start_codon,stop_codon,klst,ModelType::eAexonPrc);
      const double pcntId1 = calcPcntId(svec,start_codon,stop_codon,klst,ModelType::eAintronDn);
      const double pcntId2 = calcPcntId(svec,start_codon,stop_codon,klst,ModelType::eCexonPrc);
      const double pcntId3 = calcPcntId(svec,start_codon,stop_codon,klst,ModelType::eCintronDn);
      if( pcntId != -1 ) {
	numer = pcntId;
	denom = pcntId1==-1 ? pcntId3 : pcntId1;
      }
      if( pcntId2 != -1 ) {
	numer = pcntId2;
	denom = pcntId3;
      }
      const double dval = (numer==-1||denom==-1 ? -1 : (numer/denom));
      cout<<" eiratio: "<<(dval*100.0);

      list<SeqLabel>::const_iterator li = missing.begin();
      const list<SeqLabel>::const_iterator ls = missing.end();
      for(; li != ls; ++li) {
	const SeqLabel& sl = *li;
	cout<<" "<<sl;
      }
      cout<<endl;
    }
    ParamFile* pf = ParamFile::getInstance();
    const OrgIdx& oidx = pf->getOrgIdx();
    const bool isNon = hasNonConsensusSplice(*(svec[0]),klst);
    //cout<<"check 2: "<<targ_fs.get_header()<<" hasGc="<<(isGcDon?1:0)<<endl;
    cout<<"check 3: "<<targ_fs.get_header()<<" hasNon="<<(isNon?1:0);
    for(unsigned x = 1; x < svec.size(); ++x) {
      if( svec[x] ) {
	const string org = oidx.idx2str(x);
      const double allPcntId = calcPcntId(*(svec[0]),*(svec[x]),1,svec[0]->size());
		cout<<" "<<org<<" "<<allPcntId;
      }
    }
    cout<<endl;
  }
}

list<GffFeature*>*
RunModel::run(const fasta& aligned_targ, const string& mheader, bool runCons, pred_t predType, bool doLimitFrame, bool useExon, const string& pid) {
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  unsigned cnt = 0;
  list<GffFeature*>* finalLst = new list<GffFeature*>;
  std::ostream* osPtr = Options::getInstance()->_logStream;
  for(;iter != stop; ++iter, ++cnt) {
    const fasta_seq& aligned_fs = **iter;
    const string& header = aligned_fs.get_header();
    if( !mheader.empty() ) {
      if( mheader != header ) {
	continue;
      }
    }
    const fasta_seq& targ_fs = aligned_fs;
    vector<string*> svec = getMatchSeq(targ_fs);
    if( osPtr ) (*osPtr)<<targ_fs.get_header()<<endl;
    ParseHeader ph(ai);
    bool status = ph.parse(targ_fs);
    if( !status ) {
      if( osPtr ) (*osPtr)<<"failed: "<<targ_fs.get_header()<<endl;
      continue;
    }
    unsigned morgs=0;
    for(unsigned i = 1; i < svec.size(); ++i) {
      if( svec[i] ) {
			double pcntId = calcPcntId(*(svec[0]),*(svec[i]),1,svec[0]->size());
			if( osPtr ) (*osPtr)<<"pcnt id with org "<<i<<" "<<pcntId<<endl;
			++morgs;
      }
    }
    bool isInitExon =false, isLastExon=false;
    ExonDescrip& ed = ph.getExonDescrip();
    if( osPtr ) {
       ed.print(*osPtr);
		 (*osPtr)<<endl;
	 }
    ipair_t ip = ed.getStartStop();
    list<SeqLabel> klst = ed.getSeqLabelLst();
    vector<int> limitFrame;
    vector<int> limitNextFrame;
    if(doLimitFrame) {
      const vector<int>& lm = ed.getStartFrame();
      limitFrame.resize(lm.size());
      for(unsigned k = 0; k < limitFrame.size(); ++k) {
	limitFrame[k] = lm[k];
      }
      const vector<int>& lm2 = ed.getNextFrame();
      limitNextFrame.resize(lm2.size());
      for(unsigned k = 0; k < limitNextFrame.size(); ++k) {
	limitNextFrame[k] = lm2[k];
      }
      isInitExon = ph.getIsInitExon();
      isLastExon = ph.getIsLastExon();
    }
    const int start_codon = ip.first;
    const int stop_codon = ip.second; //3rd position of last codon
    const bool isPrc=SeqLabel::isAllPrc(klst,start_codon,stop_codon);
    const bool multiplePrc=ed.isPrcAlt();
    if( !isPrc || multiplePrc ) 
      continue;
    vector<int> incl;
    if( useExon ) {
      list<int> lst = ph.getExon(targ_fs,limitFrame,limitNextFrame,start_codon,stop_codon);
      incl.resize(2);
      assert(lst.size()==2);
      incl[0]=lst.front();
      if( incl[0]==1 ) {
	incl[0]=0;
      }
      incl[1]=lst.back();
      if( osPtr ) (*osPtr)<<"input exon: "<<incl[0]<<" "<<incl[1]<<endl;
    }
    if( osPtr ) SeqLabel::printLst(*osPtr,klst);
    if( !mheader.empty() && osPtr ) {
      ed.printAlignment(*osPtr,svec);
    }
    if( osPtr ) (*osPtr)<<"start/stop: "<<start_codon<<" "<<stop_codon<<endl;
    const LabelSeq ls(_mm);
    const bool isGcDon = hasGcDon(*(svec[0]),klst);
    if(isGcDon) {
      if( osPtr ) (*osPtr)<<"THIS SEQ has a GC donor\n";
    }
    int scoreFrame=-1;
    const spair_t result = ls.scoreF(svec,start_codon,stop_codon,klst,scoreFrame,predType);
    if( !result.first ) {
      if( osPtr ) (*osPtr)<<"Unable to score\n";
      if( mheader.empty() && osPtr ) ed.printAlignment(*osPtr,svec);
    }
    if( osPtr ) (*osPtr)<<"true score: "<<result<<endl;
    assert(klst.front().getLabel()==SeqLabel::eBeg);
    const int start_pos = klst.front().getLeft();
    assert(klst.back().getLabel()==SeqLabel::eEnd);
    const int stop_pos = klst.back().getLeft();
    LabelSeq ls1(_mm);
    int frame=-1;
    const list<SeqLabelVal>* pred = ls1.labelF(svec,start_codon,stop_codon,start_pos,stop_pos,frame,runCons,predType,limitFrame, limitNextFrame,incl,isInitExon,isLastExon);
	 if(osPtr) {
       (*osPtr)<<"We have a Prediction, please let it not be utter crap! "<<frame<<endl;
       SeqLabelVal::printLst(*osPtr,*pred);
    }
    list<GffFeature*>* glst = collectExons(svec,*pred,frame,start_codon,stop_codon,predType,header,pid,ed);
    list<GffFeature*>::const_iterator ti = glst->begin();
    const list<GffFeature*>::const_iterator si = glst->end();
    for(; ti != si; ++ti) {
      finalLst->push_back(*ti);
    }
    delete glst;
    delete pred;
  }
  if(osPtr) (*osPtr)<<"TOTAL:"<<endl;
  return finalLst;
}

list<GffFeature*>*
RunModel::collectExons(const vector<string*>& svec, const list<SeqLabelVal>& lst, const int scoreFrame, int start_codon, int stop_codon, pred_t predType, const string& header, const string& pid, const ExonDescrip& ed) {
  list<GffFeature*>* gffLst = new list<GffFeature*>;
  const bool isSize = lst.size()==2;
  std::ostream* osPtr = Options::getInstance()->_logStream;
  list<SeqLabelVal>::const_iterator it1=lst.begin();
  list<SeqLabelVal>::const_iterator it2=lst.begin();
  const list<SeqLabelVal>::const_iterator stop=lst.end();
  bool irFlag=false;
  for(; it1 != stop; ++it1) {
    it2 = it1;
    ++it2;
    for(; it2 != stop; ++it2) {
      const SeqLabelVal& left = *it1;
      const SeqLabelVal& rght = *it2;
      string type = "unclassiffied";
      // order is important here
      if( left.getLabel()==SeqLabel::eBeg && rght.getLabel()==SeqLabel::eEnd && isSize ) { 
	type = "intron";
      } 
      else if( left.getLabel()==SeqLabel::eBeg && rght.getLabel()==SeqLabel::eEnd1) {
	type = irFlag ? "exon-single-ir" : "exon-single";
      }
      else if( left.getLabel()==SeqLabel::eBeg && SeqLabel::isDon(rght.getLabel()) ) {
	list<SeqLabelVal>::const_iterator tmpi=lst.begin();
	bool foundAcc=false;
	for( ; tmpi != it2; ++tmpi ) {
	  if( SeqLabel::isAcc( (*tmpi).getLabel()) && !SeqLabel::isAccIr( (*tmpi).getLabel()) ) {
	    foundAcc=true;
	  }
	}
	if( !foundAcc  ) {
	  if( SeqLabel::isMs(rght.getLabel()) ) {
	    type = "exon-ms";
	  }
	  else if( SeqLabel::isDonIr(rght.getLabel()) || irFlag ) {
	    type = "exon-ir";
	    irFlag=true;
	  } 
	  else if( rght.getLabel() == SeqLabel::eNed ) {
	    type = "exon-cs";
	  }
	}
      }
      else if( SeqLabel::isAcc(left.getLabel()) && rght.getLabel()==SeqLabel::eEnd ) {
	list<SeqLabelVal>::const_iterator tmpi=it1;
	bool foundDon=false;
	for( ; tmpi != stop; ++tmpi ) {
	  if( SeqLabel::isDon( (*tmpi).getLabel()) && !SeqLabel::isDonIr( (*tmpi).getLabel()) ) {
	    foundDon=true;
	  }
	}
	if( !foundDon  ) {
	  if( SeqLabel::isMs(left.getLabel()) ) {
	    type = "exon-ms";
	  }
	  else if( SeqLabel::isAccIr(left.getLabel()) || irFlag ) {
	    type = "exon-ir";
	    irFlag=true;
	  } 
	  else if( left.getLabel() == SeqLabel::eNea) {
	    type = "exon-cs";
	  }
	}
      }
      else if( SeqLabel::isAcc(left.getLabel()) && SeqLabel::isDon(rght.getLabel())) {
	if( rght.getLabel()==SeqLabel::eCed ) {
	  type = "exon-ce";
	} else if( SeqLabel::isMs(rght.getLabel()) || SeqLabel::isMs(left.getLabel()) ) {
	  type = "exon-ms";
	} else if( SeqLabel::isDonIr(rght.getLabel()) || SeqLabel::isAccIr(left.getLabel()) || irFlag ) {
	  type = "exon-ir";
	  irFlag=true;
	} else if( rght.getLabel()==SeqLabel::eNed || left.getLabel()==SeqLabel::eNea) {
	  type = "exon-cs";
	}
      }
      if( type == "unclassiffied" ) {
	continue;
      }
      list<SeqLabel> klst;
      const SeqLabel::event_t event = rght.getEvent();
      const int end5 = left.getExonBnd();
      const int end3 = rght.getExonBnd();

      if( left.getLabel() != SeqLabel::eBeg ) {
	const int tb5 = left.getLeft()-1;
	const int tb3 = left.getLeft()-1;
	SeqLabel trick_beg(SeqLabel::eBeg,event,tb5,tb3);
	klst.push_back(trick_beg);
      }
      const SeqLabel& sl1 = left;
      klst.push_back(sl1);
      const SeqLabel& sl2 = rght;
      klst.push_back(sl2);
      if( !rght.isEnd() ) {
	const int te5 = rght.getRght()+1;
	const int te3 = rght.getRght()+1;
	SeqLabel trick_end(SeqLabel::eEnd,event,te5,te3);
	klst.push_back(trick_end);
      }
      LabelSeq ls(_mm);
      SPECIAL_DEBUG=true; //need to get rid of this ugly global
      const spair_t result = ls.scoreF(svec,start_codon,stop_codon,klst,scoreFrame,predType);
      if(!result.first) {
	if(osPtr) (*osPtr)<<"possible issue with predicted exon: "<<end5<<" "<<end3<<endl;
      }
      SPECIAL_DEBUG=false;
      GffFeature* gff = new GffFeature;
      const string sid = ed.getChr();
      gff->setStrId(sid);
      gff->setProgName(pid);
      gff->setFeatureName(type);
      const int aend5 = ed.compAbs(end5);
      gff->setEnd5(aend5);
      const char strnd = ed.getStrnd();
      gff->setStrnd(strnd);
      const int aend3 = ed.compAbs(end3);
      gff->setEnd3(aend3);
      gff->setFrame(scoreFrame);
      const double score = result.first ? result.second / ((end3-end5)+1) : 0;
      gff->setScore(score);
      //const string gid = ed.getId();
      gff->setComment(header);
      gffLst->push_back(gff);
    }
  }
  return gffLst;
}

#if 0
void
RunModel::discTrain(const fasta& aligned_targ, const string& mheader) {
  const pred_t predType = eDisc1HmmRates;
  double wght=0.1;
  list<const vector<string*>*> svecLst;
  list<fasta_seq*> fastaLst;
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  vector<int> debug_type(4,0);
  for(;iter != stop; ++iter) {
    bool stopRun=true;
    for(unsigned i=0; i < debug_type.size(); ++i) {
      if( debug_type[i] < cNUM_EX ) {
	stopRun=false;
	break;
      }
    }
    if( stopRun ) {
      break;
    }
    const fasta_seq& aligned_fs = **iter;
    if( !mheader.empty() ) {
      const string& header = aligned_fs.get_header();
      //cout<<"mheader: "<<mheader<<endl<<header<<endl;
      if( mheader != header ) {
	continue;
      }
    }
    const fasta_seq& targ_fs = aligned_fs;
    if(targ_fs.get_seq().length() > 800) {
      cout<<"too damn long skipping for debug phase: "<<targ_fs.get_header()<<" "<<targ_fs.get_seq().length()<<endl;
      continue;
    }
    size_t pos = targ_fs.get_header().find("_CS_");
    if(pos != string::npos) {
      if(debug_type[0]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[0];
      }
    }
    pos = targ_fs.get_header().find("_SE_");
    if(pos != string::npos) {
      if(debug_type[1]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[1];
      }
    }
    pos = targ_fs.get_header().find("_MS_");
    if(pos != string::npos) {
      if(debug_type[2]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[2];
      }
    }
    pos = targ_fs.get_header().find("_IR_");
    if(pos != string::npos) {
      if(debug_type[3]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[3];
      }
    }
    const vector<string*>* svecPtr = getMatchSeqPtr(targ_fs);
    svecLst.push_back(svecPtr);
    fastaLst.push_back(*iter);
  }
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  cout<<"discTrain size: "<<aligned_targ.size()<<endl;
  ParamSearchHashT* last_phash = NULL;
  ConsManager* last_cm = NULL;
  double last_diff = -1;
  double last_avg = -1;
  ParamSearchPair* last_param = NULL;
  ConsManager* next_cm = NULL;
  for(unsigned train_iter = 0; train_iter < 1000; ++train_iter) {
    unsigned cnt = 0, hit=0;
    cout<<"Training iteration: "<<train_iter<<endl;
    ostringstream os;
    os<<"Test iteration: "<<train_iter<<endl;
    TestModel tm(os.str());
    double sum_diff = 0, sum_score = 0;
    ParamSearchHashT* next_phash = new ParamSearchHashT;
    list<fasta_seq*>::const_iterator iter = fastaLst.begin();
    const list<fasta_seq*>::const_iterator stop = fastaLst.end();
    list<const vector<string*>*>::const_iterator si = svecLst.begin();
    for(;iter != stop; ++iter, ++si, ++cnt) {
      const vector<string*>& svec = **si;
      const fasta_seq& aligned_fs = **iter;
      const fasta_seq& targ_fs = aligned_fs;
      if( !mheader.empty() ) {
	const string& header = aligned_fs.get_header();
	//cout<<"mheader: "<<mheader<<endl<<header<<endl;
	if( mheader != header ) {
	  continue;
	}
      }
      ParseHeader ph(ai);
      bool status = ph.parse(targ_fs);
      if( !status ) {
	cout<<"skiping????"<<targ_fs.get_header()<<endl;
	continue;
      }
      cout<<targ_fs.get_header()<<endl;      
      ExonDescrip& ed = ph.getExonDescrip();
      ed.print();
      cout<<endl;
      ipair_t ip = ed.getStartStop();

      list<SeqLabel> klst = ed.getSeqLabelLst();
      SeqLabel::printLst(klst);
      if( !mheader.empty() ) {
	ed.printAlignment(svec);
      }
      const int start_codon = ip.first;
      const int stop_codon = ip.second; //3rd position of last codon

      cout<<"start/stop: "<<start_codon<<" "<<stop_codon<<endl;
      const LabelSeq ls(_mm);
      const bool isGcDon = hasGcDon(*(svec[0]),klst);
      if(isGcDon) {
	cout<<"THIS SEQ has a GC donor\n";
      }
      int scoreFrame=-1;
      const spair_t result = ls.scoreF(svec,start_codon,stop_codon,klst,scoreFrame,predType);
      if( !result.first ) {
	cout<<"MY failed\n";
	continue;
	if( mheader.empty() ) ed.printAlignment(svec);
      }
      cout<<"trueres_Score: "<<result<<endl;
      const list<SeqLabelVal>* pred = NULL;
#if 1
      assert(klst.front().getLabel()==SeqLabel::eBeg);
      const int start_pos = klst.front().getLeft();

      assert(klst.back().getLabel()==SeqLabel::eEnd);
      const int stop_pos = klst.back().getLeft();

      LabelSeq ls1(_mm);
      int labelFrame=-1;
      pred = ls1.labelF(svec,start_codon,stop_codon,start_pos,stop_pos,labelFrame,false,predType,false,false);
      cout<<"We have a Prediction, please let it not be utter crap! "<<labelFrame<<endl;
      SeqLabelVal::printLst(*pred);
      
      list<SeqLabelVal>::const_iterator ti=pred->begin();
      const list<SeqLabelVal>::const_iterator ts=pred->end();
      //const SeqLabel* lptr = NULL;
      list<SeqLabel> plst;
      for(; ti != ts; ++ti) {
	const SeqLabelVal& sl = *ti;
	plst.push_back(sl);
      }
#else
      /// 144.61
      list<SeqLabel> plst;
      SeqLabel p0(SeqLabel::eBeg,SeqLabel::eUnk,0,0);
      SeqLabel p1(SeqLabel::eNea,SeqLabel::eCon,51,74);
      SeqLabel p2(SeqLabel::eNed,SeqLabel::eCon,169,177);
      SeqLabel end(SeqLabel::eEnd,SeqLabel::eCon,287,287);
      plst.push_back(p0);
      plst.push_back(p1);
      plst.push_back(p2);
      plst.push_back(end);
      int labelFrame=-1;
      spair_t res2 = ls.scoreF(svec,start_codon,stop_codon,plst,labelFrame,predType);
      cout<<" label pscore: "<<res2<<endl;
      cout<<"PRED ALIGNMENT"<<endl;
#endif
      tm.score(targ_fs,plst,klst);
      vector<list<SeqLabel>*> cvec(2);
      cvec[0]=&klst;
      cvec[1]=&plst;
      vector<int> frames(2);
      frames[0]=scoreFrame;
      frames[1]=labelFrame;
      const int ndiff = _mm.discTrain(svec,start_codon,stop_codon,cvec,frames,*next_phash);
      sum_diff += ndiff;
      const double pval = pred ? pred->back().getVal() : 0;
      const double aval = result.second;
      const double diff = pval-aval;
      sum_score += diff;
      ++hit;
      const double avg = sum_diff / (double)hit;
      const double savg = sum_score / (double)hit;
      cout<<"diffs, "<<cnt<<" s: "<<diff<<" c: "<<ndiff<<" "<<avg<<" as: "<<savg<<endl;;
      cout<<"tracking tm: "<<cnt<<" "<<tm<<endl;
    }
    // commit changes and select a new parameter to modifiy
    const double curr_avg = tm.calcFscore();
    const double ravg = tm.calcAvg();
    cout<<"NEW DIFF: "<<sum_diff<<" LST DIFF: "<<last_diff<<" curr_avg: "<<curr_avg<<" lst avg: "<<last_avg<<" "<<ravg<<endl;
    if( last_avg <= curr_avg && last_diff != -1 ) {
      if( last_avg < curr_avg ) {
	wght = 0.1;
      }
      if( next_cm ) {
	delete last_cm;
	last_cm = next_cm;
      }
      if( last_phash ) {
	delete last_phash;
      }
      last_phash = next_phash;

      if( last_param ) {
	delete last_param;
      }
      last_param = _mm.selectParam(*next_phash);
      last_diff = sum_diff;
      last_avg = curr_avg;
    } else if( last_avg != -1 ) {
      // no improvement, remove parameter from change list (or possibly alter how it's changed)
      delete next_cm;
      next_cm=NULL;
      if( last_phash ) {
	delete next_phash;
      } else {
	last_phash = next_phash;
      }
      HmmComponent* pid = last_param->getPrd().getComp();
      (*last_phash)[pid].clear();
      if( last_param ) {
	delete last_param;
      }
      last_param = _mm.selectParam(*last_phash);
    } else {
      // first time around, select new parameter
      last_phash = next_phash;
      last_cm = _mm.getConsManager();
      last_param = _mm.selectParam(*next_phash);
      last_diff = sum_diff;
      last_avg = curr_avg;
    }
    if( !last_param ) {
      cout<<"NO new parameters to tweak"<<endl;
      wght+=0.1;
      delete last_phash;
      last_phash=NULL;
      _mm.setCM(last_cm);
      continue;
    }
    //_mm.normalize1();
    next_cm = new ConsManager(*last_cm);
    HmmComponent* pid = last_param->getPrd().getComp();
    ModelType::cname_t pcn = last_param->getPrd().getHmm()->getType();
    HmmComponent* pparam = next_cm->find(pcn,*pid);
    cout<<"huh: before: "<<endl;
    pparam->write(cout);
    pparam->update(-1.0*wght);
    cout<<"update false pred: "<<ModelType::cname2str(pcn)<<endl;
    pparam->write(cout);

    HmmComponent* tid = last_param->getAns().getComp();
    ModelType::cname_t tcn = last_param->getAns().getHmm()->getType();
    HmmComponent* tparam = next_cm->find(tcn,*tid);
    tparam->update(wght);
    cout<<"update true pred: "<<ModelType::cname2str(tcn)<<endl;
    tparam->write(cout);

    next_cm->normalize1();
    next_cm->print(cout);
    _mm.setCM(next_cm);

#if 0
    cout<<"normalize"<<endl;
    //if( (cnt%100)==0 ) {
      cout<<"Before: "<<endl;
      _mm.getConsManager().print(cout);
    //}
    _mm.normalize1();
    //if( (cnt%100)==0 ) {
      cout<<"After: "<<endl;
      _mm.getConsManager().print(cout);
    //}
#endif
    const double avg = sum_diff / (double)hit;
    const double savg = sum_score / (double)hit;
    cout<<"After round "<<train_iter<<" of conditional mods"<<endl<<tm<<" err: "<<avg<<" serr: "<<savg<<endl;
    last_cm->write("hold_rate_vals1.txt");
  }
  list<const vector<string*>*>::iterator ti = svecLst.begin();
  const list<const vector<string*>*>::iterator ts = svecLst.end();
  for(; ti != ts; ++ti) {
    delete *ti;
  }
  cout<<"TOTAL:"<<endl;
}

void
RunModel::discTrain1(const fasta& aligned_targ, const string& mheader) {
  const pred_t predType = eDisc1HmmRates;
  list<const vector<string*>*> svecLst;
  list<fasta_seq*> fastaLst;
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  vector<int> debug_type(4,0);
  for(;iter != stop; ++iter) {
    bool stopRun=true;
    for(unsigned i=0; i < debug_type.size(); ++i) {
      if( debug_type[i] < cNUM_EX ) {
	stopRun=false;
	break;
      }
    }
    if( stopRun ) {
      break;
    }
    const fasta_seq& aligned_fs = **iter;
    if( !mheader.empty() ) {
      const string& header = aligned_fs.get_header();
      //cout<<"mheader: "<<mheader<<endl<<header<<endl;
      if( mheader != header ) {
	continue;
      }
    }
    const fasta_seq& targ_fs = aligned_fs;
    if(targ_fs.get_seq().length() > 800) {
      cout<<"too damn long skipping for debug phase: "<<targ_fs.get_header()<<" "<<targ_fs.get_seq().length()<<endl;
      continue;
    }
    size_t pos = targ_fs.get_header().find("_CS_");
    if(pos != string::npos) {
      if(debug_type[0]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[0];
      }
    }
    pos = targ_fs.get_header().find("_SE_");
    if(pos != string::npos) {
      if(debug_type[1]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[1];
      }
    }
    pos = targ_fs.get_header().find("_MS_");
    if(pos != string::npos) {
      if(debug_type[2]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[2];
      }
    }
    pos = targ_fs.get_header().find("_IR_");
    if(pos != string::npos) {
      if(debug_type[3]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[3];
      }
    }
    const vector<string*>* svecPtr = getMatchSeqPtr(targ_fs);
    svecLst.push_back(svecPtr);
    fastaLst.push_back(*iter);
  }
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  cout<<"discTrain size: "<<aligned_targ.size()<<endl;
  //ParamSearchHashT* last_phash = NULL;
  //ConsManager* last_cm = NULL;
  //double last_diff = -1;
  //ParamSearchPair* last_param = NULL;
  //ConsManager* next_cm = NULL;
  for(unsigned train_iter = 0; train_iter < 1000; ++train_iter) {
    unsigned cnt = 0, hit=0;
    cout<<"Training iteration: "<<train_iter<<endl;
    ostringstream os;
    os<<"Test iteration: "<<train_iter<<endl;
    TestModel tm(os.str());
    double sum_diff = 0, sum_score = 0;
    list<fasta_seq*>::const_iterator iter = fastaLst.begin();
    const list<fasta_seq*>::const_iterator stop = fastaLst.end();
    list<const vector<string*>*>::const_iterator si = svecLst.begin();
    ParamSearchHashT* next_phash = new ParamSearchHashT;
    for(;iter != stop; ++iter, ++si, ++cnt) {
      const vector<string*>& svec = **si;
      const fasta_seq& aligned_fs = **iter;
      const fasta_seq& targ_fs = aligned_fs;
      if( !mheader.empty() ) {
	const string& header = aligned_fs.get_header();
	//cout<<"mheader: "<<mheader<<endl<<header<<endl;
	if( mheader != header ) {
	  continue;
	}
      }
      ParseHeader ph(ai);
      bool status = ph.parse(targ_fs);
      if( !status ) {
	cout<<"skiping????"<<targ_fs.get_header()<<endl;
	continue;
      }
      cout<<targ_fs.get_header()<<endl;      
      ExonDescrip& ed = ph.getExonDescrip();
      ed.print();
      cout<<endl;
      ipair_t ip = ed.getStartStop();

      list<SeqLabel> klst = ed.getSeqLabelLst();
      SeqLabel::printLst(klst);
      if( !mheader.empty() ) {
	ed.printAlignment(svec);
      }
      const int start_codon = ip.first;
      const int stop_codon = ip.second; //3rd position of last codon

      cout<<"start/stop: "<<start_codon<<" "<<stop_codon<<endl;
      const LabelSeq ls(_mm);
      const bool isGcDon = hasGcDon(*(svec[0]),klst);
      if(isGcDon) {
	cout<<"THIS SEQ has a GC donor\n";
      }
      int scoreFrame=-1;
      const spair_t result = ls.scoreF(svec,start_codon,stop_codon,klst,scoreFrame,predType);
      if( !result.first ) {
	cout<<"MY failed\n";
	continue;
	if( mheader.empty() ) ed.printAlignment(svec);
      }
      cout<<"trueres_Score: "<<result<<endl;
      const list<SeqLabelVal>* pred = NULL;
#if 1
      assert(klst.front().getLabel()==SeqLabel::eBeg);
      const int start_pos = klst.front().getLeft();

      assert(klst.back().getLabel()==SeqLabel::eEnd);
      const int stop_pos = klst.back().getLeft();

      LabelSeq ls1(_mm);
      int labelFrame=-1;
      pred = ls1.labelF(svec,start_codon,stop_codon,start_pos,stop_pos,labelFrame,false,predType);
      cout<<"We have a Prediction, please let it not be utter crap! "<<labelFrame<<endl;
      SeqLabelVal::printLst(*pred);
      
      list<SeqLabelVal>::const_iterator ti=pred->begin();
      const list<SeqLabelVal>::const_iterator ts=pred->end();
      //const SeqLabel* lptr = NULL;
      list<SeqLabel> plst;
      for(; ti != ts; ++ti) {
	const SeqLabelVal& sl = *ti;
	plst.push_back(sl);
      }
#else
      /// 144.61
      list<SeqLabel> plst;
      SeqLabel p0(SeqLabel::eBeg,SeqLabel::eUnk,0,0);
      SeqLabel p1(SeqLabel::eNea,SeqLabel::eCon,51,74);
      SeqLabel p2(SeqLabel::eNed,SeqLabel::eCon,169,177);
      SeqLabel end(SeqLabel::eEnd,SeqLabel::eCon,287,287);
      plst.push_back(p0);
      plst.push_back(p1);
      plst.push_back(p2);
      plst.push_back(end);
      int labelFrame=-1;
      spair_t res2 = ls.scoreF(svec,start_codon,stop_codon,plst,labelFrame,predType);
      cout<<" label pscore: "<<res2<<endl;
      cout<<"PRED ALIGNMENT"<<endl;
#endif
      tm.score(targ_fs,plst,klst);
      vector<list<SeqLabel>*> cvec(2);
      cvec[0]=&klst;
      cvec[1]=&plst;
      vector<int> frames(2);
      frames[0]=scoreFrame;
      frames[1]=labelFrame;
      const int ndiff = _mm.discTrain(svec,start_codon,stop_codon,cvec,frames,*next_phash);
      sum_diff += ndiff;
      const double pval = pred ? pred->back().getVal() : 0;
      const double aval = result.second;
      const double diff = pval-aval;
      sum_score += diff;
      ++hit;
      const double avg = sum_diff / (double)hit;
      const double savg = sum_score / (double)hit;
      cout<<"diffs, "<<cnt<<" s: "<<diff<<" c: "<<ndiff<<" "<<avg<<" as: "<<savg<<endl;;
      cout<<"tracking tm: "<<cnt<<" "<<tm<<endl;
    }
    _mm.updateParams(*next_phash);
    // commit changes and select a new parameter to modifiy
    cout<<"SUMDIFF = "<<sum_diff<<endl;
#if 0
    cout<<"NEW DIFF: "<<sum_diff<<" LST DIFF: "<<last_diff<<endl;
    if( sum_diff <= last_diff && last_diff != -1 ) {
      if( sum_diff < last_diff ) {
	wght = 0.1;
      }
      if( next_cm ) {
	delete last_cm;
	last_cm = next_cm;
      }
      if( last_phash ) {
	delete last_phash;
      }
      last_phash = next_phash;

      if( last_param ) {
	delete last_param;
      }
      last_param = _mm.selectParam(*next_phash);
      last_diff = sum_diff;
    } else if( last_diff != -1 ) {
      // no improvement, remove parameter from change list (or possibly alter how it's changed)
      delete next_cm;
      next_cm=NULL;
      if( last_phash ) {
	delete next_phash;
      } else {
	last_phash = next_phash;
      }
      HmmComponent* pid = last_param->getPrd().getComp();
      (*last_phash)[pid].clear();
      if( last_param ) {
	delete last_param;
      }
      last_param = _mm.selectParam(*last_phash);
    } else {
      // first time around, select new parameter
      last_phash = next_phash;
      last_cm = _mm.getConsManager();
      last_param = _mm.selectParam(*next_phash);
      last_diff = sum_diff;
    }
    if( !last_param ) {
      cout<<"NO new parameters to tweak"<<endl;
      wght+=0.1;
      delete last_phash;
      last_phash=NULL;
      _mm.setCM(last_cm);
      continue;
    }
    //_mm.normalize1();
    next_cm = new ConsManager(*last_cm);
    HmmComponent* pid = last_param->getPrd().getComp();
    ModelType::cname_t pcn = last_param->getPrd().getHmm()->getType();
    HmmComponent* pparam = next_cm->find(pcn,*pid);
    cout<<"huh: before: "<<endl;
    pparam->write(cout);
    pparam->update(-1.0*wght);
    cout<<"update false pred: "<<ModelType::cname2str(pcn)<<endl;
    pparam->write(cout);

    HmmComponent* tid = last_param->getAns().getComp();
    ModelType::cname_t tcn = last_param->getAns().getHmm()->getType();
    HmmComponent* tparam = next_cm->find(tcn,*tid);
    tparam->update(wght);
    cout<<"update true pred: "<<ModelType::cname2str(tcn)<<endl;
    tparam->write(cout);

    next_cm->normalize1();
    next_cm->print(cout);
    _mm.setCM(next_cm);
#endif

#if 0
    cout<<"normalize"<<endl;
    //if( (cnt%100)==0 ) {
      cout<<"Before: "<<endl;
      _mm.getConsManager().print(cout);
    //}
    _mm.normalize1();
    //if( (cnt%100)==0 ) {
      cout<<"After: "<<endl;
      _mm.getConsManager().print(cout);
    //}
#endif
    const double avg = sum_diff / (double)hit;
    const double savg = sum_score / (double)hit;
    cout<<"After round "<<train_iter<<" of conditional mods"<<endl<<tm<<" err: "<<avg<<" serr: "<<savg<<endl;
    _mm.getConsManager()->normalize2();
    _mm.getConsManager()->write("hold_rate_vals.txt");
  }

  list<const vector<string*>*>::iterator ti = svecLst.begin();
  const list<const vector<string*>*>::iterator ts = svecLst.end();
  for(; ti != ts; ++ti) {
    delete *ti;
  }
  cout<<"TOTAL:"<<endl;
}

#if 1
void
RunModel::paramSearch(const fasta& aligned_targ, const string& mheader) {
  cout<<"BEG paramADJUST:"<<endl;
  cout<<"discTrain size: "<<aligned_targ.size()<<endl;
  RateManager* best_rm = new RateManager;
  best_rm->create();
  TestModel* best_tm =NULL;
  for(unsigned train_iter = 0; train_iter < 200; ++train_iter ) {
    cout<<"Random parameter mutation: "<<train_iter<<endl;
    RateManager* curr_rm = new RateManager;
    curr_rm->create();
    curr_rm->setInitRates(); // random permutation
    best_tm = paramSearch1(aligned_targ,mheader,curr_rm,*best_rm,best_tm);
    delete curr_rm;
  }
  _mm.setRM(best_rm);
  cout<<"END paramADJUST:"<<endl;
}
#else
void
RunModel::paramSearch(const fasta& aligned_targ, const string& mheader) {
  cout<<"BEG paramADJUST:"<<endl;
  pred_t predType = eLearnedFixedRate;
  list<const vector<string*>*> svecLst;
  list<fasta_seq*> fastaLst;
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  vector<int> debug_type(4,0);
  for(;iter != stop; ++iter) {
    bool stopRun=true;
    for(unsigned i=0; i < debug_type.size(); ++i) {
      if( debug_type[i] < cNUM_EX ) {
	stopRun=false;
	break;
      }
    }
    if( stopRun ) {
      break;
    }
    const fasta_seq& aligned_fs = **iter;
    if( !mheader.empty() ) {
      const string& header = aligned_fs.get_header();
      //cout<<"mheader: "<<mheader<<endl<<header<<endl;
      if( mheader != header ) {
	continue;
      }
    }
    const fasta_seq& targ_fs = aligned_fs;
    if(targ_fs.get_seq().length() > 1500) {
      cout<<"too damn long skipping for debug phase: "<<targ_fs.get_header()<<" "<<targ_fs.get_seq().length()<<endl;
      continue;
    }
    size_t pos = targ_fs.get_header().find("_CS_");
    if(pos != string::npos) {
      if(debug_type[0]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[0];
      }
    }
    pos = targ_fs.get_header().find("_SE_");
    if(pos != string::npos) {
      if(debug_type[1]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[1];
      }
    }
    pos = targ_fs.get_header().find("_MS_");
    if(pos != string::npos) {
      if(debug_type[2]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[2];
      }
    }
    pos = targ_fs.get_header().find("_IR_");
    if(pos != string::npos) {
      if(debug_type[3]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[3];
      }
    }
    const vector<string*>* svecPtr = getMatchSeqPtr(targ_fs);
    svecLst.push_back(svecPtr);
    fastaLst.push_back(*iter);
  }
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  cout<<"discTrain size: "<<aligned_targ.size()<<endl;
  //double last_diff = -1;
  RateManager* change_rate = new RateManager;
  change_rate->create1();
  TestModel* best_tm = NULL;
  RateManager* best_rm = new RateManager;
  best_rm->create();
  _mm.setRM(best_rm);
  RateManager* curr_rm = curr_rm;
  double best_fscore = 0, bavg=0;
  for(unsigned train_iter = 0; train_iter < 1000; ++train_iter) {
    unsigned cnt = 0;
    cout<<"Training iteration: "<<train_iter<<endl;
    ostringstream os;
    os<<"Test iteration: "<<train_iter<<endl;
    TestModel* curr_tm = new TestModel(os.str());
    list<fasta_seq*>::const_iterator iter = fastaLst.begin();
    const list<fasta_seq*>::const_iterator stop = fastaLst.end();
    list<const vector<string*>*>::const_iterator si = svecLst.begin();
    for(;iter != stop; ++iter, ++si, ++cnt) {
      const vector<string*>& svec = **si;
      const fasta_seq& aligned_fs = **iter;
      const fasta_seq& targ_fs = aligned_fs;
      if( !mheader.empty() ) {
	const string& header = aligned_fs.get_header();
	//cout<<"mheader: "<<mheader<<endl<<header<<endl;
	if( mheader != header ) {
	  continue;
	}
      }
      ParseHeader ph(ai);
      bool status = ph.parse(targ_fs);
      if( !status ) {
	cout<<"skiping????"<<targ_fs.get_header()<<endl;
	continue;
      }
      cout<<targ_fs.get_header()<<endl;      
      ExonDescrip& ed = ph.getExonDescrip();
      ed.print();
      cout<<endl;
      ipair_t ip = ed.getStartStop();

      const list<SeqLabel>& klst = ed.getSeqLabelLst();
      SeqLabel::printLst(klst);
      if( !mheader.empty() ) {
	ed.printAlignment(svec);
      }
      const int start_codon = ip.first;\
      const int stop_codon = ip.second; //3rd position of last codon

      cout<<"start/stop: "<<start_codon<<" "<<stop_codon<<endl;
      const LabelSeq ls(_mm);
      const bool isGcDon = hasGcDon(*(svec[0]),klst);
      if(isGcDon) {
	cout<<"THIS SEQ has a GC donor\n";
      }
      int scoreFrame=-1;
      const spair_t result = ls.scoreF(svec,start_codon,stop_codon,klst,scoreFrame,predType);
      if( !result.first ) {
	cout<<"MY failed\n";
	continue;
	if( mheader.empty() ) ed.printAlignment(svec);
      }
      cout<<"trueres_Score: "<<result<<endl;

      assert(klst.front().getLabel()==SeqLabel::eBeg);
      const int start_pos = klst.front().getLeft();

      assert(klst.back().getLabel()==SeqLabel::eEnd);
      const int stop_pos = klst.back().getLeft();

      LabelSeq ls1(_mm);
      int labelFrame=-1;
      const list<SeqLabelVal>* pred = ls1.labelF(svec,start_codon,stop_codon,start_pos,stop_pos,labelFrame,false,predType);
      cout<<"This is the DiscTrain Prediction-"<<labelFrame<<endl;
      SeqLabelVal::printLst(*pred);
      
      list<SeqLabel> plst;
      list<SeqLabelVal>::const_iterator ti=pred->begin();
      const list<SeqLabelVal>::const_iterator ts=pred->end();
      for(; ti != ts; ++ti) {
	const SeqLabelVal& sl = *ti;
	plst.push_back(sl);
      }
      curr_tm->score(targ_fs,plst,klst);
      delete pred;
    }
    cout<<"tracking tm: "<<train_iter<<endl;
    cout<<"Curr:"<<endl;
    cout<<*curr_tm<<endl;
    if( best_tm ) {
      cout<<"Best:"<<endl;
      cout<<*best_tm<<endl;
    }
    // commit changes and select a new parameter to modifiy
    const double curr_fscore = curr_tm->calcFscore();
    const double cavg = curr_tm->calcAvg();
    cout<<"curr_fscore: "<<curr_fscore<<" best fscore: "<<best_fscore<<" best_avg: "<<bavg<<" curr_avg: "<<cavg<<endl;
    TestModel* prev_best_tm = NULL;
    bool delTm=true;
    if( best_tm ) {
      // save previous best performance numbers
      // so that in adjustRates, we know which direction to go in
      prev_best_tm = new TestModel(*best_tm);
      
      best_rm->updateRates(*best_tm,*curr_rm,curr_tm);
    } else {
      prev_best_tm = NULL;
      delete best_tm;
      best_tm = curr_tm;
      delTm=false; // save memory for "best_tm" initalized with first example
    }
    if( curr_fscore > best_fscore ) {
      best_fscore = curr_fscore;
      bavg = cavg;
    }
    cout<<"BestRM: "<<endl;
    best_rm->print(cout);

    if( delTm ) {
      delete curr_rm;
    }
    curr_rm = change_rate->adjustRates(*best_rm,curr_tm,prev_best_tm);
    _mm.setRM(curr_rm);
    cout<<"TryRM: "<<endl;
    curr_rm->print(cout);
    cout<<"Change: "<<endl;
    change_rate->print(cout);

    delete prev_best_tm;
    if( delTm ) {
      delete curr_tm;
    }
  }
  _mm.setRM(best_rm);
  list<const vector<string*>*>::iterator ti = svecLst.begin();
  const list<const vector<string*>*>::iterator ts = svecLst.end();
  for(; ti != ts; ++ti) {
    delete *ti;
  }
  cout<<"END paramADJUST:"<<endl;
}
#endif

void
RunModel::adjustTrans(const fasta& aligned_targ, const string& mheader) {
  cout<<"BEG transADJUST:"<<endl;
  pred_t predType = eLearnedFixedRate;
  list<const vector<string*>*> svecLst;
  list<fasta_seq*> fastaLst;
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  vector<int> debug_type(4,0);
  for(;iter != stop; ++iter) {
    bool stopRun=true;
    for(unsigned i=0; i < debug_type.size(); ++i) {
      if( debug_type[i] < cNUM_EX ) {
	stopRun=false;
	break;
      }
    }
    if( stopRun ) {
      break;
    }
    const fasta_seq& aligned_fs = **iter;
    if( !mheader.empty() ) {
      const string& header = aligned_fs.get_header();
      //cout<<"mheader: "<<mheader<<endl<<header<<endl;
      if( mheader != header ) {
	continue;
      }
    }
    const fasta_seq& targ_fs = aligned_fs;
    if(targ_fs.get_seq().length() > 1500) {
      cout<<"too damn long skipping for debug phase: "<<targ_fs.get_header()<<" "<<targ_fs.get_seq().length()<<endl;
      continue;
    }
    size_t pos = targ_fs.get_header().find("_CS_");
    if(pos != string::npos) {
      if(debug_type[0]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[0];
      }
    }
    pos = targ_fs.get_header().find("_SE_");
    if(pos != string::npos) {
      if(debug_type[1]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[1];
      }
    }
    pos = targ_fs.get_header().find("_MS_");
    if(pos != string::npos) {
      if(debug_type[2]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[2];
      }
    }
    pos = targ_fs.get_header().find("_IR_");
    if(pos != string::npos) {
      if(debug_type[3]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[3];
      }
    }
    const vector<string*>* svecPtr = getMatchSeqPtr(targ_fs);
    svecLst.push_back(svecPtr);
    fastaLst.push_back(*iter);
  }
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  cout<<"discTrain size: "<<aligned_targ.size()<<endl;
  TestModel* best_tm = NULL;
  StateTrans* best_rm = new StateTrans(_mm.getStateTrans());
  StateTrans* curr_rm = new StateTrans(_mm.getStateTrans());
  double best_fscore = 0, bavg=0;
  list<sc_t> params;
  if( params.empty() ) {
    for(unsigned ti=0; ti < (unsigned)StateTrans::eStateSize; ++ti) {
      StateTrans::state_t ts = (StateTrans::state_t)ti;
      const double tprob = best_rm->getInit(ts);
      if( tprob != 0 && tprob != 1) {
	params.push_back(sc_t(pair<StateTrans::state_t,StateTrans::state_t>(ts,StateTrans::eStateSize),cRANGE2));
      }
    }
    for(unsigned fi=0; fi < (unsigned)StateTrans::eStateSize; ++fi) {
      StateTrans::state_t fr = (StateTrans::state_t)fi;
      for(unsigned ti=0; ti < (unsigned)StateTrans::eStateSize; ++ti) {
	StateTrans::state_t to = (StateTrans::state_t)ti;
	const double tprob = best_rm->getTrans(fr,to);
	if( tprob!= 0 && tprob != 1 ) {
	  params.push_back(sc_t(pair<StateTrans::state_t,StateTrans::state_t>(fr,to),cRANGE2));
	  
	}
      }
    }
  }
  unsigned cnt = 0;
  cout<<"Total number of parameters to tweak: "<<params.size()*cRANGE2<<endl;
  while( !params.empty() ) {
    ostringstream os;
    os<<"Training transitions iteration: "<<cnt++<<endl;
    TestModel* curr_tm = new TestModel(os.str());
    list<fasta_seq*>::const_iterator iter = fastaLst.begin();
    const list<fasta_seq*>::const_iterator stop = fastaLst.end();
    list<const vector<string*>*>::const_iterator si = svecLst.begin();
    for(;iter != stop; ++iter, ++si, ++cnt) {
      const vector<string*>& svec = **si;
      const fasta_seq& aligned_fs = **iter;
      const fasta_seq& targ_fs = aligned_fs;
      if( !mheader.empty() ) {
	const string& header = aligned_fs.get_header();
	//cout<<"mheader: "<<mheader<<endl<<header<<endl;
	if( mheader != header ) {
	  continue;
	}
      }
      ParseHeader ph(ai);
      bool status = ph.parse(targ_fs);
      if( !status ) {
	cout<<"skiping????"<<targ_fs.get_header()<<endl;
	continue;
      }
      cout<<targ_fs.get_header()<<endl;      
      ExonDescrip& ed = ph.getExonDescrip();
      ed.print();
      cout<<endl;
      ipair_t ip = ed.getStartStop();

      const list<SeqLabel>& klst = ed.getSeqLabelLst();
      SeqLabel::printLst(klst);
      if( !mheader.empty() ) {
	ed.printAlignment(svec);
      }
      const int start_codon = ip.first;
      const int stop_codon = ip.second; //3rd position of last codon

      cout<<"start/stop: "<<start_codon<<" "<<stop_codon<<endl;
      const LabelSeq ls(_mm);
      const bool isGcDon = hasGcDon(*(svec[0]),klst);
      if(isGcDon) {
	cout<<"THIS SEQ has a GC donor\n";
      }
      int scoreFrame=-1;
      const spair_t result = ls.scoreF(svec,start_codon,stop_codon,klst,scoreFrame,predType);
      if( !result.first ) {
	cout<<"MY failed\n";
	continue;
	if( mheader.empty() ) ed.printAlignment(svec);
      }
      cout<<"trueres_Score: "<<result<<endl;

      assert(klst.front().getLabel()==SeqLabel::eBeg);
      const int start_pos = klst.front().getLeft();

      assert(klst.back().getLabel()==SeqLabel::eEnd);
      const int stop_pos = klst.back().getLeft();

      LabelSeq ls1(_mm);
      int labelFrame=-1;
      const list<SeqLabelVal>* pred = ls1.labelF(svec,start_codon,stop_codon,start_pos,stop_pos,labelFrame,false,predType);
      cout<<"This is the DiscTrain Prediction-"<<labelFrame<<endl;
      SeqLabelVal::printLst(*pred);
      
      list<SeqLabel> plst;
      list<SeqLabelVal>::const_iterator ti=pred->begin();
      const list<SeqLabelVal>::const_iterator ts=pred->end();
      for(; ti != ts; ++ti) {
	const SeqLabelVal& sl = *ti;
	plst.push_back(sl);
      }
      curr_tm->score(targ_fs,plst,klst);
      delete pred;
    }
    cout<<"Curr:"<<endl;
    cout<<*curr_tm<<endl;
    if( best_tm ) {
      cout<<"Best:"<<endl;
      cout<<*best_tm<<endl;
    }
    // commit changes and select a new parameter to modifiy
    const double curr_fscore = curr_tm->calcFscore();
    const double cavg = curr_tm->calcAvg();
    cout<<"curr_fscore: "<<curr_fscore<<" best fscore: "<<best_fscore<<" best_avg: "<<bavg<<" curr_avg: "<<cavg<<endl;
    TestModel* prev_best_tm = NULL;
    bool delTm=true;
    if( best_tm ) {
      // save previous best performance numbers
      // so that in adjustRates, we know which direction to go in
      prev_best_tm = new TestModel(*best_tm);
      
      best_rm->updateTrans(*best_tm,*curr_rm,*curr_tm);
    } else {
      prev_best_tm = NULL;
      delete best_tm;
      best_tm = curr_tm;
      delTm=false; // save memory for "best_tm" initalized with first example
    }
    if( curr_fscore > best_fscore ) {
      best_fscore = curr_fscore;
      bavg = cavg;
    }
    cout<<"BestRM: "<<endl;
    best_rm->print(cout);

    sc_t& scVal = params.front();
    *curr_rm = *best_rm;
    curr_rm->adjustTrans(*best_rm,scVal,cRANGE2);
    _mm.setST(curr_rm);
    cout<<"TryRM: "<<endl;
    curr_rm->print(cout);
    delete prev_best_tm;
    if(scVal.second<=0) {
      params.pop_front();
    }
  }
  // should always be true
  if( curr_rm != best_rm ) {
    delete curr_rm;
  }
  _mm.setST(best_rm);
  list<const vector<string*>*>::iterator ti = svecLst.begin();
  const list<const vector<string*>*>::iterator ts = svecLst.end();
  for(; ti != ts; ++ti) {
    delete *ti;
  }
  cout<<"END transADJUST:"<<endl;
}

TestModel*
RunModel::paramSearch1(const fasta& aligned_targ, const string& mheader, RateManager* init_rm, RateManager& global_best_rm, TestModel* global_best_tm) {
  cout<<"BEG paramADJUST1:"<<endl;
  pred_t predType = eLearnedFixedRate;
  list<const vector<string*>*> svecLst;
  list<fasta_seq*> fastaLst;
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  vector<int> debug_type(4,0);
  for(;iter != stop; ++iter) {
    bool stopRun=true;
    for(unsigned i=0; i < debug_type.size(); ++i) {
      if( debug_type[i] < cNUM_EX ) {
	stopRun=false;
	break;
      }
    }
    if( stopRun ) {
      break;
    }
    const fasta_seq& aligned_fs = **iter;
    if( !mheader.empty() ) {
      const string& header = aligned_fs.get_header();
      //cout<<"mheader: "<<mheader<<endl<<header<<endl;
      if( mheader != header ) {
	continue;
      }
    }
    const fasta_seq& targ_fs = aligned_fs;
    if(targ_fs.get_seq().length() > 1500) {
      cout<<"too damn long skipping for debug phase: "<<targ_fs.get_header()<<" "<<targ_fs.get_seq().length()<<endl;
      continue;
    }
    size_t pos = targ_fs.get_header().find("_CS_");
    if(pos != string::npos) {
      if(debug_type[0]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[0];
      }
    }
    pos = targ_fs.get_header().find("_SE_");
    if(pos != string::npos) {
      if(debug_type[1]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[1];
      }
    }
    pos = targ_fs.get_header().find("_MS_");
    if(pos != string::npos) {
      if(debug_type[2]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[2];
      }
    }
    pos = targ_fs.get_header().find("_IR_");
    if(pos != string::npos) {
      if(debug_type[3]>=cNUM_EX) {
	continue;
      } else {
	++debug_type[3];
      }
    }
    const vector<string*>* svecPtr = getMatchSeqPtr(targ_fs);
    svecLst.push_back(svecPtr);
    fastaLst.push_back(*iter);
  }
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  cout<<"discTrain size: "<<aligned_targ.size()<<endl;
  TestModel* best_tm = NULL;
  RateManager* change_rate = new RateManager;
  change_rate->create1();
  RateManager* curr_rm = new RateManager(*init_rm);
  RateManager* best_rm = new RateManager(*init_rm);
  cout<<"Starting rm: "<<endl;
  curr_rm->print(cout);
  _mm.setRM(curr_rm);
  double best_fscore = 0, bavg=0;
  list<tp2_t> params;
  // ORDER IS IMPORTANT HERE, optimal order not known
  params.push_back(tp2_t(tp_t(ModelType::eAdon,cRANGE1),0));
  params.push_back(tp2_t(tp_t(ModelType::eCdon,cRANGE1),0));
  params.push_back(tp2_t(tp_t(ModelType::eAacc,cRANGE1),0));
  params.push_back(tp2_t(tp_t(ModelType::eCacc,cRANGE1),0));
  params.push_back(tp2_t(tp_t(ModelType::eAexonPrc,cRANGE1),1));
  params.push_back(tp2_t(tp_t(ModelType::eAexonPrc,cRANGE1),0));
  params.push_back(tp2_t(tp_t(ModelType::eCexonPrc,cRANGE1),1));
  params.push_back(tp2_t(tp_t(ModelType::eCexonPrc,cRANGE1),0));
  params.push_back(tp2_t(tp_t(ModelType::eAintronDn,cRANGE1),1));
  params.push_back(tp2_t(tp_t(ModelType::eAintronDn,cRANGE1),0));
  params.push_back(tp2_t(tp_t(ModelType::eCintronDn,cRANGE1),1));
  params.push_back(tp2_t(tp_t(ModelType::eCintronDn,cRANGE1),0));
  params.push_back(tp2_t(tp_t(ModelType::eAexonNpc,cRANGE1),1));
  params.push_back(tp2_t(tp_t(ModelType::eAexonNpc,cRANGE1),0));
  params.push_back(tp2_t(tp_t(ModelType::eCexonNpc,cRANGE1),1));
  params.push_back(tp2_t(tp_t(ModelType::eCexonNpc,cRANGE1),0));
  unsigned icnt = 0;
  while(!params.empty()) {
    cout<<"Training rates iteration: "<<icnt++<<endl;
    ostringstream os;
    TestModel* curr_tm = new TestModel(os.str());
    list<fasta_seq*>::const_iterator iter = fastaLst.begin();
    const list<fasta_seq*>::const_iterator stop = fastaLst.end();
    list<const vector<string*>*>::const_iterator si = svecLst.begin();
    unsigned cnt = 0;
    for(;iter != stop; ++iter, ++si, ++cnt) {
      const vector<string*>& svec = **si;
      const fasta_seq& aligned_fs = **iter;
      const fasta_seq& targ_fs = aligned_fs;
      if( !mheader.empty() ) {
	const string& header = aligned_fs.get_header();
	//cout<<"mheader: "<<mheader<<endl<<header<<endl;
	if( mheader != header ) {
	  continue;
	}
      }
      ParseHeader ph(ai);
      bool status = ph.parse(targ_fs);
      if( !status ) {
	cout<<"skiping????"<<targ_fs.get_header()<<endl;
	continue;
      }
      cout<<targ_fs.get_header()<<endl;      
      ExonDescrip& ed = ph.getExonDescrip();
      ed.print();
      cout<<endl;
      ipair_t ip = ed.getStartStop();

      const list<SeqLabel>& klst = ed.getSeqLabelLst();
      SeqLabel::printLst(klst);
      if( !mheader.empty() ) {
	ed.printAlignment(svec);
      }
      const int start_codon = ip.first;
      const int stop_codon = ip.second; //3rd position of last codon

      cout<<"start/stop: "<<start_codon<<" "<<stop_codon<<endl;
      const LabelSeq ls(_mm);
      const bool isGcDon = hasGcDon(*(svec[0]),klst);
      if(isGcDon) {
	cout<<"THIS SEQ has a GC donor\n";
      }
      int scoreFrame=-1;
      const spair_t result = ls.scoreF(svec,start_codon,stop_codon,klst,scoreFrame,predType);
      if( !result.first ) {
	cout<<"MY failed\n";
	continue;
	if( mheader.empty() ) ed.printAlignment(svec);
      }
      cout<<"trueres_Score: "<<result<<endl;

      assert(klst.front().getLabel()==SeqLabel::eBeg);
      const int start_pos = klst.front().getLeft();

      assert(klst.back().getLabel()==SeqLabel::eEnd);
      const int stop_pos = klst.back().getLeft();

      LabelSeq ls1(_mm);
      int labelFrame=-1;
      const list<SeqLabelVal>* pred = ls1.labelF(svec,start_codon,stop_codon,start_pos,stop_pos,labelFrame,false,predType);
      cout<<"This is the DiscTrain Prediction-"<<labelFrame<<endl;
      SeqLabelVal::printLst(*pred);
      
      list<SeqLabel> plst;
      list<SeqLabelVal>::const_iterator ti=pred->begin();
      const list<SeqLabelVal>::const_iterator ts=pred->end();
      for(; ti != ts; ++ti) {
	const SeqLabelVal& sl = *ti;
	plst.push_back(sl);
      }
      curr_tm->score(targ_fs,plst,klst);
      delete pred;
    }
    cout<<"Curr:"<<endl;
    cout<<*curr_tm<<endl;
    if( best_tm ) {
      cout<<"Best:"<<endl;
      cout<<*best_tm<<endl;
    }
    // commit changes and select a new parameter to modifiy
    const double curr_fscore = curr_tm->calcFscore();
    const double cavg = curr_tm->calcAvg();
    cout<<"curr_fscore: "<<curr_fscore<<" best fscore: "<<best_fscore<<" best_avg: "<<bavg<<" curr_avg: "<<cavg<<endl;
    TestModel* prev_best_tm = NULL;
    bool delTm=true;
    if( best_tm ) {
      // save previous best performance numbers
      // so that in adjustRates, we know which direction to go in
      prev_best_tm = new TestModel(*best_tm);
      
      best_rm->updateRates(*best_tm,*curr_rm,curr_tm);
    } else {
      prev_best_tm = NULL;
      delete best_tm;
      best_tm = curr_tm;
      delTm=false; // save memory for "best_tm" initalized with first example
    }
    if( curr_fscore > best_fscore ) {
      best_fscore = curr_fscore;
      bavg = cavg;
    }
    cout<<"BestRM: "<<endl;
    best_rm->print(cout);

    tp2_t& val = params.front();
    cout<<"I'm confused on this one: "<<val.first.first<<" "<<val.first.second<<" "<<val.second<<endl;
    assert(val.second == 0 || val.second==1);
    delete curr_rm;
    curr_rm = new RateManager(*best_rm);
    change_rate->adjustRates(*curr_rm,*best_rm,val,cRANGE1);
    _mm.setRM(curr_rm);
    cout<<"TryRM: "<<endl;
    curr_rm->print(cout);
    if( val.first.second<2 ) {
      params.pop_front();
    }
    //cout<<"Change: "<<endl;
    //change_rate->print(cout);
    delete prev_best_tm;
  }
  delete change_rate;
  change_rate=NULL;
  delete curr_rm;
  curr_rm=NULL;
  list<const vector<string*>*>::iterator ti = svecLst.begin();
  const list<const vector<string*>*>::iterator ts = svecLst.end();
  for(; ti != ts; ++ti) {
    delete *ti;
  }
  if( best_tm && global_best_tm ) {
    if( best_tm->calcFscore() > global_best_tm->calcFscore() ) {
      global_best_rm.updateRates(*global_best_tm,*best_rm,best_tm);
    } else {
      delete best_tm;
      best_tm = global_best_tm;
    }
  } else if( best_tm && !global_best_tm ) {
    global_best_rm.updateRates(*best_rm);
  }
  cout<<"END paramADJUST1:"<<endl;
  return best_tm;
}
#endif


void
RunModel::run2(const fasta& aligned_targ, const vector<bool>& useOrg, bool runCons, pred_t predType) {
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  unsigned cnt = 0;
	vector<int> nFrameLimit;
  for(;iter != stop; ++iter, ++cnt) {
    const fasta_seq& aligned_fs = **iter;
    //const string& header = aligned_fs.get_header();
    const fasta_seq& targ_fs = aligned_fs;
    if(targ_fs.get_seq().length() > 10000) {
      cout<<"too damn long skipping for debug phase: "<<targ_fs.get_header()<<" "<<targ_fs.get_seq().length()<<endl;
      continue;
    }
    vector<string*> svec = getMatchSeq(targ_fs);
    bool foundAll = true;
    for(unsigned i = 0; i < svec.size(); ++i) {
      if( !svec[i] ) {
	foundAll=false;
      }
    }
    if( !foundAll ) {
      continue;
    }
    for(unsigned i = 1; i < useOrg.size(); ++i) {
      if(!useOrg[i]) {
	svec[i]=NULL;
      }
    }
    cout<<targ_fs.get_header()<<endl;
    ParseHeader ph(ai);
    bool status = ph.parse(targ_fs);
    if( !status ) {
      cout<<"failed "<<targ_fs.get_header()<<endl;
      continue;
    }
    ExonDescrip& ed = ph.getExonDescrip();
    ed.print(cout); cout<<endl;
    ipair_t ip = ed.getStartStop();
    list<SeqLabel> klst = ed.getSeqLabelLst();
    const int start_codon = ip.first;
    const int stop_codon = ip.second; //3rd position of last codon
    const bool isPrc=SeqLabel::isAllPrc(klst,start_codon,stop_codon);
    const bool multiplePrc=ed.isPrcAlt();
    if( !isPrc || multiplePrc ) 
      continue;
    SeqLabel::printLst(cout,klst);
    cout<<"start/stop: "<<start_codon<<" "<<stop_codon<<endl;
    const LabelSeq ls(_mm);
    const bool isGcDon = hasGcDon(*(svec[0]),klst);
    if(isGcDon) {
      cout<<"THIS SEQ has a GC donor\n";
    }
    int scoreFrame=-1;
    const spair_t result = ls.scoreF(svec,start_codon,stop_codon,klst,scoreFrame,predType);
    if( !result.first ) {
      cout<<"MY failed\n";
      ed.printAlignment(cout,svec);
    }
    cout<<"trueres_Score: "<<result<<endl;
    assert(klst.front().getLabel()==SeqLabel::eBeg);
    const int start_pos = klst.front().getLeft();
    assert(klst.back().getLabel()==SeqLabel::eEnd);
    const int stop_pos = klst.back().getLeft();
    LabelSeq ls1(_mm);
    int frame=-1;
    vector<int> incl;
    const list<SeqLabelVal>* pred = ls1.labelF(svec,start_codon,stop_codon,start_pos,stop_pos,frame,runCons,predType,nFrameLimit,nFrameLimit,incl,false,false);
    cout<<"We have a Prediction, please let it not be utter crap! "<<frame<<endl;
    SeqLabelVal::printLst(cout,*pred);
  }
  cout<<"TOTAL:"<<endl;
}

static
list<SeqLabel>::const_iterator
my_find(list<SeqLabel>::const_iterator start, list<SeqLabel>::const_iterator stop, SeqLabel* obj) {
  list<SeqLabel>::const_iterator match = stop;
  if(obj) {
    for(list<SeqLabel>::const_iterator iter = start; iter != stop; ++iter) {
      const SeqLabel& sl = *iter;
      if( sl.getLeft()==obj->getLeft() && sl.getRght()==obj->getRght()) {
	match = iter;
	break;
      }
    }
  }
  return match;
}


list<SeqLabel>
isAllDiCons(const vector<string*>& svec, const std::list<SeqLabel>& slst) {
  list<SeqLabel> allCons;
  const string& str = *svec[0];
  assert(svec[1]);
  list<SeqLabel>::const_iterator ti = slst.begin();
  const list<SeqLabel>::const_iterator st = slst.end();
  for(; ti != st; ++ti) {
    const SeqLabel& sl = *ti;
    const bool isAlt = (sl.getEvent()==SeqLabel::eAlt);
    string sstr;
    const string astr =isAlt?"alt":"con";
    int type = -1;
    if( SeqLabel::isDon(sl.getLabel()) ) {
      type = isAlt ? 3 : 2;
    } else if( SeqLabel::isAcc(sl.getLabel()) ) {
      type = isAlt ? 1 : 0;
    }
    if( type != -1 ) {
      const unsigned oidx = 1;
      if( !sl.isConsDi(str,*(svec[oidx])) ) {
	allCons.push_back(sl);
      }
    }
  }
  return allCons;
}
void
countDiCons(const vector<string*>& svec, const std::list<SeqLabel>& slst, vector<int>& counter, vector<int>& total) {
  const unsigned osize = 2;
  const string& str = *svec[0];
  assert(svec[1]);
  list<SeqLabel>::const_iterator ti = slst.begin();
  const list<SeqLabel>::const_iterator st = slst.end();
  for(; ti != st; ++ti) {
    const SeqLabel& sl = *ti;
    const bool isAlt = (sl.getEvent()==SeqLabel::eAlt);
    string sstr;
    const string astr =isAlt?"alt":"con";
    int type = -1;
    if( SeqLabel::isDon(sl.getLabel()) ) {
      sstr="don_";
      type = isAlt ? 3 : 2;
    } else if( SeqLabel::isAcc(sl.getLabel()) ) {
      sstr="acc_";
      type = isAlt ? 1 : 0;
    }
    if( type != -1 ) {
      for(unsigned oidx = 1; oidx < osize; ++oidx) {
	if( svec[oidx] ) {
	  if( sl.isConsDi(str,*(svec[oidx])) ) {
	    assert(type!=-1);
	    ++counter[type];
	  }
	  ++total[type];
	}
      }
    }
  }
  const int slen = (signed)str.size();
  int start_pos = slst.front().getLeft();
  const int beg = start_pos < 6 ? 6 : start_pos+1;
  for(unsigned i = beg; i < str.size(); ++i) {
    ipair_t ap = SeqLabel::getSigDim1(SeqLabel::eAcc,i);
    ipair_t dp = SeqLabel::getSigDim1(SeqLabel::eDon,i);
    SeqLabel* sl = NULL;
    string sstr;
    int type=-1;
    if( tolower(str[i-4]) == 'a' && tolower(str[i-3])=='g' &&
	ap.first > start_pos && ap.second <= slen ) {
      sl = new SeqLabel(SeqLabel::eAcc,SeqLabel::eUnk,ap.first,ap.second);
      sstr="donfalse";
      type=5;
    }
    //if( tolower(str[i-6]) == 'g' && (tolower(str[i-5])=='t' ||tolower(str[i-5])=='c')
    if( tolower(str[i-6]) == 'g' && tolower(str[i-5])=='t'
	&&
	dp.first > start_pos && dp.second <= slen ) {
      sl = new SeqLabel(SeqLabel::eDon,SeqLabel::eUnk,dp.first,dp.second);
      sstr="accfalse";
      type=4;
    }
    list<SeqLabel>::const_iterator match = my_find(slst.begin(),slst.end(),sl);
    if( sl && match == slst.end() ) {
      for(unsigned oidx = 1; oidx < osize; ++oidx) {
	if( svec[oidx] ) {
	  if( sl->isConsDi(str,*(svec[oidx])) ) {
	    ++counter[type];
	  }
	  ++total[type];
	}
      }
      delete sl;
    }
  }
}
