#ifndef _PLOTCONS_H
#define _PLOTCONS_H

#include "Phylogeny.h"
#include "Model.h"
#include "seqtype.h"
#include "Histogram3d.h"
#include "SeqLabel.h"
#include <vector>
#include <utility>
#include <string>
#include <list>

class PlotCons {
 public:
  typedef SeqLabel::blabel_t blabel_t;
  typedef std::pair<int,int> ipair_t;

 public:
  PlotCons(const Phylogeny&, const std::vector<Model*>&,const std::vector<fasta>&,
	   const std::vector<std::string>&, std::vector<Histogram::pair_t>&);

  void plot(const fasta_seq&,const std::list<SeqLabel>& ); 
  const std::vector<Histogram3d>& getHist() const { return _hist; }

 private:
  std::vector<const std::string*>
    getMatchSeq(const fasta_seq& fs) const;

  double
    computeSpliceScore(const std::vector<const std::string*>&, const ipair_t, blabel_t) const;

  double
    computeConsScore(const std::vector<const std::string*>&, const ipair_t bnd) const;

 private:
  const Phylogeny& _phylo;
  const std::vector<Model*>& _model;
  const std::vector<fasta>& _infSeq;
  std::vector<std::string> _infln;
  std::vector<Histogram::pair_t> _mmHist;
  MarkovChain* _equal;
  double _tnTvRatio, _nonFunMutRate;
  std::vector<Histogram3d> _hist;
};

#endif // _PLOTCONS_H
