#include "SeqLabel.h"
#include <string>
#include <iostream>
#include <vector>

using std::vector;
using std::string;
using std::list;
using std::cout;
using std::endl;

bool
SeqLabel::isOverlap(const SeqLabel& a, const SeqLabel& b) {
  const bool case1 = b.isIn(a.getLeft());
  const bool case2 = b.isIn(a.getRght());
  const bool case3 = a.isIn(b.getLeft());
  const bool case4 = a.isIn(b.getRght());
  return (case1 || case2 || case3 || case4);
}

bool
SeqLabel::isEnd(blabel_t bl) {
  switch(bl) {
  case eEnd:
  case eEnd1:
    return true;
  default:
    return false;
  }
}
bool
SeqLabel::isInner(blabel_t bl) {
  assert(bl!=eAcc&&bl!=eDon&&bl!=eBeg&&bl!=eEnd&&bl!=eEnd1);
  switch(bl) {
  case eCed:
  case eCea:
  case eNea:
  case eNed:
  case eFma1:
    return false;
  default:
    return true;
  }
}

bool
SeqLabel::isConsDi(const string& str1, const string& str2) const {
  int idx1=-1,idx2=-1;
  if(isDon(_label) ) {
    idx1 = getExonBnd()+1;
    idx2 = getExonBnd()+2;
  } else if( isAcc(_label) ) {
    idx1 = getExonBnd()-2;
    idx2 = getExonBnd()-1;
  } else {
    assert(0);
  }
  const bool isCons = str1[idx1-1]==str2[idx1-1] && str1[idx2-1]==str2[idx2-1];
  return isCons;
}

SeqLabel::ipair_t
SeqLabel::getSigDim1(blabel_t st, int pos) {
  switch(st) {
  case eDon:
    return ipair_t(pos-8,pos);
  case eAcc:
    return ipair_t(pos-23,pos);
  case eEnd1:
  case eEnd:
  case eBeg:
    return ipair_t(pos,pos);
  default:
    std::cerr<<"getSigDim error: "<<pos<<endl;
    assert(0);
    return ipair_t(pos,pos);
  }
}

SeqLabel::ipair_t
SeqLabel::getSigDim(blabel_t st, int pos) {
  switch(st) {
  case eDon:
    return ipair_t(pos-2,pos+6);
  case eAcc:
    return ipair_t(pos-22,pos+1);
  case eEnd1:
  case eEnd:
  case eBeg:
    return ipair_t(pos,pos);
  default:
    std::cerr<<"getSigDim error: "<<pos<<endl;
    assert(0);
    return ipair_t(pos,pos);
  }
}

bool
SeqLabel::checkThree(const std::list<SeqLabel>& lst,int startc, int stopc) {
  const list<SeqLabel>::const_iterator stop = lst.end();
  list<SeqLabel>::const_iterator iter = lst.begin();
  ++iter; // splice 1
  const SeqLabel* lPtr = &(*iter);
  bool m3 = true;
  ++iter;
  for(; iter != stop; ++iter) {
    const SeqLabel* curr = &(*iter);
    const bool isCode = ( lPtr->getExonBnd() >= startc && lPtr->getExonBnd() <= stopc &&
			  curr->getExonBnd() >= startc && curr->getExonBnd() <= stopc );
    //if(isCode && curr->isSplice() && curr->getEvent()==eAlt && lPtr->getEvent()==eAlt && curr->getLabel() != eCed && curr->getLabel() != eCea ) {
    if(isCode && curr->isSplice() && lPtr->isSplice() ) {
      assert(lPtr);
      const int ilen = (curr->getExonBnd()-lPtr->getExonBnd())+1;
      m3 = (ilen%3==0);
      if( !m3 ) break;
    }
    lPtr = curr;
  }
  return m3;
}

bool
SeqLabel::isAllPrc(const std::list<SeqLabel>& lst,int startc, int stopc) {
  const list<SeqLabel>::const_iterator stop = lst.end();
  list<SeqLabel>::const_iterator iter = lst.begin();
  ++iter; // splice 1
  bool fail = true;
  for(; iter != stop; ++iter) {
    const SeqLabel* curr = &(*iter);
    const bool isCode = ( curr->getExonBnd() >= startc && curr->getExonBnd() <= stopc );
    if(!isCode && curr->isSplice()) { 
      fail = false;
      break;
    }
  }
  return fail;
}


int 
SeqLabel::getExonBnd() const {
  blabel_t lab = getLabel();
  switch(lab) {
  case eDon:
  case eNed:
  case eCed:
  case eIrAfmd:
  case eIrFmd1:
  case eIrFmd2:
  case eIrSd:
  case eFmd1:
  case eFmd2:
  case eAfmd:
    return getLeft()+2;
  case eAcc:
  case eNea:
  case eCea:
  case eIrFma1:
  case eIrFma2:
  case eIrAfma:
  case eIrSa:
  case eFma1:
  case eFma2:
  case eAfma:
    return getLeft()+22;
  case eBeg:
    return getLeft()+1;
  case eEnd1:
  case eEnd:
    return getRght()-1;
  default:
    assert(getLeft()==getRght());
    return getLeft();
  }
}

bool 
SeqLabel::isMs(blabel_t lab) {
  switch(lab) {
  case eFmd1:
  case eFmd2:
  case eAfmd:
  case eFma1:
  case eFma2:
  case eAfma:
    return true;
  default:
    return false;
  }
}

bool 
SeqLabel::isDon(blabel_t lab) {
  switch(lab) {
  case eDon:
  case eNed:
  case eCed:
  case eIrAfmd:
  case eIrFmd2:
  case eIrFmd1:
  case eIrSd:
  case eFmd1:
  case eFmd2:
  case eAfmd:
    return true;
  default:
    return false;
  }
}

bool 
SeqLabel::isAcc(blabel_t lab) {
  switch(lab) {
  case eAcc:
  case eNea:
  case eCea:
  case eIrAfma:
  case eIrFma1:
  case eIrFma2:
  case eIrSa:
  case eFma1:
  case eFma2:
  case eAfma:
    return true;
  default:
    return false;
  }
}

bool 
SeqLabel::isAccIr(blabel_t lab) {
  switch(lab) {
  case eIrAfma:
  case eIrFma1:
  case eIrFma2:
  case eIrSa:
    return true;
  default:
    return false;
  }
}

bool 
SeqLabel::isDonIr(blabel_t lab) {
  switch(lab) {
  case eIrAfmd:
  case eIrFmd1:
  case eIrFmd2:
  case eIrSd:
    return true;
  default:
    return false;
  }
}

unsigned
SeqLabel::getSigWidth(blabel_t st) {
  blabel_t nst = st;
  if( isAcc(st) ) {
    nst = eAcc;
  } else if(isDon(st)) {
    nst = eDon;
  }
  switch(nst) {
  case eDon:
    return 9;
  case eAcc:
    return 24;
  case eBeg:
  case eEnd1:
  case eEnd:
    return 1;
  default:
    std::cerr<<"getSigWidth error: "<<st<<endl;
    assert(0);
    return 1;
  }
}

SeqLabel::ipair_t
SeqLabel::getSigDimWide(blabel_t st, int pos) {
  switch(st) {
  case eDon:
    return ipair_t(pos-23,pos+26);
  case eAcc:
    return ipair_t(pos-43,pos+21);
  case eBeg:
  case eEnd1:
  case eEnd:
    return ipair_t(pos,pos);
  default:
    std::cerr<<"getSigDimWide error: "<<pos<<endl;
    assert(0);
    return ipair_t(pos,pos);
  }
}

bool
SeqLabel::isSplice() const {
  blabel_t bl = getLabel();
  return !( bl == eBeg || bl == eEnd || bl == eEnd1 || bl==eUndef );
}

std::ostream&
operator<<(std::ostream& os, const SeqLabel& sl) {
  string str = SeqLabel::label2str(sl);
  string estr;
  if(sl.getEvent()==SeqLabel::eAlt) {
    estr = "alt";
  } else if(sl.getEvent()==SeqLabel::eCon) {
    estr = "con";
  } else if(sl.getEvent()==SeqLabel::eUnk) {
    estr = "unk";
  }
  os<<str<<" "<<estr<<" "<<sl.getLeft()<<" "<<sl.getRght();
  return os;
}

string
SeqLabel::label2str(const SeqLabel& sl) {
  string str;
  if(sl.getLabel()==SeqLabel::eDon) {
    str = "don";
  } else if(sl.getLabel()==SeqLabel::eAcc) {
    str = "acc";
  } else if(sl.getLabel()==SeqLabel::eBeg) {
    str = "beg";
  } else if(sl.getLabel()==SeqLabel::eEnd1) {
    str = "end1";
  } else if(sl.getLabel()==SeqLabel::eEnd) {
    str = "end";
  } else if(sl.getLabel()==SeqLabel::eNed) {
    str = "ned";
  } else if(sl.getLabel()==SeqLabel::eCed) {
    str = "ced";
  } else if(sl.getLabel()==SeqLabel::eIrSd) {
    str = "irsd";
  } else if(sl.getLabel()==SeqLabel::eIrFmd1) {
    str = "irfmd1";
  } else if(sl.getLabel()==SeqLabel::eIrFmd2) {
    str = "irfmd2";
  } else if(sl.getLabel()==SeqLabel::eIrAfmd) {
    str = "irafmd";
  } else if(sl.getLabel()==SeqLabel::eFmd1) {
    str = "fmd1";
  } else if(sl.getLabel()==SeqLabel::eFmd2) {
    str = "fmd2";
  } else if(sl.getLabel()==SeqLabel::eAfmd) {
    str = "afmd";
  } else if(sl.getLabel()==SeqLabel::eNea) {
    str = "nea";
  } else if(sl.getLabel()==SeqLabel::eCea) {
    str = "cea";
  } else if(sl.getLabel()==SeqLabel::eIrSa) {
    str = "irsa";
  } else if(sl.getLabel()==SeqLabel::eIrFma1) {
    str = "irfma1";
  } else if(sl.getLabel()==SeqLabel::eIrFma2) {
    str = "irfma2";
  } else if(sl.getLabel()==SeqLabel::eIrAfma) {
    str = "irafma";
  } else if(sl.getLabel()==SeqLabel::eFma1) {
    str = "fma1";
  } else if(sl.getLabel()==SeqLabel::eFma2) {
    str = "fma2";
  } else if(sl.getLabel()==SeqLabel::eAfma) {
    str = "afma";
  } else if(sl.getLabel()==SeqLabel::eUndef) {
    str = "undef";
  }
  return str;
}

void
SeqLabel::printLst(std::ostream& os, const list<SeqLabel>& lst) {
  list<SeqLabel>::const_iterator iter = lst.begin();
  list<SeqLabel>::const_iterator stop = lst.end();
  for(; iter != stop; ++iter) {
    const SeqLabel& sl = *iter;
    os<<sl<<endl;
  }
}

bool 
SeqLabel::isUpstreamIntron(blabel_t left, blabel_t rght) {
  return (left==eBeg && (rght==eAcc||rght==eFma1||rght==eCea||rght==eNea));
}

bool 
SeqLabel::isDnstreamIntron(blabel_t left, blabel_t rght) {
  return (rght==eEnd && (left==eDon||left==eFmd2||left==eAfmd||left==eCed||left==eNed));
}

bool 
SeqLabel::isExon(blabel_t left, blabel_t rght) {
  bool isUp = isUpstreamIntron(left,rght);
  bool isDn = isDnstreamIntron(left,rght);
  return (!isUp && !isDn);
}
