#ifndef _SEQLABEL_H
#define _SEQLABEL_H

#include <utility>
#include <ostream>
#include <list>
#include <string>

class SeqLabel {
 public:
  enum blabel_t { eAcc=0, eDon, 
		  //eOther
		  eBeg, eEnd,eEnd1,
		  // eNed ('N' stands for normal, really meant to signifiy constitutive)
		  eNed, eCed, eIrSd, eIrFmd1, eIrFmd2, eIrAfmd, eFmd1, eFmd2, eAfmd,
		  eNea, eCea, eIrSa, eIrFma1, eIrFma2, eIrAfma, eFma1, eFma2, eAfma, 
		  eUndef
  };
  enum sclass_t { eCassette, eIntronRetention, eConstitutive, eMultipleDonor, eMultipleAcceptor, eSpliceClassSize };
  enum event_t { eAlt=0, eCon, eNone, eUnk };
  enum code_t { eNonCoding, eProteinCoding };

  typedef std::pair<int,int> ipair_t;

 public:
  static bool isEnd(blabel_t);
  static bool isInner(blabel_t);
  static unsigned getSigWidth(blabel_t);
  static ipair_t getSigDim1(blabel_t,int);
  static ipair_t getSigDim(blabel_t,int);
  static ipair_t getSigDimWide(blabel_t,int);
  static sclass_t getSpliceClass(const std::list<SeqLabel>&);
  static std::string
    label2str(const SeqLabel&);
  static bool checkThree(const std::list<SeqLabel>&,int,int);
  static bool isAllPrc(const std::list<SeqLabel>&,int,int);
 public:
  SeqLabel() : _label(eUndef), _event(eUnk), _from(-1), _to(-1) { }
  SeqLabel(blabel_t bl, event_t ev, int f,int t) :
    _label(bl), _event(ev), _from(f), _to(t) { }
    
  SeqLabel(const SeqLabel& sl) :
      _label(sl._label), _event(sl._event), _from(sl._from), _to(sl._to) { 
  }
  bool isIn(int pos) const { return pos >= _from && pos <= _to; }
  int getLeft() const { return _from; }
  int getRght() const { return _to; }
  blabel_t getLabel() const { return _label; }
  event_t getEvent() const { return _event; }
  int getLength() const { return (_to-_from)+1; }
  bool operator==(const SeqLabel& sl) const {
    return sl._label==_label&&
      sl._from==_from && sl._to==_to; 
  }
  bool operator<(const SeqLabel& sl) const {
    return getLeft() < sl.getLeft();
  }
  bool isEnd() const { return SeqLabel::isEnd(_label); }
  bool isSplice() const;
  int getExonBnd() const;
  bool isConsDi(const std::string&, const std::string&) const;

  void setLabel(blabel_t bl) { _label=bl; }
  void setEvent(event_t ev) { _event=ev; }
  void setEnds(int f, int t) { _from = f; _to = t; }

  static bool isOverlap(const SeqLabel&, const SeqLabel&);
  static bool
    isMs(blabel_t);
  static bool
    isAccIr(blabel_t);
  static bool
    isDonIr(blabel_t);
  static bool
    isAcc(blabel_t);
  static bool
    isDon(blabel_t);

  static bool 
    isUpstreamIntron(blabel_t left, blabel_t rght);
  static bool 
    isDnstreamIntron(blabel_t left, blabel_t rght);
  static bool 
    isExon(blabel_t left, blabel_t rght);

  static bool isConnectable(blabel_t,blabel_t);
  static void
    printLst(std::ostream&,const std::list<SeqLabel>&);

 private:
  blabel_t _label;
  event_t _event;
  int _from, _to;
};

std::ostream&
operator<<(std::ostream&,const SeqLabel&);

#endif // _SEQLABEL_H
