#include "StateTrans.h"
#include <fstream>
#include <iomanip>
#include <iostream>
#include <string>
#include <cmath>

using std::setprecision;
using std::ostream;
using std::setw;
using std::ofstream;
using std::ifstream;
using std::cout;
using std::endl;
using std::string;

StateTrans::StateTrans() : _init(eStateSize,0.0), _trans(eStateSize), _length(eStateSize) {
  const double mn = 1, mx = 1000;
  const int nbin = 5;
  for(unsigned i = 0; i < _trans.size(); ++i) {
    _trans[i].resize(eStateSize,0);
    _length[i].init(mn,mx,nbin,1.0);
  }
  // this defines the actual "legal" transitions 
  // which must have non-zero prob, even if not found in training
  _init[eCUIN] = 1;
  _init[eAUIN] = 1;
  _init[eFFEIR] = 1;
  _init[eFCONE] = 1;
  _init[eFMDE] = 1;
  _init[eAINO] = 1;
  _init[eCINO] = 1;
  _init[eAEXO] = 1;
  _init[eCEXO] = 1;

  _trans[eCUIN][eSAE]=1;
  _trans[eCUIN][eNCONE]=1;
  _trans[eCUIN][eNFEIR]=1;
  _trans[eCUIN][eLCONE]=1;

  _trans[eAUIN][eMAE1]=1;
  _trans[eAUIN][eCE]=1;

  _trans[eFCONE][eCDIN]=1;
}
#if 0
static void tprn() {
  for(unsigned i =0; i < StateTrans::eSize; ++i) {
    cout<<"init["<<StateTrans::state2str((state_t)i)<<"]=1"<<endl;
  }
  for(unsigned i =0; i < StateTrans::eSize; ++i) {
    for(unsigned j =0; j < StateTrans::eSize; ++j) {
      cout<<"trans["<<StateTrans::state2str((state_t)i)<<"]["<<StateTrans::state2str((state_t)j)<<"]=1"<<endl;
    }
  }
}
#endif

StateTrans::StateTrans(const StateTrans& st) : _init(st._init.size(),0.0), _trans(st._trans.size()), _length(st._length) {
  for(unsigned i =0; i < _init.size(); ++i) {
    _init[i] = st._init[i];
  }
  for(unsigned i =0; i < _trans.size(); ++i) {
    _trans[i].resize(st._trans[i].size(),0);
    for(unsigned j =0; j < _trans[i].size(); ++j) {
      _trans[i][j] = st._trans[i][j];
    }
  }
}

StateTrans&
StateTrans::operator+=(const StateTrans& st) {
  for(unsigned i = 0; i < st._init.size(); ++i) {
    _init[i] += st._init[i];
  }
  for(unsigned i = 0; i < st._trans.size(); ++i) {
    for(unsigned j = 0; j < st._trans[i].size(); ++j) {
      _trans[i][j] += st._trans[i][j];
    }
  }
  for(unsigned i = 0; i < st._length.size(); ++i) {
    _length[i] += st._length[i];
  }
  return *this;
}

StateTrans&
StateTrans::operator/=(int den) {
  for(unsigned i = 0; i < _init.size(); ++i) {
    _init[i] /= den;
  }
  for(unsigned i = 0; i < _trans.size(); ++i) {
    for(unsigned j = 0; j < _trans[i].size(); ++j) {
      _trans[i][j] /= den;
    }
  }
  for(unsigned i = 0; i < _length.size(); ++i) {
    _length[i] /= den;
  }
  return *this;
}


StateTrans::state_t
StateTrans::label2state(const SeqLabel& left, const SeqLabel& rght, bool debug) {
  state_t res = eStateSize;
  const SeqLabel::blabel_t llabel = left.getLabel();
  const SeqLabel::blabel_t rlabel = rght.getLabel();
  if( llabel == SeqLabel::eBeg && SeqLabel::isAcc(rlabel) && rght.getEvent()==SeqLabel::eCon) {
    res = eCUIN;
  } 
  else if( llabel == SeqLabel::eBeg && rlabel == SeqLabel::eEnd1 && rght.getEvent()==SeqLabel::eCon ) {
    res = eCEXO;
  }
  else if( llabel == SeqLabel::eBeg && rlabel == SeqLabel::eEnd1 && rght.getEvent()==SeqLabel::eAlt ) {
    res = eAEXO;
  }
  else if( llabel == SeqLabel::eBeg && rlabel == SeqLabel::eEnd && rght.getEvent()==SeqLabel::eCon ) {
    res = eCINO;
  }
  else if( llabel == SeqLabel::eBeg && rlabel == SeqLabel::eEnd && rght.getEvent()==SeqLabel::eAlt ) {
    res = eAINO;
  }
  else if( llabel == SeqLabel::eBeg && rlabel == SeqLabel::eNed) {
    res = eFCONE;
  }
  else if( llabel == SeqLabel::eBeg && (rlabel==SeqLabel::eCea || rlabel==SeqLabel::eFma1)) {
    res = eAUIN;
  }
  else if( llabel == SeqLabel::eBeg && rlabel == SeqLabel::eFmd1 ) {
    res = eFMDE;
  }
  else if( llabel == SeqLabel::eFma1 && rlabel == SeqLabel::eFma2 ) {
    res = eMAE1;
  }
  else if( (llabel==SeqLabel::eAfma||llabel==SeqLabel::eFma2) && rlabel == SeqLabel::eAfma ) {
    res = eMAE2;
  }
  else if( llabel == SeqLabel::eNea && rlabel == SeqLabel::eFmd1 ) {
    res = eSAE;
  }
  else if( llabel == SeqLabel::eCea && rlabel == SeqLabel::eCed) {
    res = eCE;
  }
  else if(llabel==SeqLabel::eNea && rlabel==SeqLabel::eNed) {
    res = eNCONE;
  }
  else if((llabel==SeqLabel::eAfma||llabel==SeqLabel::eFma2) && rlabel == SeqLabel::eFmd1) {
    res = eDBLALT;
  }
  else if((llabel==SeqLabel::eAfma||llabel==SeqLabel::eFma2) && rlabel==SeqLabel::eNed ) {
    res = eSDE;
  }
  else if(llabel == SeqLabel::eFmd1 && rlabel == SeqLabel::eFmd2) {
    res = eMDE1;
  }
  else if( (llabel==SeqLabel::eAfmd||llabel==SeqLabel::eFmd2) && rlabel == SeqLabel::eAfmd ) {
    res = eMDE2;
  }
  else if( (llabel==SeqLabel::eFmd2||llabel==SeqLabel::eCed) && rlabel == SeqLabel::eEnd) {
    res = eADIN;
  }
  else if((llabel == SeqLabel::eAfma || llabel==SeqLabel::eFma2) && rlabel == SeqLabel::eEnd) {
    res = eLMAE;
  }
  else if( llabel == SeqLabel::eNea && rlabel == SeqLabel::eEnd) {
    res = eLCONE;
  }
  else if( SeqLabel::isDon(llabel) && left.getEvent()==SeqLabel::eCon && rlabel == SeqLabel::eEnd) {
    res = eCDIN;
  }
  else if( llabel==SeqLabel::eNea && (rlabel==SeqLabel::eIrSd || rlabel==SeqLabel::eIrFmd1)) {
    res = eNFEIR;
  }
  else if( llabel==SeqLabel::eBeg && (rlabel==SeqLabel::eIrSd||rlabel==SeqLabel::eIrFmd1)) {
    res = eFFEIR;
  }
  else if( (llabel==SeqLabel::eIrSa ||llabel==SeqLabel::eIrFma2||llabel==SeqLabel::eIrAfma)
	   && rlabel==SeqLabel::eNed) {
    res = eNLEIR;
  }
  else if( (llabel==SeqLabel::eIrSa||llabel==SeqLabel::eIrFma2||llabel==SeqLabel::eIrAfma) 
	   && rlabel==SeqLabel::eEnd) {
    res = eLLEIR;
  }
  else if( (llabel==SeqLabel::eIrAfmd||llabel==SeqLabel::eIrFmd2) && rlabel==SeqLabel::eIrSa ) {
    res = eRIMD;
  }
  else if( llabel==SeqLabel::eIrSd && rlabel==SeqLabel::eIrFma1 ) {
    res = eRIMA;
  }
  else if( llabel==SeqLabel::eIrSd && rlabel==SeqLabel::eIrSa ) {
    res = eRI;
  }
  else if( llabel==SeqLabel::eIrFmd1 && rlabel==SeqLabel::eIrFmd2) {
    res = eMD1IR;
  }
  else if( (llabel==SeqLabel::eIrFmd2 || llabel==SeqLabel::eIrAfmd) && rlabel==SeqLabel::eIrAfmd) {
    res = eMD2IR;
  }
  else if( (llabel==SeqLabel::eIrFma2 || llabel==SeqLabel::eIrAfma) && rlabel==SeqLabel::eIrAfma) {
    res = eMA2IR;
  }
  else if( llabel==SeqLabel::eIrFma1 && rlabel==SeqLabel::eIrFma2) {
    res = eMA1IR;
  }
  else if( (llabel==SeqLabel::eFma2 || llabel==SeqLabel::eAfma) && rlabel==SeqLabel::eIrFmd1) {
    res = eDBLALTFE;
  }
  else if( (llabel==SeqLabel::eIrFma2 || llabel==SeqLabel::eIrAfma) && rlabel==SeqLabel::eIrFmd1) {
    res = eDBLALTLE;
  }
  else if( (llabel==SeqLabel::eIrFmd2 || llabel==SeqLabel::eIrAfmd) && rlabel==SeqLabel::eIrFma1) {
    res = eDBLALTRI;
  }
  else if( (llabel==SeqLabel::eAfma || llabel==SeqLabel::eFma2) && rlabel==SeqLabel::eIrSd) {
    res = eSDIR;
  }
  else if( llabel==SeqLabel::eIrSa && rlabel==SeqLabel::eFmd1) {
    res = eSAIR;
  }
  else {
    if(debug) { 
      cout<<"Warning failed to classify: "<<left<<" "<<rght<<endl; 
    }
  }
  return res;
}

double 
StateTrans::getProb(const SeqLabel& left, const SeqLabel& rght) const {
  state_t st = label2state(left,rght,false);
  const int len = (rght.getExonBnd()-left.getExonBnd())+1;
  double val =  getProb(st,len);
  if( st == eAINO || st == eCINO || st==eAEXO || st==eCEXO ) {
    val = 0.01;
  } else if( st == eFMDE ) {
    //val = 0.05;
    if( len < 60 ) {
      val *= 0.01;
    }
  } else if( st == eFFEIR ) {
    //val = 0.05;
    if( len < 65 ) {
      val *= 0.01;
    }
  }
  return val;
}

double 
StateTrans::getProb(const SeqLabel& pleft, const SeqLabel& prght,
		    const SeqLabel& cleft, const SeqLabel& crght) const {
  state_t pst = label2state(pleft,prght,false);
  state_t cst = label2state(cleft,crght,false);
  const int len = (crght.getExonBnd()-cleft.getExonBnd())+1;
  const bool presRF = (((len-1)%3)==0);
  double val = getProb(pst,cst,len);
#if 1
  if( cst == eDBLALTRI || cst == eDBLALTLE || cst == eDBLALTFE || cst==eDBLALT ||
      cst == eMA2IR || cst == eMD2IR || cst == eMDE2 || cst == eMAE2 ||
      cst == eSDIR || cst == eSAIR || cst == eRIMA || cst == eRIMD 
      ) {
    val = 0.0000001;
  } else if( cst == eMD1IR || cst == eMA1IR ) {
    val = 0.001;
  } else if( cst == eSDE || cst == eSAE || cst==eLMAE ) {
    //val = 0.05;
    if( len < 60 ) {
      val *=0.001;
    }
  } else if( cst == eNFEIR ) {
    //val = 0.05;
    if( len < 60 ) {
      val*=0.01;
    }
  } else if( (cst == eNLEIR || cst == eLLEIR) && len < 60 ) {
    val = 0.0001;
  } else if( cst == eMAE1 && presRF && len < 20 ) {
    val *= 75.0;
    //val *= 225.0;
#if 0
  } else if( (cst == eMDE1 || cst == eMAE1) && !presRF ) {
    val = 0.001; 
#endif
  }
#endif
  return val;
}

double 
StateTrans::getProb(state_t from, state_t to, int len) const {
  if( from != eStateSize && to != eStateSize) {
    double val = _length[to].get_prob(len);
    bool isM3 = (((len-1)%3) == 0 );
      //val*=1.5; // check3.log
      //val*=2.5; // check3a.log
    //} else 
    if( len < 21 && (to==eCE||to==eNCONE)) {
    //if( len < 21 && (to==eNCONE)) {
      val=0.00001;
    }
    //else if( len < 61 && (to==eCE||to==eNCONE)) {
    else if( len < 61 && (to==eNCONE || (to==eCE && !isM3))) {
      val=0.01;
    }
    if( len < 25 && (to==eNLEIR||to==eLLEIR)) {
      val=0.001;
    }
    return _trans[from][to]*val;
  } else {
    return 2;
  }
}

void 
StateTrans::update(const SeqLabel& pleft, const SeqLabel& prght,
		   const SeqLabel& cleft, const SeqLabel& crght) {
  state_t pst = label2state(pleft,prght,false);
  state_t cst = label2state(cleft,crght,false);
  const int len = (crght.getExonBnd()-cleft.getExonBnd())+1;
  if(pst==eAUIN && cst == eSDE) {
  } else {
    update(pst,cst,len);
  }
}

void 
StateTrans::update(const SeqLabel& left, const SeqLabel& rght) {
  state_t st = label2state(left,rght,false);
  const int len = (rght.getExonBnd()-left.getExonBnd())+1;
  update(st,len);
}

void 
StateTrans::update(state_t init, int len) {
  if( init != eStateSize) {
    _init[init]++;
	 if(len > 0 ) {
	   _length[init].store(len);
    }
  }
}

void 
StateTrans::update(state_t from, state_t to, int len) {
  if( from != eStateSize && to != eStateSize) {
    _trans[from][to]++;
	 if(len > 0 ) {
	    _length[to].store(len);
    }
  }
}

double 
StateTrans::getProb(state_t is, int len) const {
  if( is != eStateSize) {
    //return 1;
    const double val = _length[is].get_prob(len);
    return _init[is]*val;
  } else {
    return 2;
  }
}


void
StateTrans::normalize() {
  double isum =0;
  for(unsigned i = 0; i < _init.size(); ++i) {
    isum += _init[i];
  }
  for(unsigned i = 0; i < _init.size(); ++i) {
    _init[i] = isum >0 ? _init[i]/isum : 0;
  }
  for(unsigned i = 0; i < _trans.size(); ++i) {
    double tsum = 0;
    for(unsigned j = 0; j < _trans.size(); ++j) {
      tsum += _trans[i][j];
    }
    for(unsigned j = 0; j < _trans.size(); ++j) {
      _trans[i][j] = tsum > 0 ? _trans[i][j]/tsum : 0;
    }
  }
  for(unsigned i = 0; i < _length.size(); ++i) {
    _length[i].finish();
  }
}

void
StateTrans::read(const std::string& str) {
  ifstream ifs(str.c_str());
  if( !ifs ) {
    cout<<"Cloud not open "<<str<<endl;
    assert(0);
  }
  string dum;
  ifs>>dum;
  for(unsigned i = 0; i < _init.size(); ++i) {
    ifs>>_init[i];
  }
  ifs>>dum;
  for(unsigned i = 0; i < _trans.size(); ++i) {
    for(unsigned j = 0; j < _trans.size(); ++j) {
      ifs>>_trans[i][j];
    }
  }
  ifs>>dum;
  for(unsigned i = 0; i < _length.size(); ++i) {
    ifs>>_length[i];
  }
  //print(cout); cout<<endl;
}

void
StateTrans::write(const std::string& str) const {
  ofstream ofs(str.c_str());
  if( !ofs ) {
    cout<<"Cloud not open "<<str<<endl;
    assert(0);
  }
  ofs<<"InitTrans"<<endl;
  for(unsigned i = 0; i < _init.size(); ++i) {
    ofs<<_init[i];
    if( i < _init.size()-1 ) {
      ofs<<" ";
    }
  }
  ofs<<endl;
  ofs<<"TransProb"<<endl;
  for(unsigned i = 0; i < _trans.size(); ++i) {
    //ofs<<i<<" ";
    for(unsigned j = 0; j < _trans.size(); ++j) {
      //ofs<<setprecision(3)<<setw(8)<<_trans[i][j];
      ofs<<_trans[i][j];
      if(j < _trans.size()-1)
	ofs<<" ";
    }
    ofs<<endl;
  }
  ofs<<"LengthProb"<<endl;
  for(unsigned i = 0; i < _length.size(); ++i) {
    ofs<<_length[i]<<endl;
  }
}

void
StateTrans::print(ostream& os) const {
  os<<"InitTrans"<<endl;
  for(unsigned i = 0; i < _init.size(); ++i) {
    if( _init[i] >0) {
      os<<i<<","<<state2str((state_t)i)<<" "<<_init[i];
      if( i < _init.size()-1 ) {
	os<<" ";
      }
    }
  }
  os<<endl;
  os<<"TransProb"<<endl;
  for(unsigned i = 0; i < _trans.size(); ++i) {
    for(unsigned j = 0; j < _trans.size(); ++j) {
      //ofs<<setprecision(3)<<setw(8)<<_trans[i][j];
      if( _trans[i][j] > 0) {
	os<<i<<","<<state2str((state_t)i)<<" "<<j<<","<<state2str((state_t)j)<<" "<<_trans[i][j];
	if(j < _trans.size()-1)
	  os<<" ";
      }
    }
    os<<endl;
  }
  os<<"LengthProb"<<endl;
  for(unsigned i = 0; i < _length.size(); ++i) {
    os<<state2str((state_t)i)<<endl;
    os<<_length[i]<<endl;
  }
}

string
StateTrans::state2str(state_t st) {
  string res;
  switch(st) {
  case eAEXO:
    res="eAEXO";
    break;
  case eCEXO:
    res="eCEXO";
    break;
  case eAINO:
    res ="eAINO";
    break;
  case eCINO:
    res ="eCINO";
    break;
  case eCUIN:
    res ="eCUIIN";
    break;
  case eAUIN: 
    res ="eAUIN";
    break;
  case eNCONE: 
    res ="eNCONE";
    break;
  case eSAE: 
    res ="eSAE";
    break;
  case eMAE1: 
    res ="eMAE1";
    break;
  case eMAE2: 
    res ="eMAE2";
    break;
  case eCE: 
    res ="eCE";
    break;
  case eSDE: 
    res ="eSDE";
    break;
  case eMDE1: 
    res ="eMDE1";
    break;
  case eMDE2: 
    res ="eMDE2";
    break;
  case eCDIN: 
    res ="eCDIN";
    break;
  case eADIN:
    res ="eADIN";
    break;
  case eNFEIR: 
    res ="eNFEIR";
    break;
  case eFFEIR: 
    res ="eFFEIR";
    break;
  case eNLEIR: 
    res ="eNLEIR";
    break;
  case eLLEIR: 
    res ="eLLEIR";
    break;
  case eRIMA: 
    res ="eRIMA";
    break;
  case eRIMD: 
    res ="eRIMD";
    break;
  case eRI: 
    res ="eRI";
    break;
  case eMD1IR: 
    res ="eMD1IR";
    break;
  case eMD2IR: 
    res ="eMD2IR";
    break;
  case eMA1IR: 
    res ="eMA1IR";
    break;
  case eMA2IR:
    res ="eMA2IR";
    break;
  case eDBLALTFE: 
    res ="eDLBALTFE";
    break;
  case eDBLALTLE: 
    res ="eDBLALTLE";
    break;
  case eDBLALTRI: 
    res ="eDBLALTRI";
    break;
  case eSDIR: 
    res ="eSDIR";
    break;
  case eSAIR:
    res ="eSAIR";
    break;
  case eFCONE: 
    res ="eFCONE";
    break;
  case eLCONE: 
    res ="eLCONE";
    break;
  case eFMDE: 
    res ="eFMDE";
    break;
  case eLMAE: 
    res ="eLMAE";
    break;
  case eDBLALT: 
    res ="eDBLALT";
    break;
  case eStateSize:
    res ="eSTATESIZE";
  default:
    res = "error";
  }
  return res;
}

void
StateTrans::drawGraphViz() const {
  cout<<"digraph G {"<<endl;

  //cout<<"size=\"9.5,10\";"<<endl;
  cout<<"fontsize=\"48\";"<<endl;
  cout<<"dpi=\"600\";"<<endl;
  cout<<"graph [color=grey,style=\"filled\"];"<<endl;
  cout<<"fontsize=24;"<<endl;
  cout<<"arrowsize=2.0;"<<endl;
  //cout<<"node [fontsize=\"48\",shape=box,style=\"filled\",color=\".6 .6 1.0\"];"<<endl;
  cout<<"node [fontsize=\"48\",shape=box];"<<endl;

  //cout<<"100[label=\"\"];"<<endl;
  for(unsigned i = 0; i < eStateSize; ++i) {
    cout<<i<<"[label=\""<<state2strReadable((state_t)i)<<"\"";
    cout<<" color=\""<<state2color((state_t)i)<<"\"";
    cout<<" fillcolor=\""<<state2color((state_t)i)<<"\"";
    const string val = state2pen((state_t)i);
    if( val == "rounded" ) {
      cout<<" style=\""<<val<<",filled";
    } else {
      cout<<" style=\"filled";
      cout<<"\" shape=\""<<val;
    }
    cout<<"\"];"<<endl;
  }
#if 0
  for(unsigned i = 0; i < eStateSize; ++i) {
    const double prob = _init[i];
    if( prob > 0 ) {
      cout<<100<<"->"<<i<<";"<<endl;
    }
  }
#endif
  for(unsigned i = 0; i < eStateSize; ++i) {
    for(unsigned j = 0; j < eStateSize; ++j) {
      const double prob = _trans[i][j];
      if( prob > 0 ) {
	cout<<i<<"->"<<j<<";"<<endl;
      }
    }
  }
  cout<<"}"<<endl;
  exit(0);
}

string
StateTrans::state2color(state_t st) {
  string res="light blue";
  switch(st) {
  case eAEXO:
    //res ="Alternative Exon\\nOnly";
    res ="purple";
    break;
  case eCEXO:
    //res ="Contitutive Exon\\nOnly";
    res ="purple";
    break;
  case eAINO:
    //res ="Alternative Intron\\nOnly";
    res ="red";
    break;
  case eCINO:
    //res ="Contitutive Intron\\nOnly";
    res = "red";
    break;
  case eCUIN:
    //res ="Uptream\\nConstitutive\\nIntron";
    res = "red";
    break;
  case eAUIN: 
    //res ="Upstream\\nAlternative\\nIntron";
    res = "red";
    break;
  case eNCONE: 
    //res= "Internal\\nConstitutive Exon";
    res ="purple";
    break;
  case eSAE: 
    //res ="Constitutive\\nAcceptor";
    res = "purple";
    break;
  case eMAE1: 
    //res ="Multiple\\nAcceptor 1";
    break;
  case eMAE2: 
    //res ="Multiple\\nAcceptor 2";
    break;
  case eCE: 
    //res ="Cassette\\nExon";
    res = "purple";
    break;
  case eSDE: 
    //res ="Constitutive\\nDonor";
    res = "purple";
    break;
  case eMDE1: 
    //res ="Multiple\\nDonor 1";
    break;
  case eMDE2: 
    //res ="Multiple\\nDonor 2";
    break;
  case eCDIN: 
    //res ="Downstream\\nConstitutive Intron";
    res = "red";
    break;
  case eADIN:
    //res ="Downstream\\nAlternative Intron";
    res = "red";
    break;
  case eNFEIR: 
    //res ="Internal\\nFirst Exon IR";
    res = "purple";
    break;
  case eFFEIR: 
    //res ="Initial\\nFirst Exon IR";
    res = "purple";
    break;
  case eNLEIR: 
    //res ="Internal\\nLast Exon IR";
    res = "purple";
    break;
  case eLLEIR: 
    //res ="Terminal\\nLast Exon IR";
    res = "purple";
    break;
  case eRIMA: 
    //res ="Retained\\nIntron MA";
    res = "red";
    break;
  case eRIMD: 
    //res ="Retained\\nIntron MD";
    res = "red";
    break;
  case eRI: 
    //res ="Retained\\nIntron";
    res = "red";
    break;
  case eMD1IR: 
    //res ="Multiple\\nDonor 1 IR";
    break;
  case eMD2IR: 
    //res ="Multiple\\nDonor 2 IR";
    break;
  case eMA1IR: 
    //res ="Multiple\\nAcceptor 1 IR";
    break;
  case eMA2IR:
    //res ="Multiple\\nAcceptor 2 IR";
    break;
  case eDBLALTFE: 
    //res ="Double Alternative\\nInitial Exon";
    res = "purple";
    break;
  case eDBLALTLE: 
    //res ="Double Alternative\\nTerminal Exon";
    res = "purple";
    break;
  case eDBLALTRI: 
    //res ="Double Alternative\\nRetained Intron";
    res = "red";
    break;
  case eSDIR: 
    //res ="Constitutive\\nDonor IR";
    res = "red";
    break;
  case eSAIR:
    //res ="Constitutive\\nAcceptor IR";
    res = "red";
    break;
  case eFCONE: 
    ////res ="Initial\\nConstitutive Exon";
    res = "purple";
    break;
  case eLCONE: 
    //res ="Terminal\\nConstitutive Exon";
    res = "purple";
    break;
  case eFMDE: 
    //res ="Initial\\nMultiple Donor";
    res = "purple";
    break;
  case eLMAE: 
    //res ="Terminal\\nMultiple Acceptor";
    res = "purple";
    break;
  case eDBLALT: 
    //res ="Double Alternative\\nInternal";
    res = "purple";
    break;
    //case eStateSize:
        //res ="eSTATESIZE";
  default:
    res = "error";
  }
  return res;
}

string
StateTrans::state2strReadable(state_t st) {
  string res;
  switch(st) {
  case eAEXO:
    res ="Alternative Exon\\nOnly";
    break;
  case eCEXO:
    res ="Contitutive Exon\\nOnly";
    break;
  case eAINO:
    res ="Alternative Intron\\nOnly";
    break;
  case eCINO:
    res ="Contitutive Intron\\nOnly";
    break;
  case eCUIN:
    res ="Uptream\\nConstitutive\\nIntron";
    break;
  case eAUIN: 
    res ="Upstream\\nAlternative\\nIntron";
    break;
  case eNCONE: 
    res= "Internal\\nConstitutive Exon";
    break;
  case eSAE: 
    res ="Single\\nAcceptor";
    break;
  case eMAE1: 
    res ="Multiple\\nAcceptor 1";
    break;
  case eMAE2: 
    res ="Multiple\\nAcceptor 2";
    break;
  case eCE: 
    res ="Cassette\\nExon";
    break;
  case eSDE: 
    res ="Single\\nDonor";
    break;
  case eMDE1: 
    res ="Multiple\\nDonor 1";
    break;
  case eMDE2: 
    res ="Multiple\\nDonor 2";
    break;
  case eCDIN: 
    res ="Downstream\\nConstitutive Intron";
    break;
  case eADIN:
    res ="Downstream\\nAlternative Intron";
    break;
  case eNFEIR: 
    res ="Internal\\nFirst Exon IR";
    break;
  case eFFEIR: 
    res ="Initial\\nFirst Exon IR";
    break;
  case eNLEIR: 
    res ="Internal\\nLast Exon IR";
    break;
  case eLLEIR: 
    res ="Terminal\\nLast Exon IR";
    break;
  case eRIMA: 
    res ="Retained\\nIntron MA";
    break;
  case eRIMD: 
    res ="Retained\\nIntron MD";
    break;
  case eRI: 
    res ="Retained\\nIntron";
    break;
  case eMD1IR: 
    res ="Multiple\\nDonor 1 IR";
    break;
  case eMD2IR: 
    res ="Multiple\\nDonor 2 IR";
    break;
  case eMA1IR: 
    res ="Multiple\\nAcceptor 1 IR";
    break;
  case eMA2IR:
    res ="Multiple\\nAcceptor 2 IR";
    break;
  case eDBLALTFE: 
    res ="Double Alternative\\nInitial Exon";
    break;
  case eDBLALTLE: 
    res ="Double Alternative\\nTerminal Exon";
    break;
  case eDBLALTRI: 
    res ="Double Alternative\\nRetained Intron";
    break;
  case eSDIR: 
    res ="Single\\nDonor IR";
    break;
  case eSAIR:
    res ="Single\\nAcceptor IR";
    break;
  case eFCONE: 
    res ="Initial\\nConstitutive Exon";
    break;
  case eLCONE: 
    res ="Terminal\\nConstitutive Exon";
    break;
  case eFMDE: 
    res ="Initial\\nMultiple Donor";
    break;
  case eLMAE: 
    res ="Terminal\\nMultiple Acceptor";
    break;
  case eDBLALT: 
    res ="Double Alternative\\nInternal";
    break;
  case eStateSize:
    res ="eSTATESIZE";
  default:
    res = "error";
  }
  return res;
}

string
StateTrans::state2pen(state_t st) {
  string res="box";
  const string def="rounded";
  const string ss = "diamond";
  switch(st) {
  case eAEXO:
    //res ="Alternative Exon\\nOnly";
    //res ="purple";
    res = ss;
    break;
  case eCEXO:
    //res ="Contitutive Exon\\nOnly";
    //res ="purple";
    res = ss;
    break;
  case eAINO:
    //res ="Alternative Intron\\nOnly";
    //res ="red";
    break;
  case eCINO:
    //res ="Contitutive Intron\\nOnly";
    //res = "red";
    res = ss;
    break;
  case eCUIN:
    //res ="Uptream\\nConstitutive\\nIntron";
    //res = "red";
    break;
  case eAUIN: 
    //res ="Upstream\\nAlternative\\nIntron";
    //res = "red";
    break;
  case eNCONE: 
    //res= "Internal\\nConstitutive Exon";
    res = def; //"purple";
    break;
  case eSAE: 
    //res ="Constitutive\\nAcceptor";
    res = def;
    //res = "purple";
    break;
  case eMAE1: 
    //res ="Multiple\\nAcceptor 1";
    res = def;
    break;
  case eMAE2: 
    //res ="Multiple\\nAcceptor 2";
    res = def;
    break;
  case eCE: 
    //res ="Cassette\\nExon";
    //res = "purple";
    res = def;
    break;
  case eSDE: 
    //res ="Constitutive\\nDonor";
    //res = "purple";
    res = def;
    break;
  case eMDE1: 
    //res ="Multiple\\nDonor 1";
    res = def;
    break;
  case eMDE2: 
    //res ="Multiple\\nDonor 2";
    res = def;
    break;
  case eCDIN: 
    //res ="Downstream\\nConstitutive Intron";
    //res = "red";
    break;
  case eADIN:
    //res ="Downstream\\nAlternative Intron";
    //res = "red";
    break;
  case eNFEIR: 
    //res ="Internal\\nFirst Exon IR";
    res = def; //"purple";
    break;
  case eFFEIR: 
    //res ="Initial\\nFirst Exon IR";
    res = def; //"purple";
    break;
  case eNLEIR: 
    //res ="Internal\\nLast Exon IR";
    res = def; //"purple";
    break;
  case eLLEIR: 
    //res ="Terminal\\nLast Exon IR";
    //res = "purple";
    break;
  case eRIMA: 
    //res ="Retained\\nIntron MA";
    res = def; //"red";
    break;
  case eRIMD: 
    //res ="Retained\\nIntron MD";
    res = def; //"red";
    break;
  case eRI: 
    //res ="Retained\\nIntron";
    res = def; //"red";
    break;
  case eMD1IR: 
    //res ="Multiple\\nDonor 1 IR";
    res = def;
    break;
  case eMD2IR: 
    //res ="Multiple\\nDonor 2 IR";
    res = def;
    break;
  case eMA1IR: 
    //res ="Multiple\\nAcceptor 1 IR";
    res = def;
    break;
  case eMA2IR:
    //res ="Multiple\\nAcceptor 2 IR";
    res = def;
    break;
  case eDBLALTFE: 
    //res ="Double Alternative\\nInitial Exon";
    res = def; //"purple";
    break;
  case eDBLALTLE: 
    //res ="Double Alternative\\nTerminal Exon";
    res = def; //"purple";
    break;
  case eDBLALTRI: 
    //res ="Double Alternative\\nRetained Intron";
    res = def; //"red";
    break;
  case eSDIR: 
    //res ="Constitutive\\nDonor IR";
    res = def; //"red";
    break;
  case eSAIR:
    //res ="Constitutive\\nAcceptor IR";
    //res = "red";
    break;
  case eFCONE: 
    ////res ="Initial\\nConstitutive Exon";
    //res = "purple";
    break;
  case eLCONE: 
    //res ="Terminal\\nConstitutive Exon";
    //res = "purple";
    break;
  case eFMDE: 
    //res ="Initial\\nMultiple Donor";
    //res = "purple";
    break;
  case eLMAE: 
    //res ="Terminal\\nMultiple Acceptor";
    //res = "purple";
    break;
  case eDBLALT: 
    //res ="Double Alternative\\nInternal";
    res = def; //"purple";
    break;
    //case eStateSize:
        //res ="eSTATESIZE";
  default:
    res = "error";
  }
  return res;
}

void 
StateTrans::updateTrans(TestModel& best_tm, const StateTrans& curr_rm, const TestModel& curr_tm) {
  const double best_score = best_tm.calcFscore();
  const double curr_score = curr_tm.calcFscore();
  if( curr_score > best_score ) {
    for(unsigned ti = 0; ti < eStateSize; ++ti) {
      state_t to = (state_t)ti;
      _init[to] = curr_rm._init[to];
    }
    for(unsigned fi = 0; fi < eStateSize; ++fi) {
      state_t fr = (state_t)fi;
      for(unsigned ti = 0; ti < eStateSize; ++ti) {
	state_t to = (state_t)ti;
	cout<<"updateTrans: from "<<state2str(fr)<<" to "<<state2str(to)<<" found better val: "<<curr_score<<" old="<<best_score<<endl;
	_trans[fr][to] = curr_rm._trans[fr][to];
      }
    }
    best_tm = curr_tm;
  }
}

void
StateTrans::adjustTrans(const StateTrans& brm, sc_t& pair1, const int cRANGE) {
  int& cnt = pair1.second;
  state_t fr = pair1.first.first;
  state_t to = pair1.first.second;
  double modVal=0;
  double half = (double)cRANGE/2.0;
  if( cnt < half ) {
    const double ex = half-cnt;
    if( to == eStateSize ) {
      _init[fr] = brm._init[fr];
    } else {
      _trans[fr][to] = brm._trans[fr][to];
    }
    modVal = pow(1.1,ex);
  } else {
    const double ex = (cRANGE+1)-cnt;
    modVal = pow(0.1,ex);
  }
  double oldVal=-1,newVal=-1;
  if( to == eStateSize ) {
    oldVal=_init[fr];
    _init[fr] *= modVal;
    newVal=_init[fr];
  } else {
    oldVal=_trans[fr][to];
    _trans[fr][to] *= modVal;
    newVal=_trans[fr][to];
  }
  // assure at least one parameter is adjusted
  cout<<"AdjustTrans: "<<(int)cnt<<" "<<state2str(fr)<<" "<<state2str(to)<<" new val="<<newVal<<" oldVal="<<oldVal<<endl;
  --cnt;
}
