#ifndef _STATETRANS_H
#define _STATETRANS_H

#include <string>
#include <vector>
#include <ostream>
#include "TestModel.h"
#include "Histogram.h"
#include "SeqLabel.h"

class StateTrans {
 public:
  enum state_t { eCUIN = 0, eAUIN, eNCONE, eSAE, eMAE1, eMAE2, eCE, eSDE, eMDE1, eMDE2, eCDIN, eADIN,
		 eNFEIR, eFFEIR, eNLEIR, eLLEIR, eRI, eRIMD, eRIMA, eMD1IR, eMD2IR, eMA1IR, eMA2IR,
		 eDBLALTFE, eDBLALTLE, eDBLALTRI, eSDIR, eSAIR,
		 eFCONE, eLCONE, eFMDE, eLMAE, 
		 eDBLALT, eAINO, eCINO,eAEXO,eCEXO,
		 eStateSize 
  };
  typedef std::pair<std::pair<state_t,state_t>,int> sc_t;
 public:
  StateTrans();
  StateTrans(const StateTrans&);

  void read(const std::string&);
  void write(const std::string&) const;
  void normalize();

  void drawGraphViz() const;

  double getProb(const SeqLabel&, const SeqLabel&) const;
  double getProb(const SeqLabel&, const SeqLabel&,
		 const SeqLabel&, const SeqLabel&) const;

  void updateTrans(TestModel&, const StateTrans&, const TestModel&);
  void adjustTrans(const StateTrans&, sc_t&, const int);

  void update(const SeqLabel&, const SeqLabel&);
  void update(const SeqLabel&, const SeqLabel&,
	      const SeqLabel&, const SeqLabel&);
  
  StateTrans& operator+=(const StateTrans&);
  StateTrans& operator/=(int);

  void print(std::ostream&) const;
  static std::string
    state2pen(state_t);
  static std::string
    state2str(state_t);
  static std::string
    state2strReadable(state_t);
  static std::string
    state2color(state_t);
  static state_t 
    label2state(const SeqLabel&, const SeqLabel&, bool);

  double getInit(state_t st) const { return _init[st]; }
  double getTrans(state_t fr, state_t to) const { return _trans[fr][to]; }

 private:
  double getProb(state_t,state_t,int) const;
  double getProb(state_t,int) const;
  void update(state_t,int);
  void update(state_t,state_t,int);

 private:
  std::vector<double> _init;
  std::vector<std::vector<double> > _trans;
  std::vector<Histogram> _length;
};

typedef StateTrans::sc_t sc_t;
#endif // _STATETRANS_H
