double
TamuraNei::getProb(const std::string& to, const std::string& fr, double blen) const {
  const char tc = to[0];
  const char fc = fr[0];
  return getProb(tc,fc,blen);
}

double
TamuraNei::getProb(char tc, char fc, double blen) const {
  const unsigned ti = bioinfo::c2idx(tc);
  const unsigned fi = bioinfo::c2idx(fc);
  //if( ti == 4 || fi == 4 ) {
    //assert(0);
  //}
  //blen *= _mRate;

  const int kd = ti==fi?1:0;
  const double alpha = isPr(fc) ? _alphar : _alphay;

  // No Events
  const double nEvent = exp((-1*(alpha+_beta))*blen)*kd;
  // Transitions
  double sum = 0;

  char base = bioinfo::idx2c(0);
  int lsb=sameBaseType(fc,base);
  sum += getBaseFreq(base)*lsb;

  base = bioinfo::idx2c(1);
  lsb=sameBaseType(fc,base);
  sum += getBaseFreq(base)*lsb;

  base = bioinfo::idx2c(2);
  lsb=sameBaseType(fc,base);
  sum += getBaseFreq(base)*lsb;

  base = bioinfo::idx2c(3);
  lsb=sameBaseType(fc,base);
  sum += getBaseFreq(base)*lsb;

  const double toFreq = getBaseFreq(tc);
  const int sb=sameBaseType(tc,fc);
  const double pcnt = sum > 0 ? (toFreq/sum)*sb : 0;
  const double transVal = exp(-1*_beta*blen)*(1-exp(-1*alpha*blen))*pcnt;
  // Transversion (type 2)
  const double type2 = (1-exp(-1*_beta*blen))*toFreq;
  double res = nEvent + transVal + type2;
  //if( res < 0 || res > 1 ) {
    //assert(0);
  //}
  assert(res>=0 && res<=1);
  return res;
}
