#ifndef _TESTMODEL_H
#define _TESTMODEL_H

#include <vector>
#include <list>
#include <utility>
#include "SeqLabelVal.h"
#include "SeqLabel.h"
#include "seqtype.h"
#include "ModelType.h"


class TestModel {
 public:
  typedef SeqLabel::sclass_t sclass_t;
  typedef SeqLabel::blabel_t blabel_t;
 public:
  TestModel(const std::string&);
  TestModel(const TestModel&);

  void score(const fasta_seq&, const std::list<SeqLabel>&, const std::list<SeqLabel>&);

  TestModel&
    operator+=(const TestModel&);

  double calcFscore() const;
  double calcAvg() const;

  //double getSnPrc() const { return _tp_nuc/(_tp_nuc+_fn_nuc); }
  //double getSpPrc() const { return _tp_nuc/(_tp_nuc+_fp_nuc); }

  double getFp(int idx) const { assert(idx < (int)_fp.size()); return _fp[idx]; }
  double getTp(int idx) const { assert(idx < (int)_tp.size()); return _tp[idx]; }
  double getFn(int idx) const { assert(idx < (int)_fn.size()); return _fn[idx]; }
  double getTn(int idx) const { assert(idx < (int)_tn.size()); return _tn[idx]; }

  void setFp(int idx, double val) { assert(idx < (int)_fp.size()); _fp[idx]=val; }
  void setTp(int idx, double val) { assert(idx < (int)_tp.size()); _tp[idx]=val; }
  void setFn(int idx, double val) { assert(idx < (int)_fn.size()); _fn[idx]=val; }
  void setTn(int idx, double val) { assert(idx < (int)_tn.size()); _tn[idx]=val; }

  double getSn(int) const;
  double getSp(int) const;

  friend std::ostream& operator<<(std::ostream&, const TestModel&);

 private:
  std::vector<double> _fp, _tp, _fn, _tn;
  std::vector<double> _fp_splice, _tp_splice, _fn_splice, _tn_splice;
  std::string _id;
  std::vector<double> _fp_class, _tp_class, _fn_class;
  double _fp_nuc, _tp_nuc, _fn_nuc, _tn_nuc;
};

#endif // _TESTMODEL_H
