#ifndef _TRELLIS_H
#define _TRELLIS_H

#include "ModelType.h"
#include "stype.h"
#include "RateHmm.h"
#include "ParamSearch.h"
#include "RateTrans.h"
#include "HmmComponent.h"
#include <iostream>
#include <utility>

class Trellis {
 public:
  typedef ModelType::cname_t cname_t;
  typedef std::pair<htype_t,int> hpair_t;

  static RateHmm* buildHmm(htype_t,cname_t);

 public:
  Trellis() { }
  Trellis(RateHmm*);
  ~Trellis();

  static hpair_t
    extract_htype(const std::vector<std::string*>& svec,int, cname_t cn, int phase);
  static std::vector<std::vector<double> > getCodedTrans(cname_t,bool);
  static std::vector<double> getCodedInit(cname_t,bool);
  HmmComponent* getBestScore1(htype_t,int, spair_t&, bool ) const;
  spair_t getBestScore(htype_t,int) const;
  unsigned size() const { return _states.size(); }
  void print(std::ostream&) const;
  RateState& getState(int idx);
  spair_t getScorePrev(const RateState&) const;
  spair_t getScoreCurr(const RateState&) const;
  void set(const RateState& cs, const spair_t&, RateTrans*, const spair_t& );
  std::vector<ParamSearchPos>* getPath(htype_t,int);
  void add(bool,htype_t,int,RateHmm*);

  bool idx2cons(htype_t ht, int id);

  static int getPrevTime(htype_t,int);
  static int getTimes(htype_t);
  static int getSize(htype_t);
  static int getLevels(htype_t);

  static htype_t
    cname2htype(ModelType::cname_t,bool);

  static htype_t mname2htype(ModelType::mname_t,bool);

 private:
  std::list<std::vector<spair_t>* > _states;
  std::list<std::vector<spair_t>* > _currs;
  std::list<std::vector<RateTrans*>* > _prev;
  std::list<RateHmm*> _hmm;
  std::list<int> _phase;
};
 
#endif // _TRELLIS_H
