//#include "Phylogeny.h"
#include "ParamFile.h"
#include "utils.h"
#include "ModelManager.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

static void
partition_data(vector<fasta>& train, vector<fasta>& test, const fasta& , const fasta&); 
static void
dispatch(const fasta& aligned_targ, int rid, ofstream& submitFile, const string& fullpath, const string&);

AnnotationInfo* AnnotationInfo::_instance = NULL;

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      string fullpath = argv[2];
      string cfile=argv[3];
      if( fullpath.size()==0 ) {
	fullpath ="./";
      } else {
	fullpath += "/";
      }
      ofstream submitFile(cfile.c_str());
      if( !submitFile ) {
	cerr<<"could not write "<<cfile<<endl;
	throw;
      }
      string executable="/fs/szgenefinding/ExAlt/bin/condor_exalt";
      submitFile<<"Executable	= "<<executable<<endl;
      submitFile<<"Universe	= standard"<<endl;

      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& altSeq = pf->getAltAlignTarg();
      fasta& conSeq = pf->getConAlignTarg();
      //altSeq.random_shuffle();
      //conSeq.random_shuffle();
      //cout<<"conSeq.size: "<<conSeq.size()<<endl;

      unsigned num_partitions = 4;
      vector<fasta> train(num_partitions), test(num_partitions);
      partition_data(train,test,altSeq,conSeq);
      for(unsigned i = 0; i < num_partitions; i++) {
	string tstr = fullpath + "models.txt";
	ostringstream ostrm;
	ostrm<<tstr<<"."<<i;
	string file = ostrm.str();
#if 0
	ModelManager mm;
	const fasta& train_set = train[i];
	mm.train(train_set);
	mm.write(file);
#endif
	const fasta& test_set = test[i];
	dispatch(test_set,i,submitFile,fullpath,file);
      }
    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}

void
partition_data_help(vector<fasta>& train, vector<fasta>& test, const fasta& masterlist) {
  const int num_p = train.size();
  //cout<<i<<" partition size: "<<partition<<endl;
  const int pwidth = masterlist.size()/num_p;
  for(int pari = 0; pari < num_p; ++pari) {
    assert((unsigned)pari < train.size() && (unsigned)pari < test.size());
    //fasta ltrain,ltest;
    fasta::const_iterator iter = masterlist.begin();
    const fasta::const_iterator stop = masterlist.end();
    int part = pari*pwidth, cnt = 0;
    //cout<<"pari: "<<pari<<" ps: "<<partsize[i]<<" "<<part<<" pwidth: "<<pwidth<<endl;
    for(; iter != stop; ++iter ) {
      if( cnt < part || cnt > (part+pwidth) ) {
	train[pari].push_back(*iter);
      } else {
	test[pari].push_back(*iter);
      }
      ++cnt;
    }
    //train.push_back(ltrain);
    //test.push_back(ltest);
  }
}

void
partition_data(vector<fasta>& train, vector<fasta>& test, const fasta& masterlist1, const fasta& masterlist2) {
  partition_data_help(train,test,masterlist1);
  ///masterlist2.print_header(cout);
  partition_data_help(train,test,masterlist2);
  //train.front().print_header(cout);
}

void
dispatch(const fasta& aligned_targ, int rid, ofstream& submitFile, const string& fullpath, const string& mfile) {
  const ParamFile* pf = ParamFile::getInstance();
  const fasta& targ = pf->getTarg();
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  fasta::const_iterator iter = aligned_targ.begin();
  fasta::const_iterator stop = aligned_targ.end();
  unsigned cnt = 0;
  for(;iter != stop; ++iter, ++cnt) {
    const fasta_seq& aligned_fs = **iter;
    fasta::const_iterator targ_match = targ.find_header(aligned_fs.get_header());
    if( targ_match == targ.end()) {
      cout<<"what happend no alignment? ";
      cout<<aligned_fs.get_header()<<endl;
      continue;
      //assert(0);
    }
    const fasta_seq& targ_fs = **targ_match;

    ParseHeader ph(ai);
    bool status = ph.parse(targ_fs);
    if( !status ) {
      cout<<"skiping????"<<targ_fs.get_header()<<endl;
      continue;
    }
    ostringstream pfile;
    pfile<<fullpath<<"pdir/tpfile."<<rid<<"."<<cnt<<".txt";
    ofstream paramFile(pfile.str().c_str());
    if(!paramFile) {
      cerr<<"error crap "<<paramFile<<endl;
    }
    paramFile<<"targ_seq dmel dum"<<endl;
    paramFile<<"annotation_info /fs/szgenefinding/ExAlt/data/bmatch/dmel-3.2.2-genelist.gff dum"<<endl;
    paramFile<<"tree /fs/szgenefinding/ExAlt/devel/ScoreColumn/mod.tree.4 dum"<<endl;
    paramFile<<"models "<<mfile<<" dum"<<endl;
    const vector<fasta>& alnSeq = pf->getAlignedSeq();
    const string& hdr = targ_fs.get_header();
    const OrgIdx& oidx = pf->getOrgIdx();
    for(unsigned i = 0; i < alnSeq.size(); ++i) {
      const fasta& fas = alnSeq[i];
      string ostr = oidx.idx2str(i);
      ostringstream ofile;
      ofile<<fullpath<<"seqs/"<<ostr<<"."<<rid<<"."<<cnt<<".fa";
      paramFile<<"alignment "<<ofile.str()<<" "<<ostr<<endl;
      //ofstream ofs(ofile.str().c_str());
      const fasta::const_iterator fnd = fas.find_header(hdr);
      if( fnd != fas.end()) {
	const fasta_seq& tf = **fnd;
	tf.write(ofile.str());
      }
    }
    ostringstream stdout;
    stdout<<fullpath<<"eoutput/stdout."<<rid<<"."<<cnt<<".txt";
    ostringstream stderr;
    stderr<<fullpath<<"eoutput/stdout."<<rid<<"."<<cnt<<".txt";
    submitFile<<"Arguments	= "<<pfile.str()<<endl;
    submitFile<<"Notification	= Error"<<endl;
    submitFile<<"Log	= "<<"cs.log"<<endl;
    submitFile<<"Output	= "<<stdout.str()<<endl;
    submitFile<<"Error	= "<<stderr.str()<<endl;
    submitFile<<"Queue"<<endl;
  }
}
