#include "ParamFile.h"
#include "ModelManager.h"
#include "utils.h"
#include "Options.h"
#include "AnnotationInfo.h"
#include "seqtype.h"
#include "stype.h"
#include "RunModel.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

AnnotationInfo* AnnotationInfo::_instance = NULL;

int 
main(int argc, char** argv) {
  int status = 0;
  const string cvs_tag = "1.0.0";
  string id = string(argv[0]) + string("-") + cvs_tag;
  try 
    {
      string cid = id + string(": prediction of alternatively spliced exons");
      Options::createInstance(argc,argv,cid);
      const Options& options = *(Options::getInstance());
      if( options._quit ) {
	usage(argv[0]);
	return 0;
      }
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

#if 0
      const string pfile = argv[1];
      const string& file = argv[2];
      const bool run_cons = atoi((argv[3]))==0 ? false : true;
      bool lim_frame = atoi((argv[4]))==0 ? false : true;
      const bool use_exon = atoi((argv[5]))==0 ? false : true;
      if(use_exon) {
	lim_frame=true;
      }
      string mheader;
      if(argc > 6 ) {
	mheader = argv[6];
      }
#endif
      const string& pfile = options._paramFile;
      const string& file = options._modelFileBase;
      const bool run_cons = options._oneExon;
      bool lim_frame = options._useFrame;
      const bool use_exon = options._useExon;
      if(use_exon) {
	lim_frame=true;
      }
      const string& mheader = options._header;

      cout<<"Reading param file: "<<pfile<<endl;
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& seq1 = pf->getAltAlignTarg();
      fasta& seq2 = pf->getConAlignTarg();
      seq1.append(seq2);
      ModelManager mm;
      cout<<"Reading model file: "<<file<<endl;
      if( mm.read(file) ) {
      	RunModel rm(mm);
      	const list<GffFeature*>* result = rm.run(seq1,mheader,run_cons,eLearnedFixedRate,lim_frame,use_exon,id);
	const string ofile = options._gffFile;
	ofstream ofs(ofile.c_str());
	if( !ofs ) {
	  cerr<<"Unable to write to ["<<ofile<<"]"<<endl;
	  status=1;
	} else {
	  GffFeature::printLst(ofs,*result);
	  delete result;
	}
      } else {
         cerr<<"Unable to read models in: ["<<file<<"]"<<endl;
      }
    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  if( status )
    usage(argv[0]);
  return status;
}
