#include "Phylogeny.h"
#include "ParamFile.h"
#include "ModelManager.h"
#include "utils.h"
#include "MarkovChain.h"
#include "AnnotationInfo.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include "SeqLabel.h"
#include "RunModel.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

AnnotationInfo* AnnotationInfo::_instance = NULL;


static void matchCount(const fasta& aligned_targ, vector<int>& mcount); 

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      initGlobalAAHashTable();
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& altSeq = pf->getAltAlignTarg();
      cout<<"altSeq.size: "<<altSeq.size()<<endl;

      const unsigned osize = oidx.size();

      vector<int> altMatchCount(osize,0);
      matchCount(altSeq,altMatchCount);

      cout<<"distr of matches in alt exons\n";
      for(unsigned i = 0; i < altMatchCount.size(); ++i) {
	cout<<i<<" "<<altMatchCount[i]<<endl;
      }

    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(const std::string& err) 
    {
      cerr<<"Unhandled exception, Problem: "<<err<<endl;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}

void matchCount(const fasta& aligned_targ, vector<int>& mcount) {
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  assert(ai);
  vector<Histogram> hist(5);
  const int mtype = 8;
  const int bsize = mtype-1;
  for(unsigned i = 0; i < hist.size(); ++i) {
    hist[i].init(1,mtype,bsize);
  }
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  for(;iter != stop; ++iter) {
    const fasta_seq& targ_fs = **iter;
    ParseHeader ph(ai);
    bool status = ph.parse(targ_fs);
    if( !status ) {
      cout<<"problem here????"<<targ_fs.get_header()<<endl;
    }
    if( targ_fs.get_header()==
	"CG8445_2R:-_CLASS:_MS_[RA_RB,11257782_11259382],_[RA,11259442_11259725:RB,11259442_11259695]_,[RA_RB,11259792_11260027]" ) {
      int stop = 0;
    }
    ExonDescrip& ed = ph.getExonDescrip();
    int cnt = ed.countCds();
    cout<<"cnt: "<<cnt<<endl;
    const string cstr = ph.getSpliceEvent(targ_fs);
    int idx = 0;
    if( cstr == "MS" ) {
      idx = 0;
    } else if( cstr == "IR" ) {
      idx = 1;
    } else if( cstr == "SE" ) {
      idx = 2;
    } else if( cstr == "CS" ) {
      idx = 3;
    } else {
      idx = 4;
    }
    hist[idx].store(cnt);
  }
  for(unsigned i = 0; i < hist.size(); ++i) {
    hist[i].finish();
    cout<<hist[i]<<endl<<endl;
  }
}
