#include "Phylogeny.h"
#include "ParamFile.h"
#include "ModelManager.h"
#include "utils.h"
#include "MarkovChain.h"
#include "AnnotationInfo.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include "SeqLabel.h"
#include "RunModel.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

AnnotationInfo* AnnotationInfo::_instance = NULL;

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      initGlobalAAHashTable();
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      const string models = argv[2];
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& altSeq = pf->getAltAlignTarg();
      fasta& conSeq = pf->getConAlignTarg();
      //altSeq.random_shuffle();
      //conSeq.random_shuffle();
      cout<<"altSeq.size: "<<altSeq.size()<<endl;
      cout<<"conSeq.size: "<<conSeq.size()<<endl;
      const bool run_cons = atoi((argv[3]))==0 ? false : true;
      string mheader;
      if(argc > 4 ) {
	mheader = argv[4];
      }
      unsigned num_partitions = 10;
      vector<fasta> train(num_partitions), test(num_partitions);
      partition_data(train,test,altSeq,conSeq);
      for(unsigned i = 0; i < num_partitions; i++) {
	//if( i != 1 ) continue; //debug
	ostringstream ostrm;
	ostrm<<models<<"."<<i;
	string file = ostrm.str();
	ModelManager mm2;
	const fasta& train_set = train[i];
	bool status = mm2.read(file);
	if( !status) {
	  ModelManager mm;
	  cout<<"Train size: "<<train_set.size()<<endl;
	  mm.train(train_set);
	  mm.write(file);
	  status = mm2.read(file);
	}
	assert(status);
	RunModel rm(mm2);
	const fasta& test_set = test[i];
	cout<<"Test size: "<<test_set.size()<<" iter: "<<i<<endl;
	rm.run(test_set,mheader,run_cons,eBasicFixedRate);
      }
    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}
