#include "Phylogeny.h"
#include "ParamFile.h"
#include "ModelManager.h"
#include "utils.h"
#include "MarkovChain.h"
#include "AnnotationInfo.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include "SeqLabel.h"
#include "RunModel.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

AnnotationInfo* AnnotationInfo::_instance = NULL;


static void matchCount(const fasta& aligned_targ, vector<int>& mcount, vector<int>& stype); 

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      initGlobalAAHashTable();
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& altSeq = pf->getAltAlignTarg();
      fasta& conSeq = pf->getConAlignTarg();
      cout<<"altSeq.size: "<<altSeq.size()<<endl;
      cout<<"conSeq.size: "<<conSeq.size()<<endl;

      const unsigned osize = oidx.size();

      vector<int> altMatchCount(osize,0);
      vector<int> stype(5,0);
      matchCount(altSeq,altMatchCount,stype);

      cout<<"distr of matches in alt exons\n";
      for(unsigned i = 0; i < altMatchCount.size(); ++i) {
	cout<<i<<" "<<altMatchCount[i]<<endl;
      }
      cout<<"distr of splice types"<<endl;
      for(unsigned i = 0; i < stype.size(); ++i) {
	cout<<i<<" "<<stype[i]<<endl;
      }

      vector<int> conMatchCount(osize,0);
      vector<int> done(5,0);
      matchCount(conSeq,conMatchCount,done);

      cout<<"distr of matches in con exons\n";
      for(unsigned i = 0; i < conMatchCount.size(); ++i) {
	cout<<i<<" "<<conMatchCount[i]<<endl;
      }

    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(const std::string& err) 
    {
      cerr<<"Unhandled exception, Problem: "<<err<<endl;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}

void matchCount(const fasta& aligned_targ, vector<int>& mcount, vector<int>& stype) {
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  fasta::const_iterator iter = aligned_targ.begin();
  const fasta::const_iterator stop = aligned_targ.end();
  for(;iter != stop; ++iter) {
    const fasta_seq& targ_fs = **iter;
    ParseHeader ph(ai);
    bool status = ph.parse(targ_fs);
    if( !status ) {
      cout<<"skiping????"<<targ_fs.get_header()<<endl;
      continue;
    }
    ExonDescrip& ed = ph.getExonDescrip();
    list<SeqLabel> klst = ed.getSeqLabelLst();
    ipair_t ip = ed.getStartStop();
    const int start_codon = ip.first;
    const int stop_codon = ip.second; //3rd position of last codon
    const bool isPrc=SeqLabel::isAllPrc(klst,start_codon,stop_codon);
    const bool multiplePrc=ed.isPrcAlt();
    if( !isPrc || multiplePrc ) 
      continue;


    int idx = 4; // other;
    size_t pos = targ_fs.get_header().find("_CS_");
    if(pos != string::npos) {
      idx = 0;
    }
    pos = targ_fs.get_header().find("_SE_");
    if(pos != string::npos) {
      idx = 1;
    }
    pos = targ_fs.get_header().find("_MS_");
    if(pos != string::npos) {
      idx = 2;
    }
    pos = targ_fs.get_header().find("_IR_");
    if(pos != string::npos) {
      idx = 3;
    }
    stype[idx]++;

    const vector<string*> svec = getMatchSeq(targ_fs);
    unsigned cnt = 0;
    for(unsigned icnt = 1; icnt < svec.size(); ++icnt) {
      if( svec[icnt] ) {
	++cnt;
      }
    }
    //cout<<"huh? "<<cnt<<endl;
    if( cnt == 0 ) {
      cout<<"what up no match: "<<targ_fs.get_header()<<endl;
    }
    ++mcount[cnt];
  }
}
