#include "Phylogeny.h"
#include "ParamFile.h"
#include "ModelManager.h"
#include "utils.h"
#include "MarkovChain.h"
#include "AnnotationInfo.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include "SeqLabel.h"
#include "RunModel.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

AnnotationInfo* AnnotationInfo::_instance = NULL;

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      initGlobalAAHashTable();
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      string mheader;
      if( argc > 2 ) {
	mheader = argv[2];
      }
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& altSeq = pf->getAltAlignTarg();
      fasta& conSeq = pf->getConAlignTarg();
		altSeq.append(conSeq);
#if 1
      vector<int> counter(6,0);
      vector<int> total(6,0);
		if( argc > 2 ) {
			ifstream ifs(mheader.c_str());
			if( !ifs ) {
				cerr<<"could not open "<<mheader<<endl;
				return 0;
         }
			string line;
			while(ifs>>line) {
      		RunModel::countPcntId1(altSeq,line,counter,total);
			}
      } else {
      RunModel::countPcntId1(altSeq,mheader,counter,total);
      for(unsigned type = 0; type < counter.size(); ++type) {
	cout<<type<<" "<<counter[type]<<endl;
	cout<<"tot: "<<type<<" "<<total[type]<<endl;
      }
      cout<<endl;
      for(unsigned type = 0; type < counter.size(); ++type) {
	cout<<type<<" "<<(double)counter[type]/(double)total[type]<<endl;
      }
      cout<<endl;
      for(unsigned type = 0; type < counter.size(); ++type) {
	const double tpcid = (double)counter[type]/(double)total[type];
	unsigned f_type = type;
	if( type == 0 || type == 1 ) {
	  f_type= 4;
	} else if( type == 2 || type == 3 ){
	  f_type= 5;
	}
	const double fpcid = (double)counter[f_type]/(double)total[f_type];
	cout<<type<<" "<<tpcid/fpcid<<endl;
      }
	}
#endif
    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(const std::string& err) 
    {
      cerr<<"Unhandled exception, Problem: "<<err<<endl;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}
