#include "Phylogeny.h"
#include "ParamFile.h"
#include "ModelManager.h"
#include "utils.h"
#include "MarkovChain.h"
#include "AnnotationInfo.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include "SeqLabel.h"
#include "RunModel.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

AnnotationInfo* AnnotationInfo::_instance = NULL;

void transfer(fasta&,fasta&);

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      initGlobalAAHashTable();
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      const string models = argv[2];
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& altSeq = pf->getAltAlignTarg();
      fasta& conSeq = pf->getConAlignTarg();
      transfer(altSeq,conSeq);
      //altSeq.random_shuffle();
      //conSeq.random_shuffle();
      cout<<"altSeq.size: "<<altSeq.size()<<endl;
      cout<<"conSeq.size: "<<conSeq.size()<<endl;
      const bool run_cons = atoi((argv[3]))==0 ? false : true;
      string mheader;
      if(argc > 4 ) {
	mheader = argv[4];
      }

      unsigned num_partitions = 10;
      vector<fasta> train(num_partitions), test(num_partitions);
      partition_data(train,test,altSeq,conSeq);
      for(unsigned i = 0; i < num_partitions; i++) {
	//if( i != 1 ) continue; //debug
	ostringstream ostrm;
	ostrm<<models<<"."<<i;
	string file = ostrm.str();
	ModelManager mm2;
	const fasta& train_set = train[i];
	//fasta mlTrain,discTrain;
	//partition_train(train_set,mlTrain,discTrain,0.1);
	bool status = mm2.read(file);
	if( !status) {
	  ModelManager mm;
	  cout<<"Train size: "<<train_set.size()<<endl;
	  mm.train(train_set);
	  mm.write(file);
	  status = mm2.read(file);
	  assert(status);
	}
	continue;
	if(status) {
	  RunModel rm(mm2);
	  rm.discTrain(discTrain,mheader);
	  //mm2.trainRates(train_set);
	  mm2.write(file);
	  exit(0);
	}
	if(status) {
	  RunModel rm(mm2);
	  const fasta& test_set = test[i];
	  cout<<"Test size: "<<test_set.size()<<" iter: "<<i<<endl;
	  rm.run(test_set,mheader,run_cons,false);
	}
      }
    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}

void 
transfer(fasta& alt, fasta& con) {
	list<fasta::iterator> rem;
	fasta::iterator iter = alt.begin();
	const fasta::iterator stop = alt.begin();
	for(; iter != stop; ++iter) {
		const fasta_seq& fs = *iter;
		const string& head =fs.get_header();
		if( strstr(head.c_str(),"_CS_") ) {
			con.push_back(fs);
			rem.push_back(iter);
      }
   }
	list<fasta::iterator>::iterator i1=rem.begin();
	const list<fasta::iterator>::iterator s1=rem.end();
	for(; i1 != s1; ++i1) {
		alt.erase(*i1);
	}
}
