#include "Phylogeny.h"
#include "ParamFile.h"
#include "Options.h"
#include "ModelManager.h"
#include "utils.h"
#include "MarkovChain.h"
#include "AnnotationInfo.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include "SeqLabel.h"
#include "RunModel.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

AnnotationInfo* AnnotationInfo::_instance = NULL;

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      initGlobalAAHashTable();
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      const string models = argv[2];
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& altSeq = pf->getAltAlignTarg();
      fasta& conSeq = pf->getConAlignTarg();
      transfer(altSeq,conSeq);
      //altSeq.random_shuffle();
      //conSeq.random_shuffle();
      cout<<"altSeq.size: "<<altSeq.size()<<endl;
      cout<<"conSeq.size: "<<conSeq.size()<<endl;
      const bool run_cons = atoi((argv[3]))==0 ? false : true;
      //const bool sep_cons = atoi((argv[4]))==0 ? false : true;
      const bool sep_cons =false;
      bool useFrame = atoi((argv[4]))==0 ? false : true;
      const bool useExon = atoi((argv[5]))==0 ? false : true;
      if( useExon ) {
	useFrame=true;
      }
      string mheader;
      if(argc > 7 ) {
	mheader = argv[7];
      }
	   const string logFile=argv[6];
		Options::createInstance(0,NULL,"dum");
		Options::getInstance()->_logStream = new ofstream(logFile.c_str());

      unsigned num_partitions = 10;
      vector<fasta> train(num_partitions), test(num_partitions);
      partition_data(train,test,altSeq,conSeq);
      for(unsigned i = 0; i < num_partitions; i++) {
	//if( i != 6 ) continue; //debug
	ostringstream ostrm;
	ostrm<<models<<"."<<i;
	string file = ostrm.str();
	ModelManager mm2(sep_cons);
	cout<<"file: "<<file<<endl;
	bool status = mm2.read(file);
	//status=false;
	if( !status) {
	  const fasta& train_set = train[i];
	  cout<<"Train size: "<<train_set.size()<<endl;
	  ModelManager mm(sep_cons);
	  mm.train(train_set);
	  //bool ts = mm.readLFR(file);
	  //assert(ts);
	  mm.write(file);
	  continue;
	}
	assert(status);
	RunModel rm2(mm2);
	const fasta& test_set = test[i];
	cout<<"Test size: "<<test_set.size()<<" iter: "<<i<<endl;
	rm2.run(test_set,mheader,run_cons,eLearnedFixedRate,useFrame,useExon,"");
      }
    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(const std::string& err) 
    {
      cerr<<"Unhandled exception, Problem: "<<err<<endl;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}
