#include "ParamFile.h"
#include "ModelManager.h"
#include "utils.h"
#include "AnnotationInfo.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

static void
filt(fasta& aligned_targ); 

AnnotationInfo* AnnotationInfo::_instance = NULL;

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      initGlobalAAHashTable();
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      fasta& altSeq = pf->getAltAlignTarg();
      fasta& conSeq = pf->getConAlignTarg();
      altSeq.random_shuffle();
      conSeq.random_shuffle();
	//reduce size by 2/3
	//unsigned asize = 2*(altSeq.size()/3);
	//for(unsigned cnt = 0; cnt < asize; ++cnt) {
		//altSeq.pop_front();
   //}
	//unsigned csize = 2*(conSeq.size()/3);
	//for(unsigned cnt = 0; cnt < csize; ++cnt) {
		//conSeq.pop_front();
   //}
		filt(altSeq);
		filt(conSeq);
      cout<<"altSeq.size: "<<altSeq.size()<<endl;
      cout<<"conSeq.size: "<<conSeq.size()<<endl;
		altSeq.write("alt.out");
		conSeq.write("con.out");
    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(const std::string& err) 
    {
      cerr<<"Unhandled exception, Problem: "<<err<<endl;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}

void
filt(fasta& aligned_targ) {
  const AnnotationInfo* ai = AnnotationInfo::getInstance();
  list<fasta::iterator> rem;
  fasta::iterator iter = aligned_targ.begin();
  const fasta::iterator stop = aligned_targ.end();
  unsigned cnt = 0;
  for(;iter != stop; ++iter, ++cnt) {
    const fasta_seq& aligned_fs = **iter;
    const fasta_seq& targ_fs = aligned_fs;
    if(targ_fs.get_seq().length() > 10000) {
      cout<<"too damn long skipping for debug phase: "<<targ_fs.get_header()<<" "<<targ_fs.get_seq().length()<<endl;
      continue;
    }
    cout<<targ_fs.get_header()<<endl;
    ParseHeader ph(ai);
    bool status = ph.parse(targ_fs);
    if( !status ) {
      cout<<"skiping????"<<targ_fs.get_header()<<endl;
      rem.push_back(iter);
		continue;
    }
    ExonDescrip& ed = ph.getExonDescrip();
    ed.print(); cout<<endl;
    ipair_t ip = ed.getStartStop();
    list<SeqLabel> klst = ed.getSeqLabelLst();
    const int start_codon = ip.first;
    const int stop_codon = ip.second; //3rd position of last codon
    bool isPrc=SeqLabel::isAllPrc(klst,start_codon,stop_codon);
    if( !isPrc ) {
      rem.push_back(iter);
		continue;
    }
    const vector<string*> svec = getMatchSeq(targ_fs);
    bool valid = true;
    for(unsigned i = 0; i < svec.size(); ++i) {
      if(svec[i] ) {
         double pcntId = calcPcntId(*(svec[0]),*(svec[i]),1,svec[0]->size());
         if( pcntId < 0.9 ) {
            valid = false;
            break;
         }
      }
    }
    if( !valid ) {
      rem.push_back(iter);
		continue;
	 }
  }
  list<fasta::iterator>::iterator it = rem.begin();
  const list<fasta::iterator>::iterator st = rem.end();
	for(; it != st; ++it) {
		aligned_targ.erase(*it);
   }
}
