#include "Phylogeny.h"
#include "ParamFile.h"
#include "ModelManager.h"
#include "utils.h"
#include "MarkovChain.h"
#include "AnnotationInfo.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include "SeqLabel.h"
#include "RunModel.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

static void
partition_data(vector<fasta>& train, vector<fasta>& test, const fasta& , const fasta&); 

AnnotationInfo* AnnotationInfo::_instance = NULL;

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);
      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& altSeq = pf->getAltAlignTarg();
      fasta& conSeq = pf->getConAlignTarg();
      //altSeq.random_shuffle();
      //conSeq.random_shuffle();
      cout<<"conSeq.size: "<<conSeq.size()<<endl;

      unsigned num_partitions = 4;
      vector<fasta> train(num_partitions), test(num_partitions);
      partition_data(train,test,altSeq,conSeq);
      for(unsigned i = 0; i < num_partitions; i++) {
	string tstr = "models.txt";
	ostringstream ostrm;
	ostrm<<tstr<<"."<<i;
	string file = ostrm.str();
#if 0
	ModelManager mm;
	const fasta& train_set = train[i];
	mm.train(train_set);
	mm.write(file);
#endif

#if 1
	ModelManager mm2;
	mm2.read(file);
	RunModel rm(mm2);
	const fasta& test_set = test[i];
	rm.run(test_set);
#endif
      }
    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}

void
partition_data_help(vector<fasta>& train, vector<fasta>& test, const fasta& masterlist) {
  const int num_p = train.size();
  //cout<<i<<" partition size: "<<partition<<endl;
  const int pwidth = masterlist.size()/num_p;
  for(int pari = 0; pari < num_p; ++pari) {
    assert((unsigned)pari < train.size() && (unsigned)pari < test.size());
    //fasta ltrain,ltest;
    fasta::const_iterator iter = masterlist.begin();
    const fasta::const_iterator stop = masterlist.end();
    int part = pari*pwidth, cnt = 0;
    //cout<<"pari: "<<pari<<" ps: "<<partsize[i]<<" "<<part<<" pwidth: "<<pwidth<<endl;
    for(; iter != stop; ++iter ) {
      if( cnt < part || cnt > (part+pwidth) ) {
	train[pari].push_back(*iter);
      } else {
	test[pari].push_back(*iter);
      }
      ++cnt;
    }
    //train.push_back(ltrain);
    //test.push_back(ltest);
  }
}

void
partition_data(vector<fasta>& train, vector<fasta>& test, const fasta& masterlist1, const fasta& masterlist2) {
  partition_data_help(train,test,masterlist1);
  ///masterlist2.print_header(cout);
  partition_data_help(train,test,masterlist2);
  //train.front().print_header(cout);
}
