#include "Phylogeny.h"
#include "ParamFile.h"
#include "ModelManager.h"
#include "utils.h"
#include "MarkovChain.h"
#include "AnnotationInfo.h"
#include "ParseHeader.h"
#include "seqtype.h"
#include "stype.h"
#include "SeqLabel.h"
#include "RunModel.h"
#include <utility>
#include <stdexcept>
#include <sstream>
#include <iostream>
#include <list>
#include <fstream>
#include <time.h>
#include <vector>
#include <string>
#include <sys/types.h> 
#include <unistd.h>
#include <iomanip>
#include <sstream>

using std::ostringstream;
using std::string;
using std::cerr;
using std::cout;
using std::endl;
using std::list;
using std::ifstream;
using std::ofstream;
using std::vector;

AnnotationInfo* AnnotationInfo::_instance = NULL;

int main(int argc, char** argv) {
  int status = 0;
  try 
    {
      //cout<<std::setprecision(10);
      time_t tm = time(NULL);
      srand(tm);      srand48(tm);  // stl uses lrand48 (when available)

      const string pfile = argv[1];
      const string ofile = argv[2];
      ParamFile::createInstance(pfile);

      ParamFile* pf = ParamFile::getInstance();
      const OrgIdx oidx = pf->getOrgIdx();

      fasta& altSeq = pf->getAltAlignTarg();
      fasta& conSeq = pf->getConAlignTarg();
      ConsManager cm_sum;
      StateTrans st_sum;
      ModelHash nm_sum;
      const unsigned num_trials = 10;
      const unsigned num_types = 23;
      cout<<"altSeq.size: "<<altSeq.size()<<endl;
      cout<<"conSeq.size: "<<conSeq.size()<<endl;
      vector<vector<Model*> > vec(num_trials);
      for(unsigned mcnt=0; mcnt < num_trials; ++mcnt) {
	cout<<"Trial "<<mcnt<<endl;
	altSeq.random_shuffle();
	conSeq.random_shuffle();
	const unsigned num_partitions = 1;
	fasta train;
	const double pcnt = 0.8;
	partition_data(train,altSeq,conSeq,pcnt);
	for(unsigned i = 0; i < 1; i++) {
	  string tstr = "models.txt";
	  ostringstream ostrm;
	  ostrm<<tstr<<"."<<i;
	  string file = ostrm.str();
	  ModelManager mm;
	  const fasta& train_set = train;
	  mm.train(train_set);
	  const ModelHash& nm = mm.getModelKmerHash();
	  //const ModelHash& nm = mm.getModelHash();
	  //const StateTrans& st = mm.getStateTrans();
	  const ConsManager& cm = mm.getConsManager();
	  cout<<"CURR "<<i<<endl;
	  //st.print(cout);
	  nm.print(cout);
	  cm.print(cout);
	  vec[mcnt].resize(num_types);
	  // in theory these should establish a baseline
	  vec[mcnt][0] =nm.compDiff(ModelType::eAintronNonConDn,ModelType::eCintronNonConDn);
	  vec[mcnt][1] =nm.compDiff(ModelType::eCintronNonConDn,ModelType::eCintronNonConUp);
	  vec[mcnt][2] =nm.compDiff(ModelType::eAintronNonConDn,ModelType::eAintronNonConUp);
	  // not expecting much difference
	  vec[mcnt][3] =nm.compDiff(ModelType::eAintronConUp,ModelType::eAintronConDn);
	  vec[mcnt][4] =nm.compDiff(ModelType::eCintronConUp,ModelType::eCintronConDn);
	  // would love to see some difference, but not sure
	  vec[mcnt][5] =nm.compDiff(ModelType::eAintronConDn,ModelType::eCintronConDn);
	  vec[mcnt][6] =nm.compDiff(ModelType::eAintronConUp,ModelType::eCintronConUp);

	  // probably the biggest diff
	  vec[mcnt][7] =nm.compDiff(ModelType::eAintronConDn,ModelType::eAintronNonConDn);
	  vec[mcnt][8] =nm.compDiff(ModelType::eCintronConDn,ModelType::eCintronNonConDn);
	  vec[mcnt][9] =nm.compDiff(ModelType::eAintronConUp,ModelType::eAintronNonConUp);
	  vec[mcnt][10] =nm.compDiff(ModelType::eCintronConUp,ModelType::eCintronNonConUp);

	  vec[mcnt][11] =nm.compDiff(ModelType::eAaccCon,ModelType::eCaccCon);
	  vec[mcnt][12] =nm.compDiff(ModelType::eAdonCon,ModelType::eCdonCon);
	  vec[mcnt][13] =nm.compDiff(ModelType::eAdonCon,ModelType::eDonNonc);
	  vec[mcnt][14] =nm.compDiff(ModelType::eAaccCon,ModelType::eAccNonc);

	  vec[mcnt][15] =nm.compDiff(ModelType::eAexonNonConPrc,ModelType::eCexonNonConPrc);
	  vec[mcnt][17] =nm.compDiff(ModelType::eAexonConPrc,ModelType::eAexonNonConPrc);
	  vec[mcnt][18] =nm.compDiff(ModelType::eCexonConPrc,ModelType::eCexonNonConPrc);
	  vec[mcnt][16] =nm.compDiff(ModelType::eAexonConPrc,ModelType::eCexonConPrc);

	  vec[mcnt][19] =nm.compDiff(ModelType::eAexonNonConNpc,ModelType::eCexonNonConNpc);
	  vec[mcnt][21] =nm.compDiff(ModelType::eAexonConNpc,ModelType::eAexonNonConNpc);
	  vec[mcnt][22] =nm.compDiff(ModelType::eCexonConNpc,ModelType::eCexonNonConNpc);
	  vec[mcnt][20] =nm.compDiff(ModelType::eAexonConNpc,ModelType::eCexonConNpc);
	}
      }
      vector<Model*> avg(num_types);
      
      // compute std dev
      for(unsigned i = 0; i < num_trials; ++i) {
	for(unsigned j = 0; j < num_types; ++j) {
	  if( i==0 ) {
	    avg[j] = vec[i][j];
	  } else {
	    *(avg[j]) += (*vec[i][j]);
	  }
	}
      }
      for(unsigned j = 0; j < num_types; ++j) {
	(*avg[j]) /= num_trials;
      }
      vector<Model*> stdev(num_types);
      for(unsigned i = 0; i < num_trials; ++i) {
	for(unsigned j = 0; j < num_types; ++j) {
	  Model* diff = *(vec[i][j])-(*avg[j]);
	  (*diff) *= (*diff);
	  if( i == 0 ) {
	    stdev[j] = diff;
	  } else {
	    *(stdev[j]) += *diff;
	    delete diff;
	  }
	}
      }
      ofstream ofs(ofile.c_str());
      if( !ofs ) {
	cerr<<"could not open "<<ofile<<endl;
	return -1;
      }
      //const double cConf = 2.57; //0.95
      const double cConf = 2.84; //0.99 sample=20
      //const double cConf = 9.92; //0.99 2 samples
      for(unsigned i = 0; i < num_types; ++i) {
	cout<<"Avg Type "<<i<<endl;
	cout<<"Avg: "; avg[i]->print(cout);cout<<endl;
	cout<<"Std: "; 
	stdev[i]->print(cout); 
	cout<<endl;
	(*stdev[i])*=cConf;
	cout<<"Std + confidence: "; 
	stdev[i]->print(cout); 
	(*avg[i])-=(*stdev[i]);
	cout<<"Res: "; avg[i]->print(cout);cout<<endl;
	avg[i]->gnuPlot(ofs);
	ofs<<endl<<endl;
      }
    }
  catch(std::exception& err)
    {
      cerr<<err.what()<<endl;
      status=1;
    }
  catch(...) 
    {
      cerr<<"Unhandled exception, Problems"<<endl;
      status=1;
    }
  return status;
}
