#ifndef _UTILS_H
#define _UTILS_H

#include "seqtype.h"
#include "stype.h"
#include "SeqLabel.h"
#include "GffFeature.h"
#include <ext/hash_map>
#include <string>
#include <vector>
#include <list>
#include <ostream>
#include <utility>


class OrgIdx;
class MarkovChain;

void usage(const char*);
bool isPrcCheck(int,int,int);
int
frame2exalt(int frame, int pos, int startc);
bool
hasNonConsensusSplice(const std::string& tseq, const std::list<SeqLabel>& lst);
bool
hasGcDon(const std::string& tseq, const std::list<SeqLabel>& lst);
void
transfer(fasta& alt, fasta& con);
double calcPcntId(const std::string&, const std::string&, int, int);
void
getPhases(std::vector<int>& ftrack, const SeqLabel& tl, int start);
void
getPhases(std::vector<int>& ftrack, int lpos, int start);

char codon2aa(const std::string&);

bool
isCons(unsigned i, const std::vector<std::string*>& svec,int);

void rghtFrm2LeftFrm(std::vector<int>&,int,int);
const std::vector<std::string*>*
getMatchSeqPtr(const fasta_seq&);

const std::vector<std::string*>
getMatchSeq(const fasta_seq&);

inline char
getDigitPos0(int val);
inline char
getDigitPos1(int val);
void
makelower(std::string&);

void
partition_train(const fasta& , fasta& , fasta&, double ); 

void
partition_data(fasta& , const fasta& , const fasta&, double ); 

void
partition_data(std::vector<fasta>& , const fasta& , const fasta& ); 

void
partition_data(std::vector<fasta>&, std::vector<fasta>&, const fasta&, const fasta&);

void
printAlignment(std::ostream&, std::list<SeqLabel>::const_iterator, const std::list<SeqLabel>::const_iterator&, const std::vector<std::string*>&);

std::pair<double,int> 
  fillColumn(std::string&,const std::vector<std::string*>&, unsigned);

fasta::const_iterator 
qfind(fasta::const_iterator iter, fasta::const_iterator stop, const std::string&);

std::vector<const std::string*>
getMatchSeq(const fasta_seq&, const std::vector<fasta>&, const std::vector<std::string>&, const OrgIdx&);

void initGlobalAAHashTable();

  struct strhash
  {
    std::size_t operator()(const std::string& __s) const
    { return __gnu_cxx::__stl_hash_string(__s.c_str()); }
  };
  struct eqstr {
    bool operator()(const std::string& a, const std::string& b) const {
      return a==b;
    }
  };

  struct dhelp {
    dhelp() : _dat(-1) { }
    dhelp(double d) : _dat(d) { }
    dhelp(const dhelp& d) : _dat(d._dat) { }
    double _dat;
  };
typedef __gnu_cxx::hash_map<std::string, char, strhash, eqstr> aahash_t;
typedef __gnu_cxx::hash_map<unsigned, unsigned> hmap_unsigned_t;
typedef __gnu_cxx::hash_map<unsigned,std::string> hmap_str_t;

#include "utils.hpp"

#endif // _UTILS_H
