#include <istream>
#include <ostream>

namespace std
{

  /**
   *  @brief Global I/O operators for bitsets.
   *
   *  Direct I/O between streams and bitsets is supported.  
   *  Input will skip whitespace, only accept valid codes defined by _BitTraits
   *  characters
   * there are some serious issues about how general this can be
   * be very careful here, you're probably going to have to rewrite this
   * based on the instantiation of _BitTraits
   *
  */
  template<class _CharT, class _Traits, class _BitTraits, size_t _Nb>
  basic_istream<_CharT, _Traits>&
  operator>>(basic_istream<_CharT, _Traits>& __is, basic_string<bitset<_Nb>, _BitTraits>& __x) {
    // for now let's do this super hoky style
    typedef typename _Traits::char_type char_type;
    static typename _Traits::int_type __eof = _Traits::eof();
    const std::size_t char_size = _BitTraits::bit_size();
    const std::size_t left_bit_idx = (_Nb-1);
    bitset<_Nb> curr_bits;
    typename basic_istream<_CharT, _Traits>::sentry __sentry(__is);
    if (__sentry) {
      basic_streambuf<_CharT, _Traits>* __buf = __is.rdbuf();
	std::size_t pos = 0;
	bool dirty = false;
	for(;;) {
	  typename _Traits::int_type __c1 = __buf->sbumpc();
	  if (_Traits::eq_int_type(__c1, __eof)) {
	    __is.setstate(ios_base::eofbit);
	    break;
	  } else {
	    char_type __c2 = _Traits::to_char_type(__c1);
	    char_type __c  = __is.narrow(__c2, '*');
	    if(_BitTraits::valid_code(__c)) {
	      dirty = true;
	      bitset<_Nb> tmp_bits = _BitTraits::bit_code(__c);
	      size_t shift_width = left_bit_idx - pos - (char_size-1);
	      tmp_bits=tmp_bits<<shift_width;
	      curr_bits |= tmp_bits; //(_BitTraits::bit_code(__c)<<(pos-bs));
	    } else if (_Traits::eq_int_type(__buf->sputbackc(__c2),__eof)) { 
	      __is.setstate(ios_base::failbit);
	      break;
	    } else {
	      //__buf->sputbackc(__c2);
	      break;
	    }
	  }
	  pos += _BitTraits::bit_size();
	  //signed state = pos - (static_cast<signed>(_BitTraits::bit_size()) - 1);
	  if( pos > left_bit_idx ) {
	    dirty = false;
	    pos = 0;
	    __x.push_back(curr_bits);
	    curr_bits.reset();
	  }
	}
	if( dirty ) {
	  __x.push_back(curr_bits);
	}
	// do I need to check if something got read?
	if(  __x.empty() ) {
	  __is.setstate(ios_base::failbit);
	}
    }
    return __is;
  }

  template <class _CharT, class _Traits, typename _BitTraits, size_t _Nb>
    basic_ostream<_CharT, _Traits>&
    operator<<(basic_ostream<_CharT, _Traits>& __os, const basic_string<bitset<_Nb>, _BitTraits>& __x)
    {
      const std::size_t char_size = _BitTraits::bit_size();
      const std::size_t left_bit_idx = (_Nb-1);
      //const signed bsize = static_cast<signed>(_BitTraits::bit_size());
      for(unsigned i = 0; i < __x.size(); ++i) {
	const bitset<_Nb>& bits = __x[i];
	//cout<<"i: "<<i<<" bits: "<<bits<<endl;
	for(std::size_t pos = 0; pos < _Nb; pos += char_size) {
	  size_t shift_width = left_bit_idx - pos - (char_size-1);
	  bitset<_Nb> mask = (bits>>shift_width);
	  //cout<<"mask: "<<mask<<endl;
	  __os<<_BitTraits::char_code(mask);
	}
      }
      return __os;
    }

#if 0
  template<class _CharT, class _Traits, class _BitTraits, size_t _Nb>
  basic_istream<_CharT, _Traits>&
  operator>>(basic_istream<_CharT, _Traits>& __is, basic_string<bitset<_Nb>, _BitTraits>& __x) {
    // for now let's do this super hoky style
    typedef typename _Traits::char_type char_type;
    static typename _Traits::int_type __eof = _Traits::eof();
    const std::size_t char_size = _BitTraits::bit_size();
    const std::size_t left_bit_idx = (_Nb-1);
    bitset<_Nb> curr_bits;
    typename basic_istream<_CharT, _Traits>::sentry __sentry(__is);
    if (__sentry) {
      basic_streambuf<_CharT, _Traits>* __buf = __is.rdbuf();
	std::size_t pos = 0;
	bool dirty = false;
	for(;;) {
	  typename _Traits::int_type __c1 = __buf->sbumpc();
	  if (_Traits::eq_int_type(__c1, __eof)) {
	    __is.setstate(ios_base::eofbit);
	    break;
	  } else {
	    char_type __c2 = _Traits::to_char_type(__c1);
	    char_type __c  = __is.narrow(__c2, '*');
	    cout<<"here's.....johnny: ["<<__c<<"]"<<endl;
	    if(_BitTraits::valid_code(__c)) {
	      dirty = true;
	      bitset<_Nb> tmp_bits = _BitTraits::bit_code(__c);
	      size_t shift_width = pos > left_bit_idx ? (char_size-1) : left_bit_idx - pos - (char_size-1);
	      tmp_bits=tmp_bits<<shift_width;
	      curr_bits |= tmp_bits; //(_BitTraits::bit_code(__c)<<(pos-bs));
	    } else if (_Traits::eq_int_type(__buf->sputbackc(__c2),__eof)) { 
	      __is.setstate(ios_base::failbit);
	      break;
	    } else {
	      //__buf->sputbackc(__c2);
	      break;
	    }
	  }
	  if(pos <= left_bit_idx)
	    pos += _BitTraits::bit_size();
	  if( pos > left_bit_idx ) {
	    dirty = false;
	    cout<<"bingo! "<<curr_bits<<endl;
	    __x.push_back(curr_bits);
	    cout<<"here i am: "<<__x<<endl;
	    curr_bits.reset();
	  }
	}
	if( dirty ) {
	  __x.push_back(curr_bits);
	  cout<<"here i am: "<<__x<<endl;
	}
	// do I need to check if something got read?
	if(  __x.empty() ) {
	  __is.setstate(ios_base::failbit);
	}
    }
    return __is;
  }
#endif

}
